// Copyright 2014 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package fs

import (
	"io"
	"os"
)

type osFS struct{}

func (osFS) Open(name string) (File, error)        { return os.Open(name) }
func (osFS) Stat(name string) (os.FileInfo, error) { return os.Stat(name) }

var fs FileSystem = osFS{}

type FileSystem interface {
	Open(name string) (File, error)
}

type File interface {
	io.ReadWriteCloser
}

// Useful for tests. Not thread safe.
func ChangeFileSystem(filesystem FileSystem) {
	fs = filesystem
}

func Open(name string) (File, error) {
	return fs.Open(name)
}
