// Package stacktemplates provides operations for working with Heat templates.
// A Cloud Orchestration template is a portable file, written in a user-readable
// language, that describes how a set of resources should be assembled and what
// software should be installed in order to produce a working stack. The template
// specifies what resources should be used, what attributes can be set, and other
// parameters that are critical to the successful, repeatable automation of a
// specific application stack.
package stacktemplates
