// AUTO-GENERATED BY mkstdlib.go

package imports

var stdlib = map[string]string{
	"adler32.Checksum":                        "hash/adler32",
	"adler32.New":                             "hash/adler32",
	"adler32.Size":                            "hash/adler32",
	"aes.BlockSize":                           "crypto/aes",
	"aes.KeySizeError":                        "crypto/aes",
	"aes.NewCipher":                           "crypto/aes",
	"ascii85.CorruptInputError":               "encoding/ascii85",
	"ascii85.Decode":                          "encoding/ascii85",
	"ascii85.Encode":                          "encoding/ascii85",
	"ascii85.MaxEncodedLen":                   "encoding/ascii85",
	"ascii85.NewDecoder":                      "encoding/ascii85",
	"ascii85.NewEncoder":                      "encoding/ascii85",
	"asn1.BitString":                          "encoding/asn1",
	"asn1.Enumerated":                         "encoding/asn1",
	"asn1.Flag":                               "encoding/asn1",
	"asn1.Marshal":                            "encoding/asn1",
	"asn1.ObjectIdentifier":                   "encoding/asn1",
	"asn1.RawContent":                         "encoding/asn1",
	"asn1.RawValue":                           "encoding/asn1",
	"asn1.StructuralError":                    "encoding/asn1",
	"asn1.SyntaxError":                        "encoding/asn1",
	"asn1.Unmarshal":                          "encoding/asn1",
	"asn1.UnmarshalWithParams":                "encoding/asn1",
	"ast.ArrayType":                           "go/ast",
	"ast.AssignStmt":                          "go/ast",
	"ast.Bad":                                 "go/ast",
	"ast.BadDecl":                             "go/ast",
	"ast.BadExpr":                             "go/ast",
	"ast.BadStmt":                             "go/ast",
	"ast.BasicLit":                            "go/ast",
	"ast.BinaryExpr":                          "go/ast",
	"ast.BlockStmt":                           "go/ast",
	"ast.BranchStmt":                          "go/ast",
	"ast.CallExpr":                            "go/ast",
	"ast.CaseClause":                          "go/ast",
	"ast.ChanDir":                             "go/ast",
	"ast.ChanType":                            "go/ast",
	"ast.CommClause":                          "go/ast",
	"ast.Comment":                             "go/ast",
	"ast.CommentGroup":                        "go/ast",
	"ast.CommentMap":                          "go/ast",
	"ast.CompositeLit":                        "go/ast",
	"ast.Con":                                 "go/ast",
	"ast.DeclStmt":                            "go/ast",
	"ast.DeferStmt":                           "go/ast",
	"ast.Ellipsis":                            "go/ast",
	"ast.EmptyStmt":                           "go/ast",
	"ast.ExprStmt":                            "go/ast",
	"ast.Field":                               "go/ast",
	"ast.FieldFilter":                         "go/ast",
	"ast.FieldList":                           "go/ast",
	"ast.File":                                "go/ast",
	"ast.FileExports":                         "go/ast",
	"ast.Filter":                              "go/ast",
	"ast.FilterDecl":                          "go/ast",
	"ast.FilterFile":                          "go/ast",
	"ast.FilterFuncDuplicates":                "go/ast",
	"ast.FilterImportDuplicates":              "go/ast",
	"ast.FilterPackage":                       "go/ast",
	"ast.FilterUnassociatedComments":          "go/ast",
	"ast.ForStmt":                             "go/ast",
	"ast.Fprint":                              "go/ast",
	"ast.Fun":                                 "go/ast",
	"ast.FuncDecl":                            "go/ast",
	"ast.FuncLit":                             "go/ast",
	"ast.FuncType":                            "go/ast",
	"ast.GenDecl":                             "go/ast",
	"ast.GoStmt":                              "go/ast",
	"ast.Ident":                               "go/ast",
	"ast.IfStmt":                              "go/ast",
	"ast.ImportSpec":                          "go/ast",
	"ast.Importer":                            "go/ast",
	"ast.IncDecStmt":                          "go/ast",
	"ast.IndexExpr":                           "go/ast",
	"ast.Inspect":                             "go/ast",
	"ast.InterfaceType":                       "go/ast",
	"ast.IsExported":                          "go/ast",
	"ast.KeyValueExpr":                        "go/ast",
	"ast.LabeledStmt":                         "go/ast",
	"ast.Lbl":                                 "go/ast",
	"ast.MapType":                             "go/ast",
	"ast.MergeMode":                           "go/ast",
	"ast.MergePackageFiles":                   "go/ast",
	"ast.NewCommentMap":                       "go/ast",
	"ast.NewIdent":                            "go/ast",
	"ast.NewObj":                              "go/ast",
	"ast.NewPackage":                          "go/ast",
	"ast.NewScope":                            "go/ast",
	"ast.Node":                                "go/ast",
	"ast.NotNilFilter":                        "go/ast",
	"ast.ObjKind":                             "go/ast",
	"ast.Object":                              "go/ast",
	"ast.Package":                             "go/ast",
	"ast.PackageExports":                      "go/ast",
	"ast.ParenExpr":                           "go/ast",
	"ast.Pkg":                                 "go/ast",
	"ast.Print":                               "go/ast",
	"ast.RECV":                                "go/ast",
	"ast.RangeStmt":                           "go/ast",
	"ast.ReturnStmt":                          "go/ast",
	"ast.SEND":                                "go/ast",
	"ast.Scope":                               "go/ast",
	"ast.SelectStmt":                          "go/ast",
	"ast.SelectorExpr":                        "go/ast",
	"ast.SendStmt":                            "go/ast",
	"ast.SliceExpr":                           "go/ast",
	"ast.SortImports":                         "go/ast",
	"ast.StarExpr":                            "go/ast",
	"ast.StructType":                          "go/ast",
	"ast.SwitchStmt":                          "go/ast",
	"ast.Typ":                                 "go/ast",
	"ast.TypeAssertExpr":                      "go/ast",
	"ast.TypeSpec":                            "go/ast",
	"ast.TypeSwitchStmt":                      "go/ast",
	"ast.UnaryExpr":                           "go/ast",
	"ast.ValueSpec":                           "go/ast",
	"ast.Var":                                 "go/ast",
	"ast.Visitor":                             "go/ast",
	"ast.Walk":                                "go/ast",
	"atomic.AddInt32":                         "sync/atomic",
	"atomic.AddInt64":                         "sync/atomic",
	"atomic.AddUint32":                        "sync/atomic",
	"atomic.AddUint64":                        "sync/atomic",
	"atomic.AddUintptr":                       "sync/atomic",
	"atomic.CompareAndSwapInt32":              "sync/atomic",
	"atomic.CompareAndSwapInt64":              "sync/atomic",
	"atomic.CompareAndSwapPointer":            "sync/atomic",
	"atomic.CompareAndSwapUint32":             "sync/atomic",
	"atomic.CompareAndSwapUint64":             "sync/atomic",
	"atomic.CompareAndSwapUintptr":            "sync/atomic",
	"atomic.LoadInt32":                        "sync/atomic",
	"atomic.LoadInt64":                        "sync/atomic",
	"atomic.LoadPointer":                      "sync/atomic",
	"atomic.LoadUint32":                       "sync/atomic",
	"atomic.LoadUint64":                       "sync/atomic",
	"atomic.LoadUintptr":                      "sync/atomic",
	"atomic.StoreInt32":                       "sync/atomic",
	"atomic.StoreInt64":                       "sync/atomic",
	"atomic.StorePointer":                     "sync/atomic",
	"atomic.StoreUint32":                      "sync/atomic",
	"atomic.StoreUint64":                      "sync/atomic",
	"atomic.StoreUintptr":                     "sync/atomic",
	"atomic.SwapInt32":                        "sync/atomic",
	"atomic.SwapInt64":                        "sync/atomic",
	"atomic.SwapPointer":                      "sync/atomic",
	"atomic.SwapUint32":                       "sync/atomic",
	"atomic.SwapUint64":                       "sync/atomic",
	"atomic.SwapUintptr":                      "sync/atomic",
	"base32.CorruptInputError":                "encoding/base32",
	"base32.Encoding":                         "encoding/base32",
	"base32.HexEncoding":                      "encoding/base32",
	"base32.NewDecoder":                       "encoding/base32",
	"base32.NewEncoder":                       "encoding/base32",
	"base32.NewEncoding":                      "encoding/base32",
	"base32.StdEncoding":                      "encoding/base32",
	"base64.CorruptInputError":                "encoding/base64",
	"base64.Encoding":                         "encoding/base64",
	"base64.NewDecoder":                       "encoding/base64",
	"base64.NewEncoder":                       "encoding/base64",
	"base64.NewEncoding":                      "encoding/base64",
	"base64.StdEncoding":                      "encoding/base64",
	"base64.URLEncoding":                      "encoding/base64",
	"big.Int":                                 "math/big",
	"big.MaxBase":                             "math/big",
	"big.NewInt":                              "math/big",
	"big.NewRat":                              "math/big",
	"big.Rat":                                 "math/big",
	"big.Word":                                "math/big",
	"binary.BigEndian":                        "encoding/binary",
	"binary.ByteOrder":                        "encoding/binary",
	"binary.LittleEndian":                     "encoding/binary",
	"binary.MaxVarintLen16":                   "encoding/binary",
	"binary.MaxVarintLen32":                   "encoding/binary",
	"binary.MaxVarintLen64":                   "encoding/binary",
	"binary.PutUvarint":                       "encoding/binary",
	"binary.PutVarint":                        "encoding/binary",
	"binary.Read":                             "encoding/binary",
	"binary.ReadUvarint":                      "encoding/binary",
	"binary.ReadVarint":                       "encoding/binary",
	"binary.Size":                             "encoding/binary",
	"binary.Uvarint":                          "encoding/binary",
	"binary.Varint":                           "encoding/binary",
	"binary.Write":                            "encoding/binary",
	"bufio.ErrAdvanceTooFar":                  "bufio",
	"bufio.ErrBufferFull":                     "bufio",
	"bufio.ErrInvalidUnreadByte":              "bufio",
	"bufio.ErrInvalidUnreadRune":              "bufio",
	"bufio.ErrNegativeAdvance":                "bufio",
	"bufio.ErrNegativeCount":                  "bufio",
	"bufio.ErrTooLong":                        "bufio",
	"bufio.MaxScanTokenSize":                  "bufio",
	"bufio.NewReadWriter":                     "bufio",
	"bufio.NewReader":                         "bufio",
	"bufio.NewReaderSize":                     "bufio",
	"bufio.NewScanner":                        "bufio",
	"bufio.NewWriter":                         "bufio",
	"bufio.NewWriterSize":                     "bufio",
	"bufio.ReadWriter":                        "bufio",
	"bufio.Reader":                            "bufio",
	"bufio.ScanBytes":                         "bufio",
	"bufio.ScanLines":                         "bufio",
	"bufio.ScanRunes":                         "bufio",
	"bufio.ScanWords":                         "bufio",
	"bufio.Scanner":                           "bufio",
	"bufio.SplitFunc":                         "bufio",
	"bufio.Writer":                            "bufio",
	"build.AllowBinary":                       "go/build",
	"build.ArchChar":                          "go/build",
	"build.Context":                           "go/build",
	"build.Default":                           "go/build",
	"build.FindOnly":                          "go/build",
	"build.Import":                            "go/build",
	"build.ImportDir":                         "go/build",
	"build.ImportMode":                        "go/build",
	"build.IsLocalImport":                     "go/build",
	"build.NoGoError":                         "go/build",
	"build.Package":                           "go/build",
	"build.ToolDir":                           "go/build",
	"bytes.Buffer":                            "bytes",
	"bytes.Compare":                           "bytes",
	"bytes.Contains":                          "bytes",
	"bytes.Count":                             "bytes",
	"bytes.Equal":                             "bytes",
	"bytes.EqualFold":                         "bytes",
	"bytes.ErrTooLarge":                       "bytes",
	"bytes.Fields":                            "bytes",
	"bytes.FieldsFunc":                        "bytes",
	"bytes.HasPrefix":                         "bytes",
	"bytes.HasSuffix":                         "bytes",
	"bytes.Index":                             "bytes",
	"bytes.IndexAny":                          "bytes",
	"bytes.IndexByte":                         "bytes",
	"bytes.IndexFunc":                         "bytes",
	"bytes.IndexRune":                         "bytes",
	"bytes.Join":                              "bytes",
	"bytes.LastIndex":                         "bytes",
	"bytes.LastIndexAny":                      "bytes",
	"bytes.LastIndexFunc":                     "bytes",
	"bytes.Map":                               "bytes",
	"bytes.MinRead":                           "bytes",
	"bytes.NewBuffer":                         "bytes",
	"bytes.NewBufferString":                   "bytes",
	"bytes.NewReader":                         "bytes",
	"bytes.Reader":                            "bytes",
	"bytes.Repeat":                            "bytes",
	"bytes.Replace":                           "bytes",
	"bytes.Runes":                             "bytes",
	"bytes.Split":                             "bytes",
	"bytes.SplitAfter":                        "bytes",
	"bytes.SplitAfterN":                       "bytes",
	"bytes.SplitN":                            "bytes",
	"bytes.Title":                             "bytes",
	"bytes.ToLower":                           "bytes",
	"bytes.ToLowerSpecial":                    "bytes",
	"bytes.ToTitle":                           "bytes",
	"bytes.ToTitleSpecial":                    "bytes",
	"bytes.ToUpper":                           "bytes",
	"bytes.ToUpperSpecial":                    "bytes",
	"bytes.Trim":                              "bytes",
	"bytes.TrimFunc":                          "bytes",
	"bytes.TrimLeft":                          "bytes",
	"bytes.TrimLeftFunc":                      "bytes",
	"bytes.TrimPrefix":                        "bytes",
	"bytes.TrimRight":                         "bytes",
	"bytes.TrimRightFunc":                     "bytes",
	"bytes.TrimSpace":                         "bytes",
	"bytes.TrimSuffix":                        "bytes",
	"bzip2.NewReader":                         "compress/bzip2",
	"bzip2.StructuralError":                   "compress/bzip2",
	"cgi.Handler":                             "net/http/cgi",
	"cgi.Request":                             "net/http/cgi",
	"cgi.RequestFromMap":                      "net/http/cgi",
	"cgi.Serve":                               "net/http/cgi",
	"cipher.AEAD":                             "crypto/cipher",
	"cipher.Block":                            "crypto/cipher",
	"cipher.BlockMode":                        "crypto/cipher",
	"cipher.NewCBCDecrypter":                  "crypto/cipher",
	"cipher.NewCBCEncrypter":                  "crypto/cipher",
	"cipher.NewCFBDecrypter":                  "crypto/cipher",
	"cipher.NewCFBEncrypter":                  "crypto/cipher",
	"cipher.NewCTR":                           "crypto/cipher",
	"cipher.NewGCM":                           "crypto/cipher",
	"cipher.NewOFB":                           "crypto/cipher",
	"cipher.Stream":                           "crypto/cipher",
	"cipher.StreamReader":                     "crypto/cipher",
	"cipher.StreamWriter":                     "crypto/cipher",
	"cmplx.Abs":                               "math/cmplx",
	"cmplx.Acos":                              "math/cmplx",
	"cmplx.Acosh":                             "math/cmplx",
	"cmplx.Asin":                              "math/cmplx",
	"cmplx.Asinh":                             "math/cmplx",
	"cmplx.Atan":                              "math/cmplx",
	"cmplx.Atanh":                             "math/cmplx",
	"cmplx.Conj":                              "math/cmplx",
	"cmplx.Cos":                               "math/cmplx",
	"cmplx.Cosh":                              "math/cmplx",
	"cmplx.Cot":                               "math/cmplx",
	"cmplx.Exp":                               "math/cmplx",
	"cmplx.Inf":                               "math/cmplx",
	"cmplx.IsInf":                             "math/cmplx",
	"cmplx.IsNaN":                             "math/cmplx",
	"cmplx.Log":                               "math/cmplx",
	"cmplx.Log10":                             "math/cmplx",
	"cmplx.NaN":                               "math/cmplx",
	"cmplx.Phase":                             "math/cmplx",
	"cmplx.Polar":                             "math/cmplx",
	"cmplx.Pow":                               "math/cmplx",
	"cmplx.Rect":                              "math/cmplx",
	"cmplx.Sin":                               "math/cmplx",
	"cmplx.Sinh":                              "math/cmplx",
	"cmplx.Sqrt":                              "math/cmplx",
	"cmplx.Tan":                               "math/cmplx",
	"cmplx.Tanh":                              "math/cmplx",
	"color.Alpha":                             "image/color",
	"color.Alpha16":                           "image/color",
	"color.Alpha16Model":                      "image/color",
	"color.AlphaModel":                        "image/color",
	"color.Black":                             "image/color",
	"color.Color":                             "image/color",
	"color.Gray":                              "image/color",
	"color.Gray16":                            "image/color",
	"color.Gray16Model":                       "image/color",
	"color.GrayModel":                         "image/color",
	"color.Model":                             "image/color",
	"color.ModelFunc":                         "image/color",
	"color.NRGBA":                             "image/color",
	"color.NRGBA64":                           "image/color",
	"color.NRGBA64Model":                      "image/color",
	"color.NRGBAModel":                        "image/color",
	"color.Opaque":                            "image/color",
	"color.Palette":                           "image/color",
	"color.RGBA":                              "image/color",
	"color.RGBA64":                            "image/color",
	"color.RGBA64Model":                       "image/color",
	"color.RGBAModel":                         "image/color",
	"color.RGBToYCbCr":                        "image/color",
	"color.Transparent":                       "image/color",
	"color.White":                             "image/color",
	"color.YCbCr":                             "image/color",
	"color.YCbCrModel":                        "image/color",
	"color.YCbCrToRGB":                        "image/color",
	"cookiejar.Jar":                           "net/http/cookiejar",
	"cookiejar.New":                           "net/http/cookiejar",
	"cookiejar.Options":                       "net/http/cookiejar",
	"cookiejar.PublicSuffixList":              "net/http/cookiejar",
	"crc32.Castagnoli":                        "hash/crc32",
	"crc32.Checksum":                          "hash/crc32",
	"crc32.ChecksumIEEE":                      "hash/crc32",
	"crc32.IEEE":                              "hash/crc32",
	"crc32.IEEETable":                         "hash/crc32",
	"crc32.Koopman":                           "hash/crc32",
	"crc32.MakeTable":                         "hash/crc32",
	"crc32.New":                               "hash/crc32",
	"crc32.NewIEEE":                           "hash/crc32",
	"crc32.Size":                              "hash/crc32",
	"crc32.Table":                             "hash/crc32",
	"crc32.Update":                            "hash/crc32",
	"crc64.Checksum":                          "hash/crc64",
	"crc64.ECMA":                              "hash/crc64",
	"crc64.ISO":                               "hash/crc64",
	"crc64.MakeTable":                         "hash/crc64",
	"crc64.New":                               "hash/crc64",
	"crc64.Size":                              "hash/crc64",
	"crc64.Table":                             "hash/crc64",
	"crc64.Update":                            "hash/crc64",
	"crypto.Hash":                             "crypto",
	"crypto.MD4":                              "crypto",
	"crypto.MD5":                              "crypto",
	"crypto.MD5SHA1":                          "crypto",
	"crypto.PrivateKey":                       "crypto",
	"crypto.PublicKey":                        "crypto",
	"crypto.RIPEMD160":                        "crypto",
	"crypto.RegisterHash":                     "crypto",
	"crypto.SHA1":                             "crypto",
	"crypto.SHA224":                           "crypto",
	"crypto.SHA256":                           "crypto",
	"crypto.SHA384":                           "crypto",
	"crypto.SHA512":                           "crypto",
	"csv.ErrBareQuote":                        "encoding/csv",
	"csv.ErrFieldCount":                       "encoding/csv",
	"csv.ErrQuote":                            "encoding/csv",
	"csv.ErrTrailingComma":                    "encoding/csv",
	"csv.NewReader":                           "encoding/csv",
	"csv.NewWriter":                           "encoding/csv",
	"csv.ParseError":                          "encoding/csv",
	"csv.Reader":                              "encoding/csv",
	"csv.Writer":                              "encoding/csv",
	"debug.FreeOSMemory":                      "runtime/debug",
	"debug.GCStats":                           "runtime/debug",
	"debug.PrintStack":                        "runtime/debug",
	"debug.ReadGCStats":                       "runtime/debug",
	"debug.SetGCPercent":                      "runtime/debug",
	"debug.SetMaxStack":                       "runtime/debug",
	"debug.SetMaxThreads":                     "runtime/debug",
	"debug.Stack":                             "runtime/debug",
	"des.BlockSize":                           "crypto/des",
	"des.KeySizeError":                        "crypto/des",
	"des.NewCipher":                           "crypto/des",
	"des.NewTripleDESCipher":                  "crypto/des",
	"doc.AllDecls":                            "go/doc",
	"doc.AllMethods":                          "go/doc",
	"doc.Example":                             "go/doc",
	"doc.Examples":                            "go/doc",
	"doc.Filter":                              "go/doc",
	"doc.Func":                                "go/doc",
	"doc.IllegalPrefixes":                     "go/doc",
	"doc.Mode":                                "go/doc",
	"doc.New":                                 "go/doc",
	"doc.Note":                                "go/doc",
	"doc.Package":                             "go/doc",
	"doc.Synopsis":                            "go/doc",
	"doc.ToHTML":                              "go/doc",
	"doc.ToText":                              "go/doc",
	"doc.Type":                                "go/doc",
	"doc.Value":                               "go/doc",
	"draw.Draw":                               "image/draw",
	"draw.DrawMask":                           "image/draw",
	"draw.Drawer":                             "image/draw",
	"draw.FloydSteinberg":                     "image/draw",
	"draw.Image":                              "image/draw",
	"draw.Op":                                 "image/draw",
	"draw.Over":                               "image/draw",
	"draw.Quantizer":                          "image/draw",
	"draw.Src":                                "image/draw",
	"driver.Bool":                             "database/sql/driver",
	"driver.ColumnConverter":                  "database/sql/driver",
	"driver.Conn":                             "database/sql/driver",
	"driver.DefaultParameterConverter":        "database/sql/driver",
	"driver.Driver":                           "database/sql/driver",
	"driver.ErrBadConn":                       "database/sql/driver",
	"driver.ErrSkip":                          "database/sql/driver",
	"driver.Execer":                           "database/sql/driver",
	"driver.Int32":                            "database/sql/driver",
	"driver.IsScanValue":                      "database/sql/driver",
	"driver.IsValue":                          "database/sql/driver",
	"driver.NotNull":                          "database/sql/driver",
	"driver.Null":                             "database/sql/driver",
	"driver.Queryer":                          "database/sql/driver",
	"driver.Result":                           "database/sql/driver",
	"driver.ResultNoRows":                     "database/sql/driver",
	"driver.Rows":                             "database/sql/driver",
	"driver.RowsAffected":                     "database/sql/driver",
	"driver.Stmt":                             "database/sql/driver",
	"driver.String":                           "database/sql/driver",
	"driver.Tx":                               "database/sql/driver",
	"driver.Value":                            "database/sql/driver",
	"driver.ValueConverter":                   "database/sql/driver",
	"driver.Valuer":                           "database/sql/driver",
	"dsa.ErrInvalidPublicKey":                 "crypto/dsa",
	"dsa.GenerateKey":                         "crypto/dsa",
	"dsa.GenerateParameters":                  "crypto/dsa",
	"dsa.L1024N160":                           "crypto/dsa",
	"dsa.L2048N224":                           "crypto/dsa",
	"dsa.L2048N256":                           "crypto/dsa",
	"dsa.L3072N256":                           "crypto/dsa",
	"dsa.ParameterSizes":                      "crypto/dsa",
	"dsa.Parameters":                          "crypto/dsa",
	"dsa.PrivateKey":                          "crypto/dsa",
	"dsa.PublicKey":                           "crypto/dsa",
	"dsa.Sign":                                "crypto/dsa",
	"dsa.Verify":                              "crypto/dsa",
	"dwarf.AddrType":                          "debug/dwarf",
	"dwarf.ArrayType":                         "debug/dwarf",
	"dwarf.Attr":                              "debug/dwarf",
	"dwarf.AttrAbstractOrigin":                "debug/dwarf",
	"dwarf.AttrAccessibility":                 "debug/dwarf",
	"dwarf.AttrAddrClass":                     "debug/dwarf",
	"dwarf.AttrAllocated":                     "debug/dwarf",
	"dwarf.AttrArtificial":                    "debug/dwarf",
	"dwarf.AttrAssociated":                    "debug/dwarf",
	"dwarf.AttrBaseTypes":                     "debug/dwarf",
	"dwarf.AttrBitOffset":                     "debug/dwarf",
	"dwarf.AttrBitSize":                       "debug/dwarf",
	"dwarf.AttrByteSize":                      "debug/dwarf",
	"dwarf.AttrCallColumn":                    "debug/dwarf",
	"dwarf.AttrCallFile":                      "debug/dwarf",
	"dwarf.AttrCallLine":                      "debug/dwarf",
	"dwarf.AttrCalling":                       "debug/dwarf",
	"dwarf.AttrCommonRef":                     "debug/dwarf",
	"dwarf.AttrCompDir":                       "debug/dwarf",
	"dwarf.AttrConstValue":                    "debug/dwarf",
	"dwarf.AttrContainingType":                "debug/dwarf",
	"dwarf.AttrCount":                         "debug/dwarf",
	"dwarf.AttrDataLocation":                  "debug/dwarf",
	"dwarf.AttrDataMemberLoc":                 "debug/dwarf",
	"dwarf.AttrDeclColumn":                    "debug/dwarf",
	"dwarf.AttrDeclFile":                      "debug/dwarf",
	"dwarf.AttrDeclLine":                      "debug/dwarf",
	"dwarf.AttrDeclaration":                   "debug/dwarf",
	"dwarf.AttrDefaultValue":                  "debug/dwarf",
	"dwarf.AttrDescription":                   "debug/dwarf",
	"dwarf.AttrDiscr":                         "debug/dwarf",
	"dwarf.AttrDiscrList":                     "debug/dwarf",
	"dwarf.AttrDiscrValue":                    "debug/dwarf",
	"dwarf.AttrEncoding":                      "debug/dwarf",
	"dwarf.AttrEntrypc":                       "debug/dwarf",
	"dwarf.AttrExtension":                     "debug/dwarf",
	"dwarf.AttrExternal":                      "debug/dwarf",
	"dwarf.AttrFrameBase":                     "debug/dwarf",
	"dwarf.AttrFriend":                        "debug/dwarf",
	"dwarf.AttrHighpc":                        "debug/dwarf",
	"dwarf.AttrIdentifierCase":                "debug/dwarf",
	"dwarf.AttrImport":                        "debug/dwarf",
	"dwarf.AttrInline":                        "debug/dwarf",
	"dwarf.AttrIsOptional":                    "debug/dwarf",
	"dwarf.AttrLanguage":                      "debug/dwarf",
	"dwarf.AttrLocation":                      "debug/dwarf",
	"dwarf.AttrLowerBound":                    "debug/dwarf",
	"dwarf.AttrLowpc":                         "debug/dwarf",
	"dwarf.AttrMacroInfo":                     "debug/dwarf",
	"dwarf.AttrName":                          "debug/dwarf",
	"dwarf.AttrNamelistItem":                  "debug/dwarf",
	"dwarf.AttrOrdering":                      "debug/dwarf",
	"dwarf.AttrPriority":                      "debug/dwarf",
	"dwarf.AttrProducer":                      "debug/dwarf",
	"dwarf.AttrPrototyped":                    "debug/dwarf",
	"dwarf.AttrRanges":                        "debug/dwarf",
	"dwarf.AttrReturnAddr":                    "debug/dwarf",
	"dwarf.AttrSegment":                       "debug/dwarf",
	"dwarf.AttrSibling":                       "debug/dwarf",
	"dwarf.AttrSpecification":                 "debug/dwarf",
	"dwarf.AttrStartScope":                    "debug/dwarf",
	"dwarf.AttrStaticLink":                    "debug/dwarf",
	"dwarf.AttrStmtList":                      "debug/dwarf",
	"dwarf.AttrStride":                        "debug/dwarf",
	"dwarf.AttrStrideSize":                    "debug/dwarf",
	"dwarf.AttrStringLength":                  "debug/dwarf",
	"dwarf.AttrTrampoline":                    "debug/dwarf",
	"dwarf.AttrType":                          "debug/dwarf",
	"dwarf.AttrUpperBound":                    "debug/dwarf",
	"dwarf.AttrUseLocation":                   "debug/dwarf",
	"dwarf.AttrUseUTF8":                       "debug/dwarf",
	"dwarf.AttrVarParam":                      "debug/dwarf",
	"dwarf.AttrVirtuality":                    "debug/dwarf",
	"dwarf.AttrVisibility":                    "debug/dwarf",
	"dwarf.AttrVtableElemLoc":                 "debug/dwarf",
	"dwarf.BasicType":                         "debug/dwarf",
	"dwarf.BoolType":                          "debug/dwarf",
	"dwarf.CharType":                          "debug/dwarf",
	"dwarf.CommonType":                        "debug/dwarf",
	"dwarf.ComplexType":                       "debug/dwarf",
	"dwarf.Data":                              "debug/dwarf",
	"dwarf.DecodeError":                       "debug/dwarf",
	"dwarf.DotDotDotType":                     "debug/dwarf",
	"dwarf.Entry":                             "debug/dwarf",
	"dwarf.EnumType":                          "debug/dwarf",
	"dwarf.EnumValue":                         "debug/dwarf",
	"dwarf.Field":                             "debug/dwarf",
	"dwarf.FloatType":                         "debug/dwarf",
	"dwarf.FuncType":                          "debug/dwarf",
	"dwarf.IntType":                           "debug/dwarf",
	"dwarf.New":                               "debug/dwarf",
	"dwarf.Offset":                            "debug/dwarf",
	"dwarf.PtrType":                           "debug/dwarf",
	"dwarf.QualType":                          "debug/dwarf",
	"dwarf.Reader":                            "debug/dwarf",
	"dwarf.StructField":                       "debug/dwarf",
	"dwarf.StructType":                        "debug/dwarf",
	"dwarf.Tag":                               "debug/dwarf",
	"dwarf.TagAccessDeclaration":              "debug/dwarf",
	"dwarf.TagArrayType":                      "debug/dwarf",
	"dwarf.TagBaseType":                       "debug/dwarf",
	"dwarf.TagCatchDwarfBlock":                "debug/dwarf",
	"dwarf.TagClassType":                      "debug/dwarf",
	"dwarf.TagCommonDwarfBlock":               "debug/dwarf",
	"dwarf.TagCommonInclusion":                "debug/dwarf",
	"dwarf.TagCompileUnit":                    "debug/dwarf",
	"dwarf.TagConstType":                      "debug/dwarf",
	"dwarf.TagConstant":                       "debug/dwarf",
	"dwarf.TagDwarfProcedure":                 "debug/dwarf",
	"dwarf.TagEntryPoint":                     "debug/dwarf",
	"dwarf.TagEnumerationType":                "debug/dwarf",
	"dwarf.TagEnumerator":                     "debug/dwarf",
	"dwarf.TagFileType":                       "debug/dwarf",
	"dwarf.TagFormalParameter":                "debug/dwarf",
	"dwarf.TagFriend":                         "debug/dwarf",
	"dwarf.TagImportedDeclaration":            "debug/dwarf",
	"dwarf.TagImportedModule":                 "debug/dwarf",
	"dwarf.TagImportedUnit":                   "debug/dwarf",
	"dwarf.TagInheritance":                    "debug/dwarf",
	"dwarf.TagInlinedSubroutine":              "debug/dwarf",
	"dwarf.TagInterfaceType":                  "debug/dwarf",
	"dwarf.TagLabel":                          "debug/dwarf",
	"dwarf.TagLexDwarfBlock":                  "debug/dwarf",
	"dwarf.TagMember":                         "debug/dwarf",
	"dwarf.TagModule":                         "debug/dwarf",
	"dwarf.TagMutableType":                    "debug/dwarf",
	"dwarf.TagNamelist":                       "debug/dwarf",
	"dwarf.TagNamelistItem":                   "debug/dwarf",
	"dwarf.TagNamespace":                      "debug/dwarf",
	"dwarf.TagPackedType":                     "debug/dwarf",
	"dwarf.TagPartialUnit":                    "debug/dwarf",
	"dwarf.TagPointerType":                    "debug/dwarf",
	"dwarf.TagPtrToMemberType":                "debug/dwarf",
	"dwarf.TagReferenceType":                  "debug/dwarf",
	"dwarf.TagRestrictType":                   "debug/dwarf",
	"dwarf.TagSetType":                        "debug/dwarf",
	"dwarf.TagStringType":                     "debug/dwarf",
	"dwarf.TagStructType":                     "debug/dwarf",
	"dwarf.TagSubprogram":                     "debug/dwarf",
	"dwarf.TagSubrangeType":                   "debug/dwarf",
	"dwarf.TagSubroutineType":                 "debug/dwarf",
	"dwarf.TagTemplateTypeParameter":          "debug/dwarf",
	"dwarf.TagTemplateValueParameter":         "debug/dwarf",
	"dwarf.TagThrownType":                     "debug/dwarf",
	"dwarf.TagTryDwarfBlock":                  "debug/dwarf",
	"dwarf.TagTypedef":                        "debug/dwarf",
	"dwarf.TagUnionType":                      "debug/dwarf",
	"dwarf.TagUnspecifiedParameters":          "debug/dwarf",
	"dwarf.TagUnspecifiedType":                "debug/dwarf",
	"dwarf.TagVariable":                       "debug/dwarf",
	"dwarf.TagVariant":                        "debug/dwarf",
	"dwarf.TagVariantPart":                    "debug/dwarf",
	"dwarf.TagVolatileType":                   "debug/dwarf",
	"dwarf.TagWithStmt":                       "debug/dwarf",
	"dwarf.Type":                              "debug/dwarf",
	"dwarf.TypedefType":                       "debug/dwarf",
	"dwarf.UcharType":                         "debug/dwarf",
	"dwarf.UintType":                          "debug/dwarf",
	"dwarf.VoidType":                          "debug/dwarf",
	"ecdsa.GenerateKey":                       "crypto/ecdsa",
	"ecdsa.PrivateKey":                        "crypto/ecdsa",
	"ecdsa.PublicKey":                         "crypto/ecdsa",
	"ecdsa.Sign":                              "crypto/ecdsa",
	"ecdsa.Verify":                            "crypto/ecdsa",
	"elf.ARM_MAGIC_TRAMP_NUMBER":              "debug/elf",
	"elf.Class":                               "debug/elf",
	"elf.DF_BIND_NOW":                         "debug/elf",
	"elf.DF_ORIGIN":                           "debug/elf",
	"elf.DF_STATIC_TLS":                       "debug/elf",
	"elf.DF_SYMBOLIC":                         "debug/elf",
	"elf.DF_TEXTREL":                          "debug/elf",
	"elf.DT_BIND_NOW":                         "debug/elf",
	"elf.DT_DEBUG":                            "debug/elf",
	"elf.DT_ENCODING":                         "debug/elf",
	"elf.DT_FINI":                             "debug/elf",
	"elf.DT_FINI_ARRAY":                       "debug/elf",
	"elf.DT_FINI_ARRAYSZ":                     "debug/elf",
	"elf.DT_FLAGS":                            "debug/elf",
	"elf.DT_HASH":                             "debug/elf",
	"elf.DT_HIOS":                             "debug/elf",
	"elf.DT_HIPROC":                           "debug/elf",
	"elf.DT_INIT":                             "debug/elf",
	"elf.DT_INIT_ARRAY":                       "debug/elf",
	"elf.DT_INIT_ARRAYSZ":                     "debug/elf",
	"elf.DT_JMPREL":                           "debug/elf",
	"elf.DT_LOOS":                             "debug/elf",
	"elf.DT_LOPROC":                           "debug/elf",
	"elf.DT_NEEDED":                           "debug/elf",
	"elf.DT_NULL":                             "debug/elf",
	"elf.DT_PLTGOT":                           "debug/elf",
	"elf.DT_PLTREL":                           "debug/elf",
	"elf.DT_PLTRELSZ":                         "debug/elf",
	"elf.DT_PREINIT_ARRAY":                    "debug/elf",
	"elf.DT_PREINIT_ARRAYSZ":                  "debug/elf",
	"elf.DT_REL":                              "debug/elf",
	"elf.DT_RELA":                             "debug/elf",
	"elf.DT_RELAENT":                          "debug/elf",
	"elf.DT_RELASZ":                           "debug/elf",
	"elf.DT_RELENT":                           "debug/elf",
	"elf.DT_RELSZ":                            "debug/elf",
	"elf.DT_RPATH":                            "debug/elf",
	"elf.DT_RUNPATH":                          "debug/elf",
	"elf.DT_SONAME":                           "debug/elf",
	"elf.DT_STRSZ":                            "debug/elf",
	"elf.DT_STRTAB":                           "debug/elf",
	"elf.DT_SYMBOLIC":                         "debug/elf",
	"elf.DT_SYMENT":                           "debug/elf",
	"elf.DT_SYMTAB":                           "debug/elf",
	"elf.DT_TEXTREL":                          "debug/elf",
	"elf.DT_VERNEED":                          "debug/elf",
	"elf.DT_VERNEEDNUM":                       "debug/elf",
	"elf.DT_VERSYM":                           "debug/elf",
	"elf.Data":                                "debug/elf",
	"elf.Dyn32":                               "debug/elf",
	"elf.Dyn64":                               "debug/elf",
	"elf.DynFlag":                             "debug/elf",
	"elf.DynTag":                              "debug/elf",
	"elf.EI_ABIVERSION":                       "debug/elf",
	"elf.EI_CLASS":                            "debug/elf",
	"elf.EI_DATA":                             "debug/elf",
	"elf.EI_NIDENT":                           "debug/elf",
	"elf.EI_OSABI":                            "debug/elf",
	"elf.EI_PAD":                              "debug/elf",
	"elf.EI_VERSION":                          "debug/elf",
	"elf.ELFCLASS32":                          "debug/elf",
	"elf.ELFCLASS64":                          "debug/elf",
	"elf.ELFCLASSNONE":                        "debug/elf",
	"elf.ELFDATA2LSB":                         "debug/elf",
	"elf.ELFDATA2MSB":                         "debug/elf",
	"elf.ELFDATANONE":                         "debug/elf",
	"elf.ELFMAG":                              "debug/elf",
	"elf.ELFOSABI_86OPEN":                     "debug/elf",
	"elf.ELFOSABI_AIX":                        "debug/elf",
	"elf.ELFOSABI_ARM":                        "debug/elf",
	"elf.ELFOSABI_FREEBSD":                    "debug/elf",
	"elf.ELFOSABI_HPUX":                       "debug/elf",
	"elf.ELFOSABI_HURD":                       "debug/elf",
	"elf.ELFOSABI_IRIX":                       "debug/elf",
	"elf.ELFOSABI_LINUX":                      "debug/elf",
	"elf.ELFOSABI_MODESTO":                    "debug/elf",
	"elf.ELFOSABI_NETBSD":                     "debug/elf",
	"elf.ELFOSABI_NONE":                       "debug/elf",
	"elf.ELFOSABI_NSK":                        "debug/elf",
	"elf.ELFOSABI_OPENBSD":                    "debug/elf",
	"elf.ELFOSABI_OPENVMS":                    "debug/elf",
	"elf.ELFOSABI_SOLARIS":                    "debug/elf",
	"elf.ELFOSABI_STANDALONE":                 "debug/elf",
	"elf.ELFOSABI_TRU64":                      "debug/elf",
	"elf.EM_386":                              "debug/elf",
	"elf.EM_486":                              "debug/elf",
	"elf.EM_68HC12":                           "debug/elf",
	"elf.EM_68K":                              "debug/elf",
	"elf.EM_860":                              "debug/elf",
	"elf.EM_88K":                              "debug/elf",
	"elf.EM_960":                              "debug/elf",
	"elf.EM_ALPHA":                            "debug/elf",
	"elf.EM_ALPHA_STD":                        "debug/elf",
	"elf.EM_ARC":                              "debug/elf",
	"elf.EM_ARM":                              "debug/elf",
	"elf.EM_COLDFIRE":                         "debug/elf",
	"elf.EM_FR20":                             "debug/elf",
	"elf.EM_H8S":                              "debug/elf",
	"elf.EM_H8_300":                           "debug/elf",
	"elf.EM_H8_300H":                          "debug/elf",
	"elf.EM_H8_500":                           "debug/elf",
	"elf.EM_IA_64":                            "debug/elf",
	"elf.EM_M32":                              "debug/elf",
	"elf.EM_ME16":                             "debug/elf",
	"elf.EM_MIPS":                             "debug/elf",
	"elf.EM_MIPS_RS3_LE":                      "debug/elf",
	"elf.EM_MIPS_RS4_BE":                      "debug/elf",
	"elf.EM_MIPS_X":                           "debug/elf",
	"elf.EM_MMA":                              "debug/elf",
	"elf.EM_NCPU":                             "debug/elf",
	"elf.EM_NDR1":                             "debug/elf",
	"elf.EM_NONE":                             "debug/elf",
	"elf.EM_PARISC":                           "debug/elf",
	"elf.EM_PCP":                              "debug/elf",
	"elf.EM_PPC":                              "debug/elf",
	"elf.EM_PPC64":                            "debug/elf",
	"elf.EM_RCE":                              "debug/elf",
	"elf.EM_RH32":                             "debug/elf",
	"elf.EM_S370":                             "debug/elf",
	"elf.EM_S390":                             "debug/elf",
	"elf.EM_SH":                               "debug/elf",
	"elf.EM_SPARC":                            "debug/elf",
	"elf.EM_SPARC32PLUS":                      "debug/elf",
	"elf.EM_SPARCV9":                          "debug/elf",
	"elf.EM_ST100":                            "debug/elf",
	"elf.EM_STARCORE":                         "debug/elf",
	"elf.EM_TINYJ":                            "debug/elf",
	"elf.EM_TRICORE":                          "debug/elf",
	"elf.EM_V800":                             "debug/elf",
	"elf.EM_VPP500":                           "debug/elf",
	"elf.EM_X86_64":                           "debug/elf",
	"elf.ET_CORE":                             "debug/elf",
	"elf.ET_DYN":                              "debug/elf",
	"elf.ET_EXEC":                             "debug/elf",
	"elf.ET_HIOS":                             "debug/elf",
	"elf.ET_HIPROC":                           "debug/elf",
	"elf.ET_LOOS":                             "debug/elf",
	"elf.ET_LOPROC":                           "debug/elf",
	"elf.ET_NONE":                             "debug/elf",
	"elf.ET_REL":                              "debug/elf",
	"elf.EV_CURRENT":                          "debug/elf",
	"elf.EV_NONE":                             "debug/elf",
	"elf.File":                                "debug/elf",
	"elf.FileHeader":                          "debug/elf",
	"elf.FormatError":                         "debug/elf",
	"elf.Header32":                            "debug/elf",
	"elf.Header64":                            "debug/elf",
	"elf.ImportedSymbol":                      "debug/elf",
	"elf.Machine":                             "debug/elf",
	"elf.NT_FPREGSET":                         "debug/elf",
	"elf.NT_PRPSINFO":                         "debug/elf",
	"elf.NT_PRSTATUS":                         "debug/elf",
	"elf.NType":                               "debug/elf",
	"elf.NewFile":                             "debug/elf",
	"elf.OSABI":                               "debug/elf",
	"elf.Open":                                "debug/elf",
	"elf.PF_MASKOS":                           "debug/elf",
	"elf.PF_MASKPROC":                         "debug/elf",
	"elf.PF_R":                                "debug/elf",
	"elf.PF_W":                                "debug/elf",
	"elf.PF_X":                                "debug/elf",
	"elf.PT_DYNAMIC":                          "debug/elf",
	"elf.PT_HIOS":                             "debug/elf",
	"elf.PT_HIPROC":                           "debug/elf",
	"elf.PT_INTERP":                           "debug/elf",
	"elf.PT_LOAD":                             "debug/elf",
	"elf.PT_LOOS":                             "debug/elf",
	"elf.PT_LOPROC":                           "debug/elf",
	"elf.PT_NOTE":                             "debug/elf",
	"elf.PT_NULL":                             "debug/elf",
	"elf.PT_PHDR":                             "debug/elf",
	"elf.PT_SHLIB":                            "debug/elf",
	"elf.PT_TLS":                              "debug/elf",
	"elf.Prog":                                "debug/elf",
	"elf.Prog32":                              "debug/elf",
	"elf.Prog64":                              "debug/elf",
	"elf.ProgFlag":                            "debug/elf",
	"elf.ProgHeader":                          "debug/elf",
	"elf.ProgType":                            "debug/elf",
	"elf.R_386":                               "debug/elf",
	"elf.R_386_32":                            "debug/elf",
	"elf.R_386_COPY":                          "debug/elf",
	"elf.R_386_GLOB_DAT":                      "debug/elf",
	"elf.R_386_GOT32":                         "debug/elf",
	"elf.R_386_GOTOFF":                        "debug/elf",
	"elf.R_386_GOTPC":                         "debug/elf",
	"elf.R_386_JMP_SLOT":                      "debug/elf",
	"elf.R_386_NONE":                          "debug/elf",
	"elf.R_386_PC32":                          "debug/elf",
	"elf.R_386_PLT32":                         "debug/elf",
	"elf.R_386_RELATIVE":                      "debug/elf",
	"elf.R_386_TLS_DTPMOD32":                  "debug/elf",
	"elf.R_386_TLS_DTPOFF32":                  "debug/elf",
	"elf.R_386_TLS_GD":                        "debug/elf",
	"elf.R_386_TLS_GD_32":                     "debug/elf",
	"elf.R_386_TLS_GD_CALL":                   "debug/elf",
	"elf.R_386_TLS_GD_POP":                    "debug/elf",
	"elf.R_386_TLS_GD_PUSH":                   "debug/elf",
	"elf.R_386_TLS_GOTIE":                     "debug/elf",
	"elf.R_386_TLS_IE":                        "debug/elf",
	"elf.R_386_TLS_IE_32":                     "debug/elf",
	"elf.R_386_TLS_LDM":                       "debug/elf",
	"elf.R_386_TLS_LDM_32":                    "debug/elf",
	"elf.R_386_TLS_LDM_CALL":                  "debug/elf",
	"elf.R_386_TLS_LDM_POP":                   "debug/elf",
	"elf.R_386_TLS_LDM_PUSH":                  "debug/elf",
	"elf.R_386_TLS_LDO_32":                    "debug/elf",
	"elf.R_386_TLS_LE":                        "debug/elf",
	"elf.R_386_TLS_LE_32":                     "debug/elf",
	"elf.R_386_TLS_TPOFF":                     "debug/elf",
	"elf.R_386_TLS_TPOFF32":                   "debug/elf",
	"elf.R_ALPHA":                             "debug/elf",
	"elf.R_ALPHA_BRADDR":                      "debug/elf",
	"elf.R_ALPHA_COPY":                        "debug/elf",
	"elf.R_ALPHA_GLOB_DAT":                    "debug/elf",
	"elf.R_ALPHA_GPDISP":                      "debug/elf",
	"elf.R_ALPHA_GPREL32":                     "debug/elf",
	"elf.R_ALPHA_GPRELHIGH":                   "debug/elf",
	"elf.R_ALPHA_GPRELLOW":                    "debug/elf",
	"elf.R_ALPHA_GPVALUE":                     "debug/elf",
	"elf.R_ALPHA_HINT":                        "debug/elf",
	"elf.R_ALPHA_IMMED_BR_HI32":               "debug/elf",
	"elf.R_ALPHA_IMMED_GP_16":                 "debug/elf",
	"elf.R_ALPHA_IMMED_GP_HI32":               "debug/elf",
	"elf.R_ALPHA_IMMED_LO32":                  "debug/elf",
	"elf.R_ALPHA_IMMED_SCN_HI32":              "debug/elf",
	"elf.R_ALPHA_JMP_SLOT":                    "debug/elf",
	"elf.R_ALPHA_LITERAL":                     "debug/elf",
	"elf.R_ALPHA_LITUSE":                      "debug/elf",
	"elf.R_ALPHA_NONE":                        "debug/elf",
	"elf.R_ALPHA_OP_PRSHIFT":                  "debug/elf",
	"elf.R_ALPHA_OP_PSUB":                     "debug/elf",
	"elf.R_ALPHA_OP_PUSH":                     "debug/elf",
	"elf.R_ALPHA_OP_STORE":                    "debug/elf",
	"elf.R_ALPHA_REFLONG":                     "debug/elf",
	"elf.R_ALPHA_REFQUAD":                     "debug/elf",
	"elf.R_ALPHA_RELATIVE":                    "debug/elf",
	"elf.R_ALPHA_SREL16":                      "debug/elf",
	"elf.R_ALPHA_SREL32":                      "debug/elf",
	"elf.R_ALPHA_SREL64":                      "debug/elf",
	"elf.R_ARM":                               "debug/elf",
	"elf.R_ARM_ABS12":                         "debug/elf",
	"elf.R_ARM_ABS16":                         "debug/elf",
	"elf.R_ARM_ABS32":                         "debug/elf",
	"elf.R_ARM_ABS8":                          "debug/elf",
	"elf.R_ARM_AMP_VCALL9":                    "debug/elf",
	"elf.R_ARM_COPY":                          "debug/elf",
	"elf.R_ARM_GLOB_DAT":                      "debug/elf",
	"elf.R_ARM_GNU_VTENTRY":                   "debug/elf",
	"elf.R_ARM_GNU_VTINHERIT":                 "debug/elf",
	"elf.R_ARM_GOT32":                         "debug/elf",
	"elf.R_ARM_GOTOFF":                        "debug/elf",
	"elf.R_ARM_GOTPC":                         "debug/elf",
	"elf.R_ARM_JUMP_SLOT":                     "debug/elf",
	"elf.R_ARM_NONE":                          "debug/elf",
	"elf.R_ARM_PC13":                          "debug/elf",
	"elf.R_ARM_PC24":                          "debug/elf",
	"elf.R_ARM_PLT32":                         "debug/elf",
	"elf.R_ARM_RABS32":                        "debug/elf",
	"elf.R_ARM_RBASE":                         "debug/elf",
	"elf.R_ARM_REL32":                         "debug/elf",
	"elf.R_ARM_RELATIVE":                      "debug/elf",
	"elf.R_ARM_RPC24":                         "debug/elf",
	"elf.R_ARM_RREL32":                        "debug/elf",
	"elf.R_ARM_RSBREL32":                      "debug/elf",
	"elf.R_ARM_SBREL32":                       "debug/elf",
	"elf.R_ARM_SWI24":                         "debug/elf",
	"elf.R_ARM_THM_ABS5":                      "debug/elf",
	"elf.R_ARM_THM_PC22":                      "debug/elf",
	"elf.R_ARM_THM_PC8":                       "debug/elf",
	"elf.R_ARM_THM_RPC22":                     "debug/elf",
	"elf.R_ARM_THM_SWI8":                      "debug/elf",
	"elf.R_ARM_THM_XPC22":                     "debug/elf",
	"elf.R_ARM_XPC25":                         "debug/elf",
	"elf.R_INFO":                              "debug/elf",
	"elf.R_INFO32":                            "debug/elf",
	"elf.R_PPC":                               "debug/elf",
	"elf.R_PPC_ADDR14":                        "debug/elf",
	"elf.R_PPC_ADDR14_BRNTAKEN":               "debug/elf",
	"elf.R_PPC_ADDR14_BRTAKEN":                "debug/elf",
	"elf.R_PPC_ADDR16":                        "debug/elf",
	"elf.R_PPC_ADDR16_HA":                     "debug/elf",
	"elf.R_PPC_ADDR16_HI":                     "debug/elf",
	"elf.R_PPC_ADDR16_LO":                     "debug/elf",
	"elf.R_PPC_ADDR24":                        "debug/elf",
	"elf.R_PPC_ADDR32":                        "debug/elf",
	"elf.R_PPC_COPY":                          "debug/elf",
	"elf.R_PPC_DTPMOD32":                      "debug/elf",
	"elf.R_PPC_DTPREL16":                      "debug/elf",
	"elf.R_PPC_DTPREL16_HA":                   "debug/elf",
	"elf.R_PPC_DTPREL16_HI":                   "debug/elf",
	"elf.R_PPC_DTPREL16_LO":                   "debug/elf",
	"elf.R_PPC_DTPREL32":                      "debug/elf",
	"elf.R_PPC_EMB_BIT_FLD":                   "debug/elf",
	"elf.R_PPC_EMB_MRKREF":                    "debug/elf",
	"elf.R_PPC_EMB_NADDR16":                   "debug/elf",
	"elf.R_PPC_EMB_NADDR16_HA":                "debug/elf",
	"elf.R_PPC_EMB_NADDR16_HI":                "debug/elf",
	"elf.R_PPC_EMB_NADDR16_LO":                "debug/elf",
	"elf.R_PPC_EMB_NADDR32":                   "debug/elf",
	"elf.R_PPC_EMB_RELSDA":                    "debug/elf",
	"elf.R_PPC_EMB_RELSEC16":                  "debug/elf",
	"elf.R_PPC_EMB_RELST_HA":                  "debug/elf",
	"elf.R_PPC_EMB_RELST_HI":                  "debug/elf",
	"elf.R_PPC_EMB_RELST_LO":                  "debug/elf",
	"elf.R_PPC_EMB_SDA21":                     "debug/elf",
	"elf.R_PPC_EMB_SDA2I16":                   "debug/elf",
	"elf.R_PPC_EMB_SDA2REL":                   "debug/elf",
	"elf.R_PPC_EMB_SDAI16":                    "debug/elf",
	"elf.R_PPC_GLOB_DAT":                      "debug/elf",
	"elf.R_PPC_GOT16":                         "debug/elf",
	"elf.R_PPC_GOT16_HA":                      "debug/elf",
	"elf.R_PPC_GOT16_HI":                      "debug/elf",
	"elf.R_PPC_GOT16_LO":                      "debug/elf",
	"elf.R_PPC_GOT_TLSGD16":                   "debug/elf",
	"elf.R_PPC_GOT_TLSGD16_HA":                "debug/elf",
	"elf.R_PPC_GOT_TLSGD16_HI":                "debug/elf",
	"elf.R_PPC_GOT_TLSGD16_LO":                "debug/elf",
	"elf.R_PPC_GOT_TLSLD16":                   "debug/elf",
	"elf.R_PPC_GOT_TLSLD16_HA":                "debug/elf",
	"elf.R_PPC_GOT_TLSLD16_HI":                "debug/elf",
	"elf.R_PPC_GOT_TLSLD16_LO":                "debug/elf",
	"elf.R_PPC_GOT_TPREL16":                   "debug/elf",
	"elf.R_PPC_GOT_TPREL16_HA":                "debug/elf",
	"elf.R_PPC_GOT_TPREL16_HI":                "debug/elf",
	"elf.R_PPC_GOT_TPREL16_LO":                "debug/elf",
	"elf.R_PPC_JMP_SLOT":                      "debug/elf",
	"elf.R_PPC_LOCAL24PC":                     "debug/elf",
	"elf.R_PPC_NONE":                          "debug/elf",
	"elf.R_PPC_PLT16_HA":                      "debug/elf",
	"elf.R_PPC_PLT16_HI":                      "debug/elf",
	"elf.R_PPC_PLT16_LO":                      "debug/elf",
	"elf.R_PPC_PLT32":                         "debug/elf",
	"elf.R_PPC_PLTREL24":                      "debug/elf",
	"elf.R_PPC_PLTREL32":                      "debug/elf",
	"elf.R_PPC_REL14":                         "debug/elf",
	"elf.R_PPC_REL14_BRNTAKEN":                "debug/elf",
	"elf.R_PPC_REL14_BRTAKEN":                 "debug/elf",
	"elf.R_PPC_REL24":                         "debug/elf",
	"elf.R_PPC_REL32":                         "debug/elf",
	"elf.R_PPC_RELATIVE":                      "debug/elf",
	"elf.R_PPC_SDAREL16":                      "debug/elf",
	"elf.R_PPC_SECTOFF":                       "debug/elf",
	"elf.R_PPC_SECTOFF_HA":                    "debug/elf",
	"elf.R_PPC_SECTOFF_HI":                    "debug/elf",
	"elf.R_PPC_SECTOFF_LO":                    "debug/elf",
	"elf.R_PPC_TLS":                           "debug/elf",
	"elf.R_PPC_TPREL16":                       "debug/elf",
	"elf.R_PPC_TPREL16_HA":                    "debug/elf",
	"elf.R_PPC_TPREL16_HI":                    "debug/elf",
	"elf.R_PPC_TPREL16_LO":                    "debug/elf",
	"elf.R_PPC_TPREL32":                       "debug/elf",
	"elf.R_PPC_UADDR16":                       "debug/elf",
	"elf.R_PPC_UADDR32":                       "debug/elf",
	"elf.R_SPARC":                             "debug/elf",
	"elf.R_SPARC_10":                          "debug/elf",
	"elf.R_SPARC_11":                          "debug/elf",
	"elf.R_SPARC_13":                          "debug/elf",
	"elf.R_SPARC_16":                          "debug/elf",
	"elf.R_SPARC_22":                          "debug/elf",
	"elf.R_SPARC_32":                          "debug/elf",
	"elf.R_SPARC_5":                           "debug/elf",
	"elf.R_SPARC_6":                           "debug/elf",
	"elf.R_SPARC_64":                          "debug/elf",
	"elf.R_SPARC_7":                           "debug/elf",
	"elf.R_SPARC_8":                           "debug/elf",
	"elf.R_SPARC_COPY":                        "debug/elf",
	"elf.R_SPARC_DISP16":                      "debug/elf",
	"elf.R_SPARC_DISP32":                      "debug/elf",
	"elf.R_SPARC_DISP64":                      "debug/elf",
	"elf.R_SPARC_DISP8":                       "debug/elf",
	"elf.R_SPARC_GLOB_DAT":                    "debug/elf",
	"elf.R_SPARC_GLOB_JMP":                    "debug/elf",
	"elf.R_SPARC_GOT10":                       "debug/elf",
	"elf.R_SPARC_GOT13":                       "debug/elf",
	"elf.R_SPARC_GOT22":                       "debug/elf",
	"elf.R_SPARC_H44":                         "debug/elf",
	"elf.R_SPARC_HH22":                        "debug/elf",
	"elf.R_SPARC_HI22":                        "debug/elf",
	"elf.R_SPARC_HIPLT22":                     "debug/elf",
	"elf.R_SPARC_HIX22":                       "debug/elf",
	"elf.R_SPARC_HM10":                        "debug/elf",
	"elf.R_SPARC_JMP_SLOT":                    "debug/elf",
	"elf.R_SPARC_L44":                         "debug/elf",
	"elf.R_SPARC_LM22":                        "debug/elf",
	"elf.R_SPARC_LO10":                        "debug/elf",
	"elf.R_SPARC_LOPLT10":                     "debug/elf",
	"elf.R_SPARC_LOX10":                       "debug/elf",
	"elf.R_SPARC_M44":                         "debug/elf",
	"elf.R_SPARC_NONE":                        "debug/elf",
	"elf.R_SPARC_OLO10":                       "debug/elf",
	"elf.R_SPARC_PC10":                        "debug/elf",
	"elf.R_SPARC_PC22":                        "debug/elf",
	"elf.R_SPARC_PCPLT10":                     "debug/elf",
	"elf.R_SPARC_PCPLT22":                     "debug/elf",
	"elf.R_SPARC_PCPLT32":                     "debug/elf",
	"elf.R_SPARC_PC_HH22":                     "debug/elf",
	"elf.R_SPARC_PC_HM10":                     "debug/elf",
	"elf.R_SPARC_PC_LM22":                     "debug/elf",
	"elf.R_SPARC_PLT32":                       "debug/elf",
	"elf.R_SPARC_PLT64":                       "debug/elf",
	"elf.R_SPARC_REGISTER":                    "debug/elf",
	"elf.R_SPARC_RELATIVE":                    "debug/elf",
	"elf.R_SPARC_UA16":                        "debug/elf",
	"elf.R_SPARC_UA32":                        "debug/elf",
	"elf.R_SPARC_UA64":                        "debug/elf",
	"elf.R_SPARC_WDISP16":                     "debug/elf",
	"elf.R_SPARC_WDISP19":                     "debug/elf",
	"elf.R_SPARC_WDISP22":                     "debug/elf",
	"elf.R_SPARC_WDISP30":                     "debug/elf",
	"elf.R_SPARC_WPLT30":                      "debug/elf",
	"elf.R_SYM32":                             "debug/elf",
	"elf.R_SYM64":                             "debug/elf",
	"elf.R_TYPE32":                            "debug/elf",
	"elf.R_TYPE64":                            "debug/elf",
	"elf.R_X86_64":                            "debug/elf",
	"elf.R_X86_64_16":                         "debug/elf",
	"elf.R_X86_64_32":                         "debug/elf",
	"elf.R_X86_64_32S":                        "debug/elf",
	"elf.R_X86_64_64":                         "debug/elf",
	"elf.R_X86_64_8":                          "debug/elf",
	"elf.R_X86_64_COPY":                       "debug/elf",
	"elf.R_X86_64_DTPMOD64":                   "debug/elf",
	"elf.R_X86_64_DTPOFF32":                   "debug/elf",
	"elf.R_X86_64_DTPOFF64":                   "debug/elf",
	"elf.R_X86_64_GLOB_DAT":                   "debug/elf",
	"elf.R_X86_64_GOT32":                      "debug/elf",
	"elf.R_X86_64_GOTPCREL":                   "debug/elf",
	"elf.R_X86_64_GOTTPOFF":                   "debug/elf",
	"elf.R_X86_64_JMP_SLOT":                   "debug/elf",
	"elf.R_X86_64_NONE":                       "debug/elf",
	"elf.R_X86_64_PC16":                       "debug/elf",
	"elf.R_X86_64_PC32":                       "debug/elf",
	"elf.R_X86_64_PC8":                        "debug/elf",
	"elf.R_X86_64_PLT32":                      "debug/elf",
	"elf.R_X86_64_RELATIVE":                   "debug/elf",
	"elf.R_X86_64_TLSGD":                      "debug/elf",
	"elf.R_X86_64_TLSLD":                      "debug/elf",
	"elf.R_X86_64_TPOFF32":                    "debug/elf",
	"elf.R_X86_64_TPOFF64":                    "debug/elf",
	"elf.Rel32":                               "debug/elf",
	"elf.Rel64":                               "debug/elf",
	"elf.Rela32":                              "debug/elf",
	"elf.Rela64":                              "debug/elf",
	"elf.SHF_ALLOC":                           "debug/elf",
	"elf.SHF_EXECINSTR":                       "debug/elf",
	"elf.SHF_GROUP":                           "debug/elf",
	"elf.SHF_INFO_LINK":                       "debug/elf",
	"elf.SHF_LINK_ORDER":                      "debug/elf",
	"elf.SHF_MASKOS":                          "debug/elf",
	"elf.SHF_MASKPROC":                        "debug/elf",
	"elf.SHF_MERGE":                           "debug/elf",
	"elf.SHF_OS_NONCONFORMING":                "debug/elf",
	"elf.SHF_STRINGS":                         "debug/elf",
	"elf.SHF_TLS":                             "debug/elf",
	"elf.SHF_WRITE":                           "debug/elf",
	"elf.SHN_ABS":                             "debug/elf",
	"elf.SHN_COMMON":                          "debug/elf",
	"elf.SHN_HIOS":                            "debug/elf",
	"elf.SHN_HIPROC":                          "debug/elf",
	"elf.SHN_HIRESERVE":                       "debug/elf",
	"elf.SHN_LOOS":                            "debug/elf",
	"elf.SHN_LOPROC":                          "debug/elf",
	"elf.SHN_LORESERVE":                       "debug/elf",
	"elf.SHN_UNDEF":                           "debug/elf",
	"elf.SHN_XINDEX":                          "debug/elf",
	"elf.SHT_DYNAMIC":                         "debug/elf",
	"elf.SHT_DYNSYM":                          "debug/elf",
	"elf.SHT_FINI_ARRAY":                      "debug/elf",
	"elf.SHT_GNU_ATTRIBUTES":                  "debug/elf",
	"elf.SHT_GNU_HASH":                        "debug/elf",
	"elf.SHT_GNU_LIBLIST":                     "debug/elf",
	"elf.SHT_GNU_VERDEF":                      "debug/elf",
	"elf.SHT_GNU_VERNEED":                     "debug/elf",
	"elf.SHT_GNU_VERSYM":                      "debug/elf",
	"elf.SHT_GROUP":                           "debug/elf",
	"elf.SHT_HASH":                            "debug/elf",
	"elf.SHT_HIOS":                            "debug/elf",
	"elf.SHT_HIPROC":                          "debug/elf",
	"elf.SHT_HIUSER":                          "debug/elf",
	"elf.SHT_INIT_ARRAY":                      "debug/elf",
	"elf.SHT_LOOS":                            "debug/elf",
	"elf.SHT_LOPROC":                          "debug/elf",
	"elf.SHT_LOUSER":                          "debug/elf",
	"elf.SHT_NOBITS":                          "debug/elf",
	"elf.SHT_NOTE":                            "debug/elf",
	"elf.SHT_NULL":                            "debug/elf",
	"elf.SHT_PREINIT_ARRAY":                   "debug/elf",
	"elf.SHT_PROGBITS":                        "debug/elf",
	"elf.SHT_REL":                             "debug/elf",
	"elf.SHT_RELA":                            "debug/elf",
	"elf.SHT_SHLIB":                           "debug/elf",
	"elf.SHT_STRTAB":                          "debug/elf",
	"elf.SHT_SYMTAB":                          "debug/elf",
	"elf.SHT_SYMTAB_SHNDX":                    "debug/elf",
	"elf.STB_GLOBAL":                          "debug/elf",
	"elf.STB_HIOS":                            "debug/elf",
	"elf.STB_HIPROC":                          "debug/elf",
	"elf.STB_LOCAL":                           "debug/elf",
	"elf.STB_LOOS":                            "debug/elf",
	"elf.STB_LOPROC":                          "debug/elf",
	"elf.STB_WEAK":                            "debug/elf",
	"elf.STT_COMMON":                          "debug/elf",
	"elf.STT_FILE":                            "debug/elf",
	"elf.STT_FUNC":                            "debug/elf",
	"elf.STT_HIOS":                            "debug/elf",
	"elf.STT_HIPROC":                          "debug/elf",
	"elf.STT_LOOS":                            "debug/elf",
	"elf.STT_LOPROC":                          "debug/elf",
	"elf.STT_NOTYPE":                          "debug/elf",
	"elf.STT_OBJECT":                          "debug/elf",
	"elf.STT_SECTION":                         "debug/elf",
	"elf.STT_TLS":                             "debug/elf",
	"elf.STV_DEFAULT":                         "debug/elf",
	"elf.STV_HIDDEN":                          "debug/elf",
	"elf.STV_INTERNAL":                        "debug/elf",
	"elf.STV_PROTECTED":                       "debug/elf",
	"elf.ST_BIND":                             "debug/elf",
	"elf.ST_INFO":                             "debug/elf",
	"elf.ST_TYPE":                             "debug/elf",
	"elf.ST_VISIBILITY":                       "debug/elf",
	"elf.Section":                             "debug/elf",
	"elf.Section32":                           "debug/elf",
	"elf.Section64":                           "debug/elf",
	"elf.SectionFlag":                         "debug/elf",
	"elf.SectionHeader":                       "debug/elf",
	"elf.SectionIndex":                        "debug/elf",
	"elf.SectionType":                         "debug/elf",
	"elf.Sym32":                               "debug/elf",
	"elf.Sym32Size":                           "debug/elf",
	"elf.Sym64":                               "debug/elf",
	"elf.Sym64Size":                           "debug/elf",
	"elf.SymBind":                             "debug/elf",
	"elf.SymType":                             "debug/elf",
	"elf.SymVis":                              "debug/elf",
	"elf.Symbol":                              "debug/elf",
	"elf.Type":                                "debug/elf",
	"elf.Version":                             "debug/elf",
	"elliptic.Curve":                          "crypto/elliptic",
	"elliptic.CurveParams":                    "crypto/elliptic",
	"elliptic.GenerateKey":                    "crypto/elliptic",
	"elliptic.Marshal":                        "crypto/elliptic",
	"elliptic.P224":                           "crypto/elliptic",
	"elliptic.P256":                           "crypto/elliptic",
	"elliptic.P384":                           "crypto/elliptic",
	"elliptic.P521":                           "crypto/elliptic",
	"elliptic.Unmarshal":                      "crypto/elliptic",
	"encoding.BinaryMarshaler":                "encoding",
	"encoding.BinaryUnmarshaler":              "encoding",
	"encoding.TextMarshaler":                  "encoding",
	"encoding.TextUnmarshaler":                "encoding",
	"errors.New":                              "errors",
	"exec.Cmd":                                "os/exec",
	"exec.Command":                            "os/exec",
	"exec.ErrNotFound":                        "os/exec",
	"exec.Error":                              "os/exec",
	"exec.ExitError":                          "os/exec",
	"exec.LookPath":                           "os/exec",
	"expvar.Do":                               "expvar",
	"expvar.Float":                            "expvar",
	"expvar.Func":                             "expvar",
	"expvar.Get":                              "expvar",
	"expvar.Int":                              "expvar",
	"expvar.KeyValue":                         "expvar",
	"expvar.Map":                              "expvar",
	"expvar.NewFloat":                         "expvar",
	"expvar.NewInt":                           "expvar",
	"expvar.NewMap":                           "expvar",
	"expvar.NewString":                        "expvar",
	"expvar.Publish":                          "expvar",
	"expvar.String":                           "expvar",
	"expvar.Var":                              "expvar",
	"fcgi.Serve":                              "net/http/fcgi",
	"filepath.Abs":                            "path/filepath",
	"filepath.Base":                           "path/filepath",
	"filepath.Clean":                          "path/filepath",
	"filepath.Dir":                            "path/filepath",
	"filepath.ErrBadPattern":                  "path/filepath",
	"filepath.EvalSymlinks":                   "path/filepath",
	"filepath.Ext":                            "path/filepath",
	"filepath.FromSlash":                      "path/filepath",
	"filepath.Glob":                           "path/filepath",
	"filepath.HasPrefix":                      "path/filepath",
	"filepath.IsAbs":                          "path/filepath",
	"filepath.Join":                           "path/filepath",
	"filepath.ListSeparator":                  "path/filepath",
	"filepath.Match":                          "path/filepath",
	"filepath.Rel":                            "path/filepath",
	"filepath.Separator":                      "path/filepath",
	"filepath.SkipDir":                        "path/filepath",
	"filepath.Split":                          "path/filepath",
	"filepath.SplitList":                      "path/filepath",
	"filepath.ToSlash":                        "path/filepath",
	"filepath.VolumeName":                     "path/filepath",
	"filepath.Walk":                           "path/filepath",
	"filepath.WalkFunc":                       "path/filepath",
	"flag.Arg":                                "flag",
	"flag.Args":                               "flag",
	"flag.Bool":                               "flag",
	"flag.BoolVar":                            "flag",
	"flag.CommandLine":                        "flag",
	"flag.ContinueOnError":                    "flag",
	"flag.Duration":                           "flag",
	"flag.DurationVar":                        "flag",
	"flag.ErrHelp":                            "flag",
	"flag.ErrorHandling":                      "flag",
	"flag.ExitOnError":                        "flag",
	"flag.Flag":                               "flag",
	"flag.FlagSet":                            "flag",
	"flag.Float64":                            "flag",
	"flag.Float64Var":                         "flag",
	"flag.Getter":                             "flag",
	"flag.Int":                                "flag",
	"flag.Int64":                              "flag",
	"flag.Int64Var":                           "flag",
	"flag.IntVar":                             "flag",
	"flag.Lookup":                             "flag",
	"flag.NArg":                               "flag",
	"flag.NFlag":                              "flag",
	"flag.NewFlagSet":                         "flag",
	"flag.PanicOnError":                       "flag",
	"flag.Parse":                              "flag",
	"flag.Parsed":                             "flag",
	"flag.PrintDefaults":                      "flag",
	"flag.Set":                                "flag",
	"flag.String":                             "flag",
	"flag.StringVar":                          "flag",
	"flag.Uint":                               "flag",
	"flag.Uint64":                             "flag",
	"flag.Uint64Var":                          "flag",
	"flag.UintVar":                            "flag",
	"flag.Usage":                              "flag",
	"flag.Value":                              "flag",
	"flag.Var":                                "flag",
	"flag.Visit":                              "flag",
	"flag.VisitAll":                           "flag",
	"flate.BestCompression":                   "compress/flate",
	"flate.BestSpeed":                         "compress/flate",
	"flate.CorruptInputError":                 "compress/flate",
	"flate.DefaultCompression":                "compress/flate",
	"flate.InternalError":                     "compress/flate",
	"flate.NewReader":                         "compress/flate",
	"flate.NewReaderDict":                     "compress/flate",
	"flate.NewWriter":                         "compress/flate",
	"flate.NewWriterDict":                     "compress/flate",
	"flate.NoCompression":                     "compress/flate",
	"flate.ReadError":                         "compress/flate",
	"flate.Reader":                            "compress/flate",
	"flate.WriteError":                        "compress/flate",
	"flate.Writer":                            "compress/flate",
	"fmt.Errorf":                              "fmt",
	"fmt.Formatter":                           "fmt",
	"fmt.Fprint":                              "fmt",
	"fmt.Fprintf":                             "fmt",
	"fmt.Fprintln":                            "fmt",
	"fmt.Fscan":                               "fmt",
	"fmt.Fscanf":                              "fmt",
	"fmt.Fscanln":                             "fmt",
	"fmt.GoStringer":                          "fmt",
	"fmt.Print":                               "fmt",
	"fmt.Printf":                              "fmt",
	"fmt.Println":                             "fmt",
	"fmt.Scan":                                "fmt",
	"fmt.ScanState":                           "fmt",
	"fmt.Scanf":                               "fmt",
	"fmt.Scanln":                              "fmt",
	"fmt.Scanner":                             "fmt",
	"fmt.Sprint":                              "fmt",
	"fmt.Sprintf":                             "fmt",
	"fmt.Sprintln":                            "fmt",
	"fmt.Sscan":                               "fmt",
	"fmt.Sscanf":                              "fmt",
	"fmt.Sscanln":                             "fmt",
	"fmt.State":                               "fmt",
	"fmt.Stringer":                            "fmt",
	"fnv.New32":                               "hash/fnv",
	"fnv.New32a":                              "hash/fnv",
	"fnv.New64":                               "hash/fnv",
	"fnv.New64a":                              "hash/fnv",
	"format.Node":                             "go/format",
	"format.Source":                           "go/format",
	"gif.Decode":                              "image/gif",
	"gif.DecodeAll":                           "image/gif",
	"gif.DecodeConfig":                        "image/gif",
	"gif.Encode":                              "image/gif",
	"gif.EncodeAll":                           "image/gif",
	"gif.GIF":                                 "image/gif",
	"gif.Options":                             "image/gif",
	"gob.CommonType":                          "encoding/gob",
	"gob.Decoder":                             "encoding/gob",
	"gob.Encoder":                             "encoding/gob",
	"gob.GobDecoder":                          "encoding/gob",
	"gob.GobEncoder":                          "encoding/gob",
	"gob.NewDecoder":                          "encoding/gob",
	"gob.NewEncoder":                          "encoding/gob",
	"gob.Register":                            "encoding/gob",
	"gob.RegisterName":                        "encoding/gob",
	"gosym.DecodingError":                     "debug/gosym",
	"gosym.Func":                              "debug/gosym",
	"gosym.LineTable":                         "debug/gosym",
	"gosym.NewLineTable":                      "debug/gosym",
	"gosym.NewTable":                          "debug/gosym",
	"gosym.Obj":                               "debug/gosym",
	"gosym.Sym":                               "debug/gosym",
	"gosym.Table":                             "debug/gosym",
	"gosym.UnknownFileError":                  "debug/gosym",
	"gosym.UnknownLineError":                  "debug/gosym",
	"gzip.BestCompression":                    "compress/gzip",
	"gzip.BestSpeed":                          "compress/gzip",
	"gzip.DefaultCompression":                 "compress/gzip",
	"gzip.ErrChecksum":                        "compress/gzip",
	"gzip.ErrHeader":                          "compress/gzip",
	"gzip.Header":                             "compress/gzip",
	"gzip.NewReader":                          "compress/gzip",
	"gzip.NewWriter":                          "compress/gzip",
	"gzip.NewWriterLevel":                     "compress/gzip",
	"gzip.NoCompression":                      "compress/gzip",
	"gzip.Reader":                             "compress/gzip",
	"gzip.Writer":                             "compress/gzip",
	"hash.Hash":                               "hash",
	"hash.Hash32":                             "hash",
	"hash.Hash64":                             "hash",
	"heap.Fix":                                "container/heap",
	"heap.Init":                               "container/heap",
	"heap.Interface":                          "container/heap",
	"heap.Pop":                                "container/heap",
	"heap.Push":                               "container/heap",
	"heap.Remove":                             "container/heap",
	"hex.Decode":                              "encoding/hex",
	"hex.DecodeString":                        "encoding/hex",
	"hex.DecodedLen":                          "encoding/hex",
	"hex.Dump":                                "encoding/hex",
	"hex.Dumper":                              "encoding/hex",
	"hex.Encode":                              "encoding/hex",
	"hex.EncodeToString":                      "encoding/hex",
	"hex.EncodedLen":                          "encoding/hex",
	"hex.ErrLength":                           "encoding/hex",
	"hex.InvalidByteError":                    "encoding/hex",
	"hmac.Equal":                              "crypto/hmac",
	"hmac.New":                                "crypto/hmac",
	"html.EscapeString":                       "html",
	"html.UnescapeString":                     "html",
	"http.CanonicalHeaderKey":                 "net/http",
	"http.Client":                             "net/http",
	"http.CloseNotifier":                      "net/http",
	"http.Cookie":                             "net/http",
	"http.CookieJar":                          "net/http",
	"http.DefaultClient":                      "net/http",
	"http.DefaultMaxHeaderBytes":              "net/http",
	"http.DefaultMaxIdleConnsPerHost":         "net/http",
	"http.DefaultServeMux":                    "net/http",
	"http.DefaultTransport":                   "net/http",
	"http.DetectContentType":                  "net/http",
	"http.Dir":                                "net/http",
	"http.ErrBodyNotAllowed":                  "net/http",
	"http.ErrBodyReadAfterClose":              "net/http",
	"http.ErrContentLength":                   "net/http",
	"http.ErrHandlerTimeout":                  "net/http",
	"http.ErrHeaderTooLong":                   "net/http",
	"http.ErrHijacked":                        "net/http",
	"http.ErrLineTooLong":                     "net/http",
	"http.ErrMissingBoundary":                 "net/http",
	"http.ErrMissingContentLength":            "net/http",
	"http.ErrMissingFile":                     "net/http",
	"http.ErrNoCookie":                        "net/http",
	"http.ErrNoLocation":                      "net/http",
	"http.ErrNotMultipart":                    "net/http",
	"http.ErrNotSupported":                    "net/http",
	"http.ErrShortBody":                       "net/http",
	"http.ErrUnexpectedTrailer":               "net/http",
	"http.ErrWriteAfterFlush":                 "net/http",
	"http.Error":                              "net/http",
	"http.File":                               "net/http",
	"http.FileServer":                         "net/http",
	"http.FileSystem":                         "net/http",
	"http.Flusher":                            "net/http",
	"http.Get":                                "net/http",
	"http.Handle":                             "net/http",
	"http.HandleFunc":                         "net/http",
	"http.Handler":                            "net/http",
	"http.HandlerFunc":                        "net/http",
	"http.Head":                               "net/http",
	"http.Header":                             "net/http",
	"http.Hijacker":                           "net/http",
	"http.ListenAndServe":                     "net/http",
	"http.ListenAndServeTLS":                  "net/http",
	"http.MaxBytesReader":                     "net/http",
	"http.NewFileTransport":                   "net/http",
	"http.NewRequest":                         "net/http",
	"http.NewServeMux":                        "net/http",
	"http.NotFound":                           "net/http",
	"http.NotFoundHandler":                    "net/http",
	"http.ParseHTTPVersion":                   "net/http",
	"http.ParseTime":                          "net/http",
	"http.Post":                               "net/http",
	"http.PostForm":                           "net/http",
	"http.ProtocolError":                      "net/http",
	"http.ProxyFromEnvironment":               "net/http",
	"http.ProxyURL":                           "net/http",
	"http.ReadRequest":                        "net/http",
	"http.ReadResponse":                       "net/http",
	"http.Redirect":                           "net/http",
	"http.RedirectHandler":                    "net/http",
	"http.Request":                            "net/http",
	"http.Response":                           "net/http",
	"http.ResponseWriter":                     "net/http",
	"http.RoundTripper":                       "net/http",
	"http.Serve":                              "net/http",
	"http.ServeContent":                       "net/http",
	"http.ServeFile":                          "net/http",
	"http.ServeMux":                           "net/http",
	"http.Server":                             "net/http",
	"http.SetCookie":                          "net/http",
	"http.StatusAccepted":                     "net/http",
	"http.StatusBadGateway":                   "net/http",
	"http.StatusBadRequest":                   "net/http",
	"http.StatusConflict":                     "net/http",
	"http.StatusContinue":                     "net/http",
	"http.StatusCreated":                      "net/http",
	"http.StatusExpectationFailed":            "net/http",
	"http.StatusForbidden":                    "net/http",
	"http.StatusFound":                        "net/http",
	"http.StatusGatewayTimeout":               "net/http",
	"http.StatusGone":                         "net/http",
	"http.StatusHTTPVersionNotSupported":      "net/http",
	"http.StatusInternalServerError":          "net/http",
	"http.StatusLengthRequired":               "net/http",
	"http.StatusMethodNotAllowed":             "net/http",
	"http.StatusMovedPermanently":             "net/http",
	"http.StatusMultipleChoices":              "net/http",
	"http.StatusNoContent":                    "net/http",
	"http.StatusNonAuthoritativeInfo":         "net/http",
	"http.StatusNotAcceptable":                "net/http",
	"http.StatusNotFound":                     "net/http",
	"http.StatusNotImplemented":               "net/http",
	"http.StatusNotModified":                  "net/http",
	"http.StatusOK":                           "net/http",
	"http.StatusPartialContent":               "net/http",
	"http.StatusPaymentRequired":              "net/http",
	"http.StatusPreconditionFailed":           "net/http",
	"http.StatusProxyAuthRequired":            "net/http",
	"http.StatusRequestEntityTooLarge":        "net/http",
	"http.StatusRequestTimeout":               "net/http",
	"http.StatusRequestURITooLong":            "net/http",
	"http.StatusRequestedRangeNotSatisfiable": "net/http",
	"http.StatusResetContent":                 "net/http",
	"http.StatusSeeOther":                     "net/http",
	"http.StatusServiceUnavailable":           "net/http",
	"http.StatusSwitchingProtocols":           "net/http",
	"http.StatusTeapot":                       "net/http",
	"http.StatusTemporaryRedirect":            "net/http",
	"http.StatusText":                         "net/http",
	"http.StatusUnauthorized":                 "net/http",
	"http.StatusUnsupportedMediaType":         "net/http",
	"http.StatusUseProxy":                     "net/http",
	"http.StripPrefix":                        "net/http",
	"http.TimeFormat":                         "net/http",
	"http.TimeoutHandler":                     "net/http",
	"http.Transport":                          "net/http",
	"httptest.DefaultRemoteAddr":              "net/http/httptest",
	"httptest.NewRecorder":                    "net/http/httptest",
	"httptest.NewServer":                      "net/http/httptest",
	"httptest.NewTLSServer":                   "net/http/httptest",
	"httptest.NewUnstartedServer":             "net/http/httptest",
	"httptest.ResponseRecorder":               "net/http/httptest",
	"httptest.Server":                         "net/http/httptest",
	"httputil.ClientConn":                     "net/http/httputil",
	"httputil.DumpRequest":                    "net/http/httputil",
	"httputil.DumpRequestOut":                 "net/http/httputil",
	"httputil.DumpResponse":                   "net/http/httputil",
	"httputil.ErrClosed":                      "net/http/httputil",
	"httputil.ErrLineTooLong":                 "net/http/httputil",
	"httputil.ErrPersistEOF":                  "net/http/httputil",
	"httputil.ErrPipeline":                    "net/http/httputil",
	"httputil.NewChunkedReader":               "net/http/httputil",
	"httputil.NewChunkedWriter":               "net/http/httputil",
	"httputil.NewClientConn":                  "net/http/httputil",
	"httputil.NewProxyClientConn":             "net/http/httputil",
	"httputil.NewServerConn":                  "net/http/httputil",
	"httputil.NewSingleHostReverseProxy":      "net/http/httputil",
	"httputil.ReverseProxy":                   "net/http/httputil",
	"httputil.ServerConn":                     "net/http/httputil",
	"image.Alpha":                             "image",
	"image.Alpha16":                           "image",
	"image.Black":                             "image",
	"image.Config":                            "image",
	"image.Decode":                            "image",
	"image.DecodeConfig":                      "image",
	"image.ErrFormat":                         "image",
	"image.Gray":                              "image",
	"image.Gray16":                            "image",
	"image.Image":                             "image",
	"image.NRGBA":                             "image",
	"image.NRGBA64":                           "image",
	"image.NewAlpha":                          "image",
	"image.NewAlpha16":                        "image",
	"image.NewGray":                           "image",
	"image.NewGray16":                         "image",
	"image.NewNRGBA":                          "image",
	"image.NewNRGBA64":                        "image",
	"image.NewPaletted":                       "image",
	"image.NewRGBA":                           "image",
	"image.NewRGBA64":                         "image",
	"image.NewUniform":                        "image",
	"image.NewYCbCr":                          "image",
	"image.Opaque":                            "image",
	"image.Paletted":                          "image",
	"image.PalettedImage":                     "image",
	"image.Point":                             "image",
	"image.Pt":                                "image",
	"image.RGBA":                              "image",
	"image.RGBA64":                            "image",
	"image.Rect":                              "image",
	"image.Rectangle":                         "image",
	"image.RegisterFormat":                    "image",
	"image.Transparent":                       "image",
	"image.Uniform":                           "image",
	"image.White":                             "image",
	"image.YCbCr":                             "image",
	"image.YCbCrSubsampleRatio":               "image",
	"image.YCbCrSubsampleRatio420":            "image",
	"image.YCbCrSubsampleRatio422":            "image",
	"image.YCbCrSubsampleRatio440":            "image",
	"image.YCbCrSubsampleRatio444":            "image",
	"image.ZP":                                "image",
	"image.ZR":                                "image",
	"io.ByteReader":                           "io",
	"io.ByteScanner":                          "io",
	"io.ByteWriter":                           "io",
	"io.Closer":                               "io",
	"io.Copy":                                 "io",
	"io.CopyN":                                "io",
	"io.EOF":                                  "io",
	"io.ErrClosedPipe":                        "io",
	"io.ErrNoProgress":                        "io",
	"io.ErrShortBuffer":                       "io",
	"io.ErrShortWrite":                        "io",
	"io.ErrUnexpectedEOF":                     "io",
	"io.LimitReader":                          "io",
	"io.LimitedReader":                        "io",
	"io.MultiReader":                          "io",
	"io.MultiWriter":                          "io",
	"io.NewSectionReader":                     "io",
	"io.Pipe":                                 "io",
	"io.PipeReader":                           "io",
	"io.PipeWriter":                           "io",
	"io.ReadAtLeast":                          "io",
	"io.ReadCloser":                           "io",
	"io.ReadFull":                             "io",
	"io.ReadSeeker":                           "io",
	"io.ReadWriteCloser":                      "io",
	"io.ReadWriteSeeker":                      "io",
	"io.ReadWriter":                           "io",
	"io.Reader":                               "io",
	"io.ReaderAt":                             "io",
	"io.ReaderFrom":                           "io",
	"io.RuneReader":                           "io",
	"io.RuneScanner":                          "io",
	"io.SectionReader":                        "io",
	"io.Seeker":                               "io",
	"io.TeeReader":                            "io",
	"io.WriteCloser":                          "io",
	"io.WriteSeeker":                          "io",
	"io.WriteString":                          "io",
	"io.Writer":                               "io",
	"io.WriterAt":                             "io",
	"io.WriterTo":                             "io",
	"iotest.DataErrReader":                    "testing/iotest",
	"iotest.ErrTimeout":                       "testing/iotest",
	"iotest.HalfReader":                       "testing/iotest",
	"iotest.NewReadLogger":                    "testing/iotest",
	"iotest.NewWriteLogger":                   "testing/iotest",
	"iotest.OneByteReader":                    "testing/iotest",
	"iotest.TimeoutReader":                    "testing/iotest",
	"iotest.TruncateWriter":                   "testing/iotest",
	"ioutil.Discard":                          "io/ioutil",
	"ioutil.NopCloser":                        "io/ioutil",
	"ioutil.ReadAll":                          "io/ioutil",
	"ioutil.ReadDir":                          "io/ioutil",
	"ioutil.ReadFile":                         "io/ioutil",
	"ioutil.TempDir":                          "io/ioutil",
	"ioutil.TempFile":                         "io/ioutil",
	"ioutil.WriteFile":                        "io/ioutil",
	"jpeg.Decode":                             "image/jpeg",
	"jpeg.DecodeConfig":                       "image/jpeg",
	"jpeg.DefaultQuality":                     "image/jpeg",
	"jpeg.Encode":                             "image/jpeg",
	"jpeg.FormatError":                        "image/jpeg",
	"jpeg.Options":                            "image/jpeg",
	"jpeg.Reader":                             "image/jpeg",
	"jpeg.UnsupportedError":                   "image/jpeg",
	"json.Compact":                            "encoding/json",
	"json.Decoder":                            "encoding/json",
	"json.Encoder":                            "encoding/json",
	"json.HTMLEscape":                         "encoding/json",
	"json.Indent":                             "encoding/json",
	"json.InvalidUTF8Error":                   "encoding/json",
	"json.InvalidUnmarshalError":              "encoding/json",
	"json.Marshal":                            "encoding/json",
	"json.MarshalIndent":                      "encoding/json",
	"json.Marshaler":                          "encoding/json",
	"json.MarshalerError":                     "encoding/json",
	"json.NewDecoder":                         "encoding/json",
	"json.NewEncoder":                         "encoding/json",
	"json.Number":                             "encoding/json",
	"json.RawMessage":                         "encoding/json",
	"json.SyntaxError":                        "encoding/json",
	"json.Unmarshal":                          "encoding/json",
	"json.UnmarshalFieldError":                "encoding/json",
	"json.UnmarshalTypeError":                 "encoding/json",
	"json.Unmarshaler":                        "encoding/json",
	"json.UnsupportedTypeError":               "encoding/json",
	"json.UnsupportedValueError":              "encoding/json",
	"jsonrpc.Dial":                            "net/rpc/jsonrpc",
	"jsonrpc.NewClient":                       "net/rpc/jsonrpc",
	"jsonrpc.NewClientCodec":                  "net/rpc/jsonrpc",
	"jsonrpc.NewServerCodec":                  "net/rpc/jsonrpc",
	"jsonrpc.ServeConn":                       "net/rpc/jsonrpc",
	"list.Element":                            "container/list",
	"list.List":                               "container/list",
	"list.New":                                "container/list",
	"log.Fatal":                               "log",
	"log.Fatalf":                              "log",
	"log.Fatalln":                             "log",
	"log.Flags":                               "log",
	"log.Ldate":                               "log",
	"log.Llongfile":                           "log",
	"log.Lmicroseconds":                       "log",
	"log.Logger":                              "log",
	"log.Lshortfile":                          "log",
	"log.LstdFlags":                           "log",
	"log.Ltime":                               "log",
	"log.New":                                 "log",
	"log.Panic":                               "log",
	"log.Panicf":                              "log",
	"log.Panicln":                             "log",
	"log.Prefix":                              "log",
	"log.Print":                               "log",
	"log.Printf":                              "log",
	"log.Println":                             "log",
	"log.SetFlags":                            "log",
	"log.SetOutput":                           "log",
	"log.SetPrefix":                           "log",
	"lzw.LSB":                                 "compress/lzw",
	"lzw.MSB":                                 "compress/lzw",
	"lzw.NewReader":                           "compress/lzw",
	"lzw.NewWriter":                           "compress/lzw",
	"lzw.Order":                               "compress/lzw",
	"macho.Cpu":                               "debug/macho",
	"macho.Cpu386":                            "debug/macho",
	"macho.CpuAmd64":                          "debug/macho",
	"macho.Dylib":                             "debug/macho",
	"macho.DylibCmd":                          "debug/macho",
	"macho.Dysymtab":                          "debug/macho",
	"macho.DysymtabCmd":                       "debug/macho",
	"macho.File":                              "debug/macho",
	"macho.FileHeader":                        "debug/macho",
	"macho.FormatError":                       "debug/macho",
	"macho.Load":                              "debug/macho",
	"macho.LoadBytes":                         "debug/macho",
	"macho.LoadCmd":                           "debug/macho",
	"macho.LoadCmdDylib":                      "debug/macho",
	"macho.LoadCmdDylinker":                   "debug/macho",
	"macho.LoadCmdDysymtab":                   "debug/macho",
	"macho.LoadCmdSegment":                    "debug/macho",
	"macho.LoadCmdSegment64":                  "debug/macho",
	"macho.LoadCmdSymtab":                     "debug/macho",
	"macho.LoadCmdThread":                     "debug/macho",
	"macho.LoadCmdUnixThread":                 "debug/macho",
	"macho.Magic32":                           "debug/macho",
	"macho.Magic64":                           "debug/macho",
	"macho.NewFile":                           "debug/macho",
	"macho.Nlist32":                           "debug/macho",
	"macho.Nlist64":                           "debug/macho",
	"macho.Open":                              "debug/macho",
	"macho.Regs386":                           "debug/macho",
	"macho.RegsAMD64":                         "debug/macho",
	"macho.Section":                           "debug/macho",
	"macho.Section32":                         "debug/macho",
	"macho.Section64":                         "debug/macho",
	"macho.SectionHeader":                     "debug/macho",
	"macho.Segment":                           "debug/macho",
	"macho.Segment32":                         "debug/macho",
	"macho.Segment64":                         "debug/macho",
	"macho.SegmentHeader":                     "debug/macho",
	"macho.Symbol":                            "debug/macho",
	"macho.Symtab":                            "debug/macho",
	"macho.SymtabCmd":                         "debug/macho",
	"macho.Thread":                            "debug/macho",
	"macho.Type":                              "debug/macho",
	"macho.TypeExec":                          "debug/macho",
	"macho.TypeObj":                           "debug/macho",
	"mail.Address":                            "net/mail",
	"mail.ErrHeaderNotPresent":                "net/mail",
	"mail.Header":                             "net/mail",
	"mail.Message":                            "net/mail",
	"mail.ParseAddress":                       "net/mail",
	"mail.ParseAddressList":                   "net/mail",
	"mail.ReadMessage":                        "net/mail",
	"math.Abs":                                "math",
	"math.Acos":                               "math",
	"math.Acosh":                              "math",
	"math.Asin":                               "math",
	"math.Asinh":                              "math",
	"math.Atan":                               "math",
	"math.Atan2":                              "math",
	"math.Atanh":                              "math",
	"math.Cbrt":                               "math",
	"math.Ceil":                               "math",
	"math.Copysign":                           "math",
	"math.Cos":                                "math",
	"math.Cosh":                               "math",
	"math.Dim":                                "math",
	"math.E":                                  "math",
	"math.Erf":                                "math",
	"math.Erfc":                               "math",
	"math.Exp":                                "math",
	"math.Exp2":                               "math",
	"math.Expm1":                              "math",
	"math.Float32bits":                        "math",
	"math.Float32frombits":                    "math",
	"math.Float64bits":                        "math",
	"math.Float64frombits":                    "math",
	"math.Floor":                              "math",
	"math.Frexp":                              "math",
	"math.Gamma":                              "math",
	"math.Hypot":                              "math",
	"math.Ilogb":                              "math",
	"math.Inf":                                "math",
	"math.IsInf":                              "math",
	"math.IsNaN":                              "math",
	"math.J0":                                 "math",
	"math.J1":                                 "math",
	"math.Jn":                                 "math",
	"math.Ldexp":                              "math",
	"math.Lgamma":                             "math",
	"math.Ln10":                               "math",
	"math.Ln2":                                "math",
	"math.Log":                                "math",
	"math.Log10":                              "math",
	"math.Log10E":                             "math",
	"math.Log1p":                              "math",
	"math.Log2":                               "math",
	"math.Log2E":                              "math",
	"math.Logb":                               "math",
	"math.Max":                                "math",
	"math.MaxFloat32":                         "math",
	"math.MaxFloat64":                         "math",
	"math.MaxInt16":                           "math",
	"math.MaxInt32":                           "math",
	"math.MaxInt64":                           "math",
	"math.MaxInt8":                            "math",
	"math.MaxUint16":                          "math",
	"math.MaxUint32":                          "math",
	"math.MaxUint64":                          "math",
	"math.MaxUint8":                           "math",
	"math.Min":                                "math",
	"math.MinInt16":                           "math",
	"math.MinInt32":                           "math",
	"math.MinInt64":                           "math",
	"math.MinInt8":                            "math",
	"math.Mod":                                "math",
	"math.Modf":                               "math",
	"math.NaN":                                "math",
	"math.Nextafter":                          "math",
	"math.Phi":                                "math",
	"math.Pi":                                 "math",
	"math.Pow":                                "math",
	"math.Pow10":                              "math",
	"math.Remainder":                          "math",
	"math.Signbit":                            "math",
	"math.Sin":                                "math",
	"math.Sincos":                             "math",
	"math.Sinh":                               "math",
	"math.SmallestNonzeroFloat32":             "math",
	"math.SmallestNonzeroFloat64":             "math",
	"math.Sqrt":                               "math",
	"math.Sqrt2":                              "math",
	"math.SqrtE":                              "math",
	"math.SqrtPhi":                            "math",
	"math.SqrtPi":                             "math",
	"math.Tan":                                "math",
	"math.Tanh":                               "math",
	"math.Trunc":                              "math",
	"math.Y0":                                 "math",
	"math.Y1":                                 "math",
	"math.Yn":                                 "math",
	"md5.BlockSize":                           "crypto/md5",
	"md5.New":                                 "crypto/md5",
	"md5.Size":                                "crypto/md5",
	"md5.Sum":                                 "crypto/md5",
	"mime.AddExtensionType":                   "mime",
	"mime.FormatMediaType":                    "mime",
	"mime.ParseMediaType":                     "mime",
	"mime.TypeByExtension":                    "mime",
	"multipart.File":                          "mime/multipart",
	"multipart.FileHeader":                    "mime/multipart",
	"multipart.Form":                          "mime/multipart",
	"multipart.NewReader":                     "mime/multipart",
	"multipart.NewWriter":                     "mime/multipart",
	"multipart.Part":                          "mime/multipart",
	"multipart.Reader":                        "mime/multipart",
	"multipart.Writer":                        "mime/multipart",
	"net.Addr":                                "net",
	"net.AddrError":                           "net",
	"net.CIDRMask":                            "net",
	"net.Conn":                                "net",
	"net.DNSConfigError":                      "net",
	"net.DNSError":                            "net",
	"net.Dial":                                "net",
	"net.DialIP":                              "net",
	"net.DialTCP":                             "net",
	"net.DialTimeout":                         "net",
	"net.DialUDP":                             "net",
	"net.DialUnix":                            "net",
	"net.Dialer":                              "net",
	"net.ErrWriteToConnected":                 "net",
	"net.Error":                               "net",
	"net.FileConn":                            "net",
	"net.FileListener":                        "net",
	"net.FilePacketConn":                      "net",
	"net.FlagBroadcast":                       "net",
	"net.FlagLoopback":                        "net",
	"net.FlagMulticast":                       "net",
	"net.FlagPointToPoint":                    "net",
	"net.FlagUp":                              "net",
	"net.Flags":                               "net",
	"net.HardwareAddr":                        "net",
	"net.IP":                                  "net",
	"net.IPAddr":                              "net",
	"net.IPConn":                              "net",
	"net.IPMask":                              "net",
	"net.IPNet":                               "net",
	"net.IPv4":                                "net",
	"net.IPv4Mask":                            "net",
	"net.IPv4allrouter":                       "net",
	"net.IPv4allsys":                          "net",
	"net.IPv4bcast":                           "net",
	"net.IPv4len":                             "net",
	"net.IPv4zero":                            "net",
	"net.IPv6interfacelocalallnodes":          "net",
	"net.IPv6len":                             "net",
	"net.IPv6linklocalallnodes":               "net",
	"net.IPv6linklocalallrouters":             "net",
	"net.IPv6loopback":                        "net",
	"net.IPv6unspecified":                     "net",
	"net.IPv6zero":                            "net",
	"net.Interface":                           "net",
	"net.InterfaceAddrs":                      "net",
	"net.InterfaceByIndex":                    "net",
	"net.InterfaceByName":                     "net",
	"net.Interfaces":                          "net",
	"net.InvalidAddrError":                    "net",
	"net.JoinHostPort":                        "net",
	"net.Listen":                              "net",
	"net.ListenIP":                            "net",
	"net.ListenMulticastUDP":                  "net",
	"net.ListenPacket":                        "net",
	"net.ListenTCP":                           "net",
	"net.ListenUDP":                           "net",
	"net.ListenUnix":                          "net",
	"net.ListenUnixgram":                      "net",
	"net.Listener":                            "net",
	"net.LookupAddr":                          "net",
	"net.LookupCNAME":                         "net",
	"net.LookupHost":                          "net",
	"net.LookupIP":                            "net",
	"net.LookupMX":                            "net",
	"net.LookupNS":                            "net",
	"net.LookupPort":                          "net",
	"net.LookupSRV":                           "net",
	"net.LookupTXT":                           "net",
	"net.MX":                                  "net",
	"net.NS":                                  "net",
	"net.OpError":                             "net",
	"net.PacketConn":                          "net",
	"net.ParseCIDR":                           "net",
	"net.ParseError":                          "net",
	"net.ParseIP":                             "net",
	"net.ParseMAC":                            "net",
	"net.Pipe":                                "net",
	"net.ResolveIPAddr":                       "net",
	"net.ResolveTCPAddr":                      "net",
	"net.ResolveUDPAddr":                      "net",
	"net.ResolveUnixAddr":                     "net",
	"net.SRV":                                 "net",
	"net.SplitHostPort":                       "net",
	"net.TCPAddr":                             "net",
	"net.TCPConn":                             "net",
	"net.TCPListener":                         "net",
	"net.UDPAddr":                             "net",
	"net.UDPConn":                             "net",
	"net.UnixAddr":                            "net",
	"net.UnixConn":                            "net",
	"net.UnixListener":                        "net",
	"net.UnknownNetworkError":                 "net",
	"os.Args":                                 "os",
	"os.Chdir":                                "os",
	"os.Chmod":                                "os",
	"os.Chown":                                "os",
	"os.Chtimes":                              "os",
	"os.Clearenv":                             "os",
	"os.Create":                               "os",
	"os.DevNull":                              "os",
	"os.Environ":                              "os",
	"os.ErrExist":                             "os",
	"os.ErrInvalid":                           "os",
	"os.ErrNotExist":                          "os",
	"os.ErrPermission":                        "os",
	"os.Exit":                                 "os",
	"os.Expand":                               "os",
	"os.ExpandEnv":                            "os",
	"os.File":                                 "os",
	"os.FileInfo":                             "os",
	"os.FileMode":                             "os",
	"os.FindProcess":                          "os",
	"os.Getegid":                              "os",
	"os.Getenv":                               "os",
	"os.Geteuid":                              "os",
	"os.Getgid":                               "os",
	"os.Getgroups":                            "os",
	"os.Getpagesize":                          "os",
	"os.Getpid":                               "os",
	"os.Getppid":                              "os",
	"os.Getuid":                               "os",
	"os.Getwd":                                "os",
	"os.Hostname":                             "os",
	"os.Interrupt":                            "os",
	"os.IsExist":                              "os",
	"os.IsNotExist":                           "os",
	"os.IsPathSeparator":                      "os",
	"os.IsPermission":                         "os",
	"os.Kill":                                 "os",
	"os.Lchown":                               "os",
	"os.Link":                                 "os",
	"os.LinkError":                            "os",
	"os.Lstat":                                "os",
	"os.Mkdir":                                "os",
	"os.MkdirAll":                             "os",
	"os.ModeAppend":                           "os",
	"os.ModeCharDevice":                       "os",
	"os.ModeDevice":                           "os",
	"os.ModeDir":                              "os",
	"os.ModeExclusive":                        "os",
	"os.ModeNamedPipe":                        "os",
	"os.ModePerm":                             "os",
	"os.ModeSetgid":                           "os",
	"os.ModeSetuid":                           "os",
	"os.ModeSocket":                           "os",
	"os.ModeSticky":                           "os",
	"os.ModeSymlink":                          "os",
	"os.ModeTemporary":                        "os",
	"os.ModeType":                             "os",
	"os.NewFile":                              "os",
	"os.NewSyscallError":                      "os",
	"os.O_APPEND":                             "os",
	"os.O_CREATE":                             "os",
	"os.O_EXCL":                               "os",
	"os.O_RDONLY":                             "os",
	"os.O_RDWR":                               "os",
	"os.O_SYNC":                               "os",
	"os.O_TRUNC":                              "os",
	"os.O_WRONLY":                             "os",
	"os.Open":                                 "os",
	"os.OpenFile":                             "os",
	"os.PathError":                            "os",
	"os.PathListSeparator":                    "os",
	"os.PathSeparator":                        "os",
	"os.Pipe":                                 "os",
	"os.ProcAttr":                             "os",
	"os.Process":                              "os",
	"os.ProcessState":                         "os",
	"os.Readlink":                             "os",
	"os.Remove":                               "os",
	"os.RemoveAll":                            "os",
	"os.Rename":                               "os",
	"os.SEEK_CUR":                             "os",
	"os.SEEK_END":                             "os",
	"os.SEEK_SET":                             "os",
	"os.SameFile":                             "os",
	"os.Setenv":                               "os",
	"os.Signal":                               "os",
	"os.StartProcess":                         "os",
	"os.Stat":                                 "os",
	"os.Stderr":                               "os",
	"os.Stdin":                                "os",
	"os.Stdout":                               "os",
	"os.Symlink":                              "os",
	"os.SyscallError":                         "os",
	"os.TempDir":                              "os",
	"os.Truncate":                             "os",
	"palette.Plan9":                           "image/color/palette",
	"palette.WebSafe":                         "image/color/palette",
	"parse.ActionNode":                        "text/template/parse",
	"parse.BoolNode":                          "text/template/parse",
	"parse.BranchNode":                        "text/template/parse",
	"parse.ChainNode":                         "text/template/parse",
	"parse.CommandNode":                       "text/template/parse",
	"parse.DotNode":                           "text/template/parse",
	"parse.FieldNode":                         "text/template/parse",
	"parse.IdentifierNode":                    "text/template/parse",
	"parse.IfNode":                            "text/template/parse",
	"parse.IsEmptyTree":                       "text/template/parse",
	"parse.ListNode":                          "text/template/parse",
	"parse.New":                               "text/template/parse",
	"parse.NewIdentifier":                     "text/template/parse",
	"parse.NilNode":                           "text/template/parse",
	"parse.Node":                              "text/template/parse",
	"parse.NodeAction":                        "text/template/parse",
	"parse.NodeBool":                          "text/template/parse",
	"parse.NodeChain":                         "text/template/parse",
	"parse.NodeCommand":                       "text/template/parse",
	"parse.NodeDot":                           "text/template/parse",
	"parse.NodeField":                         "text/template/parse",
	"parse.NodeIdentifier":                    "text/template/parse",
	"parse.NodeIf":                            "text/template/parse",
	"parse.NodeList":                          "text/template/parse",
	"parse.NodeNil":                           "text/template/parse",
	"parse.NodeNumber":                        "text/template/parse",
	"parse.NodePipe":                          "text/template/parse",
	"parse.NodeRange":                         "text/template/parse",
	"parse.NodeString":                        "text/template/parse",
	"parse.NodeTemplate":                      "text/template/parse",
	"parse.NodeText":                          "text/template/parse",
	"parse.NodeType":                          "text/template/parse",
	"parse.NodeVariable":                      "text/template/parse",
	"parse.NodeWith":                          "text/template/parse",
	"parse.NumberNode":                        "text/template/parse",
	"parse.Parse":                             "text/template/parse",
	"parse.PipeNode":                          "text/template/parse",
	"parse.Pos":                               "text/template/parse",
	"parse.RangeNode":                         "text/template/parse",
	"parse.StringNode":                        "text/template/parse",
	"parse.TemplateNode":                      "text/template/parse",
	"parse.TextNode":                          "text/template/parse",
	"parse.Tree":                              "text/template/parse",
	"parse.VariableNode":                      "text/template/parse",
	"parse.WithNode":                          "text/template/parse",
	"parser.AllErrors":                        "go/parser",
	"parser.DeclarationErrors":                "go/parser",
	"parser.ImportsOnly":                      "go/parser",
	"parser.Mode":                             "go/parser",
	"parser.PackageClauseOnly":                "go/parser",
	"parser.ParseComments":                    "go/parser",
	"parser.ParseDir":                         "go/parser",
	"parser.ParseExpr":                        "go/parser",
	"parser.ParseFile":                        "go/parser",
	"parser.SpuriousErrors":                   "go/parser",
	"parser.Trace":                            "go/parser",
	"path.Base":                               "path",
	"path.Clean":                              "path",
	"path.Dir":                                "path",
	"path.ErrBadPattern":                      "path",
	"path.Ext":                                "path",
	"path.IsAbs":                              "path",
	"path.Join":                               "path",
	"path.Match":                              "path",
	"path.Split":                              "path",
	"pe.COFFSymbol":                           "debug/pe",
	"pe.COFFSymbolSize":                       "debug/pe",
	"pe.File":                                 "debug/pe",
	"pe.FileHeader":                           "debug/pe",
	"pe.FormatError":                          "debug/pe",
	"pe.IMAGE_FILE_MACHINE_AM33":              "debug/pe",
	"pe.IMAGE_FILE_MACHINE_AMD64":             "debug/pe",
	"pe.IMAGE_FILE_MACHINE_ARM":               "debug/pe",
	"pe.IMAGE_FILE_MACHINE_EBC":               "debug/pe",
	"pe.IMAGE_FILE_MACHINE_I386":              "debug/pe",
	"pe.IMAGE_FILE_MACHINE_IA64":              "debug/pe",
	"pe.IMAGE_FILE_MACHINE_M32R":              "debug/pe",
	"pe.IMAGE_FILE_MACHINE_MIPS16":            "debug/pe",
	"pe.IMAGE_FILE_MACHINE_MIPSFPU":           "debug/pe",
	"pe.IMAGE_FILE_MACHINE_MIPSFPU16":         "debug/pe",
	"pe.IMAGE_FILE_MACHINE_POWERPC":           "debug/pe",
	"pe.IMAGE_FILE_MACHINE_POWERPCFP":         "debug/pe",
	"pe.IMAGE_FILE_MACHINE_R4000":             "debug/pe",
	"pe.IMAGE_FILE_MACHINE_SH3":               "debug/pe",
	"pe.IMAGE_FILE_MACHINE_SH3DSP":            "debug/pe",
	"pe.IMAGE_FILE_MACHINE_SH4":               "debug/pe",
	"pe.IMAGE_FILE_MACHINE_SH5":               "debug/pe",
	"pe.IMAGE_FILE_MACHINE_THUMB":             "debug/pe",
	"pe.IMAGE_FILE_MACHINE_UNKNOWN":           "debug/pe",
	"pe.IMAGE_FILE_MACHINE_WCEMIPSV2":         "debug/pe",
	"pe.ImportDirectory":                      "debug/pe",
	"pe.NewFile":                              "debug/pe",
	"pe.Open":                                 "debug/pe",
	"pe.Section":                              "debug/pe",
	"pe.SectionHeader":                        "debug/pe",
	"pe.SectionHeader32":                      "debug/pe",
	"pe.Symbol":                               "debug/pe",
	"pem.Block":                               "encoding/pem",
	"pem.Decode":                              "encoding/pem",
	"pem.Encode":                              "encoding/pem",
	"pem.EncodeToMemory":                      "encoding/pem",
	"pkix.AlgorithmIdentifier":                "crypto/x509/pkix",
	"pkix.AttributeTypeAndValue":              "crypto/x509/pkix",
	"pkix.CertificateList":                    "crypto/x509/pkix",
	"pkix.Extension":                          "crypto/x509/pkix",
	"pkix.Name":                               "crypto/x509/pkix",
	"pkix.RDNSequence":                        "crypto/x509/pkix",
	"pkix.RelativeDistinguishedNameSET":       "crypto/x509/pkix",
	"pkix.RevokedCertificate":                 "crypto/x509/pkix",
	"pkix.TBSCertificateList":                 "crypto/x509/pkix",
	"png.Decode":                              "image/png",
	"png.DecodeConfig":                        "image/png",
	"png.Encode":                              "image/png",
	"png.FormatError":                         "image/png",
	"png.UnsupportedError":                    "image/png",
	"pprof.Cmdline":                           "net/http/pprof",
	"pprof.Handler":                           "net/http/pprof",
	"pprof.Index":                             "net/http/pprof",
	"pprof.Lookup":                            "runtime/pprof",
	"pprof.NewProfile":                        "runtime/pprof",
	// "pprof.Profile" is ambiguous
	"pprof.Profiles":         "runtime/pprof",
	"pprof.StartCPUProfile":  "runtime/pprof",
	"pprof.StopCPUProfile":   "runtime/pprof",
	"pprof.Symbol":           "net/http/pprof",
	"pprof.WriteHeapProfile": "runtime/pprof",
	"printer.CommentedNode":  "go/printer",
	"printer.Config":         "go/printer",
	"printer.Fprint":         "go/printer",
	"printer.Mode":           "go/printer",
	"printer.RawFormat":      "go/printer",
	"printer.SourcePos":      "go/printer",
	"printer.TabIndent":      "go/printer",
	"printer.UseSpaces":      "go/printer",
	"quick.Check":            "testing/quick",
	"quick.CheckEqual":       "testing/quick",
	"quick.CheckEqualError":  "testing/quick",
	"quick.CheckError":       "testing/quick",
	"quick.Config":           "testing/quick",
	"quick.Generator":        "testing/quick",
	"quick.SetupError":       "testing/quick",
	"quick.Value":            "testing/quick",
	"rand.ExpFloat64":        "math/rand",
	"rand.Float32":           "math/rand",
	"rand.Float64":           "math/rand",
	// "rand.Int" is ambiguous
	"rand.Int31":                    "math/rand",
	"rand.Int31n":                   "math/rand",
	"rand.Int63":                    "math/rand",
	"rand.Int63n":                   "math/rand",
	"rand.Intn":                     "math/rand",
	"rand.New":                      "math/rand",
	"rand.NewSource":                "math/rand",
	"rand.NewZipf":                  "math/rand",
	"rand.NormFloat64":              "math/rand",
	"rand.Perm":                     "math/rand",
	"rand.Prime":                    "crypto/rand",
	"rand.Rand":                     "math/rand",
	"rand.Read":                     "crypto/rand",
	"rand.Reader":                   "crypto/rand",
	"rand.Seed":                     "math/rand",
	"rand.Source":                   "math/rand",
	"rand.Uint32":                   "math/rand",
	"rand.Zipf":                     "math/rand",
	"rc4.Cipher":                    "crypto/rc4",
	"rc4.KeySizeError":              "crypto/rc4",
	"rc4.NewCipher":                 "crypto/rc4",
	"reflect.Append":                "reflect",
	"reflect.AppendSlice":           "reflect",
	"reflect.Array":                 "reflect",
	"reflect.Bool":                  "reflect",
	"reflect.BothDir":               "reflect",
	"reflect.Chan":                  "reflect",
	"reflect.ChanDir":               "reflect",
	"reflect.ChanOf":                "reflect",
	"reflect.Complex128":            "reflect",
	"reflect.Complex64":             "reflect",
	"reflect.Copy":                  "reflect",
	"reflect.DeepEqual":             "reflect",
	"reflect.Float32":               "reflect",
	"reflect.Float64":               "reflect",
	"reflect.Func":                  "reflect",
	"reflect.Indirect":              "reflect",
	"reflect.Int":                   "reflect",
	"reflect.Int16":                 "reflect",
	"reflect.Int32":                 "reflect",
	"reflect.Int64":                 "reflect",
	"reflect.Int8":                  "reflect",
	"reflect.Interface":             "reflect",
	"reflect.Invalid":               "reflect",
	"reflect.Kind":                  "reflect",
	"reflect.MakeChan":              "reflect",
	"reflect.MakeFunc":              "reflect",
	"reflect.MakeMap":               "reflect",
	"reflect.MakeSlice":             "reflect",
	"reflect.Map":                   "reflect",
	"reflect.MapOf":                 "reflect",
	"reflect.Method":                "reflect",
	"reflect.New":                   "reflect",
	"reflect.NewAt":                 "reflect",
	"reflect.Ptr":                   "reflect",
	"reflect.PtrTo":                 "reflect",
	"reflect.RecvDir":               "reflect",
	"reflect.Select":                "reflect",
	"reflect.SelectCase":            "reflect",
	"reflect.SelectDefault":         "reflect",
	"reflect.SelectDir":             "reflect",
	"reflect.SelectRecv":            "reflect",
	"reflect.SelectSend":            "reflect",
	"reflect.SendDir":               "reflect",
	"reflect.Slice":                 "reflect",
	"reflect.SliceHeader":           "reflect",
	"reflect.SliceOf":               "reflect",
	"reflect.String":                "reflect",
	"reflect.StringHeader":          "reflect",
	"reflect.Struct":                "reflect",
	"reflect.StructField":           "reflect",
	"reflect.StructTag":             "reflect",
	"reflect.TypeOf":                "reflect",
	"reflect.Uint":                  "reflect",
	"reflect.Uint16":                "reflect",
	"reflect.Uint32":                "reflect",
	"reflect.Uint64":                "reflect",
	"reflect.Uint8":                 "reflect",
	"reflect.Uintptr":               "reflect",
	"reflect.UnsafePointer":         "reflect",
	"reflect.Value":                 "reflect",
	"reflect.ValueError":            "reflect",
	"reflect.ValueOf":               "reflect",
	"reflect.Zero":                  "reflect",
	"regexp.Compile":                "regexp",
	"regexp.CompilePOSIX":           "regexp",
	"regexp.Match":                  "regexp",
	"regexp.MatchReader":            "regexp",
	"regexp.MatchString":            "regexp",
	"regexp.MustCompile":            "regexp",
	"regexp.MustCompilePOSIX":       "regexp",
	"regexp.QuoteMeta":              "regexp",
	"regexp.Regexp":                 "regexp",
	"ring.New":                      "container/ring",
	"ring.Ring":                     "container/ring",
	"rpc.Accept":                    "net/rpc",
	"rpc.Call":                      "net/rpc",
	"rpc.Client":                    "net/rpc",
	"rpc.ClientCodec":               "net/rpc",
	"rpc.DefaultDebugPath":          "net/rpc",
	"rpc.DefaultRPCPath":            "net/rpc",
	"rpc.DefaultServer":             "net/rpc",
	"rpc.Dial":                      "net/rpc",
	"rpc.DialHTTP":                  "net/rpc",
	"rpc.DialHTTPPath":              "net/rpc",
	"rpc.ErrShutdown":               "net/rpc",
	"rpc.HandleHTTP":                "net/rpc",
	"rpc.NewClient":                 "net/rpc",
	"rpc.NewClientWithCodec":        "net/rpc",
	"rpc.NewServer":                 "net/rpc",
	"rpc.Register":                  "net/rpc",
	"rpc.RegisterName":              "net/rpc",
	"rpc.Request":                   "net/rpc",
	"rpc.Response":                  "net/rpc",
	"rpc.ServeCodec":                "net/rpc",
	"rpc.ServeConn":                 "net/rpc",
	"rpc.ServeRequest":              "net/rpc",
	"rpc.Server":                    "net/rpc",
	"rpc.ServerCodec":               "net/rpc",
	"rpc.ServerError":               "net/rpc",
	"rsa.CRTValue":                  "crypto/rsa",
	"rsa.DecryptOAEP":               "crypto/rsa",
	"rsa.DecryptPKCS1v15":           "crypto/rsa",
	"rsa.DecryptPKCS1v15SessionKey": "crypto/rsa",
	"rsa.EncryptOAEP":               "crypto/rsa",
	"rsa.EncryptPKCS1v15":           "crypto/rsa",
	"rsa.ErrDecryption":             "crypto/rsa",
	"rsa.ErrMessageTooLong":         "crypto/rsa",
	"rsa.ErrVerification":           "crypto/rsa",
	"rsa.GenerateKey":               "crypto/rsa",
	"rsa.GenerateMultiPrimeKey":     "crypto/rsa",
	"rsa.PSSOptions":                "crypto/rsa",
	"rsa.PSSSaltLengthAuto":         "crypto/rsa",
	"rsa.PSSSaltLengthEqualsHash":   "crypto/rsa",
	"rsa.PrecomputedValues":         "crypto/rsa",
	"rsa.PrivateKey":                "crypto/rsa",
	"rsa.PublicKey":                 "crypto/rsa",
	"rsa.SignPKCS1v15":              "crypto/rsa",
	"rsa.SignPSS":                   "crypto/rsa",
	"rsa.VerifyPKCS1v15":            "crypto/rsa",
	"rsa.VerifyPSS":                 "crypto/rsa",
	"runtime.BlockProfile":          "runtime",
	"runtime.BlockProfileRecord":    "runtime",
	"runtime.Breakpoint":            "runtime",
	"runtime.CPUProfile":            "runtime",
	"runtime.Caller":                "runtime",
	"runtime.Callers":               "runtime",
	"runtime.Compiler":              "runtime",
	"runtime.Error":                 "runtime",
	"runtime.Func":                  "runtime",
	"runtime.FuncForPC":             "runtime",
	"runtime.GC":                    "runtime",
	"runtime.GOARCH":                "runtime",
	"runtime.GOMAXPROCS":            "runtime",
	"runtime.GOOS":                  "runtime",
	"runtime.GOROOT":                "runtime",
	"runtime.Goexit":                "runtime",
	"runtime.GoroutineProfile":      "runtime",
	"runtime.Gosched":               "runtime",
	"runtime.LockOSThread":          "runtime",
	"runtime.MemProfile":            "runtime",
	"runtime.MemProfileRate":        "runtime",
	"runtime.MemProfileRecord":      "runtime",
	"runtime.MemStats":              "runtime",
	"runtime.NumCPU":                "runtime",
	"runtime.NumCgoCall":            "runtime",
	"runtime.NumGoroutine":          "runtime",
	"runtime.ReadMemStats":          "runtime",
	"runtime.SetBlockProfileRate":   "runtime",
	"runtime.SetCPUProfileRate":     "runtime",
	"runtime.SetFinalizer":          "runtime",
	"runtime.Stack":                 "runtime",
	"runtime.StackRecord":           "runtime",
	"runtime.ThreadCreateProfile":   "runtime",
	"runtime.TypeAssertionError":    "runtime",
	"runtime.UnlockOSThread":        "runtime",
	"runtime.Version":               "runtime",
	"scanner.Char":                  "text/scanner",
	"scanner.Comment":               "text/scanner",
	"scanner.EOF":                   "text/scanner",
	"scanner.Error":                 "go/scanner",
	"scanner.ErrorHandler":          "go/scanner",
	"scanner.ErrorList":             "go/scanner",
	"scanner.Float":                 "text/scanner",
	"scanner.GoTokens":              "text/scanner",
	"scanner.GoWhitespace":          "text/scanner",
	"scanner.Ident":                 "text/scanner",
	"scanner.Int":                   "text/scanner",
	"scanner.Mode":                  "go/scanner",
	"scanner.Position":              "text/scanner",
	"scanner.PrintError":            "go/scanner",
	"scanner.RawString":             "text/scanner",
	"scanner.ScanChars":             "text/scanner",
	// "scanner.ScanComments" is ambiguous
	"scanner.ScanFloats":     "text/scanner",
	"scanner.ScanIdents":     "text/scanner",
	"scanner.ScanInts":       "text/scanner",
	"scanner.ScanRawStrings": "text/scanner",
	"scanner.ScanStrings":    "text/scanner",
	// "scanner.Scanner" is ambiguous
	"scanner.SkipComments":                                 "text/scanner",
	"scanner.String":                                       "text/scanner",
	"scanner.TokenString":                                  "text/scanner",
	"sha1.BlockSize":                                       "crypto/sha1",
	"sha1.New":                                             "crypto/sha1",
	"sha1.Size":                                            "crypto/sha1",
	"sha1.Sum":                                             "crypto/sha1",
	"sha256.BlockSize":                                     "crypto/sha256",
	"sha256.New":                                           "crypto/sha256",
	"sha256.New224":                                        "crypto/sha256",
	"sha256.Size":                                          "crypto/sha256",
	"sha256.Size224":                                       "crypto/sha256",
	"sha256.Sum224":                                        "crypto/sha256",
	"sha256.Sum256":                                        "crypto/sha256",
	"sha512.BlockSize":                                     "crypto/sha512",
	"sha512.New":                                           "crypto/sha512",
	"sha512.New384":                                        "crypto/sha512",
	"sha512.Size":                                          "crypto/sha512",
	"sha512.Size384":                                       "crypto/sha512",
	"sha512.Sum384":                                        "crypto/sha512",
	"sha512.Sum512":                                        "crypto/sha512",
	"signal.Notify":                                        "os/signal",
	"signal.Stop":                                          "os/signal",
	"smtp.Auth":                                            "net/smtp",
	"smtp.CRAMMD5Auth":                                     "net/smtp",
	"smtp.Client":                                          "net/smtp",
	"smtp.Dial":                                            "net/smtp",
	"smtp.NewClient":                                       "net/smtp",
	"smtp.PlainAuth":                                       "net/smtp",
	"smtp.SendMail":                                        "net/smtp",
	"smtp.ServerInfo":                                      "net/smtp",
	"sort.Float64Slice":                                    "sort",
	"sort.Float64s":                                        "sort",
	"sort.Float64sAreSorted":                               "sort",
	"sort.IntSlice":                                        "sort",
	"sort.Interface":                                       "sort",
	"sort.Ints":                                            "sort",
	"sort.IntsAreSorted":                                   "sort",
	"sort.IsSorted":                                        "sort",
	"sort.Reverse":                                         "sort",
	"sort.Search":                                          "sort",
	"sort.SearchFloat64s":                                  "sort",
	"sort.SearchInts":                                      "sort",
	"sort.SearchStrings":                                   "sort",
	"sort.Sort":                                            "sort",
	"sort.Stable":                                          "sort",
	"sort.StringSlice":                                     "sort",
	"sort.Strings":                                         "sort",
	"sort.StringsAreSorted":                                "sort",
	"sql.DB":                                               "database/sql",
	"sql.ErrNoRows":                                        "database/sql",
	"sql.ErrTxDone":                                        "database/sql",
	"sql.NullBool":                                         "database/sql",
	"sql.NullFloat64":                                      "database/sql",
	"sql.NullInt64":                                        "database/sql",
	"sql.NullString":                                       "database/sql",
	"sql.Open":                                             "database/sql",
	"sql.RawBytes":                                         "database/sql",
	"sql.Register":                                         "database/sql",
	"sql.Result":                                           "database/sql",
	"sql.Row":                                              "database/sql",
	"sql.Rows":                                             "database/sql",
	"sql.Scanner":                                          "database/sql",
	"sql.Stmt":                                             "database/sql",
	"sql.Tx":                                               "database/sql",
	"strconv.AppendBool":                                   "strconv",
	"strconv.AppendFloat":                                  "strconv",
	"strconv.AppendInt":                                    "strconv",
	"strconv.AppendQuote":                                  "strconv",
	"strconv.AppendQuoteRune":                              "strconv",
	"strconv.AppendQuoteRuneToASCII":                       "strconv",
	"strconv.AppendQuoteToASCII":                           "strconv",
	"strconv.AppendUint":                                   "strconv",
	"strconv.Atoi":                                         "strconv",
	"strconv.CanBackquote":                                 "strconv",
	"strconv.ErrRange":                                     "strconv",
	"strconv.ErrSyntax":                                    "strconv",
	"strconv.FormatBool":                                   "strconv",
	"strconv.FormatFloat":                                  "strconv",
	"strconv.FormatInt":                                    "strconv",
	"strconv.FormatUint":                                   "strconv",
	"strconv.IntSize":                                      "strconv",
	"strconv.IsPrint":                                      "strconv",
	"strconv.Itoa":                                         "strconv",
	"strconv.NumError":                                     "strconv",
	"strconv.ParseBool":                                    "strconv",
	"strconv.ParseFloat":                                   "strconv",
	"strconv.ParseInt":                                     "strconv",
	"strconv.ParseUint":                                    "strconv",
	"strconv.Quote":                                        "strconv",
	"strconv.QuoteRune":                                    "strconv",
	"strconv.QuoteRuneToASCII":                             "strconv",
	"strconv.QuoteToASCII":                                 "strconv",
	"strconv.Unquote":                                      "strconv",
	"strconv.UnquoteChar":                                  "strconv",
	"strings.Contains":                                     "strings",
	"strings.ContainsAny":                                  "strings",
	"strings.ContainsRune":                                 "strings",
	"strings.Count":                                        "strings",
	"strings.EqualFold":                                    "strings",
	"strings.Fields":                                       "strings",
	"strings.FieldsFunc":                                   "strings",
	"strings.HasPrefix":                                    "strings",
	"strings.HasSuffix":                                    "strings",
	"strings.Index":                                        "strings",
	"strings.IndexAny":                                     "strings",
	"strings.IndexByte":                                    "strings",
	"strings.IndexFunc":                                    "strings",
	"strings.IndexRune":                                    "strings",
	"strings.Join":                                         "strings",
	"strings.LastIndex":                                    "strings",
	"strings.LastIndexAny":                                 "strings",
	"strings.LastIndexFunc":                                "strings",
	"strings.Map":                                          "strings",
	"strings.NewReader":                                    "strings",
	"strings.NewReplacer":                                  "strings",
	"strings.Reader":                                       "strings",
	"strings.Repeat":                                       "strings",
	"strings.Replace":                                      "strings",
	"strings.Replacer":                                     "strings",
	"strings.Split":                                        "strings",
	"strings.SplitAfter":                                   "strings",
	"strings.SplitAfterN":                                  "strings",
	"strings.SplitN":                                       "strings",
	"strings.Title":                                        "strings",
	"strings.ToLower":                                      "strings",
	"strings.ToLowerSpecial":                               "strings",
	"strings.ToTitle":                                      "strings",
	"strings.ToTitleSpecial":                               "strings",
	"strings.ToUpper":                                      "strings",
	"strings.ToUpperSpecial":                               "strings",
	"strings.Trim":                                         "strings",
	"strings.TrimFunc":                                     "strings",
	"strings.TrimLeft":                                     "strings",
	"strings.TrimLeftFunc":                                 "strings",
	"strings.TrimPrefix":                                   "strings",
	"strings.TrimRight":                                    "strings",
	"strings.TrimRightFunc":                                "strings",
	"strings.TrimSpace":                                    "strings",
	"strings.TrimSuffix":                                   "strings",
	"subtle.ConstantTimeByteEq":                            "crypto/subtle",
	"subtle.ConstantTimeCompare":                           "crypto/subtle",
	"subtle.ConstantTimeCopy":                              "crypto/subtle",
	"subtle.ConstantTimeEq":                                "crypto/subtle",
	"subtle.ConstantTimeLessOrEq":                          "crypto/subtle",
	"subtle.ConstantTimeSelect":                            "crypto/subtle",
	"suffixarray.Index":                                    "index/suffixarray",
	"suffixarray.New":                                      "index/suffixarray",
	"sync.Cond":                                            "sync",
	"sync.Locker":                                          "sync",
	"sync.Mutex":                                           "sync",
	"sync.NewCond":                                         "sync",
	"sync.Once":                                            "sync",
	"sync.RWMutex":                                         "sync",
	"sync.WaitGroup":                                       "sync",
	"syntax.ClassNL":                                       "regexp/syntax",
	"syntax.Compile":                                       "regexp/syntax",
	"syntax.DotNL":                                         "regexp/syntax",
	"syntax.EmptyBeginLine":                                "regexp/syntax",
	"syntax.EmptyBeginText":                                "regexp/syntax",
	"syntax.EmptyEndLine":                                  "regexp/syntax",
	"syntax.EmptyEndText":                                  "regexp/syntax",
	"syntax.EmptyNoWordBoundary":                           "regexp/syntax",
	"syntax.EmptyOp":                                       "regexp/syntax",
	"syntax.EmptyOpContext":                                "regexp/syntax",
	"syntax.EmptyWordBoundary":                             "regexp/syntax",
	"syntax.ErrInternalError":                              "regexp/syntax",
	"syntax.ErrInvalidCharClass":                           "regexp/syntax",
	"syntax.ErrInvalidCharRange":                           "regexp/syntax",
	"syntax.ErrInvalidEscape":                              "regexp/syntax",
	"syntax.ErrInvalidNamedCapture":                        "regexp/syntax",
	"syntax.ErrInvalidPerlOp":                              "regexp/syntax",
	"syntax.ErrInvalidRepeatOp":                            "regexp/syntax",
	"syntax.ErrInvalidRepeatSize":                          "regexp/syntax",
	"syntax.ErrInvalidUTF8":                                "regexp/syntax",
	"syntax.ErrMissingBracket":                             "regexp/syntax",
	"syntax.ErrMissingParen":                               "regexp/syntax",
	"syntax.ErrMissingRepeatArgument":                      "regexp/syntax",
	"syntax.ErrTrailingBackslash":                          "regexp/syntax",
	"syntax.ErrUnexpectedParen":                            "regexp/syntax",
	"syntax.Error":                                         "regexp/syntax",
	"syntax.ErrorCode":                                     "regexp/syntax",
	"syntax.Flags":                                         "regexp/syntax",
	"syntax.FoldCase":                                      "regexp/syntax",
	"syntax.Inst":                                          "regexp/syntax",
	"syntax.InstAlt":                                       "regexp/syntax",
	"syntax.InstAltMatch":                                  "regexp/syntax",
	"syntax.InstCapture":                                   "regexp/syntax",
	"syntax.InstEmptyWidth":                                "regexp/syntax",
	"syntax.InstFail":                                      "regexp/syntax",
	"syntax.InstMatch":                                     "regexp/syntax",
	"syntax.InstNop":                                       "regexp/syntax",
	"syntax.InstOp":                                        "regexp/syntax",
	"syntax.InstRune":                                      "regexp/syntax",
	"syntax.InstRune1":                                     "regexp/syntax",
	"syntax.InstRuneAny":                                   "regexp/syntax",
	"syntax.InstRuneAnyNotNL":                              "regexp/syntax",
	"syntax.IsWordChar":                                    "regexp/syntax",
	"syntax.Literal":                                       "regexp/syntax",
	"syntax.MatchNL":                                       "regexp/syntax",
	"syntax.NonGreedy":                                     "regexp/syntax",
	"syntax.OneLine":                                       "regexp/syntax",
	"syntax.Op":                                            "regexp/syntax",
	"syntax.OpAlternate":                                   "regexp/syntax",
	"syntax.OpAnyChar":                                     "regexp/syntax",
	"syntax.OpAnyCharNotNL":                                "regexp/syntax",
	"syntax.OpBeginLine":                                   "regexp/syntax",
	"syntax.OpBeginText":                                   "regexp/syntax",
	"syntax.OpCapture":                                     "regexp/syntax",
	"syntax.OpCharClass":                                   "regexp/syntax",
	"syntax.OpConcat":                                      "regexp/syntax",
	"syntax.OpEmptyMatch":                                  "regexp/syntax",
	"syntax.OpEndLine":                                     "regexp/syntax",
	"syntax.OpEndText":                                     "regexp/syntax",
	"syntax.OpLiteral":                                     "regexp/syntax",
	"syntax.OpNoMatch":                                     "regexp/syntax",
	"syntax.OpNoWordBoundary":                              "regexp/syntax",
	"syntax.OpPlus":                                        "regexp/syntax",
	"syntax.OpQuest":                                       "regexp/syntax",
	"syntax.OpRepeat":                                      "regexp/syntax",
	"syntax.OpStar":                                        "regexp/syntax",
	"syntax.OpWordBoundary":                                "regexp/syntax",
	"syntax.POSIX":                                         "regexp/syntax",
	"syntax.Parse":                                         "regexp/syntax",
	"syntax.Perl":                                          "regexp/syntax",
	"syntax.PerlX":                                         "regexp/syntax",
	"syntax.Prog":                                          "regexp/syntax",
	"syntax.Regexp":                                        "regexp/syntax",
	"syntax.Simple":                                        "regexp/syntax",
	"syntax.UnicodeGroups":                                 "regexp/syntax",
	"syntax.WasDollar":                                     "regexp/syntax",
	"syscall.AF_ALG":                                       "syscall",
	"syscall.AF_APPLETALK":                                 "syscall",
	"syscall.AF_ARP":                                       "syscall",
	"syscall.AF_ASH":                                       "syscall",
	"syscall.AF_ATM":                                       "syscall",
	"syscall.AF_ATMPVC":                                    "syscall",
	"syscall.AF_ATMSVC":                                    "syscall",
	"syscall.AF_AX25":                                      "syscall",
	"syscall.AF_BLUETOOTH":                                 "syscall",
	"syscall.AF_BRIDGE":                                    "syscall",
	"syscall.AF_CAIF":                                      "syscall",
	"syscall.AF_CAN":                                       "syscall",
	"syscall.AF_CCITT":                                     "syscall",
	"syscall.AF_CHAOS":                                     "syscall",
	"syscall.AF_CNT":                                       "syscall",
	"syscall.AF_COIP":                                      "syscall",
	"syscall.AF_DATAKIT":                                   "syscall",
	"syscall.AF_DECnet":                                    "syscall",
	"syscall.AF_DLI":                                       "syscall",
	"syscall.AF_E164":                                      "syscall",
	"syscall.AF_ECMA":                                      "syscall",
	"syscall.AF_ECONET":                                    "syscall",
	"syscall.AF_ENCAP":                                     "syscall",
	"syscall.AF_FILE":                                      "syscall",
	"syscall.AF_HYLINK":                                    "syscall",
	"syscall.AF_IEEE80211":                                 "syscall",
	"syscall.AF_IEEE802154":                                "syscall",
	"syscall.AF_IMPLINK":                                   "syscall",
	"syscall.AF_INET":                                      "syscall",
	"syscall.AF_INET6":                                     "syscall",
	"syscall.AF_IPX":                                       "syscall",
	"syscall.AF_IRDA":                                      "syscall",
	"syscall.AF_ISDN":                                      "syscall",
	"syscall.AF_ISO":                                       "syscall",
	"syscall.AF_IUCV":                                      "syscall",
	"syscall.AF_KEY":                                       "syscall",
	"syscall.AF_LAT":                                       "syscall",
	"syscall.AF_LINK":                                      "syscall",
	"syscall.AF_LLC":                                       "syscall",
	"syscall.AF_LOCAL":                                     "syscall",
	"syscall.AF_MAX":                                       "syscall",
	"syscall.AF_MPLS":                                      "syscall",
	"syscall.AF_NATM":                                      "syscall",
	"syscall.AF_NDRV":                                      "syscall",
	"syscall.AF_NETBEUI":                                   "syscall",
	"syscall.AF_NETBIOS":                                   "syscall",
	"syscall.AF_NETGRAPH":                                  "syscall",
	"syscall.AF_NETLINK":                                   "syscall",
	"syscall.AF_NETROM":                                    "syscall",
	"syscall.AF_NS":                                        "syscall",
	"syscall.AF_OROUTE":                                    "syscall",
	"syscall.AF_OSI":                                       "syscall",
	"syscall.AF_PACKET":                                    "syscall",
	"syscall.AF_PHONET":                                    "syscall",
	"syscall.AF_PPP":                                       "syscall",
	"syscall.AF_PPPOX":                                     "syscall",
	"syscall.AF_PUP":                                       "syscall",
	"syscall.AF_RDS":                                       "syscall",
	"syscall.AF_RESERVED_36":                               "syscall",
	"syscall.AF_ROSE":                                      "syscall",
	"syscall.AF_ROUTE":                                     "syscall",
	"syscall.AF_RXRPC":                                     "syscall",
	"syscall.AF_SCLUSTER":                                  "syscall",
	"syscall.AF_SECURITY":                                  "syscall",
	"syscall.AF_SIP":                                       "syscall",
	"syscall.AF_SLOW":                                      "syscall",
	"syscall.AF_SNA":                                       "syscall",
	"syscall.AF_SYSTEM":                                    "syscall",
	"syscall.AF_TIPC":                                      "syscall",
	"syscall.AF_UNIX":                                      "syscall",
	"syscall.AF_UNSPEC":                                    "syscall",
	"syscall.AF_VENDOR00":                                  "syscall",
	"syscall.AF_VENDOR01":                                  "syscall",
	"syscall.AF_VENDOR02":                                  "syscall",
	"syscall.AF_VENDOR03":                                  "syscall",
	"syscall.AF_VENDOR04":                                  "syscall",
	"syscall.AF_VENDOR05":                                  "syscall",
	"syscall.AF_VENDOR06":                                  "syscall",
	"syscall.AF_VENDOR07":                                  "syscall",
	"syscall.AF_VENDOR08":                                  "syscall",
	"syscall.AF_VENDOR09":                                  "syscall",
	"syscall.AF_VENDOR10":                                  "syscall",
	"syscall.AF_VENDOR11":                                  "syscall",
	"syscall.AF_VENDOR12":                                  "syscall",
	"syscall.AF_VENDOR13":                                  "syscall",
	"syscall.AF_VENDOR14":                                  "syscall",
	"syscall.AF_VENDOR15":                                  "syscall",
	"syscall.AF_VENDOR16":                                  "syscall",
	"syscall.AF_VENDOR17":                                  "syscall",
	"syscall.AF_VENDOR18":                                  "syscall",
	"syscall.AF_VENDOR19":                                  "syscall",
	"syscall.AF_VENDOR20":                                  "syscall",
	"syscall.AF_VENDOR21":                                  "syscall",
	"syscall.AF_VENDOR22":                                  "syscall",
	"syscall.AF_VENDOR23":                                  "syscall",
	"syscall.AF_VENDOR24":                                  "syscall",
	"syscall.AF_VENDOR25":                                  "syscall",
	"syscall.AF_VENDOR26":                                  "syscall",
	"syscall.AF_VENDOR27":                                  "syscall",
	"syscall.AF_VENDOR28":                                  "syscall",
	"syscall.AF_VENDOR29":                                  "syscall",
	"syscall.AF_VENDOR30":                                  "syscall",
	"syscall.AF_VENDOR31":                                  "syscall",
	"syscall.AF_VENDOR32":                                  "syscall",
	"syscall.AF_VENDOR33":                                  "syscall",
	"syscall.AF_VENDOR34":                                  "syscall",
	"syscall.AF_VENDOR35":                                  "syscall",
	"syscall.AF_VENDOR36":                                  "syscall",
	"syscall.AF_VENDOR37":                                  "syscall",
	"syscall.AF_VENDOR38":                                  "syscall",
	"syscall.AF_VENDOR39":                                  "syscall",
	"syscall.AF_VENDOR40":                                  "syscall",
	"syscall.AF_VENDOR41":                                  "syscall",
	"syscall.AF_VENDOR42":                                  "syscall",
	"syscall.AF_VENDOR43":                                  "syscall",
	"syscall.AF_VENDOR44":                                  "syscall",
	"syscall.AF_VENDOR45":                                  "syscall",
	"syscall.AF_VENDOR46":                                  "syscall",
	"syscall.AF_VENDOR47":                                  "syscall",
	"syscall.AF_WANPIPE":                                   "syscall",
	"syscall.AF_X25":                                       "syscall",
	"syscall.AI_CANONNAME":                                 "syscall",
	"syscall.AI_NUMERICHOST":                               "syscall",
	"syscall.AI_PASSIVE":                                   "syscall",
	"syscall.APPLICATION_ERROR":                            "syscall",
	"syscall.ARPHRD_ADAPT":                                 "syscall",
	"syscall.ARPHRD_APPLETLK":                              "syscall",
	"syscall.ARPHRD_ARCNET":                                "syscall",
	"syscall.ARPHRD_ASH":                                   "syscall",
	"syscall.ARPHRD_ATM":                                   "syscall",
	"syscall.ARPHRD_AX25":                                  "syscall",
	"syscall.ARPHRD_BIF":                                   "syscall",
	"syscall.ARPHRD_CHAOS":                                 "syscall",
	"syscall.ARPHRD_CISCO":                                 "syscall",
	"syscall.ARPHRD_CSLIP":                                 "syscall",
	"syscall.ARPHRD_CSLIP6":                                "syscall",
	"syscall.ARPHRD_DDCMP":                                 "syscall",
	"syscall.ARPHRD_DLCI":                                  "syscall",
	"syscall.ARPHRD_ECONET":                                "syscall",
	"syscall.ARPHRD_EETHER":                                "syscall",
	"syscall.ARPHRD_ETHER":                                 "syscall",
	"syscall.ARPHRD_EUI64":                                 "syscall",
	"syscall.ARPHRD_FCAL":                                  "syscall",
	"syscall.ARPHRD_FCFABRIC":                              "syscall",
	"syscall.ARPHRD_FCPL":                                  "syscall",
	"syscall.ARPHRD_FCPP":                                  "syscall",
	"syscall.ARPHRD_FDDI":                                  "syscall",
	"syscall.ARPHRD_FRAD":                                  "syscall",
	"syscall.ARPHRD_FRELAY":                                "syscall",
	"syscall.ARPHRD_HDLC":                                  "syscall",
	"syscall.ARPHRD_HIPPI":                                 "syscall",
	"syscall.ARPHRD_HWX25":                                 "syscall",
	"syscall.ARPHRD_IEEE1394":                              "syscall",
	"syscall.ARPHRD_IEEE802":                               "syscall",
	"syscall.ARPHRD_IEEE80211":                             "syscall",
	"syscall.ARPHRD_IEEE80211_PRISM":                       "syscall",
	"syscall.ARPHRD_IEEE80211_RADIOTAP":                    "syscall",
	"syscall.ARPHRD_IEEE802154":                            "syscall",
	"syscall.ARPHRD_IEEE802154_PHY":                        "syscall",
	"syscall.ARPHRD_IEEE802_TR":                            "syscall",
	"syscall.ARPHRD_INFINIBAND":                            "syscall",
	"syscall.ARPHRD_IPDDP":                                 "syscall",
	"syscall.ARPHRD_IPGRE":                                 "syscall",
	"syscall.ARPHRD_IRDA":                                  "syscall",
	"syscall.ARPHRD_LAPB":                                  "syscall",
	"syscall.ARPHRD_LOCALTLK":                              "syscall",
	"syscall.ARPHRD_LOOPBACK":                              "syscall",
	"syscall.ARPHRD_METRICOM":                              "syscall",
	"syscall.ARPHRD_NETROM":                                "syscall",
	"syscall.ARPHRD_NONE":                                  "syscall",
	"syscall.ARPHRD_PIMREG":                                "syscall",
	"syscall.ARPHRD_PPP":                                   "syscall",
	"syscall.ARPHRD_PRONET":                                "syscall",
	"syscall.ARPHRD_RAWHDLC":                               "syscall",
	"syscall.ARPHRD_ROSE":                                  "syscall",
	"syscall.ARPHRD_RSRVD":                                 "syscall",
	"syscall.ARPHRD_SIT":                                   "syscall",
	"syscall.ARPHRD_SKIP":                                  "syscall",
	"syscall.ARPHRD_SLIP":                                  "syscall",
	"syscall.ARPHRD_SLIP6":                                 "syscall",
	"syscall.ARPHRD_STRIP":                                 "syscall",
	"syscall.ARPHRD_TUNNEL":                                "syscall",
	"syscall.ARPHRD_TUNNEL6":                               "syscall",
	"syscall.ARPHRD_VOID":                                  "syscall",
	"syscall.ARPHRD_X25":                                   "syscall",
	"syscall.AUTHTYPE_CLIENT":                              "syscall",
	"syscall.AUTHTYPE_SERVER":                              "syscall",
	"syscall.Accept":                                       "syscall",
	"syscall.Accept4":                                      "syscall",
	"syscall.AcceptEx":                                     "syscall",
	"syscall.Access":                                       "syscall",
	"syscall.Acct":                                         "syscall",
	"syscall.AddrinfoW":                                    "syscall",
	"syscall.Adjtime":                                      "syscall",
	"syscall.Adjtimex":                                     "syscall",
	"syscall.AttachLsf":                                    "syscall",
	"syscall.B0":                                           "syscall",
	"syscall.B1000000":                                     "syscall",
	"syscall.B110":                                         "syscall",
	"syscall.B115200":                                      "syscall",
	"syscall.B1152000":                                     "syscall",
	"syscall.B1200":                                        "syscall",
	"syscall.B134":                                         "syscall",
	"syscall.B14400":                                       "syscall",
	"syscall.B150":                                         "syscall",
	"syscall.B1500000":                                     "syscall",
	"syscall.B1800":                                        "syscall",
	"syscall.B19200":                                       "syscall",
	"syscall.B200":                                         "syscall",
	"syscall.B2000000":                                     "syscall",
	"syscall.B230400":                                      "syscall",
	"syscall.B2400":                                        "syscall",
	"syscall.B2500000":                                     "syscall",
	"syscall.B28800":                                       "syscall",
	"syscall.B300":                                         "syscall",
	"syscall.B3000000":                                     "syscall",
	"syscall.B3500000":                                     "syscall",
	"syscall.B38400":                                       "syscall",
	"syscall.B4000000":                                     "syscall",
	"syscall.B460800":                                      "syscall",
	"syscall.B4800":                                        "syscall",
	"syscall.B50":                                          "syscall",
	"syscall.B500000":                                      "syscall",
	"syscall.B57600":                                       "syscall",
	"syscall.B576000":                                      "syscall",
	"syscall.B600":                                         "syscall",
	"syscall.B7200":                                        "syscall",
	"syscall.B75":                                          "syscall",
	"syscall.B76800":                                       "syscall",
	"syscall.B921600":                                      "syscall",
	"syscall.B9600":                                        "syscall",
	"syscall.BASE_PROTOCOL":                                "syscall",
	"syscall.BIOCFEEDBACK":                                 "syscall",
	"syscall.BIOCFLUSH":                                    "syscall",
	"syscall.BIOCGBLEN":                                    "syscall",
	"syscall.BIOCGDIRECTION":                               "syscall",
	"syscall.BIOCGDIRFILT":                                 "syscall",
	"syscall.BIOCGDLT":                                     "syscall",
	"syscall.BIOCGDLTLIST":                                 "syscall",
	"syscall.BIOCGETBUFMODE":                               "syscall",
	"syscall.BIOCGETIF":                                    "syscall",
	"syscall.BIOCGETZMAX":                                  "syscall",
	"syscall.BIOCGFEEDBACK":                                "syscall",
	"syscall.BIOCGFILDROP":                                 "syscall",
	"syscall.BIOCGHDRCMPLT":                                "syscall",
	"syscall.BIOCGRSIG":                                    "syscall",
	"syscall.BIOCGRTIMEOUT":                                "syscall",
	"syscall.BIOCGSEESENT":                                 "syscall",
	"syscall.BIOCGSTATS":                                   "syscall",
	"syscall.BIOCGSTATSOLD":                                "syscall",
	"syscall.BIOCGTSTAMP":                                  "syscall",
	"syscall.BIOCIMMEDIATE":                                "syscall",
	"syscall.BIOCLOCK":                                     "syscall",
	"syscall.BIOCPROMISC":                                  "syscall",
	"syscall.BIOCROTZBUF":                                  "syscall",
	"syscall.BIOCSBLEN":                                    "syscall",
	"syscall.BIOCSDIRECTION":                               "syscall",
	"syscall.BIOCSDIRFILT":                                 "syscall",
	"syscall.BIOCSDLT":                                     "syscall",
	"syscall.BIOCSETBUFMODE":                               "syscall",
	"syscall.BIOCSETF":                                     "syscall",
	"syscall.BIOCSETFNR":                                   "syscall",
	"syscall.BIOCSETIF":                                    "syscall",
	"syscall.BIOCSETWF":                                    "syscall",
	"syscall.BIOCSETZBUF":                                  "syscall",
	"syscall.BIOCSFEEDBACK":                                "syscall",
	"syscall.BIOCSFILDROP":                                 "syscall",
	"syscall.BIOCSHDRCMPLT":                                "syscall",
	"syscall.BIOCSRSIG":                                    "syscall",
	"syscall.BIOCSRTIMEOUT":                                "syscall",
	"syscall.BIOCSSEESENT":                                 "syscall",
	"syscall.BIOCSTCPF":                                    "syscall",
	"syscall.BIOCSTSTAMP":                                  "syscall",
	"syscall.BIOCSUDPF":                                    "syscall",
	"syscall.BIOCVERSION":                                  "syscall",
	"syscall.BPF_A":                                        "syscall",
	"syscall.BPF_ABS":                                      "syscall",
	"syscall.BPF_ADD":                                      "syscall",
	"syscall.BPF_ALIGNMENT":                                "syscall",
	"syscall.BPF_ALIGNMENT32":                              "syscall",
	"syscall.BPF_ALU":                                      "syscall",
	"syscall.BPF_AND":                                      "syscall",
	"syscall.BPF_B":                                        "syscall",
	"syscall.BPF_BUFMODE_BUFFER":                           "syscall",
	"syscall.BPF_BUFMODE_ZBUF":                             "syscall",
	"syscall.BPF_DFLTBUFSIZE":                              "syscall",
	"syscall.BPF_DIRECTION_IN":                             "syscall",
	"syscall.BPF_DIRECTION_OUT":                            "syscall",
	"syscall.BPF_DIV":                                      "syscall",
	"syscall.BPF_H":                                        "syscall",
	"syscall.BPF_IMM":                                      "syscall",
	"syscall.BPF_IND":                                      "syscall",
	"syscall.BPF_JA":                                       "syscall",
	"syscall.BPF_JEQ":                                      "syscall",
	"syscall.BPF_JGE":                                      "syscall",
	"syscall.BPF_JGT":                                      "syscall",
	"syscall.BPF_JMP":                                      "syscall",
	"syscall.BPF_JSET":                                     "syscall",
	"syscall.BPF_K":                                        "syscall",
	"syscall.BPF_LD":                                       "syscall",
	"syscall.BPF_LDX":                                      "syscall",
	"syscall.BPF_LEN":                                      "syscall",
	"syscall.BPF_LSH":                                      "syscall",
	"syscall.BPF_MAJOR_VERSION":                            "syscall",
	"syscall.BPF_MAXBUFSIZE":                               "syscall",
	"syscall.BPF_MAXINSNS":                                 "syscall",
	"syscall.BPF_MEM":                                      "syscall",
	"syscall.BPF_MEMWORDS":                                 "syscall",
	"syscall.BPF_MINBUFSIZE":                               "syscall",
	"syscall.BPF_MINOR_VERSION":                            "syscall",
	"syscall.BPF_MISC":                                     "syscall",
	"syscall.BPF_MSH":                                      "syscall",
	"syscall.BPF_MUL":                                      "syscall",
	"syscall.BPF_NEG":                                      "syscall",
	"syscall.BPF_OR":                                       "syscall",
	"syscall.BPF_RELEASE":                                  "syscall",
	"syscall.BPF_RET":                                      "syscall",
	"syscall.BPF_RSH":                                      "syscall",
	"syscall.BPF_ST":                                       "syscall",
	"syscall.BPF_STX":                                      "syscall",
	"syscall.BPF_SUB":                                      "syscall",
	"syscall.BPF_TAX":                                      "syscall",
	"syscall.BPF_TXA":                                      "syscall",
	"syscall.BPF_T_BINTIME":                                "syscall",
	"syscall.BPF_T_BINTIME_FAST":                           "syscall",
	"syscall.BPF_T_BINTIME_MONOTONIC":                      "syscall",
	"syscall.BPF_T_BINTIME_MONOTONIC_FAST":                 "syscall",
	"syscall.BPF_T_FAST":                                   "syscall",
	"syscall.BPF_T_FLAG_MASK":                              "syscall",
	"syscall.BPF_T_FORMAT_MASK":                            "syscall",
	"syscall.BPF_T_MICROTIME":                              "syscall",
	"syscall.BPF_T_MICROTIME_FAST":                         "syscall",
	"syscall.BPF_T_MICROTIME_MONOTONIC":                    "syscall",
	"syscall.BPF_T_MICROTIME_MONOTONIC_FAST":               "syscall",
	"syscall.BPF_T_MONOTONIC":                              "syscall",
	"syscall.BPF_T_MONOTONIC_FAST":                         "syscall",
	"syscall.BPF_T_NANOTIME":                               "syscall",
	"syscall.BPF_T_NANOTIME_FAST":                          "syscall",
	"syscall.BPF_T_NANOTIME_MONOTONIC":                     "syscall",
	"syscall.BPF_T_NANOTIME_MONOTONIC_FAST":                "syscall",
	"syscall.BPF_T_NONE":                                   "syscall",
	"syscall.BPF_T_NORMAL":                                 "syscall",
	"syscall.BPF_W":                                        "syscall",
	"syscall.BPF_X":                                        "syscall",
	"syscall.BRKINT":                                       "syscall",
	"syscall.Bind":                                         "syscall",
	"syscall.BindToDevice":                                 "syscall",
	"syscall.BpfBuflen":                                    "syscall",
	"syscall.BpfDatalink":                                  "syscall",
	"syscall.BpfHdr":                                       "syscall",
	"syscall.BpfHeadercmpl":                                "syscall",
	"syscall.BpfInsn":                                      "syscall",
	"syscall.BpfInterface":                                 "syscall",
	"syscall.BpfJump":                                      "syscall",
	"syscall.BpfProgram":                                   "syscall",
	"syscall.BpfStat":                                      "syscall",
	"syscall.BpfStats":                                     "syscall",
	"syscall.BpfStmt":                                      "syscall",
	"syscall.BpfTimeout":                                   "syscall",
	"syscall.BpfTimeval":                                   "syscall",
	"syscall.BpfVersion":                                   "syscall",
	"syscall.BpfZbuf":                                      "syscall",
	"syscall.BpfZbufHeader":                                "syscall",
	"syscall.ByHandleFileInformation":                      "syscall",
	"syscall.BytePtrFromString":                            "syscall",
	"syscall.ByteSliceFromString":                          "syscall",
	"syscall.CCR0_FLUSH":                                   "syscall",
	"syscall.CERT_CHAIN_POLICY_AUTHENTICODE":               "syscall",
	"syscall.CERT_CHAIN_POLICY_AUTHENTICODE_TS":            "syscall",
	"syscall.CERT_CHAIN_POLICY_BASE":                       "syscall",
	"syscall.CERT_CHAIN_POLICY_BASIC_CONSTRAINTS":          "syscall",
	"syscall.CERT_CHAIN_POLICY_EV":                         "syscall",
	"syscall.CERT_CHAIN_POLICY_MICROSOFT_ROOT":             "syscall",
	"syscall.CERT_CHAIN_POLICY_NT_AUTH":                    "syscall",
	"syscall.CERT_CHAIN_POLICY_SSL":                        "syscall",
	"syscall.CERT_E_CN_NO_MATCH":                           "syscall",
	"syscall.CERT_E_EXPIRED":                               "syscall",
	"syscall.CERT_E_PURPOSE":                               "syscall",
	"syscall.CERT_E_ROLE":                                  "syscall",
	"syscall.CERT_E_UNTRUSTEDROOT":                         "syscall",
	"syscall.CERT_STORE_ADD_ALWAYS":                        "syscall",
	"syscall.CERT_STORE_DEFER_CLOSE_UNTIL_LAST_FREE_FLAG":  "syscall",
	"syscall.CERT_STORE_PROV_MEMORY":                       "syscall",
	"syscall.CERT_TRUST_HAS_EXCLUDED_NAME_CONSTRAINT":      "syscall",
	"syscall.CERT_TRUST_HAS_NOT_DEFINED_NAME_CONSTRAINT":   "syscall",
	"syscall.CERT_TRUST_HAS_NOT_PERMITTED_NAME_CONSTRAINT": "syscall",
	"syscall.CERT_TRUST_HAS_NOT_SUPPORTED_CRITICAL_EXT":    "syscall",
	"syscall.CERT_TRUST_HAS_NOT_SUPPORTED_NAME_CONSTRAINT": "syscall",
	"syscall.CERT_TRUST_INVALID_BASIC_CONSTRAINTS":         "syscall",
	"syscall.CERT_TRUST_INVALID_EXTENSION":                 "syscall",
	"syscall.CERT_TRUST_INVALID_NAME_CONSTRAINTS":          "syscall",
	"syscall.CERT_TRUST_INVALID_POLICY_CONSTRAINTS":        "syscall",
	"syscall.CERT_TRUST_IS_CYCLIC":                         "syscall",
	"syscall.CERT_TRUST_IS_EXPLICIT_DISTRUST":              "syscall",
	"syscall.CERT_TRUST_IS_NOT_SIGNATURE_VALID":            "syscall",
	"syscall.CERT_TRUST_IS_NOT_TIME_VALID":                 "syscall",
	"syscall.CERT_TRUST_IS_NOT_VALID_FOR_USAGE":            "syscall",
	"syscall.CERT_TRUST_IS_OFFLINE_REVOCATION":             "syscall",
	"syscall.CERT_TRUST_IS_REVOKED":                        "syscall",
	"syscall.CERT_TRUST_IS_UNTRUSTED_ROOT":                 "syscall",
	"syscall.CERT_TRUST_NO_ERROR":                          "syscall",
	"syscall.CERT_TRUST_NO_ISSUANCE_CHAIN_POLICY":          "syscall",
	"syscall.CERT_TRUST_REVOCATION_STATUS_UNKNOWN":         "syscall",
	"syscall.CFLUSH":                                       "syscall",
	"syscall.CLOCAL":                                       "syscall",
	"syscall.CLONE_CHILD_CLEARTID":                         "syscall",
	"syscall.CLONE_CHILD_SETTID":                           "syscall",
	"syscall.CLONE_DETACHED":                               "syscall",
	"syscall.CLONE_FILES":                                  "syscall",
	"syscall.CLONE_FS":                                     "syscall",
	"syscall.CLONE_IO":                                     "syscall",
	"syscall.CLONE_NEWIPC":                                 "syscall",
	"syscall.CLONE_NEWNET":                                 "syscall",
	"syscall.CLONE_NEWNS":                                  "syscall",
	"syscall.CLONE_NEWPID":                                 "syscall",
	"syscall.CLONE_NEWUSER":                                "syscall",
	"syscall.CLONE_NEWUTS":                                 "syscall",
	"syscall.CLONE_PARENT":                                 "syscall",
	"syscall.CLONE_PARENT_SETTID":                          "syscall",
	"syscall.CLONE_PTRACE":                                 "syscall",
	"syscall.CLONE_SETTLS":                                 "syscall",
	"syscall.CLONE_SIGHAND":                                "syscall",
	"syscall.CLONE_SYSVSEM":                                "syscall",
	"syscall.CLONE_THREAD":                                 "syscall",
	"syscall.CLONE_UNTRACED":                               "syscall",
	"syscall.CLONE_VFORK":                                  "syscall",
	"syscall.CLONE_VM":                                     "syscall",
	"syscall.CPUID_CFLUSH":                                 "syscall",
	"syscall.CREAD":                                        "syscall",
	"syscall.CREATE_ALWAYS":                                "syscall",
	"syscall.CREATE_NEW":                                   "syscall",
	"syscall.CREATE_NEW_PROCESS_GROUP":                     "syscall",
	"syscall.CREATE_UNICODE_ENVIRONMENT":                   "syscall",
	"syscall.CRYPT_DEFAULT_CONTAINER_OPTIONAL":             "syscall",
	"syscall.CRYPT_DELETEKEYSET":                           "syscall",
	"syscall.CRYPT_MACHINE_KEYSET":                         "syscall",
	"syscall.CRYPT_NEWKEYSET":                              "syscall",
	"syscall.CRYPT_SILENT":                                 "syscall",
	"syscall.CRYPT_VERIFYCONTEXT":                          "syscall",
	"syscall.CS5":                                          "syscall",
	"syscall.CS6":                                          "syscall",
	"syscall.CS7":                                          "syscall",
	"syscall.CS8":                                          "syscall",
	"syscall.CSIZE":                                        "syscall",
	"syscall.CSTART":                                       "syscall",
	"syscall.CSTATUS":                                      "syscall",
	"syscall.CSTOP":                                        "syscall",
	"syscall.CSTOPB":                                       "syscall",
	"syscall.CSUSP":                                        "syscall",
	"syscall.CTL_MAXNAME":                                  "syscall",
	"syscall.CTL_NET":                                      "syscall",
	"syscall.CTL_QUERY":                                    "syscall",
	"syscall.CTRL_BREAK_EVENT":                             "syscall",
	"syscall.CTRL_C_EVENT":                                 "syscall",
	"syscall.CancelIo":                                     "syscall",
	"syscall.CancelIoEx":                                   "syscall",
	"syscall.CertAddCertificateContextToStore":             "syscall",
	"syscall.CertChainContext":                             "syscall",
	"syscall.CertChainElement":                             "syscall",
	"syscall.CertChainPara":                                "syscall",
	"syscall.CertChainPolicyPara":                          "syscall",
	"syscall.CertChainPolicyStatus":                        "syscall",
	"syscall.CertCloseStore":                               "syscall",
	"syscall.CertContext":                                  "syscall",
	"syscall.CertCreateCertificateContext":                 "syscall",
	"syscall.CertEnhKeyUsage":                              "syscall",
	"syscall.CertEnumCertificatesInStore":                  "syscall",
	"syscall.CertFreeCertificateChain":                     "syscall",
	"syscall.CertFreeCertificateContext":                   "syscall",
	"syscall.CertGetCertificateChain":                      "syscall",
	"syscall.CertOpenStore":                                "syscall",
	"syscall.CertOpenSystemStore":                          "syscall",
	"syscall.CertRevocationInfo":                           "syscall",
	"syscall.CertSimpleChain":                              "syscall",
	"syscall.CertTrustStatus":                              "syscall",
	"syscall.CertUsageMatch":                               "syscall",
	"syscall.CertVerifyCertificateChainPolicy":             "syscall",
	"syscall.Chdir":                                        "syscall",
	"syscall.CheckBpfVersion":                              "syscall",
	"syscall.Chflags":                                      "syscall",
	"syscall.Chmod":                                        "syscall",
	"syscall.Chown":                                        "syscall",
	"syscall.Chroot":                                       "syscall",
	"syscall.Clearenv":                                     "syscall",
	"syscall.Close":                                        "syscall",
	"syscall.CloseHandle":                                  "syscall",
	"syscall.CloseOnExec":                                  "syscall",
	"syscall.Closesocket":                                  "syscall",
	"syscall.CmsgLen":                                      "syscall",
	"syscall.CmsgSpace":                                    "syscall",
	"syscall.Cmsghdr":                                      "syscall",
	"syscall.CommandLineToArgv":                            "syscall",
	"syscall.ComputerName":                                 "syscall",
	"syscall.Connect":                                      "syscall",
	"syscall.ConnectEx":                                    "syscall",
	"syscall.ConvertSidToStringSid":                        "syscall",
	"syscall.ConvertStringSidToSid":                        "syscall",
	"syscall.CopySid":                                      "syscall",
	"syscall.Creat":                                        "syscall",
	"syscall.CreateDirectory":                              "syscall",
	"syscall.CreateFile":                                   "syscall",
	"syscall.CreateFileMapping":                            "syscall",
	"syscall.CreateIoCompletionPort":                       "syscall",
	"syscall.CreatePipe":                                   "syscall",
	"syscall.CreateProcess":                                "syscall",
	"syscall.Credential":                                   "syscall",
	"syscall.CryptAcquireContext":                          "syscall",
	"syscall.CryptGenRandom":                               "syscall",
	"syscall.CryptReleaseContext":                          "syscall",
	"syscall.DIOCBSFLUSH":                                  "syscall",
	"syscall.DIOCOSFPFLUSH":                                "syscall",
	"syscall.DLL":                                          "syscall",
	"syscall.DLLError":                                     "syscall",
	"syscall.DLT_A429":                                     "syscall",
	"syscall.DLT_A653_ICM":                                 "syscall",
	"syscall.DLT_AIRONET_HEADER":                           "syscall",
	"syscall.DLT_AOS":                                      "syscall",
	"syscall.DLT_APPLE_IP_OVER_IEEE1394":                   "syscall",
	"syscall.DLT_ARCNET":                                   "syscall",
	"syscall.DLT_ARCNET_LINUX":                             "syscall",
	"syscall.DLT_ATM_CLIP":                                 "syscall",
	"syscall.DLT_ATM_RFC1483":                              "syscall",
	"syscall.DLT_AURORA":                                   "syscall",
	"syscall.DLT_AX25":                                     "syscall",
	"syscall.DLT_AX25_KISS":                                "syscall",
	"syscall.DLT_BACNET_MS_TP":                             "syscall",
	"syscall.DLT_BLUETOOTH_HCI_H4":                         "syscall",
	"syscall.DLT_BLUETOOTH_HCI_H4_WITH_PHDR":               "syscall",
	"syscall.DLT_CAN20B":                                   "syscall",
	"syscall.DLT_CAN_SOCKETCAN":                            "syscall",
	"syscall.DLT_CHAOS":                                    "syscall",
	"syscall.DLT_CHDLC":                                    "syscall",
	"syscall.DLT_CISCO_IOS":                                "syscall",
	"syscall.DLT_C_HDLC":                                   "syscall",
	"syscall.DLT_C_HDLC_WITH_DIR":                          "syscall",
	"syscall.DLT_DBUS":                                     "syscall",
	"syscall.DLT_DECT":                                     "syscall",
	"syscall.DLT_DOCSIS":                                   "syscall",
	"syscall.DLT_DVB_CI":                                   "syscall",
	"syscall.DLT_ECONET":                                   "syscall",
	"syscall.DLT_EN10MB":                                   "syscall",
	"syscall.DLT_EN3MB":                                    "syscall",
	"syscall.DLT_ENC":                                      "syscall",
	"syscall.DLT_ERF":                                      "syscall",
	"syscall.DLT_ERF_ETH":                                  "syscall",
	"syscall.DLT_ERF_POS":                                  "syscall",
	"syscall.DLT_FC_2":                                     "syscall",
	"syscall.DLT_FC_2_WITH_FRAME_DELIMS":                   "syscall",
	"syscall.DLT_FDDI":                                     "syscall",
	"syscall.DLT_FLEXRAY":                                  "syscall",
	"syscall.DLT_FRELAY":                                   "syscall",
	"syscall.DLT_FRELAY_WITH_DIR":                          "syscall",
	"syscall.DLT_GCOM_SERIAL":                              "syscall",
	"syscall.DLT_GCOM_T1E1":                                "syscall",
	"syscall.DLT_GPF_F":                                    "syscall",
	"syscall.DLT_GPF_T":                                    "syscall",
	"syscall.DLT_GPRS_LLC":                                 "syscall",
	"syscall.DLT_GSMTAP_ABIS":                              "syscall",
	"syscall.DLT_GSMTAP_UM":                                "syscall",
	"syscall.DLT_HDLC":                                     "syscall",
	"syscall.DLT_HHDLC":                                    "syscall",
	"syscall.DLT_HIPPI":                                    "syscall",
	"syscall.DLT_IBM_SN":                                   "syscall",
	"syscall.DLT_IBM_SP":                                   "syscall",
	"syscall.DLT_IEEE802":                                  "syscall",
	"syscall.DLT_IEEE802_11":                               "syscall",
	"syscall.DLT_IEEE802_11_RADIO":                         "syscall",
	"syscall.DLT_IEEE802_11_RADIO_AVS":                     "syscall",
	"syscall.DLT_IEEE802_15_4":                             "syscall",
	"syscall.DLT_IEEE802_15_4_LINUX":                       "syscall",
	"syscall.DLT_IEEE802_15_4_NOFCS":                       "syscall",
	"syscall.DLT_IEEE802_15_4_NONASK_PHY":                  "syscall",
	"syscall.DLT_IEEE802_16_MAC_CPS":                       "syscall",
	"syscall.DLT_IEEE802_16_MAC_CPS_RADIO":                 "syscall",
	"syscall.DLT_IPFILTER":                                 "syscall",
	"syscall.DLT_IPMB":                                     "syscall",
	"syscall.DLT_IPMB_LINUX":                               "syscall",
	"syscall.DLT_IPNET":                                    "syscall",
	"syscall.DLT_IPOIB":                                    "syscall",
	"syscall.DLT_IPV4":                                     "syscall",
	"syscall.DLT_IPV6":                                     "syscall",
	"syscall.DLT_IP_OVER_FC":                               "syscall",
	"syscall.DLT_JUNIPER_ATM1":                             "syscall",
	"syscall.DLT_JUNIPER_ATM2":                             "syscall",
	"syscall.DLT_JUNIPER_ATM_CEMIC":                        "syscall",
	"syscall.DLT_JUNIPER_CHDLC":                            "syscall",
	"syscall.DLT_JUNIPER_ES":                               "syscall",
	"syscall.DLT_JUNIPER_ETHER":                            "syscall",
	"syscall.DLT_JUNIPER_FIBRECHANNEL":                     "syscall",
	"syscall.DLT_JUNIPER_FRELAY":                           "syscall",
	"syscall.DLT_JUNIPER_GGSN":                             "syscall",
	"syscall.DLT_JUNIPER_ISM":                              "syscall",
	"syscall.DLT_JUNIPER_MFR":                              "syscall",
	"syscall.DLT_JUNIPER_MLFR":                             "syscall",
	"syscall.DLT_JUNIPER_MLPPP":                            "syscall",
	"syscall.DLT_JUNIPER_MONITOR":                          "syscall",
	"syscall.DLT_JUNIPER_PIC_PEER":                         "syscall",
	"syscall.DLT_JUNIPER_PPP":                              "syscall",
	"syscall.DLT_JUNIPER_PPPOE":                            "syscall",
	"syscall.DLT_JUNIPER_PPPOE_ATM":                        "syscall",
	"syscall.DLT_JUNIPER_SERVICES":                         "syscall",
	"syscall.DLT_JUNIPER_SRX_E2E":                          "syscall",
	"syscall.DLT_JUNIPER_ST":                               "syscall",
	"syscall.DLT_JUNIPER_VP":                               "syscall",
	"syscall.DLT_JUNIPER_VS":                               "syscall",
	"syscall.DLT_LAPB_WITH_DIR":                            "syscall",
	"syscall.DLT_LAPD":                                     "syscall",
	"syscall.DLT_LIN":                                      "syscall",
	"syscall.DLT_LINUX_EVDEV":                              "syscall",
	"syscall.DLT_LINUX_IRDA":                               "syscall",
	"syscall.DLT_LINUX_LAPD":                               "syscall",
	"syscall.DLT_LINUX_PPP_WITHDIRECTION":                  "syscall",
	"syscall.DLT_LINUX_SLL":                                "syscall",
	"syscall.DLT_LOOP":                                     "syscall",
	"syscall.DLT_LTALK":                                    "syscall",
	"syscall.DLT_MATCHING_MAX":                             "syscall",
	"syscall.DLT_MATCHING_MIN":                             "syscall",
	"syscall.DLT_MFR":                                      "syscall",
	"syscall.DLT_MOST":                                     "syscall",
	"syscall.DLT_MPEG_2_TS":                                "syscall",
	"syscall.DLT_MPLS":                                     "syscall",
	"syscall.DLT_MTP2":                                     "syscall",
	"syscall.DLT_MTP2_WITH_PHDR":                           "syscall",
	"syscall.DLT_MTP3":                                     "syscall",
	"syscall.DLT_MUX27010":                                 "syscall",
	"syscall.DLT_NETANALYZER":                              "syscall",
	"syscall.DLT_NETANALYZER_TRANSPARENT":                  "syscall",
	"syscall.DLT_NFC_LLCP":                                 "syscall",
	"syscall.DLT_NFLOG":                                    "syscall",
	"syscall.DLT_NG40":                                     "syscall",
	"syscall.DLT_NULL":                                     "syscall",
	"syscall.DLT_PCI_EXP":                                  "syscall",
	"syscall.DLT_PFLOG":                                    "syscall",
	"syscall.DLT_PFSYNC":                                   "syscall",
	"syscall.DLT_PPI":                                      "syscall",
	"syscall.DLT_PPP":                                      "syscall",
	"syscall.DLT_PPP_BSDOS":                                "syscall",
	"syscall.DLT_PPP_ETHER":                                "syscall",
	"syscall.DLT_PPP_PPPD":                                 "syscall",
	"syscall.DLT_PPP_SERIAL":                               "syscall",
	"syscall.DLT_PPP_WITH_DIR":                             "syscall",
	"syscall.DLT_PPP_WITH_DIRECTION":                       "syscall",
	"syscall.DLT_PRISM_HEADER":                             "syscall",
	"syscall.DLT_PRONET":                                   "syscall",
	"syscall.DLT_RAIF1":                                    "syscall",
	"syscall.DLT_RAW":                                      "syscall",
	"syscall.DLT_RAWAF_MASK":                               "syscall",
	"syscall.DLT_RIO":                                      "syscall",
	"syscall.DLT_SCCP":                                     "syscall",
	"syscall.DLT_SITA":                                     "syscall",
	"syscall.DLT_SLIP":                                     "syscall",
	"syscall.DLT_SLIP_BSDOS":                               "syscall",
	"syscall.DLT_STANAG_5066_D_PDU":                        "syscall",
	"syscall.DLT_SUNATM":                                   "syscall",
	"syscall.DLT_SYMANTEC_FIREWALL":                        "syscall",
	"syscall.DLT_TZSP":                                     "syscall",
	"syscall.DLT_USB":                                      "syscall",
	"syscall.DLT_USB_LINUX":                                "syscall",
	"syscall.DLT_USB_LINUX_MMAPPED":                        "syscall",
	"syscall.DLT_USER0":                                    "syscall",
	"syscall.DLT_USER1":                                    "syscall",
	"syscall.DLT_USER10":                                   "syscall",
	"syscall.DLT_USER11":                                   "syscall",
	"syscall.DLT_USER12":                                   "syscall",
	"syscall.DLT_USER13":                                   "syscall",
	"syscall.DLT_USER14":                                   "syscall",
	"syscall.DLT_USER15":                                   "syscall",
	"syscall.DLT_USER2":                                    "syscall",
	"syscall.DLT_USER3":                                    "syscall",
	"syscall.DLT_USER4":                                    "syscall",
	"syscall.DLT_USER5":                                    "syscall",
	"syscall.DLT_USER6":                                    "syscall",
	"syscall.DLT_USER7":                                    "syscall",
	"syscall.DLT_USER8":                                    "syscall",
	"syscall.DLT_USER9":                                    "syscall",
	"syscall.DLT_WIHART":                                   "syscall",
	"syscall.DLT_X2E_SERIAL":                               "syscall",
	"syscall.DLT_X2E_XORAYA":                               "syscall",
	"syscall.DNSMXData":                                    "syscall",
	"syscall.DNSPTRData":                                   "syscall",
	"syscall.DNSRecord":                                    "syscall",
	"syscall.DNSSRVData":                                   "syscall",
	"syscall.DNSTXTData":                                   "syscall",
	"syscall.DNS_TYPE_A":                                   "syscall",
	"syscall.DNS_TYPE_A6":                                  "syscall",
	"syscall.DNS_TYPE_AAAA":                                "syscall",
	"syscall.DNS_TYPE_ADDRS":                               "syscall",
	"syscall.DNS_TYPE_AFSDB":                               "syscall",
	"syscall.DNS_TYPE_ALL":                                 "syscall",
	"syscall.DNS_TYPE_ANY":                                 "syscall",
	"syscall.DNS_TYPE_ATMA":                                "syscall",
	"syscall.DNS_TYPE_AXFR":                                "syscall",
	"syscall.DNS_TYPE_CERT":                                "syscall",
	"syscall.DNS_TYPE_CNAME":                               "syscall",
	"syscall.DNS_TYPE_DHCID":                               "syscall",
	"syscall.DNS_TYPE_DNAME":                               "syscall",
	"syscall.DNS_TYPE_DNSKEY":                              "syscall",
	"syscall.DNS_TYPE_DS":                                  "syscall",
	"syscall.DNS_TYPE_EID":                                 "syscall",
	"syscall.DNS_TYPE_GID":                                 "syscall",
	"syscall.DNS_TYPE_GPOS":                                "syscall",
	"syscall.DNS_TYPE_HINFO":                               "syscall",
	"syscall.DNS_TYPE_ISDN":                                "syscall",
	"syscall.DNS_TYPE_IXFR":                                "syscall",
	"syscall.DNS_TYPE_KEY":                                 "syscall",
	"syscall.DNS_TYPE_KX":                                  "syscall",
	"syscall.DNS_TYPE_LOC":                                 "syscall",
	"syscall.DNS_TYPE_MAILA":                               "syscall",
	"syscall.DNS_TYPE_MAILB":                               "syscall",
	"syscall.DNS_TYPE_MB":                                  "syscall",
	"syscall.DNS_TYPE_MD":                                  "syscall",
	"syscall.DNS_TYPE_MF":                                  "syscall",
	"syscall.DNS_TYPE_MG":                                  "syscall",
	"syscall.DNS_TYPE_MINFO":                               "syscall",
	"syscall.DNS_TYPE_MR":                                  "syscall",
	"syscall.DNS_TYPE_MX":                                  "syscall",
	"syscall.DNS_TYPE_NAPTR":                               "syscall",
	"syscall.DNS_TYPE_NBSTAT":                              "syscall",
	"syscall.DNS_TYPE_NIMLOC":                              "syscall",
	"syscall.DNS_TYPE_NS":                                  "syscall",
	"syscall.DNS_TYPE_NSAP":                                "syscall",
	"syscall.DNS_TYPE_NSAPPTR":                             "syscall",
	"syscall.DNS_TYPE_NSEC":                                "syscall",
	"syscall.DNS_TYPE_NULL":                                "syscall",
	"syscall.DNS_TYPE_NXT":                                 "syscall",
	"syscall.DNS_TYPE_OPT":                                 "syscall",
	"syscall.DNS_TYPE_PTR":                                 "syscall",
	"syscall.DNS_TYPE_PX":                                  "syscall",
	"syscall.DNS_TYPE_RP":                                  "syscall",
	"syscall.DNS_TYPE_RRSIG":                               "syscall",
	"syscall.DNS_TYPE_RT":                                  "syscall",
	"syscall.DNS_TYPE_SIG":                                 "syscall",
	"syscall.DNS_TYPE_SINK":                                "syscall",
	"syscall.DNS_TYPE_SOA":                                 "syscall",
	"syscall.DNS_TYPE_SRV":                                 "syscall",
	"syscall.DNS_TYPE_TEXT":                                "syscall",
	"syscall.DNS_TYPE_TKEY":                                "syscall",
	"syscall.DNS_TYPE_TSIG":                                "syscall",
	"syscall.DNS_TYPE_UID":                                 "syscall",
	"syscall.DNS_TYPE_UINFO":                               "syscall",
	"syscall.DNS_TYPE_UNSPEC":                              "syscall",
	"syscall.DNS_TYPE_WINS":                                "syscall",
	"syscall.DNS_TYPE_WINSR":                               "syscall",
	"syscall.DNS_TYPE_WKS":                                 "syscall",
	"syscall.DNS_TYPE_X25":                                 "syscall",
	"syscall.DT_BLK":                                       "syscall",
	"syscall.DT_CHR":                                       "syscall",
	"syscall.DT_DIR":                                       "syscall",
	"syscall.DT_FIFO":                                      "syscall",
	"syscall.DT_LNK":                                       "syscall",
	"syscall.DT_REG":                                       "syscall",
	"syscall.DT_SOCK":                                      "syscall",
	"syscall.DT_UNKNOWN":                                   "syscall",
	"syscall.DT_WHT":                                       "syscall",
	"syscall.DUPLICATE_CLOSE_SOURCE":                       "syscall",
	"syscall.DUPLICATE_SAME_ACCESS":                        "syscall",
	"syscall.DeleteFile":                                   "syscall",
	"syscall.DetachLsf":                                    "syscall",
	"syscall.Dirent":                                       "syscall",
	"syscall.DnsQuery":                                     "syscall",
	"syscall.DnsRecordListFree":                            "syscall",
	"syscall.Dup":                                          "syscall",
	"syscall.Dup2":                                         "syscall",
	"syscall.Dup3":                                         "syscall",
	"syscall.DuplicateHandle":                              "syscall",
	"syscall.E2BIG":                                        "syscall",
	"syscall.EACCES":                                       "syscall",
	"syscall.EADDRINUSE":                                   "syscall",
	"syscall.EADDRNOTAVAIL":                                "syscall",
	"syscall.EADV":                                         "syscall",
	"syscall.EAFNOSUPPORT":                                 "syscall",
	"syscall.EAGAIN":                                       "syscall",
	"syscall.EALREADY":                                     "syscall",
	"syscall.EAUTH":                                        "syscall",
	"syscall.EBADARCH":                                     "syscall",
	"syscall.EBADE":                                        "syscall",
	"syscall.EBADEXEC":                                     "syscall",
	"syscall.EBADF":                                        "syscall",
	"syscall.EBADFD":                                       "syscall",
	"syscall.EBADMACHO":                                    "syscall",
	"syscall.EBADMSG":                                      "syscall",
	"syscall.EBADR":                                        "syscall",
	"syscall.EBADRPC":                                      "syscall",
	"syscall.EBADRQC":                                      "syscall",
	"syscall.EBADSLT":                                      "syscall",
	"syscall.EBFONT":                                       "syscall",
	"syscall.EBUSY":                                        "syscall",
	"syscall.ECANCELED":                                    "syscall",
	"syscall.ECAPMODE":                                     "syscall",
	"syscall.ECHILD":                                       "syscall",
	"syscall.ECHO":                                         "syscall",
	"syscall.ECHOCTL":                                      "syscall",
	"syscall.ECHOE":                                        "syscall",
	"syscall.ECHOK":                                        "syscall",
	"syscall.ECHOKE":                                       "syscall",
	"syscall.ECHONL":                                       "syscall",
	"syscall.ECHOPRT":                                      "syscall",
	"syscall.ECHRNG":                                       "syscall",
	"syscall.ECOMM":                                        "syscall",
	"syscall.ECONNABORTED":                                 "syscall",
	"syscall.ECONNREFUSED":                                 "syscall",
	"syscall.ECONNRESET":                                   "syscall",
	"syscall.EDEADLK":                                      "syscall",
	"syscall.EDEADLOCK":                                    "syscall",
	"syscall.EDESTADDRREQ":                                 "syscall",
	"syscall.EDEVERR":                                      "syscall",
	"syscall.EDOM":                                         "syscall",
	"syscall.EDOOFUS":                                      "syscall",
	"syscall.EDOTDOT":                                      "syscall",
	"syscall.EDQUOT":                                       "syscall",
	"syscall.EEXIST":                                       "syscall",
	"syscall.EFAULT":                                       "syscall",
	"syscall.EFBIG":                                        "syscall",
	"syscall.EFER_LMA":                                     "syscall",
	"syscall.EFER_LME":                                     "syscall",
	"syscall.EFER_NXE":                                     "syscall",
	"syscall.EFER_SCE":                                     "syscall",
	"syscall.EFTYPE":                                       "syscall",
	"syscall.EHOSTDOWN":                                    "syscall",
	"syscall.EHOSTUNREACH":                                 "syscall",
	"syscall.EHWPOISON":                                    "syscall",
	"syscall.EIDRM":                                        "syscall",
	"syscall.EILSEQ":                                       "syscall",
	"syscall.EINPROGRESS":                                  "syscall",
	"syscall.EINTR":                                        "syscall",
	"syscall.EINVAL":                                       "syscall",
	"syscall.EIO":                                          "syscall",
	"syscall.EIPSEC":                                       "syscall",
	"syscall.EISCONN":                                      "syscall",
	"syscall.EISDIR":                                       "syscall",
	"syscall.EISNAM":                                       "syscall",
	"syscall.EKEYEXPIRED":                                  "syscall",
	"syscall.EKEYREJECTED":                                 "syscall",
	"syscall.EKEYREVOKED":                                  "syscall",
	"syscall.EL2HLT":                                       "syscall",
	"syscall.EL2NSYNC":                                     "syscall",
	"syscall.EL3HLT":                                       "syscall",
	"syscall.EL3RST":                                       "syscall",
	"syscall.ELAST":                                        "syscall",
	"syscall.ELF_NGREG":                                    "syscall",
	"syscall.ELF_PRARGSZ":                                  "syscall",
	"syscall.ELIBACC":                                      "syscall",
	"syscall.ELIBBAD":                                      "syscall",
	"syscall.ELIBEXEC":                                     "syscall",
	"syscall.ELIBMAX":                                      "syscall",
	"syscall.ELIBSCN":                                      "syscall",
	"syscall.ELNRNG":                                       "syscall",
	"syscall.ELOOP":                                        "syscall",
	"syscall.EMEDIUMTYPE":                                  "syscall",
	"syscall.EMFILE":                                       "syscall",
	"syscall.EMLINK":                                       "syscall",
	"syscall.EMSGSIZE":                                     "syscall",
	"syscall.EMT_TAGOVF":                                   "syscall",
	"syscall.EMULTIHOP":                                    "syscall",
	"syscall.EMUL_ENABLED":                                 "syscall",
	"syscall.EMUL_LINUX":                                   "syscall",
	"syscall.EMUL_LINUX32":                                 "syscall",
	"syscall.EMUL_MAXID":                                   "syscall",
	"syscall.EMUL_NATIVE":                                  "syscall",
	"syscall.ENAMETOOLONG":                                 "syscall",
	"syscall.ENAVAIL":                                      "syscall",
	"syscall.ENDRUNDISC":                                   "syscall",
	"syscall.ENEEDAUTH":                                    "syscall",
	"syscall.ENETDOWN":                                     "syscall",
	"syscall.ENETRESET":                                    "syscall",
	"syscall.ENETUNREACH":                                  "syscall",
	"syscall.ENFILE":                                       "syscall",
	"syscall.ENOANO":                                       "syscall",
	"syscall.ENOATTR":                                      "syscall",
	"syscall.ENOBUFS":                                      "syscall",
	"syscall.ENOCSI":                                       "syscall",
	"syscall.ENODATA":                                      "syscall",
	"syscall.ENODEV":                                       "syscall",
	"syscall.ENOENT":                                       "syscall",
	"syscall.ENOEXEC":                                      "syscall",
	"syscall.ENOKEY":                                       "syscall",
	"syscall.ENOLCK":                                       "syscall",
	"syscall.ENOLINK":                                      "syscall",
	"syscall.ENOMEDIUM":                                    "syscall",
	"syscall.ENOMEM":                                       "syscall",
	"syscall.ENOMSG":                                       "syscall",
	"syscall.ENONET":                                       "syscall",
	"syscall.ENOPKG":                                       "syscall",
	"syscall.ENOPOLICY":                                    "syscall",
	"syscall.ENOPROTOOPT":                                  "syscall",
	"syscall.ENOSPC":                                       "syscall",
	"syscall.ENOSR":                                        "syscall",
	"syscall.ENOSTR":                                       "syscall",
	"syscall.ENOSYS":                                       "syscall",
	"syscall.ENOTBLK":                                      "syscall",
	"syscall.ENOTCAPABLE":                                  "syscall",
	"syscall.ENOTCONN":                                     "syscall",
	"syscall.ENOTDIR":                                      "syscall",
	"syscall.ENOTEMPTY":                                    "syscall",
	"syscall.ENOTNAM":                                      "syscall",
	"syscall.ENOTRECOVERABLE":                              "syscall",
	"syscall.ENOTSOCK":                                     "syscall",
	"syscall.ENOTSUP":                                      "syscall",
	"syscall.ENOTTY":                                       "syscall",
	"syscall.ENOTUNIQ":                                     "syscall",
	"syscall.ENXIO":                                        "syscall",
	"syscall.EN_SW_CTL_INF":                                "syscall",
	"syscall.EN_SW_CTL_PREC":                               "syscall",
	"syscall.EN_SW_CTL_ROUND":                              "syscall",
	"syscall.EN_SW_DATACHAIN":                              "syscall",
	"syscall.EN_SW_DENORM":                                 "syscall",
	"syscall.EN_SW_INVOP":                                  "syscall",
	"syscall.EN_SW_OVERFLOW":                               "syscall",
	"syscall.EN_SW_PRECLOSS":                               "syscall",
	"syscall.EN_SW_UNDERFLOW":                              "syscall",
	"syscall.EN_SW_ZERODIV":                                "syscall",
	"syscall.EOPNOTSUPP":                                   "syscall",
	"syscall.EOVERFLOW":                                    "syscall",
	"syscall.EOWNERDEAD":                                   "syscall",
	"syscall.EPERM":                                        "syscall",
	"syscall.EPFNOSUPPORT":                                 "syscall",
	"syscall.EPIPE":                                        "syscall",
	"syscall.EPOLLERR":                                     "syscall",
	"syscall.EPOLLET":                                      "syscall",
	"syscall.EPOLLHUP":                                     "syscall",
	"syscall.EPOLLIN":                                      "syscall",
	"syscall.EPOLLMSG":                                     "syscall",
	"syscall.EPOLLONESHOT":                                 "syscall",
	"syscall.EPOLLOUT":                                     "syscall",
	"syscall.EPOLLPRI":                                     "syscall",
	"syscall.EPOLLRDBAND":                                  "syscall",
	"syscall.EPOLLRDHUP":                                   "syscall",
	"syscall.EPOLLRDNORM":                                  "syscall",
	"syscall.EPOLLWRBAND":                                  "syscall",
	"syscall.EPOLLWRNORM":                                  "syscall",
	"syscall.EPOLL_CLOEXEC":                                "syscall",
	"syscall.EPOLL_CTL_ADD":                                "syscall",
	"syscall.EPOLL_CTL_DEL":                                "syscall",
	"syscall.EPOLL_CTL_MOD":                                "syscall",
	"syscall.EPOLL_NONBLOCK":                               "syscall",
	"syscall.EPROCLIM":                                     "syscall",
	"syscall.EPROCUNAVAIL":                                 "syscall",
	"syscall.EPROGMISMATCH":                                "syscall",
	"syscall.EPROGUNAVAIL":                                 "syscall",
	"syscall.EPROTO":                                       "syscall",
	"syscall.EPROTONOSUPPORT":                              "syscall",
	"syscall.EPROTOTYPE":                                   "syscall",
	"syscall.EPWROFF":                                      "syscall",
	"syscall.ERANGE":                                       "syscall",
	"syscall.EREMCHG":                                      "syscall",
	"syscall.EREMOTE":                                      "syscall",
	"syscall.EREMOTEIO":                                    "syscall",
	"syscall.ERESTART":                                     "syscall",
	"syscall.ERFKILL":                                      "syscall",
	"syscall.EROFS":                                        "syscall",
	"syscall.ERPCMISMATCH":                                 "syscall",
	"syscall.ERROR_ACCESS_DENIED":                          "syscall",
	"syscall.ERROR_ALREADY_EXISTS":                         "syscall",
	"syscall.ERROR_BROKEN_PIPE":                            "syscall",
	"syscall.ERROR_BUFFER_OVERFLOW":                        "syscall",
	"syscall.ERROR_ENVVAR_NOT_FOUND":                       "syscall",
	"syscall.ERROR_FILE_EXISTS":                            "syscall",
	"syscall.ERROR_FILE_NOT_FOUND":                         "syscall",
	"syscall.ERROR_HANDLE_EOF":                             "syscall",
	"syscall.ERROR_INSUFFICIENT_BUFFER":                    "syscall",
	"syscall.ERROR_IO_PENDING":                             "syscall",
	"syscall.ERROR_MOD_NOT_FOUND":                          "syscall",
	"syscall.ERROR_NOT_FOUND":                              "syscall",
	"syscall.ERROR_NO_MORE_FILES":                          "syscall",
	"syscall.ERROR_OPERATION_ABORTED":                      "syscall",
	"syscall.ERROR_PATH_NOT_FOUND":                         "syscall",
	"syscall.ERROR_PROC_NOT_FOUND":                         "syscall",
	"syscall.ESHLIBVERS":                                   "syscall",
	"syscall.ESHUTDOWN":                                    "syscall",
	"syscall.ESOCKTNOSUPPORT":                              "syscall",
	"syscall.ESPIPE":                                       "syscall",
	"syscall.ESRCH":                                        "syscall",
	"syscall.ESRMNT":                                       "syscall",
	"syscall.ESTALE":                                       "syscall",
	"syscall.ESTRPIPE":                                     "syscall",
	"syscall.ETHERCAP_JUMBO_MTU":                           "syscall",
	"syscall.ETHERCAP_VLAN_HWTAGGING":                      "syscall",
	"syscall.ETHERCAP_VLAN_MTU":                            "syscall",
	"syscall.ETHERMIN":                                     "syscall",
	"syscall.ETHERMTU":                                     "syscall",
	"syscall.ETHERMTU_JUMBO":                               "syscall",
	"syscall.ETHERTYPE_8023":                               "syscall",
	"syscall.ETHERTYPE_AARP":                               "syscall",
	"syscall.ETHERTYPE_ACCTON":                             "syscall",
	"syscall.ETHERTYPE_AEONIC":                             "syscall",
	"syscall.ETHERTYPE_ALPHA":                              "syscall",
	"syscall.ETHERTYPE_AMBER":                              "syscall",
	"syscall.ETHERTYPE_AMOEBA":                             "syscall",
	"syscall.ETHERTYPE_AOE":                                "syscall",
	"syscall.ETHERTYPE_APOLLO":                             "syscall",
	"syscall.ETHERTYPE_APOLLODOMAIN":                       "syscall",
	"syscall.ETHERTYPE_APPLETALK":                          "syscall",
	"syscall.ETHERTYPE_APPLITEK":                           "syscall",
	"syscall.ETHERTYPE_ARGONAUT":                           "syscall",
	"syscall.ETHERTYPE_ARP":                                "syscall",
	"syscall.ETHERTYPE_AT":                                 "syscall",
	"syscall.ETHERTYPE_ATALK":                              "syscall",
	"syscall.ETHERTYPE_ATOMIC":                             "syscall",
	"syscall.ETHERTYPE_ATT":                                "syscall",
	"syscall.ETHERTYPE_ATTSTANFORD":                        "syscall",
	"syscall.ETHERTYPE_AUTOPHON":                           "syscall",
	"syscall.ETHERTYPE_AXIS":                               "syscall",
	"syscall.ETHERTYPE_BCLOOP":                             "syscall",
	"syscall.ETHERTYPE_BOFL":                               "syscall",
	"syscall.ETHERTYPE_CABLETRON":                          "syscall",
	"syscall.ETHERTYPE_CHAOS":                              "syscall",
	"syscall.ETHERTYPE_COMDESIGN":                          "syscall",
	"syscall.ETHERTYPE_COMPUGRAPHIC":                       "syscall",
	"syscall.ETHERTYPE_COUNTERPOINT":                       "syscall",
	"syscall.ETHERTYPE_CRONUS":                             "syscall",
	"syscall.ETHERTYPE_CRONUSVLN":                          "syscall",
	"syscall.ETHERTYPE_DCA":                                "syscall",
	"syscall.ETHERTYPE_DDE":                                "syscall",
	"syscall.ETHERTYPE_DEBNI":                              "syscall",
	"syscall.ETHERTYPE_DECAM":                              "syscall",
	"syscall.ETHERTYPE_DECCUST":                            "syscall",
	"syscall.ETHERTYPE_DECDIAG":                            "syscall",
	"syscall.ETHERTYPE_DECDNS":                             "syscall",
	"syscall.ETHERTYPE_DECDTS":                             "syscall",
	"syscall.ETHERTYPE_DECEXPER":                           "syscall",
	"syscall.ETHERTYPE_DECLAST":                            "syscall",
	"syscall.ETHERTYPE_DECLTM":                             "syscall",
	"syscall.ETHERTYPE_DECMUMPS":                           "syscall",
	"syscall.ETHERTYPE_DECNETBIOS":                         "syscall",
	"syscall.ETHERTYPE_DELTACON":                           "syscall",
	"syscall.ETHERTYPE_DIDDLE":                             "syscall",
	"syscall.ETHERTYPE_DLOG1":                              "syscall",
	"syscall.ETHERTYPE_DLOG2":                              "syscall",
	"syscall.ETHERTYPE_DN":                                 "syscall",
	"syscall.ETHERTYPE_DOGFIGHT":                           "syscall",
	"syscall.ETHERTYPE_DSMD":                               "syscall",
	"syscall.ETHERTYPE_ECMA":                               "syscall",
	"syscall.ETHERTYPE_ENCRYPT":                            "syscall",
	"syscall.ETHERTYPE_ES":                                 "syscall",
	"syscall.ETHERTYPE_EXCELAN":                            "syscall",
	"syscall.ETHERTYPE_EXPERDATA":                          "syscall",
	"syscall.ETHERTYPE_FLIP":                               "syscall",
	"syscall.ETHERTYPE_FLOWCONTROL":                        "syscall",
	"syscall.ETHERTYPE_FRARP":                              "syscall",
	"syscall.ETHERTYPE_GENDYN":                             "syscall",
	"syscall.ETHERTYPE_HAYES":                              "syscall",
	"syscall.ETHERTYPE_HIPPI_FP":                           "syscall",
	"syscall.ETHERTYPE_HITACHI":                            "syscall",
	"syscall.ETHERTYPE_HP":                                 "syscall",
	"syscall.ETHERTYPE_IEEEPUP":                            "syscall",
	"syscall.ETHERTYPE_IEEEPUPAT":                          "syscall",
	"syscall.ETHERTYPE_IMLBL":                              "syscall",
	"syscall.ETHERTYPE_IMLBLDIAG":                          "syscall",
	"syscall.ETHERTYPE_IP":                                 "syscall",
	"syscall.ETHERTYPE_IPAS":                               "syscall",
	"syscall.ETHERTYPE_IPV6":                               "syscall",
	"syscall.ETHERTYPE_IPX":                                "syscall",
	"syscall.ETHERTYPE_IPXNEW":                             "syscall",
	"syscall.ETHERTYPE_KALPANA":                            "syscall",
	"syscall.ETHERTYPE_LANBRIDGE":                          "syscall",
	"syscall.ETHERTYPE_LANPROBE":                           "syscall",
	"syscall.ETHERTYPE_LAT":                                "syscall",
	"syscall.ETHERTYPE_LBACK":                              "syscall",
	"syscall.ETHERTYPE_LITTLE":                             "syscall",
	"syscall.ETHERTYPE_LLDP":                               "syscall",
	"syscall.ETHERTYPE_LOGICRAFT":                          "syscall",
	"syscall.ETHERTYPE_LOOPBACK":                           "syscall",
	"syscall.ETHERTYPE_MATRA":                              "syscall",
	"syscall.ETHERTYPE_MAX":                                "syscall",
	"syscall.ETHERTYPE_MERIT":                              "syscall",
	"syscall.ETHERTYPE_MICP":                               "syscall",
	"syscall.ETHERTYPE_MOPDL":                              "syscall",
	"syscall.ETHERTYPE_MOPRC":                              "syscall",
	"syscall.ETHERTYPE_MOTOROLA":                           "syscall",
	"syscall.ETHERTYPE_MPLS":                               "syscall",
	"syscall.ETHERTYPE_MPLS_MCAST":                         "syscall",
	"syscall.ETHERTYPE_MUMPS":                              "syscall",
	"syscall.ETHERTYPE_NBPCC":                              "syscall",
	"syscall.ETHERTYPE_NBPCLAIM":                           "syscall",
	"syscall.ETHERTYPE_NBPCLREQ":                           "syscall",
	"syscall.ETHERTYPE_NBPCLRSP":                           "syscall",
	"syscall.ETHERTYPE_NBPCREQ":                            "syscall",
	"syscall.ETHERTYPE_NBPCRSP":                            "syscall",
	"syscall.ETHERTYPE_NBPDG":                              "syscall",
	"syscall.ETHERTYPE_NBPDGB":                             "syscall",
	"syscall.ETHERTYPE_NBPDLTE":                            "syscall",
	"syscall.ETHERTYPE_NBPRAR":                             "syscall",
	"syscall.ETHERTYPE_NBPRAS":                             "syscall",
	"syscall.ETHERTYPE_NBPRST":                             "syscall",
	"syscall.ETHERTYPE_NBPSCD":                             "syscall",
	"syscall.ETHERTYPE_NBPVCD":                             "syscall",
	"syscall.ETHERTYPE_NBS":                                "syscall",
	"syscall.ETHERTYPE_NCD":                                "syscall",
	"syscall.ETHERTYPE_NESTAR":                             "syscall",
	"syscall.ETHERTYPE_NETBEUI":                            "syscall",
	"syscall.ETHERTYPE_NOVELL":                             "syscall",
	"syscall.ETHERTYPE_NS":                                 "syscall",
	"syscall.ETHERTYPE_NSAT":                               "syscall",
	"syscall.ETHERTYPE_NSCOMPAT":                           "syscall",
	"syscall.ETHERTYPE_NTRAILER":                           "syscall",
	"syscall.ETHERTYPE_OS9":                                "syscall",
	"syscall.ETHERTYPE_OS9NET":                             "syscall",
	"syscall.ETHERTYPE_PACER":                              "syscall",
	"syscall.ETHERTYPE_PAE":                                "syscall",
	"syscall.ETHERTYPE_PCS":                                "syscall",
	"syscall.ETHERTYPE_PLANNING":                           "syscall",
	"syscall.ETHERTYPE_PPP":                                "syscall",
	"syscall.ETHERTYPE_PPPOE":                              "syscall",
	"syscall.ETHERTYPE_PPPOEDISC":                          "syscall",
	"syscall.ETHERTYPE_PRIMENTS":                           "syscall",
	"syscall.ETHERTYPE_PUP":                                "syscall",
	"syscall.ETHERTYPE_PUPAT":                              "syscall",
	"syscall.ETHERTYPE_QINQ":                               "syscall",
	"syscall.ETHERTYPE_RACAL":                              "syscall",
	"syscall.ETHERTYPE_RATIONAL":                           "syscall",
	"syscall.ETHERTYPE_RAWFR":                              "syscall",
	"syscall.ETHERTYPE_RCL":                                "syscall",
	"syscall.ETHERTYPE_RDP":                                "syscall",
	"syscall.ETHERTYPE_RETIX":                              "syscall",
	"syscall.ETHERTYPE_REVARP":                             "syscall",
	"syscall.ETHERTYPE_SCA":                                "syscall",
	"syscall.ETHERTYPE_SECTRA":                             "syscall",
	"syscall.ETHERTYPE_SECUREDATA":                         "syscall",
	"syscall.ETHERTYPE_SGITW":                              "syscall",
	"syscall.ETHERTYPE_SG_BOUNCE":                          "syscall",
	"syscall.ETHERTYPE_SG_DIAG":                            "syscall",
	"syscall.ETHERTYPE_SG_NETGAMES":                        "syscall",
	"syscall.ETHERTYPE_SG_RESV":                            "syscall",
	"syscall.ETHERTYPE_SIMNET":                             "syscall",
	"syscall.ETHERTYPE_SLOW":                               "syscall",
	"syscall.ETHERTYPE_SLOWPROTOCOLS":                      "syscall",
	"syscall.ETHERTYPE_SNA":                                "syscall",
	"syscall.ETHERTYPE_SNMP":                               "syscall",
	"syscall.ETHERTYPE_SONIX":                              "syscall",
	"syscall.ETHERTYPE_SPIDER":                             "syscall",
	"syscall.ETHERTYPE_SPRITE":                             "syscall",
	"syscall.ETHERTYPE_STP":                                "syscall",
	"syscall.ETHERTYPE_TALARIS":                            "syscall",
	"syscall.ETHERTYPE_TALARISMC":                          "syscall",
	"syscall.ETHERTYPE_TCPCOMP":                            "syscall",
	"syscall.ETHERTYPE_TCPSM":                              "syscall",
	"syscall.ETHERTYPE_TEC":                                "syscall",
	"syscall.ETHERTYPE_TIGAN":                              "syscall",
	"syscall.ETHERTYPE_TRAIL":                              "syscall",
	"syscall.ETHERTYPE_TRANSETHER":                         "syscall",
	"syscall.ETHERTYPE_TYMSHARE":                           "syscall",
	"syscall.ETHERTYPE_UBBST":                              "syscall",
	"syscall.ETHERTYPE_UBDEBUG":                            "syscall",
	"syscall.ETHERTYPE_UBDIAGLOOP":                         "syscall",
	"syscall.ETHERTYPE_UBDL":                               "syscall",
	"syscall.ETHERTYPE_UBNIU":                              "syscall",
	"syscall.ETHERTYPE_UBNMC":                              "syscall",
	"syscall.ETHERTYPE_VALID":                              "syscall",
	"syscall.ETHERTYPE_VARIAN":                             "syscall",
	"syscall.ETHERTYPE_VAXELN":                             "syscall",
	"syscall.ETHERTYPE_VEECO":                              "syscall",
	"syscall.ETHERTYPE_VEXP":                               "syscall",
	"syscall.ETHERTYPE_VGLAB":                              "syscall",
	"syscall.ETHERTYPE_VINES":                              "syscall",
	"syscall.ETHERTYPE_VINESECHO":                          "syscall",
	"syscall.ETHERTYPE_VINESLOOP":                          "syscall",
	"syscall.ETHERTYPE_VITAL":                              "syscall",
	"syscall.ETHERTYPE_VLAN":                               "syscall",
	"syscall.ETHERTYPE_VLTLMAN":                            "syscall",
	"syscall.ETHERTYPE_VPROD":                              "syscall",
	"syscall.ETHERTYPE_VURESERVED":                         "syscall",
	"syscall.ETHERTYPE_WATERLOO":                           "syscall",
	"syscall.ETHERTYPE_WELLFLEET":                          "syscall",
	"syscall.ETHERTYPE_X25":                                "syscall",
	"syscall.ETHERTYPE_X75":                                "syscall",
	"syscall.ETHERTYPE_XNSSM":                              "syscall",
	"syscall.ETHERTYPE_XTP":                                "syscall",
	"syscall.ETHER_ADDR_LEN":                               "syscall",
	"syscall.ETHER_ALIGN":                                  "syscall",
	"syscall.ETHER_CRC_LEN":                                "syscall",
	"syscall.ETHER_CRC_POLY_BE":                            "syscall",
	"syscall.ETHER_CRC_POLY_LE":                            "syscall",
	"syscall.ETHER_HDR_LEN":                                "syscall",
	"syscall.ETHER_MAX_DIX_LEN":                            "syscall",
	"syscall.ETHER_MAX_LEN":                                "syscall",
	"syscall.ETHER_MAX_LEN_JUMBO":                          "syscall",
	"syscall.ETHER_MIN_LEN":                                "syscall",
	"syscall.ETHER_PPPOE_ENCAP_LEN":                        "syscall",
	"syscall.ETHER_TYPE_LEN":                               "syscall",
	"syscall.ETHER_VLAN_ENCAP_LEN":                         "syscall",
	"syscall.ETH_P_1588":                                   "syscall",
	"syscall.ETH_P_8021Q":                                  "syscall",
	"syscall.ETH_P_802_2":                                  "syscall",
	"syscall.ETH_P_802_3":                                  "syscall",
	"syscall.ETH_P_AARP":                                   "syscall",
	"syscall.ETH_P_ALL":                                    "syscall",
	"syscall.ETH_P_AOE":                                    "syscall",
	"syscall.ETH_P_ARCNET":                                 "syscall",
	"syscall.ETH_P_ARP":                                    "syscall",
	"syscall.ETH_P_ATALK":                                  "syscall",
	"syscall.ETH_P_ATMFATE":                                "syscall",
	"syscall.ETH_P_ATMMPOA":                                "syscall",
	"syscall.ETH_P_AX25":                                   "syscall",
	"syscall.ETH_P_BPQ":                                    "syscall",
	"syscall.ETH_P_CAIF":                                   "syscall",
	"syscall.ETH_P_CAN":                                    "syscall",
	"syscall.ETH_P_CONTROL":                                "syscall",
	"syscall.ETH_P_CUST":                                   "syscall",
	"syscall.ETH_P_DDCMP":                                  "syscall",
	"syscall.ETH_P_DEC":                                    "syscall",
	"syscall.ETH_P_DIAG":                                   "syscall",
	"syscall.ETH_P_DNA_DL":                                 "syscall",
	"syscall.ETH_P_DNA_RC":                                 "syscall",
	"syscall.ETH_P_DNA_RT":                                 "syscall",
	"syscall.ETH_P_DSA":                                    "syscall",
	"syscall.ETH_P_ECONET":                                 "syscall",
	"syscall.ETH_P_EDSA":                                   "syscall",
	"syscall.ETH_P_FCOE":                                   "syscall",
	"syscall.ETH_P_FIP":                                    "syscall",
	"syscall.ETH_P_HDLC":                                   "syscall",
	"syscall.ETH_P_IEEE802154":                             "syscall",
	"syscall.ETH_P_IEEEPUP":                                "syscall",
	"syscall.ETH_P_IEEEPUPAT":                              "syscall",
	"syscall.ETH_P_IP":                                     "syscall",
	"syscall.ETH_P_IPV6":                                   "syscall",
	"syscall.ETH_P_IPX":                                    "syscall",
	"syscall.ETH_P_IRDA":                                   "syscall",
	"syscall.ETH_P_LAT":                                    "syscall",
	"syscall.ETH_P_LINK_CTL":                               "syscall",
	"syscall.ETH_P_LOCALTALK":                              "syscall",
	"syscall.ETH_P_LOOP":                                   "syscall",
	"syscall.ETH_P_MOBITEX":                                "syscall",
	"syscall.ETH_P_MPLS_MC":                                "syscall",
	"syscall.ETH_P_MPLS_UC":                                "syscall",
	"syscall.ETH_P_PAE":                                    "syscall",
	"syscall.ETH_P_PAUSE":                                  "syscall",
	"syscall.ETH_P_PHONET":                                 "syscall",
	"syscall.ETH_P_PPPTALK":                                "syscall",
	"syscall.ETH_P_PPP_DISC":                               "syscall",
	"syscall.ETH_P_PPP_MP":                                 "syscall",
	"syscall.ETH_P_PPP_SES":                                "syscall",
	"syscall.ETH_P_PUP":                                    "syscall",
	"syscall.ETH_P_PUPAT":                                  "syscall",
	"syscall.ETH_P_RARP":                                   "syscall",
	"syscall.ETH_P_SCA":                                    "syscall",
	"syscall.ETH_P_SLOW":                                   "syscall",
	"syscall.ETH_P_SNAP":                                   "syscall",
	"syscall.ETH_P_TEB":                                    "syscall",
	"syscall.ETH_P_TIPC":                                   "syscall",
	"syscall.ETH_P_TRAILER":                                "syscall",
	"syscall.ETH_P_TR_802_2":                               "syscall",
	"syscall.ETH_P_WAN_PPP":                                "syscall",
	"syscall.ETH_P_WCCP":                                   "syscall",
	"syscall.ETH_P_X25":                                    "syscall",
	"syscall.ETIME":                                        "syscall",
	"syscall.ETIMEDOUT":                                    "syscall",
	"syscall.ETOOMANYREFS":                                 "syscall",
	"syscall.ETXTBSY":                                      "syscall",
	"syscall.EUCLEAN":                                      "syscall",
	"syscall.EUNATCH":                                      "syscall",
	"syscall.EUSERS":                                       "syscall",
	"syscall.EVFILT_AIO":                                   "syscall",
	"syscall.EVFILT_FS":                                    "syscall",
	"syscall.EVFILT_LIO":                                   "syscall",
	"syscall.EVFILT_MACHPORT":                              "syscall",
	"syscall.EVFILT_PROC":                                  "syscall",
	"syscall.EVFILT_READ":                                  "syscall",
	"syscall.EVFILT_SIGNAL":                                "syscall",
	"syscall.EVFILT_SYSCOUNT":                              "syscall",
	"syscall.EVFILT_THREADMARKER":                          "syscall",
	"syscall.EVFILT_TIMER":                                 "syscall",
	"syscall.EVFILT_USER":                                  "syscall",
	"syscall.EVFILT_VM":                                    "syscall",
	"syscall.EVFILT_VNODE":                                 "syscall",
	"syscall.EVFILT_WRITE":                                 "syscall",
	"syscall.EV_ADD":                                       "syscall",
	"syscall.EV_CLEAR":                                     "syscall",
	"syscall.EV_DELETE":                                    "syscall",
	"syscall.EV_DISABLE":                                   "syscall",
	"syscall.EV_DISPATCH":                                  "syscall",
	"syscall.EV_ENABLE":                                    "syscall",
	"syscall.EV_EOF":                                       "syscall",
	"syscall.EV_ERROR":                                     "syscall",
	"syscall.EV_FLAG0":                                     "syscall",
	"syscall.EV_FLAG1":                                     "syscall",
	"syscall.EV_ONESHOT":                                   "syscall",
	"syscall.EV_OOBAND":                                    "syscall",
	"syscall.EV_POLL":                                      "syscall",
	"syscall.EV_RECEIPT":                                   "syscall",
	"syscall.EV_SYSFLAGS":                                  "syscall",
	"syscall.EWINDOWS":                                     "syscall",
	"syscall.EWOULDBLOCK":                                  "syscall",
	"syscall.EXDEV":                                        "syscall",
	"syscall.EXFULL":                                       "syscall",
	"syscall.EXTA":                                         "syscall",
	"syscall.EXTB":                                         "syscall",
	"syscall.EXTPROC":                                      "syscall",
	"syscall.Environ":                                      "syscall",
	"syscall.EpollCreate":                                  "syscall",
	"syscall.EpollCreate1":                                 "syscall",
	"syscall.EpollCtl":                                     "syscall",
	"syscall.EpollEvent":                                   "syscall",
	"syscall.EpollWait":                                    "syscall",
	"syscall.Errno":                                        "syscall",
	"syscall.EscapeArg":                                    "syscall",
	"syscall.Exchangedata":                                 "syscall",
	"syscall.Exec":                                         "syscall",
	"syscall.Exit":                                         "syscall",
	"syscall.ExitProcess":                                  "syscall",
	"syscall.FD_CLOEXEC":                                   "syscall",
	"syscall.FD_SETSIZE":                                   "syscall",
	"syscall.FILE_ACTION_ADDED":                            "syscall",
	"syscall.FILE_ACTION_MODIFIED":                         "syscall",
	"syscall.FILE_ACTION_REMOVED":                          "syscall",
	"syscall.FILE_ACTION_RENAMED_NEW_NAME":                 "syscall",
	"syscall.FILE_ACTION_RENAMED_OLD_NAME":                 "syscall",
	"syscall.FILE_APPEND_DATA":                             "syscall",
	"syscall.FILE_ATTRIBUTE_ARCHIVE":                       "syscall",
	"syscall.FILE_ATTRIBUTE_DIRECTORY":                     "syscall",
	"syscall.FILE_ATTRIBUTE_HIDDEN":                        "syscall",
	"syscall.FILE_ATTRIBUTE_NORMAL":                        "syscall",
	"syscall.FILE_ATTRIBUTE_READONLY":                      "syscall",
	"syscall.FILE_ATTRIBUTE_SYSTEM":                        "syscall",
	"syscall.FILE_BEGIN":                                   "syscall",
	"syscall.FILE_CURRENT":                                 "syscall",
	"syscall.FILE_END":                                     "syscall",
	"syscall.FILE_FLAG_BACKUP_SEMANTICS":                   "syscall",
	"syscall.FILE_FLAG_OVERLAPPED":                         "syscall",
	"syscall.FILE_LIST_DIRECTORY":                          "syscall",
	"syscall.FILE_MAP_COPY":                                "syscall",
	"syscall.FILE_MAP_EXECUTE":                             "syscall",
	"syscall.FILE_MAP_READ":                                "syscall",
	"syscall.FILE_MAP_WRITE":                               "syscall",
	"syscall.FILE_NOTIFY_CHANGE_ATTRIBUTES":                "syscall",
	"syscall.FILE_NOTIFY_CHANGE_CREATION":                  "syscall",
	"syscall.FILE_NOTIFY_CHANGE_DIR_NAME":                  "syscall",
	"syscall.FILE_NOTIFY_CHANGE_FILE_NAME":                 "syscall",
	"syscall.FILE_NOTIFY_CHANGE_LAST_ACCESS":               "syscall",
	"syscall.FILE_NOTIFY_CHANGE_LAST_WRITE":                "syscall",
	"syscall.FILE_NOTIFY_CHANGE_SIZE":                      "syscall",
	"syscall.FILE_SHARE_DELETE":                            "syscall",
	"syscall.FILE_SHARE_READ":                              "syscall",
	"syscall.FILE_SHARE_WRITE":                             "syscall",
	"syscall.FILE_SKIP_COMPLETION_PORT_ON_SUCCESS":         "syscall",
	"syscall.FILE_SKIP_SET_EVENT_ON_HANDLE":                "syscall",
	"syscall.FILE_TYPE_CHAR":                               "syscall",
	"syscall.FILE_TYPE_DISK":                               "syscall",
	"syscall.FILE_TYPE_PIPE":                               "syscall",
	"syscall.FILE_TYPE_REMOTE":                             "syscall",
	"syscall.FILE_TYPE_UNKNOWN":                            "syscall",
	"syscall.FILE_WRITE_ATTRIBUTES":                        "syscall",
	"syscall.FLUSHO":                                       "syscall",
	"syscall.FORMAT_MESSAGE_ALLOCATE_BUFFER":               "syscall",
	"syscall.FORMAT_MESSAGE_ARGUMENT_ARRAY":                "syscall",
	"syscall.FORMAT_MESSAGE_FROM_HMODULE":                  "syscall",
	"syscall.FORMAT_MESSAGE_FROM_STRING":                   "syscall",
	"syscall.FORMAT_MESSAGE_FROM_SYSTEM":                   "syscall",
	"syscall.FORMAT_MESSAGE_IGNORE_INSERTS":                "syscall",
	"syscall.FORMAT_MESSAGE_MAX_WIDTH_MASK":                "syscall",
	"syscall.F_ADDFILESIGS":                                "syscall",
	"syscall.F_ADDSIGS":                                    "syscall",
	"syscall.F_ALLOCATEALL":                                "syscall",
	"syscall.F_ALLOCATECONTIG":                             "syscall",
	"syscall.F_CANCEL":                                     "syscall",
	"syscall.F_CHKCLEAN":                                   "syscall",
	"syscall.F_CLOSEM":                                     "syscall",
	"syscall.F_DUP2FD":                                     "syscall",
	"syscall.F_DUP2FD_CLOEXEC":                             "syscall",
	"syscall.F_DUPFD":                                      "syscall",
	"syscall.F_DUPFD_CLOEXEC":                              "syscall",
	"syscall.F_EXLCK":                                      "syscall",
	"syscall.F_FLUSH_DATA":                                 "syscall",
	"syscall.F_FREEZE_FS":                                  "syscall",
	"syscall.F_FSCTL":                                      "syscall",
	"syscall.F_FSDIRMASK":                                  "syscall",
	"syscall.F_FSIN":                                       "syscall",
	"syscall.F_FSINOUT":                                    "syscall",
	"syscall.F_FSOUT":                                      "syscall",
	"syscall.F_FSPRIV":                                     "syscall",
	"syscall.F_FSVOID":                                     "syscall",
	"syscall.F_FULLFSYNC":                                  "syscall",
	"syscall.F_GETFD":                                      "syscall",
	"syscall.F_GETFL":                                      "syscall",
	"syscall.F_GETLEASE":                                   "syscall",
	"syscall.F_GETLK":                                      "syscall",
	"syscall.F_GETLK64":                                    "syscall",
	"syscall.F_GETLKPID":                                   "syscall",
	"syscall.F_GETNOSIGPIPE":                               "syscall",
	"syscall.F_GETOWN":                                     "syscall",
	"syscall.F_GETOWN_EX":                                  "syscall",
	"syscall.F_GETPATH":                                    "syscall",
	"syscall.F_GETPATH_MTMINFO":                            "syscall",
	"syscall.F_GETPIPE_SZ":                                 "syscall",
	"syscall.F_GETPROTECTIONCLASS":                         "syscall",
	"syscall.F_GETSIG":                                     "syscall",
	"syscall.F_GLOBAL_NOCACHE":                             "syscall",
	"syscall.F_LOCK":                                       "syscall",
	"syscall.F_LOG2PHYS":                                   "syscall",
	"syscall.F_LOG2PHYS_EXT":                               "syscall",
	"syscall.F_MARKDEPENDENCY":                             "syscall",
	"syscall.F_MAXFD":                                      "syscall",
	"syscall.F_NOCACHE":                                    "syscall",
	"syscall.F_NODIRECT":                                   "syscall",
	"syscall.F_NOTIFY":                                     "syscall",
	"syscall.F_OGETLK":                                     "syscall",
	"syscall.F_OK":                                         "syscall",
	"syscall.F_OSETLK":                                     "syscall",
	"syscall.F_OSETLKW":                                    "syscall",
	"syscall.F_PARAM_MASK":                                 "syscall",
	"syscall.F_PARAM_MAX":                                  "syscall",
	"syscall.F_PATHPKG_CHECK":                              "syscall",
	"syscall.F_PEOFPOSMODE":                                "syscall",
	"syscall.F_PREALLOCATE":                                "syscall",
	"syscall.F_RDADVISE":                                   "syscall",
	"syscall.F_RDAHEAD":                                    "syscall",
	"syscall.F_RDLCK":                                      "syscall",
	"syscall.F_READAHEAD":                                  "syscall",
	"syscall.F_READBOOTSTRAP":                              "syscall",
	"syscall.F_SETBACKINGSTORE":                            "syscall",
	"syscall.F_SETFD":                                      "syscall",
	"syscall.F_SETFL":                                      "syscall",
	"syscall.F_SETLEASE":                                   "syscall",
	"syscall.F_SETLK":                                      "syscall",
	"syscall.F_SETLK64":                                    "syscall",
	"syscall.F_SETLKW":                                     "syscall",
	"syscall.F_SETLKW64":                                   "syscall",
	"syscall.F_SETLK_REMOTE":                               "syscall",
	"syscall.F_SETNOSIGPIPE":                               "syscall",
	"syscall.F_SETOWN":                                     "syscall",
	"syscall.F_SETOWN_EX":                                  "syscall",
	"syscall.F_SETPIPE_SZ":                                 "syscall",
	"syscall.F_SETPROTECTIONCLASS":                         "syscall",
	"syscall.F_SETSIG":                                     "syscall",
	"syscall.F_SETSIZE":                                    "syscall",
	"syscall.F_SHLCK":                                      "syscall",
	"syscall.F_TEST":                                       "syscall",
	"syscall.F_THAW_FS":                                    "syscall",
	"syscall.F_TLOCK":                                      "syscall",
	"syscall.F_ULOCK":                                      "syscall",
	"syscall.F_UNLCK":                                      "syscall",
	"syscall.F_UNLCKSYS":                                   "syscall",
	"syscall.F_VOLPOSMODE":                                 "syscall",
	"syscall.F_WRITEBOOTSTRAP":                             "syscall",
	"syscall.F_WRLCK":                                      "syscall",
	"syscall.Faccessat":                                    "syscall",
	"syscall.Fallocate":                                    "syscall",
	"syscall.Fbootstraptransfer_t":                         "syscall",
	"syscall.Fchdir":                                       "syscall",
	"syscall.Fchflags":                                     "syscall",
	"syscall.Fchmod":                                       "syscall",
	"syscall.Fchmodat":                                     "syscall",
	"syscall.Fchown":                                       "syscall",
	"syscall.Fchownat":                                     "syscall",
	"syscall.FdSet":                                        "syscall",
	"syscall.Fdatasync":                                    "syscall",
	"syscall.FileNotifyInformation":                        "syscall",
	"syscall.Filetime":                                     "syscall",
	"syscall.FindClose":                                    "syscall",
	"syscall.FindFirstFile":                                "syscall",
	"syscall.FindNextFile":                                 "syscall",
	"syscall.Flock":                                        "syscall",
	"syscall.Flock_t":                                      "syscall",
	"syscall.FlushBpf":                                     "syscall",
	"syscall.FlushFileBuffers":                             "syscall",
	"syscall.FlushViewOfFile":                              "syscall",
	"syscall.ForkExec":                                     "syscall",
	"syscall.ForkLock":                                     "syscall",
	"syscall.FormatMessage":                                "syscall",
	"syscall.Fpathconf":                                    "syscall",
	"syscall.FreeAddrInfoW":                                "syscall",
	"syscall.FreeEnvironmentStrings":                       "syscall",
	"syscall.FreeLibrary":                                  "syscall",
	"syscall.Fsid":                                         "syscall",
	"syscall.Fstat":                                        "syscall",
	"syscall.Fstatfs":                                      "syscall",
	"syscall.Fstore_t":                                     "syscall",
	"syscall.Fsync":                                        "syscall",
	"syscall.Ftruncate":                                    "syscall",
	"syscall.Futimes":                                      "syscall",
	"syscall.Futimesat":                                    "syscall",
	"syscall.GENERIC_ALL":                                  "syscall",
	"syscall.GENERIC_EXECUTE":                              "syscall",
	"syscall.GENERIC_READ":                                 "syscall",
	"syscall.GENERIC_WRITE":                                "syscall",
	"syscall.GUID":                                         "syscall",
	"syscall.GetAcceptExSockaddrs":                         "syscall",
	"syscall.GetAdaptersInfo":                              "syscall",
	"syscall.GetAddrInfoW":                                 "syscall",
	"syscall.GetCommandLine":                               "syscall",
	"syscall.GetComputerName":                              "syscall",
	"syscall.GetConsoleMode":                               "syscall",
	"syscall.GetCurrentDirectory":                          "syscall",
	"syscall.GetCurrentProcess":                            "syscall",
	"syscall.GetEnvironmentStrings":                        "syscall",
	"syscall.GetEnvironmentVariable":                       "syscall",
	"syscall.GetExitCodeProcess":                           "syscall",
	"syscall.GetFileAttributes":                            "syscall",
	"syscall.GetFileAttributesEx":                          "syscall",
	"syscall.GetFileExInfoStandard":                        "syscall",
	"syscall.GetFileExMaxInfoLevel":                        "syscall",
	"syscall.GetFileInformationByHandle":                   "syscall",
	"syscall.GetFileType":                                  "syscall",
	"syscall.GetFullPathName":                              "syscall",
	"syscall.GetHostByName":                                "syscall",
	"syscall.GetIfEntry":                                   "syscall",
	"syscall.GetLastError":                                 "syscall",
	"syscall.GetLengthSid":                                 "syscall",
	"syscall.GetLongPathName":                              "syscall",
	"syscall.GetProcAddress":                               "syscall",
	"syscall.GetProcessTimes":                              "syscall",
	"syscall.GetProtoByName":                               "syscall",
	"syscall.GetQueuedCompletionStatus":                    "syscall",
	"syscall.GetServByName":                                "syscall",
	"syscall.GetShortPathName":                             "syscall",
	"syscall.GetStartupInfo":                               "syscall",
	"syscall.GetStdHandle":                                 "syscall",
	"syscall.GetSystemTimeAsFileTime":                      "syscall",
	"syscall.GetTempPath":                                  "syscall",
	"syscall.GetTimeZoneInformation":                       "syscall",
	"syscall.GetTokenInformation":                          "syscall",
	"syscall.GetUserNameEx":                                "syscall",
	"syscall.GetUserProfileDirectory":                      "syscall",
	"syscall.GetVersion":                                   "syscall",
	"syscall.Getcwd":                                       "syscall",
	"syscall.Getdents":                                     "syscall",
	"syscall.Getdirentries":                                "syscall",
	"syscall.Getdtablesize":                                "syscall",
	"syscall.Getegid":                                      "syscall",
	"syscall.Getenv":                                       "syscall",
	"syscall.Geteuid":                                      "syscall",
	"syscall.Getfsstat":                                    "syscall",
	"syscall.Getgid":                                       "syscall",
	"syscall.Getgroups":                                    "syscall",
	"syscall.Getpagesize":                                  "syscall",
	"syscall.Getpeername":                                  "syscall",
	"syscall.Getpgid":                                      "syscall",
	"syscall.Getpgrp":                                      "syscall",
	"syscall.Getpid":                                       "syscall",
	"syscall.Getppid":                                      "syscall",
	"syscall.Getpriority":                                  "syscall",
	"syscall.Getrlimit":                                    "syscall",
	"syscall.Getrusage":                                    "syscall",
	"syscall.Getsid":                                       "syscall",
	"syscall.Getsockname":                                  "syscall",
	"syscall.Getsockopt":                                   "syscall",
	"syscall.GetsockoptByte":                               "syscall",
	"syscall.GetsockoptICMPv6Filter":                       "syscall",
	"syscall.GetsockoptIPMreq":                             "syscall",
	"syscall.GetsockoptIPMreqn":                            "syscall",
	"syscall.GetsockoptIPv6MTUInfo":                        "syscall",
	"syscall.GetsockoptIPv6Mreq":                           "syscall",
	"syscall.GetsockoptInet4Addr":                          "syscall",
	"syscall.GetsockoptInt":                                "syscall",
	"syscall.GetsockoptUcred":                              "syscall",
	"syscall.Gettid":                                       "syscall",
	"syscall.Gettimeofday":                                 "syscall",
	"syscall.Getuid":                                       "syscall",
	"syscall.Getwd":                                        "syscall",
	"syscall.Getxattr":                                     "syscall",
	"syscall.HANDLE_FLAG_INHERIT":                          "syscall",
	"syscall.HKEY_CLASSES_ROOT":                            "syscall",
	"syscall.HKEY_CURRENT_CONFIG":                          "syscall",
	"syscall.HKEY_CURRENT_USER":                            "syscall",
	"syscall.HKEY_DYN_DATA":                                "syscall",
	"syscall.HKEY_LOCAL_MACHINE":                           "syscall",
	"syscall.HKEY_PERFORMANCE_DATA":                        "syscall",
	"syscall.HKEY_USERS":                                   "syscall",
	"syscall.HUPCL":                                        "syscall",
	"syscall.Handle":                                       "syscall",
	"syscall.Hostent":                                      "syscall",
	"syscall.ICANON":                                       "syscall",
	"syscall.ICMP6_FILTER":                                 "syscall",
	"syscall.ICMPV6_FILTER":                                "syscall",
	"syscall.ICMPv6Filter":                                 "syscall",
	"syscall.ICRNL":                                        "syscall",
	"syscall.IEXTEN":                                       "syscall",
	"syscall.IFAN_ARRIVAL":                                 "syscall",
	"syscall.IFAN_DEPARTURE":                               "syscall",
	"syscall.IFA_ADDRESS":                                  "syscall",
	"syscall.IFA_ANYCAST":                                  "syscall",
	"syscall.IFA_BROADCAST":                                "syscall",
	"syscall.IFA_CACHEINFO":                                "syscall",
	"syscall.IFA_F_DADFAILED":                              "syscall",
	"syscall.IFA_F_DEPRECATED":                             "syscall",
	"syscall.IFA_F_HOMEADDRESS":                            "syscall",
	"syscall.IFA_F_NODAD":                                  "syscall",
	"syscall.IFA_F_OPTIMISTIC":                             "syscall",
	"syscall.IFA_F_PERMANENT":                              "syscall",
	"syscall.IFA_F_SECONDARY":                              "syscall",
	"syscall.IFA_F_TEMPORARY":                              "syscall",
	"syscall.IFA_F_TENTATIVE":                              "syscall",
	"syscall.IFA_LABEL":                                    "syscall",
	"syscall.IFA_LOCAL":                                    "syscall",
	"syscall.IFA_MAX":                                      "syscall",
	"syscall.IFA_MULTICAST":                                "syscall",
	"syscall.IFA_ROUTE":                                    "syscall",
	"syscall.IFA_UNSPEC":                                   "syscall",
	"syscall.IFF_ALLMULTI":                                 "syscall",
	"syscall.IFF_ALTPHYS":                                  "syscall",
	"syscall.IFF_AUTOMEDIA":                                "syscall",
	"syscall.IFF_BROADCAST":                                "syscall",
	"syscall.IFF_CANTCHANGE":                               "syscall",
	"syscall.IFF_CANTCONFIG":                               "syscall",
	"syscall.IFF_DEBUG":                                    "syscall",
	"syscall.IFF_DRV_OACTIVE":                              "syscall",
	"syscall.IFF_DRV_RUNNING":                              "syscall",
	"syscall.IFF_DYING":                                    "syscall",
	"syscall.IFF_DYNAMIC":                                  "syscall",
	"syscall.IFF_LINK0":                                    "syscall",
	"syscall.IFF_LINK1":                                    "syscall",
	"syscall.IFF_LINK2":                                    "syscall",
	"syscall.IFF_LOOPBACK":                                 "syscall",
	"syscall.IFF_MASTER":                                   "syscall",
	"syscall.IFF_MONITOR":                                  "syscall",
	"syscall.IFF_MULTICAST":                                "syscall",
	"syscall.IFF_NOARP":                                    "syscall",
	"syscall.IFF_NOTRAILERS":                               "syscall",
	"syscall.IFF_NO_PI":                                    "syscall",
	"syscall.IFF_OACTIVE":                                  "syscall",
	"syscall.IFF_ONE_QUEUE":                                "syscall",
	"syscall.IFF_POINTOPOINT":                              "syscall",
	"syscall.IFF_POINTTOPOINT":                             "syscall",
	"syscall.IFF_PORTSEL":                                  "syscall",
	"syscall.IFF_PPROMISC":                                 "syscall",
	"syscall.IFF_PROMISC":                                  "syscall",
	"syscall.IFF_RENAMING":                                 "syscall",
	"syscall.IFF_RUNNING":                                  "syscall",
	"syscall.IFF_SIMPLEX":                                  "syscall",
	"syscall.IFF_SLAVE":                                    "syscall",
	"syscall.IFF_SMART":                                    "syscall",
	"syscall.IFF_STATICARP":                                "syscall",
	"syscall.IFF_TAP":                                      "syscall",
	"syscall.IFF_TUN":                                      "syscall",
	"syscall.IFF_TUN_EXCL":                                 "syscall",
	"syscall.IFF_UP":                                       "syscall",
	"syscall.IFF_VNET_HDR":                                 "syscall",
	"syscall.IFLA_ADDRESS":                                 "syscall",
	"syscall.IFLA_BROADCAST":                               "syscall",
	"syscall.IFLA_COST":                                    "syscall",
	"syscall.IFLA_IFALIAS":                                 "syscall",
	"syscall.IFLA_IFNAME":                                  "syscall",
	"syscall.IFLA_LINK":                                    "syscall",
	"syscall.IFLA_LINKINFO":                                "syscall",
	"syscall.IFLA_LINKMODE":                                "syscall",
	"syscall.IFLA_MAP":                                     "syscall",
	"syscall.IFLA_MASTER":                                  "syscall",
	"syscall.IFLA_MAX":                                     "syscall",
	"syscall.IFLA_MTU":                                     "syscall",
	"syscall.IFLA_NET_NS_PID":                              "syscall",
	"syscall.IFLA_OPERSTATE":                               "syscall",
	"syscall.IFLA_PRIORITY":                                "syscall",
	"syscall.IFLA_PROTINFO":                                "syscall",
	"syscall.IFLA_QDISC":                                   "syscall",
	"syscall.IFLA_STATS":                                   "syscall",
	"syscall.IFLA_TXQLEN":                                  "syscall",
	"syscall.IFLA_UNSPEC":                                  "syscall",
	"syscall.IFLA_WEIGHT":                                  "syscall",
	"syscall.IFLA_WIRELESS":                                "syscall",
	"syscall.IFNAMSIZ":                                     "syscall",
	"syscall.IFT_1822":                                     "syscall",
	"syscall.IFT_A12MPPSWITCH":                             "syscall",
	"syscall.IFT_AAL2":                                     "syscall",
	"syscall.IFT_AAL5":                                     "syscall",
	"syscall.IFT_ADSL":                                     "syscall",
	"syscall.IFT_AFLANE8023":                               "syscall",
	"syscall.IFT_AFLANE8025":                               "syscall",
	"syscall.IFT_ARAP":                                     "syscall",
	"syscall.IFT_ARCNET":                                   "syscall",
	"syscall.IFT_ARCNETPLUS":                               "syscall",
	"syscall.IFT_ASYNC":                                    "syscall",
	"syscall.IFT_ATM":                                      "syscall",
	"syscall.IFT_ATMDXI":                                   "syscall",
	"syscall.IFT_ATMFUNI":                                  "syscall",
	"syscall.IFT_ATMIMA":                                   "syscall",
	"syscall.IFT_ATMLOGICAL":                               "syscall",
	"syscall.IFT_ATMRADIO":                                 "syscall",
	"syscall.IFT_ATMSUBINTERFACE":                          "syscall",
	"syscall.IFT_ATMVCIENDPT":                              "syscall",
	"syscall.IFT_ATMVIRTUAL":                               "syscall",
	"syscall.IFT_BGPPOLICYACCOUNTING":                      "syscall",
	"syscall.IFT_BLUETOOTH":                                "syscall",
	"syscall.IFT_BRIDGE":                                   "syscall",
	"syscall.IFT_BSC":                                      "syscall",
	"syscall.IFT_CARP":                                     "syscall",
	"syscall.IFT_CCTEMUL":                                  "syscall",
	"syscall.IFT_CELLULAR":                                 "syscall",
	"syscall.IFT_CEPT":                                     "syscall",
	"syscall.IFT_CES":                                      "syscall",
	"syscall.IFT_CHANNEL":                                  "syscall",
	"syscall.IFT_CNR":                                      "syscall",
	"syscall.IFT_COFFEE":                                   "syscall",
	"syscall.IFT_COMPOSITELINK":                            "syscall",
	"syscall.IFT_DCN":                                      "syscall",
	"syscall.IFT_DIGITALPOWERLINE":                         "syscall",
	"syscall.IFT_DIGITALWRAPPEROVERHEADCHANNEL":            "syscall",
	"syscall.IFT_DLSW":                                     "syscall",
	"syscall.IFT_DOCSCABLEDOWNSTREAM":                      "syscall",
	"syscall.IFT_DOCSCABLEMACLAYER":                        "syscall",
	"syscall.IFT_DOCSCABLEUPSTREAM":                        "syscall",
	"syscall.IFT_DOCSCABLEUPSTREAMCHANNEL":                 "syscall",
	"syscall.IFT_DS0":                                      "syscall",
	"syscall.IFT_DS0BUNDLE":                                "syscall",
	"syscall.IFT_DS1FDL":                                   "syscall",
	"syscall.IFT_DS3":                                      "syscall",
	"syscall.IFT_DTM":                                      "syscall",
	"syscall.IFT_DUMMY":                                    "syscall",
	"syscall.IFT_DVBASILN":                                 "syscall",
	"syscall.IFT_DVBASIOUT":                                "syscall",
	"syscall.IFT_DVBRCCDOWNSTREAM":                         "syscall",
	"syscall.IFT_DVBRCCMACLAYER":                           "syscall",
	"syscall.IFT_DVBRCCUPSTREAM":                           "syscall",
	"syscall.IFT_ECONET":                                   "syscall",
	"syscall.IFT_ENC":                                      "syscall",
	"syscall.IFT_EON":                                      "syscall",
	"syscall.IFT_EPLRS":                                    "syscall",
	"syscall.IFT_ESCON":                                    "syscall",
	"syscall.IFT_ETHER":                                    "syscall",
	"syscall.IFT_FAITH":                                    "syscall",
	"syscall.IFT_FAST":                                     "syscall",
	"syscall.IFT_FASTETHER":                                "syscall",
	"syscall.IFT_FASTETHERFX":                              "syscall",
	"syscall.IFT_FDDI":                                     "syscall",
	"syscall.IFT_FIBRECHANNEL":                             "syscall",
	"syscall.IFT_FRAMERELAYINTERCONNECT":                   "syscall",
	"syscall.IFT_FRAMERELAYMPI":                            "syscall",
	"syscall.IFT_FRDLCIENDPT":                              "syscall",
	"syscall.IFT_FRELAY":                                   "syscall",
	"syscall.IFT_FRELAYDCE":                                "syscall",
	"syscall.IFT_FRF16MFRBUNDLE":                           "syscall",
	"syscall.IFT_FRFORWARD":                                "syscall",
	"syscall.IFT_G703AT2MB":                                "syscall",
	"syscall.IFT_G703AT64K":                                "syscall",
	"syscall.IFT_GIF":                                      "syscall",
	"syscall.IFT_GIGABITETHERNET":                          "syscall",
	"syscall.IFT_GR303IDT":                                 "syscall",
	"syscall.IFT_GR303RDT":                                 "syscall",
	"syscall.IFT_H323GATEKEEPER":                           "syscall",
	"syscall.IFT_H323PROXY":                                "syscall",
	"syscall.IFT_HDH1822":                                  "syscall",
	"syscall.IFT_HDLC":                                     "syscall",
	"syscall.IFT_HDSL2":                                    "syscall",
	"syscall.IFT_HIPERLAN2":                                "syscall",
	"syscall.IFT_HIPPI":                                    "syscall",
	"syscall.IFT_HIPPIINTERFACE":                           "syscall",
	"syscall.IFT_HOSTPAD":                                  "syscall",
	"syscall.IFT_HSSI":                                     "syscall",
	"syscall.IFT_HY":                                       "syscall",
	"syscall.IFT_IBM370PARCHAN":                            "syscall",
	"syscall.IFT_IDSL":                                     "syscall",
	"syscall.IFT_IEEE1394":                                 "syscall",
	"syscall.IFT_IEEE80211":                                "syscall",
	"syscall.IFT_IEEE80212":                                "syscall",
	"syscall.IFT_IEEE8023ADLAG":                            "syscall",
	"syscall.IFT_IFGSN":                                    "syscall",
	"syscall.IFT_IMT":                                      "syscall",
	"syscall.IFT_INFINIBAND":                               "syscall",
	"syscall.IFT_INTERLEAVE":                               "syscall",
	"syscall.IFT_IP":                                       "syscall",
	"syscall.IFT_IPFORWARD":                                "syscall",
	"syscall.IFT_IPOVERATM":                                "syscall",
	"syscall.IFT_IPOVERCDLC":                               "syscall",
	"syscall.IFT_IPOVERCLAW":                               "syscall",
	"syscall.IFT_IPSWITCH":                                 "syscall",
	"syscall.IFT_IPXIP":                                    "syscall",
	"syscall.IFT_ISDN":                                     "syscall",
	"syscall.IFT_ISDNBASIC":                                "syscall",
	"syscall.IFT_ISDNPRIMARY":                              "syscall",
	"syscall.IFT_ISDNS":                                    "syscall",
	"syscall.IFT_ISDNU":                                    "syscall",
	"syscall.IFT_ISO88022LLC":                              "syscall",
	"syscall.IFT_ISO88023":                                 "syscall",
	"syscall.IFT_ISO88024":                                 "syscall",
	"syscall.IFT_ISO88025":                                 "syscall",
	"syscall.IFT_ISO88025CRFPINT":                          "syscall",
	"syscall.IFT_ISO88025DTR":                              "syscall",
	"syscall.IFT_ISO88025FIBER":                            "syscall",
	"syscall.IFT_ISO88026":                                 "syscall",
	"syscall.IFT_ISUP":                                     "syscall",
	"syscall.IFT_L2VLAN":                                   "syscall",
	"syscall.IFT_L3IPVLAN":                                 "syscall",
	"syscall.IFT_L3IPXVLAN":                                "syscall",
	"syscall.IFT_LAPB":                                     "syscall",
	"syscall.IFT_LAPD":                                     "syscall",
	"syscall.IFT_LAPF":                                     "syscall",
	"syscall.IFT_LINEGROUP":                                "syscall",
	"syscall.IFT_LOCALTALK":                                "syscall",
	"syscall.IFT_LOOP":                                     "syscall",
	"syscall.IFT_MEDIAMAILOVERIP":                          "syscall",
	"syscall.IFT_MFSIGLINK":                                "syscall",
	"syscall.IFT_MIOX25":                                   "syscall",
	"syscall.IFT_MODEM":                                    "syscall",
	"syscall.IFT_MPC":                                      "syscall",
	"syscall.IFT_MPLS":                                     "syscall",
	"syscall.IFT_MPLSTUNNEL":                               "syscall",
	"syscall.IFT_MSDSL":                                    "syscall",
	"syscall.IFT_MVL":                                      "syscall",
	"syscall.IFT_MYRINET":                                  "syscall",
	"syscall.IFT_NFAS":                                     "syscall",
	"syscall.IFT_NSIP":                                     "syscall",
	"syscall.IFT_OPTICALCHANNEL":                           "syscall",
	"syscall.IFT_OPTICALTRANSPORT":                         "syscall",
	"syscall.IFT_OTHER":                                    "syscall",
	"syscall.IFT_P10":                                      "syscall",
	"syscall.IFT_P80":                                      "syscall",
	"syscall.IFT_PARA":                                     "syscall",
	"syscall.IFT_PDP":                                      "syscall",
	"syscall.IFT_PFLOG":                                    "syscall",
	"syscall.IFT_PFLOW":                                    "syscall",
	"syscall.IFT_PFSYNC":                                   "syscall",
	"syscall.IFT_PLC":                                      "syscall",
	"syscall.IFT_PON155":                                   "syscall",
	"syscall.IFT_PON622":                                   "syscall",
	"syscall.IFT_POS":                                      "syscall",
	"syscall.IFT_PPP":                                      "syscall",
	"syscall.IFT_PPPMULTILINKBUNDLE":                       "syscall",
	"syscall.IFT_PROPATM":                                  "syscall",
	"syscall.IFT_PROPBWAP2MP":                              "syscall",
	"syscall.IFT_PROPCNLS":                                 "syscall",
	"syscall.IFT_PROPDOCSWIRELESSDOWNSTREAM":               "syscall",
	"syscall.IFT_PROPDOCSWIRELESSMACLAYER":                 "syscall",
	"syscall.IFT_PROPDOCSWIRELESSUPSTREAM":                 "syscall",
	"syscall.IFT_PROPMUX":                                  "syscall",
	"syscall.IFT_PROPVIRTUAL":                              "syscall",
	"syscall.IFT_PROPWIRELESSP2P":                          "syscall",
	"syscall.IFT_PTPSERIAL":                                "syscall",
	"syscall.IFT_PVC":                                      "syscall",
	"syscall.IFT_Q2931":                                    "syscall",
	"syscall.IFT_QLLC":                                     "syscall",
	"syscall.IFT_RADIOMAC":                                 "syscall",
	"syscall.IFT_RADSL":                                    "syscall",
	"syscall.IFT_REACHDSL":                                 "syscall",
	"syscall.IFT_RFC1483":                                  "syscall",
	"syscall.IFT_RS232":                                    "syscall",
	"syscall.IFT_RSRB":                                     "syscall",
	"syscall.IFT_SDLC":                                     "syscall",
	"syscall.IFT_SDSL":                                     "syscall",
	"syscall.IFT_SHDSL":                                    "syscall",
	"syscall.IFT_SIP":                                      "syscall",
	"syscall.IFT_SIPSIG":                                   "syscall",
	"syscall.IFT_SIPTG":                                    "syscall",
	"syscall.IFT_SLIP":                                     "syscall",
	"syscall.IFT_SMDSDXI":                                  "syscall",
	"syscall.IFT_SMDSICIP":                                 "syscall",
	"syscall.IFT_SONET":                                    "syscall",
	"syscall.IFT_SONETOVERHEADCHANNEL":                     "syscall",
	"syscall.IFT_SONETPATH":                                "syscall",
	"syscall.IFT_SONETVT":                                  "syscall",
	"syscall.IFT_SRP":                                      "syscall",
	"syscall.IFT_SS7SIGLINK":                               "syscall",
	"syscall.IFT_STACKTOSTACK":                             "syscall",
	"syscall.IFT_STARLAN":                                  "syscall",
	"syscall.IFT_STF":                                      "syscall",
	"syscall.IFT_T1":                                       "syscall",
	"syscall.IFT_TDLC":                                     "syscall",
	"syscall.IFT_TELINK":                                   "syscall",
	"syscall.IFT_TERMPAD":                                  "syscall",
	"syscall.IFT_TR008":                                    "syscall",
	"syscall.IFT_TRANSPHDLC":                               "syscall",
	"syscall.IFT_TUNNEL":                                   "syscall",
	"syscall.IFT_ULTRA":                                    "syscall",
	"syscall.IFT_USB":                                      "syscall",
	"syscall.IFT_V11":                                      "syscall",
	"syscall.IFT_V35":                                      "syscall",
	"syscall.IFT_V36":                                      "syscall",
	"syscall.IFT_V37":                                      "syscall",
	"syscall.IFT_VDSL":                                     "syscall",
	"syscall.IFT_VIRTUALIPADDRESS":                         "syscall",
	"syscall.IFT_VIRTUALTG":                                "syscall",
	"syscall.IFT_VOICEDID":                                 "syscall",
	"syscall.IFT_VOICEEM":                                  "syscall",
	"syscall.IFT_VOICEEMFGD":                               "syscall",
	"syscall.IFT_VOICEENCAP":                               "syscall",
	"syscall.IFT_VOICEFGDEANA":                             "syscall",
	"syscall.IFT_VOICEFXO":                                 "syscall",
	"syscall.IFT_VOICEFXS":                                 "syscall",
	"syscall.IFT_VOICEOVERATM":                             "syscall",
	"syscall.IFT_VOICEOVERCABLE":                           "syscall",
	"syscall.IFT_VOICEOVERFRAMERELAY":                      "syscall",
	"syscall.IFT_VOICEOVERIP":                              "syscall",
	"syscall.IFT_X213":                                     "syscall",
	"syscall.IFT_X25":                                      "syscall",
	"syscall.IFT_X25DDN":                                   "syscall",
	"syscall.IFT_X25HUNTGROUP":                             "syscall",
	"syscall.IFT_X25MLP":                                   "syscall",
	"syscall.IFT_X25PLE":                                   "syscall",
	"syscall.IFT_XETHER":                                   "syscall",
	"syscall.IGNBRK":                                       "syscall",
	"syscall.IGNCR":                                        "syscall",
	"syscall.IGNORE":                                       "syscall",
	"syscall.IGNPAR":                                       "syscall",
	"syscall.IMAXBEL":                                      "syscall",
	"syscall.INFINITE":                                     "syscall",
	"syscall.INLCR":                                        "syscall",
	"syscall.INPCK":                                        "syscall",
	"syscall.INVALID_FILE_ATTRIBUTES":                      "syscall",
	"syscall.IN_ACCESS":                                    "syscall",
	"syscall.IN_ALL_EVENTS":                                "syscall",
	"syscall.IN_ATTRIB":                                    "syscall",
	"syscall.IN_CLASSA_HOST":                               "syscall",
	"syscall.IN_CLASSA_MAX":                                "syscall",
	"syscall.IN_CLASSA_NET":                                "syscall",
	"syscall.IN_CLASSA_NSHIFT":                             "syscall",
	"syscall.IN_CLASSB_HOST":                               "syscall",
	"syscall.IN_CLASSB_MAX":                                "syscall",
	"syscall.IN_CLASSB_NET":                                "syscall",
	"syscall.IN_CLASSB_NSHIFT":                             "syscall",
	"syscall.IN_CLASSC_HOST":                               "syscall",
	"syscall.IN_CLASSC_NET":                                "syscall",
	"syscall.IN_CLASSC_NSHIFT":                             "syscall",
	"syscall.IN_CLASSD_HOST":                               "syscall",
	"syscall.IN_CLASSD_NET":                                "syscall",
	"syscall.IN_CLASSD_NSHIFT":                             "syscall",
	"syscall.IN_CLOEXEC":                                   "syscall",
	"syscall.IN_CLOSE":                                     "syscall",
	"syscall.IN_CLOSE_NOWRITE":                             "syscall",
	"syscall.IN_CLOSE_WRITE":                               "syscall",
	"syscall.IN_CREATE":                                    "syscall",
	"syscall.IN_DELETE":                                    "syscall",
	"syscall.IN_DELETE_SELF":                               "syscall",
	"syscall.IN_DONT_FOLLOW":                               "syscall",
	"syscall.IN_EXCL_UNLINK":                               "syscall",
	"syscall.IN_IGNORED":                                   "syscall",
	"syscall.IN_ISDIR":                                     "syscall",
	"syscall.IN_LINKLOCALNETNUM":                           "syscall",
	"syscall.IN_LOOPBACKNET":                               "syscall",
	"syscall.IN_MASK_ADD":                                  "syscall",
	"syscall.IN_MODIFY":                                    "syscall",
	"syscall.IN_MOVE":                                      "syscall",
	"syscall.IN_MOVED_FROM":                                "syscall",
	"syscall.IN_MOVED_TO":                                  "syscall",
	"syscall.IN_MOVE_SELF":                                 "syscall",
	"syscall.IN_NONBLOCK":                                  "syscall",
	"syscall.IN_ONESHOT":                                   "syscall",
	"syscall.IN_ONLYDIR":                                   "syscall",
	"syscall.IN_OPEN":                                      "syscall",
	"syscall.IN_Q_OVERFLOW":                                "syscall",
	"syscall.IN_RFC3021_HOST":                              "syscall",
	"syscall.IN_RFC3021_MASK":                              "syscall",
	"syscall.IN_RFC3021_NET":                               "syscall",
	"syscall.IN_RFC3021_NSHIFT":                            "syscall",
	"syscall.IN_UNMOUNT":                                   "syscall",
	"syscall.IOC_IN":                                       "syscall",
	"syscall.IOC_INOUT":                                    "syscall",
	"syscall.IOC_OUT":                                      "syscall",
	"syscall.IOC_WS2":                                      "syscall",
	"syscall.IPMreq":                                       "syscall",
	"syscall.IPMreqn":                                      "syscall",
	"syscall.IPPROTO_3PC":                                  "syscall",
	"syscall.IPPROTO_ADFS":                                 "syscall",
	"syscall.IPPROTO_AH":                                   "syscall",
	"syscall.IPPROTO_AHIP":                                 "syscall",
	"syscall.IPPROTO_APES":                                 "syscall",
	"syscall.IPPROTO_ARGUS":                                "syscall",
	"syscall.IPPROTO_AX25":                                 "syscall",
	"syscall.IPPROTO_BHA":                                  "syscall",
	"syscall.IPPROTO_BLT":                                  "syscall",
	"syscall.IPPROTO_BRSATMON":                             "syscall",
	"syscall.IPPROTO_CARP":                                 "syscall",
	"syscall.IPPROTO_CFTP":                                 "syscall",
	"syscall.IPPROTO_CHAOS":                                "syscall",
	"syscall.IPPROTO_CMTP":                                 "syscall",
	"syscall.IPPROTO_COMP":                                 "syscall",
	"syscall.IPPROTO_CPHB":                                 "syscall",
	"syscall.IPPROTO_CPNX":                                 "syscall",
	"syscall.IPPROTO_DCCP":                                 "syscall",
	"syscall.IPPROTO_DDP":                                  "syscall",
	"syscall.IPPROTO_DGP":                                  "syscall",
	"syscall.IPPROTO_DIVERT":                               "syscall",
	"syscall.IPPROTO_DONE":                                 "syscall",
	"syscall.IPPROTO_DSTOPTS":                              "syscall",
	"syscall.IPPROTO_EGP":                                  "syscall",
	"syscall.IPPROTO_EMCON":                                "syscall",
	"syscall.IPPROTO_ENCAP":                                "syscall",
	"syscall.IPPROTO_EON":                                  "syscall",
	"syscall.IPPROTO_ESP":                                  "syscall",
	"syscall.IPPROTO_ETHERIP":                              "syscall",
	"syscall.IPPROTO_FRAGMENT":                             "syscall",
	"syscall.IPPROTO_GGP":                                  "syscall",
	"syscall.IPPROTO_GMTP":                                 "syscall",
	"syscall.IPPROTO_GRE":                                  "syscall",
	"syscall.IPPROTO_HELLO":                                "syscall",
	"syscall.IPPROTO_HMP":                                  "syscall",
	"syscall.IPPROTO_HOPOPTS":                              "syscall",
	"syscall.IPPROTO_ICMP":                                 "syscall",
	"syscall.IPPROTO_ICMPV6":                               "syscall",
	"syscall.IPPROTO_IDP":                                  "syscall",
	"syscall.IPPROTO_IDPR":                                 "syscall",
	"syscall.IPPROTO_IDRP":                                 "syscall",
	"syscall.IPPROTO_IGMP":                                 "syscall",
	"syscall.IPPROTO_IGP":                                  "syscall",
	"syscall.IPPROTO_IGRP":                                 "syscall",
	"syscall.IPPROTO_IL":                                   "syscall",
	"syscall.IPPROTO_INLSP":                                "syscall",
	"syscall.IPPROTO_INP":                                  "syscall",
	"syscall.IPPROTO_IP":                                   "syscall",
	"syscall.IPPROTO_IPCOMP":                               "syscall",
	"syscall.IPPROTO_IPCV":                                 "syscall",
	"syscall.IPPROTO_IPEIP":                                "syscall",
	"syscall.IPPROTO_IPIP":                                 "syscall",
	"syscall.IPPROTO_IPPC":                                 "syscall",
	"syscall.IPPROTO_IPV4":                                 "syscall",
	"syscall.IPPROTO_IPV6":                                 "syscall",
	"syscall.IPPROTO_IPV6_ICMP":                            "syscall",
	"syscall.IPPROTO_IRTP":                                 "syscall",
	"syscall.IPPROTO_KRYPTOLAN":                            "syscall",
	"syscall.IPPROTO_LARP":                                 "syscall",
	"syscall.IPPROTO_LEAF1":                                "syscall",
	"syscall.IPPROTO_LEAF2":                                "syscall",
	"syscall.IPPROTO_MAX":                                  "syscall",
	"syscall.IPPROTO_MAXID":                                "syscall",
	"syscall.IPPROTO_MEAS":                                 "syscall",
	"syscall.IPPROTO_MH":                                   "syscall",
	"syscall.IPPROTO_MHRP":                                 "syscall",
	"syscall.IPPROTO_MICP":                                 "syscall",
	"syscall.IPPROTO_MOBILE":                               "syscall",
	"syscall.IPPROTO_MPLS":                                 "syscall",
	"syscall.IPPROTO_MTP":                                  "syscall",
	"syscall.IPPROTO_MUX":                                  "syscall",
	"syscall.IPPROTO_ND":                                   "syscall",
	"syscall.IPPROTO_NHRP":                                 "syscall",
	"syscall.IPPROTO_NONE":                                 "syscall",
	"syscall.IPPROTO_NSP":                                  "syscall",
	"syscall.IPPROTO_NVPII":                                "syscall",
	"syscall.IPPROTO_OLD_DIVERT":                           "syscall",
	"syscall.IPPROTO_OSPFIGP":                              "syscall",
	"syscall.IPPROTO_PFSYNC":                               "syscall",
	"syscall.IPPROTO_PGM":                                  "syscall",
	"syscall.IPPROTO_PIGP":                                 "syscall",
	"syscall.IPPROTO_PIM":                                  "syscall",
	"syscall.IPPROTO_PRM":                                  "syscall",
	"syscall.IPPROTO_PUP":                                  "syscall",
	"syscall.IPPROTO_PVP":                                  "syscall",
	"syscall.IPPROTO_RAW":                                  "syscall",
	"syscall.IPPROTO_RCCMON":                               "syscall",
	"syscall.IPPROTO_RDP":                                  "syscall",
	"syscall.IPPROTO_ROUTING":                              "syscall",
	"syscall.IPPROTO_RSVP":                                 "syscall",
	"syscall.IPPROTO_RVD":                                  "syscall",
	"syscall.IPPROTO_SATEXPAK":                             "syscall",
	"syscall.IPPROTO_SATMON":                               "syscall",
	"syscall.IPPROTO_SCCSP":                                "syscall",
	"syscall.IPPROTO_SCTP":                                 "syscall",
	"syscall.IPPROTO_SDRP":                                 "syscall",
	"syscall.IPPROTO_SEND":                                 "syscall",
	"syscall.IPPROTO_SEP":                                  "syscall",
	"syscall.IPPROTO_SKIP":                                 "syscall",
	"syscall.IPPROTO_SPACER":                               "syscall",
	"syscall.IPPROTO_SRPC":                                 "syscall",
	"syscall.IPPROTO_ST":                                   "syscall",
	"syscall.IPPROTO_SVMTP":                                "syscall",
	"syscall.IPPROTO_SWIPE":                                "syscall",
	"syscall.IPPROTO_TCF":                                  "syscall",
	"syscall.IPPROTO_TCP":                                  "syscall",
	"syscall.IPPROTO_TLSP":                                 "syscall",
	"syscall.IPPROTO_TP":                                   "syscall",
	"syscall.IPPROTO_TPXX":                                 "syscall",
	"syscall.IPPROTO_TRUNK1":                               "syscall",
	"syscall.IPPROTO_TRUNK2":                               "syscall",
	"syscall.IPPROTO_TTP":                                  "syscall",
	"syscall.IPPROTO_UDP":                                  "syscall",
	"syscall.IPPROTO_UDPLITE":                              "syscall",
	"syscall.IPPROTO_VINES":                                "syscall",
	"syscall.IPPROTO_VISA":                                 "syscall",
	"syscall.IPPROTO_VMTP":                                 "syscall",
	"syscall.IPPROTO_VRRP":                                 "syscall",
	"syscall.IPPROTO_WBEXPAK":                              "syscall",
	"syscall.IPPROTO_WBMON":                                "syscall",
	"syscall.IPPROTO_WSN":                                  "syscall",
	"syscall.IPPROTO_XNET":                                 "syscall",
	"syscall.IPPROTO_XTP":                                  "syscall",
	"syscall.IPV6_2292DSTOPTS":                             "syscall",
	"syscall.IPV6_2292HOPLIMIT":                            "syscall",
	"syscall.IPV6_2292HOPOPTS":                             "syscall",
	"syscall.IPV6_2292NEXTHOP":                             "syscall",
	"syscall.IPV6_2292PKTINFO":                             "syscall",
	"syscall.IPV6_2292PKTOPTIONS":                          "syscall",
	"syscall.IPV6_2292RTHDR":                               "syscall",
	"syscall.IPV6_ADDRFORM":                                "syscall",
	"syscall.IPV6_ADD_MEMBERSHIP":                          "syscall",
	"syscall.IPV6_AUTHHDR":                                 "syscall",
	"syscall.IPV6_AUTH_LEVEL":                              "syscall",
	"syscall.IPV6_AUTOFLOWLABEL":                           "syscall",
	"syscall.IPV6_BINDANY":                                 "syscall",
	"syscall.IPV6_BINDV6ONLY":                              "syscall",
	"syscall.IPV6_BOUND_IF":                                "syscall",
	"syscall.IPV6_CHECKSUM":                                "syscall",
	"syscall.IPV6_DEFAULT_MULTICAST_HOPS":                  "syscall",
	"syscall.IPV6_DEFAULT_MULTICAST_LOOP":                  "syscall",
	"syscall.IPV6_DEFHLIM":                                 "syscall",
	"syscall.IPV6_DONTFRAG":                                "syscall",
	"syscall.IPV6_DROP_MEMBERSHIP":                         "syscall",
	"syscall.IPV6_DSTOPTS":                                 "syscall",
	"syscall.IPV6_ESP_NETWORK_LEVEL":                       "syscall",
	"syscall.IPV6_ESP_TRANS_LEVEL":                         "syscall",
	"syscall.IPV6_FAITH":                                   "syscall",
	"syscall.IPV6_FLOWINFO_MASK":                           "syscall",
	"syscall.IPV6_FLOWLABEL_MASK":                          "syscall",
	"syscall.IPV6_FRAGTTL":                                 "syscall",
	"syscall.IPV6_FW_ADD":                                  "syscall",
	"syscall.IPV6_FW_DEL":                                  "syscall",
	"syscall.IPV6_FW_FLUSH":                                "syscall",
	"syscall.IPV6_FW_GET":                                  "syscall",
	"syscall.IPV6_FW_ZERO":                                 "syscall",
	"syscall.IPV6_HLIMDEC":                                 "syscall",
	"syscall.IPV6_HOPLIMIT":                                "syscall",
	"syscall.IPV6_HOPOPTS":                                 "syscall",
	"syscall.IPV6_IPCOMP_LEVEL":                            "syscall",
	"syscall.IPV6_IPSEC_POLICY":                            "syscall",
	"syscall.IPV6_JOIN_ANYCAST":                            "syscall",
	"syscall.IPV6_JOIN_GROUP":                              "syscall",
	"syscall.IPV6_LEAVE_ANYCAST":                           "syscall",
	"syscall.IPV6_LEAVE_GROUP":                             "syscall",
	"syscall.IPV6_MAXHLIM":                                 "syscall",
	"syscall.IPV6_MAXOPTHDR":                               "syscall",
	"syscall.IPV6_MAXPACKET":                               "syscall",
	"syscall.IPV6_MAX_GROUP_SRC_FILTER":                    "syscall",
	"syscall.IPV6_MAX_MEMBERSHIPS":                         "syscall",
	"syscall.IPV6_MAX_SOCK_SRC_FILTER":                     "syscall",
	"syscall.IPV6_MIN_MEMBERSHIPS":                         "syscall",
	"syscall.IPV6_MMTU":                                    "syscall",
	"syscall.IPV6_MSFILTER":                                "syscall",
	"syscall.IPV6_MTU":                                     "syscall",
	"syscall.IPV6_MTU_DISCOVER":                            "syscall",
	"syscall.IPV6_MULTICAST_HOPS":                          "syscall",
	"syscall.IPV6_MULTICAST_IF":                            "syscall",
	"syscall.IPV6_MULTICAST_LOOP":                          "syscall",
	"syscall.IPV6_NEXTHOP":                                 "syscall",
	"syscall.IPV6_OPTIONS":                                 "syscall",
	"syscall.IPV6_PATHMTU":                                 "syscall",
	"syscall.IPV6_PIPEX":                                   "syscall",
	"syscall.IPV6_PKTINFO":                                 "syscall",
	"syscall.IPV6_PMTUDISC_DO":                             "syscall",
	"syscall.IPV6_PMTUDISC_DONT":                           "syscall",
	"syscall.IPV6_PMTUDISC_PROBE":                          "syscall",
	"syscall.IPV6_PMTUDISC_WANT":                           "syscall",
	"syscall.IPV6_PORTRANGE":                               "syscall",
	"syscall.IPV6_PORTRANGE_DEFAULT":                       "syscall",
	"syscall.IPV6_PORTRANGE_HIGH":                          "syscall",
	"syscall.IPV6_PORTRANGE_LOW":                           "syscall",
	"syscall.IPV6_PREFER_TEMPADDR":                         "syscall",
	"syscall.IPV6_RECVDSTOPTS":                             "syscall",
	"syscall.IPV6_RECVERR":                                 "syscall",
	"syscall.IPV6_RECVHOPLIMIT":                            "syscall",
	"syscall.IPV6_RECVHOPOPTS":                             "syscall",
	"syscall.IPV6_RECVPATHMTU":                             "syscall",
	"syscall.IPV6_RECVPKTINFO":                             "syscall",
	"syscall.IPV6_RECVRTHDR":                               "syscall",
	"syscall.IPV6_RECVTCLASS":                              "syscall",
	"syscall.IPV6_ROUTER_ALERT":                            "syscall",
	"syscall.IPV6_RTABLE":                                  "syscall",
	"syscall.IPV6_RTHDR":                                   "syscall",
	"syscall.IPV6_RTHDRDSTOPTS":                            "syscall",
	"syscall.IPV6_RTHDR_LOOSE":                             "syscall",
	"syscall.IPV6_RTHDR_STRICT":                            "syscall",
	"syscall.IPV6_RTHDR_TYPE_0":                            "syscall",
	"syscall.IPV6_RXDSTOPTS":                               "syscall",
	"syscall.IPV6_RXHOPOPTS":                               "syscall",
	"syscall.IPV6_SOCKOPT_RESERVED1":                       "syscall",
	"syscall.IPV6_TCLASS":                                  "syscall",
	"syscall.IPV6_UNICAST_HOPS":                            "syscall",
	"syscall.IPV6_USE_MIN_MTU":                             "syscall",
	"syscall.IPV6_V6ONLY":                                  "syscall",
	"syscall.IPV6_VERSION":                                 "syscall",
	"syscall.IPV6_VERSION_MASK":                            "syscall",
	"syscall.IPV6_XFRM_POLICY":                             "syscall",
	"syscall.IP_ADD_MEMBERSHIP":                            "syscall",
	"syscall.IP_ADD_SOURCE_MEMBERSHIP":                     "syscall",
	"syscall.IP_AUTH_LEVEL":                                "syscall",
	"syscall.IP_BINDANY":                                   "syscall",
	"syscall.IP_BLOCK_SOURCE":                              "syscall",
	"syscall.IP_BOUND_IF":                                  "syscall",
	"syscall.IP_DEFAULT_MULTICAST_LOOP":                    "syscall",
	"syscall.IP_DEFAULT_MULTICAST_TTL":                     "syscall",
	"syscall.IP_DF":                                        "syscall",
	"syscall.IP_DONTFRAG":                                  "syscall",
	"syscall.IP_DROP_MEMBERSHIP":                           "syscall",
	"syscall.IP_DROP_SOURCE_MEMBERSHIP":                    "syscall",
	"syscall.IP_DUMMYNET3":                                 "syscall",
	"syscall.IP_DUMMYNET_CONFIGURE":                        "syscall",
	"syscall.IP_DUMMYNET_DEL":                              "syscall",
	"syscall.IP_DUMMYNET_FLUSH":                            "syscall",
	"syscall.IP_DUMMYNET_GET":                              "syscall",
	"syscall.IP_EF":                                        "syscall",
	"syscall.IP_ERRORMTU":                                  "syscall",
	"syscall.IP_ESP_NETWORK_LEVEL":                         "syscall",
	"syscall.IP_ESP_TRANS_LEVEL":                           "syscall",
	"syscall.IP_FAITH":                                     "syscall",
	"syscall.IP_FREEBIND":                                  "syscall",
	"syscall.IP_FW3":                                       "syscall",
	"syscall.IP_FW_ADD":                                    "syscall",
	"syscall.IP_FW_DEL":                                    "syscall",
	"syscall.IP_FW_FLUSH":                                  "syscall",
	"syscall.IP_FW_GET":                                    "syscall",
	"syscall.IP_FW_NAT_CFG":                                "syscall",
	"syscall.IP_FW_NAT_DEL":                                "syscall",
	"syscall.IP_FW_NAT_GET_CONFIG":                         "syscall",
	"syscall.IP_FW_NAT_GET_LOG":                            "syscall",
	"syscall.IP_FW_RESETLOG":                               "syscall",
	"syscall.IP_FW_TABLE_ADD":                              "syscall",
	"syscall.IP_FW_TABLE_DEL":                              "syscall",
	"syscall.IP_FW_TABLE_FLUSH":                            "syscall",
	"syscall.IP_FW_TABLE_GETSIZE":                          "syscall",
	"syscall.IP_FW_TABLE_LIST":                             "syscall",
	"syscall.IP_FW_ZERO":                                   "syscall",
	"syscall.IP_HDRINCL":                                   "syscall",
	"syscall.IP_IPCOMP_LEVEL":                              "syscall",
	"syscall.IP_IPSECFLOWINFO":                             "syscall",
	"syscall.IP_IPSEC_LOCAL_AUTH":                          "syscall",
	"syscall.IP_IPSEC_LOCAL_CRED":                          "syscall",
	"syscall.IP_IPSEC_LOCAL_ID":                            "syscall",
	"syscall.IP_IPSEC_POLICY":                              "syscall",
	"syscall.IP_IPSEC_REMOTE_AUTH":                         "syscall",
	"syscall.IP_IPSEC_REMOTE_CRED":                         "syscall",
	"syscall.IP_IPSEC_REMOTE_ID":                           "syscall",
	"syscall.IP_MAXPACKET":                                 "syscall",
	"syscall.IP_MAX_GROUP_SRC_FILTER":                      "syscall",
	"syscall.IP_MAX_MEMBERSHIPS":                           "syscall",
	"syscall.IP_MAX_SOCK_MUTE_FILTER":                      "syscall",
	"syscall.IP_MAX_SOCK_SRC_FILTER":                       "syscall",
	"syscall.IP_MAX_SOURCE_FILTER":                         "syscall",
	"syscall.IP_MF":                                        "syscall",
	"syscall.IP_MINFRAGSIZE":                               "syscall",
	"syscall.IP_MINTTL":                                    "syscall",
	"syscall.IP_MIN_MEMBERSHIPS":                           "syscall",
	"syscall.IP_MSFILTER":                                  "syscall",
	"syscall.IP_MSS":                                       "syscall",
	"syscall.IP_MTU":                                       "syscall",
	"syscall.IP_MTU_DISCOVER":                              "syscall",
	"syscall.IP_MULTICAST_IF":                              "syscall",
	"syscall.IP_MULTICAST_IFINDEX":                         "syscall",
	"syscall.IP_MULTICAST_LOOP":                            "syscall",
	"syscall.IP_MULTICAST_TTL":                             "syscall",
	"syscall.IP_MULTICAST_VIF":                             "syscall",
	"syscall.IP_NAT__XXX":                                  "syscall",
	"syscall.IP_OFFMASK":                                   "syscall",
	"syscall.IP_OLD_FW_ADD":                                "syscall",
	"syscall.IP_OLD_FW_DEL":                                "syscall",
	"syscall.IP_OLD_FW_FLUSH":                              "syscall",
	"syscall.IP_OLD_FW_GET":                                "syscall",
	"syscall.IP_OLD_FW_RESETLOG":                           "syscall",
	"syscall.IP_OLD_FW_ZERO":                               "syscall",
	"syscall.IP_ONESBCAST":                                 "syscall",
	"syscall.IP_OPTIONS":                                   "syscall",
	"syscall.IP_ORIGDSTADDR":                               "syscall",
	"syscall.IP_PASSSEC":                                   "syscall",
	"syscall.IP_PIPEX":                                     "syscall",
	"syscall.IP_PKTINFO":                                   "syscall",
	"syscall.IP_PKTOPTIONS":                                "syscall",
	"syscall.IP_PMTUDISC":                                  "syscall",
	"syscall.IP_PMTUDISC_DO":                               "syscall",
	"syscall.IP_PMTUDISC_DONT":                             "syscall",
	"syscall.IP_PMTUDISC_PROBE":                            "syscall",
	"syscall.IP_PMTUDISC_WANT":                             "syscall",
	"syscall.IP_PORTRANGE":                                 "syscall",
	"syscall.IP_PORTRANGE_DEFAULT":                         "syscall",
	"syscall.IP_PORTRANGE_HIGH":                            "syscall",
	"syscall.IP_PORTRANGE_LOW":                             "syscall",
	"syscall.IP_RECVDSTADDR":                               "syscall",
	"syscall.IP_RECVDSTPORT":                               "syscall",
	"syscall.IP_RECVERR":                                   "syscall",
	"syscall.IP_RECVIF":                                    "syscall",
	"syscall.IP_RECVOPTS":                                  "syscall",
	"syscall.IP_RECVORIGDSTADDR":                           "syscall",
	"syscall.IP_RECVPKTINFO":                               "syscall",
	"syscall.IP_RECVRETOPTS":                               "syscall",
	"syscall.IP_RECVRTABLE":                                "syscall",
	"syscall.IP_RECVTOS":                                   "syscall",
	"syscall.IP_RECVTTL":                                   "syscall",
	"syscall.IP_RETOPTS":                                   "syscall",
	"syscall.IP_RF":                                        "syscall",
	"syscall.IP_ROUTER_ALERT":                              "syscall",
	"syscall.IP_RSVP_OFF":                                  "syscall",
	"syscall.IP_RSVP_ON":                                   "syscall",
	"syscall.IP_RSVP_VIF_OFF":                              "syscall",
	"syscall.IP_RSVP_VIF_ON":                               "syscall",
	"syscall.IP_RTABLE":                                    "syscall",
	"syscall.IP_SENDSRCADDR":                               "syscall",
	"syscall.IP_STRIPHDR":                                  "syscall",
	"syscall.IP_TOS":                                       "syscall",
	"syscall.IP_TRAFFIC_MGT_BACKGROUND":                    "syscall",
	"syscall.IP_TRANSPARENT":                               "syscall",
	"syscall.IP_TTL":                                       "syscall",
	"syscall.IP_UNBLOCK_SOURCE":                            "syscall",
	"syscall.IP_XFRM_POLICY":                               "syscall",
	"syscall.IPv6MTUInfo":                                  "syscall",
	"syscall.IPv6Mreq":                                     "syscall",
	"syscall.ISIG":                                         "syscall",
	"syscall.ISTRIP":                                       "syscall",
	"syscall.IUCLC":                                        "syscall",
	"syscall.IUTF8":                                        "syscall",
	"syscall.IXANY":                                        "syscall",
	"syscall.IXOFF":                                        "syscall",
	"syscall.IXON":                                         "syscall",
	"syscall.IfAddrmsg":                                    "syscall",
	"syscall.IfAnnounceMsghdr":                             "syscall",
	"syscall.IfData":                                       "syscall",
	"syscall.IfInfomsg":                                    "syscall",
	"syscall.IfMsghdr":                                     "syscall",
	"syscall.IfaMsghdr":                                    "syscall",
	"syscall.IfmaMsghdr":                                   "syscall",
	"syscall.IfmaMsghdr2":                                  "syscall",
	"syscall.ImplementsGetwd":                              "syscall",
	"syscall.Inet4Pktinfo":                                 "syscall",
	"syscall.Inet6Pktinfo":                                 "syscall",
	"syscall.InotifyAddWatch":                              "syscall",
	"syscall.InotifyEvent":                                 "syscall",
	"syscall.InotifyInit":                                  "syscall",
	"syscall.InotifyInit1":                                 "syscall",
	"syscall.InotifyRmWatch":                               "syscall",
	"syscall.InterfaceAddrMessage":                         "syscall",
	"syscall.InterfaceAnnounceMessage":                     "syscall",
	"syscall.InterfaceInfo":                                "syscall",
	"syscall.InterfaceMessage":                             "syscall",
	"syscall.InterfaceMulticastAddrMessage":                "syscall",
	"syscall.InvalidHandle":                                "syscall",
	"syscall.Ioperm":                                       "syscall",
	"syscall.Iopl":                                         "syscall",
	"syscall.Iovec":                                        "syscall",
	"syscall.IpAdapterInfo":                                "syscall",
	"syscall.IpAddrString":                                 "syscall",
	"syscall.IpAddressString":                              "syscall",
	"syscall.IpMaskString":                                 "syscall",
	"syscall.Issetugid":                                    "syscall",
	"syscall.KEY_ALL_ACCESS":                               "syscall",
	"syscall.KEY_CREATE_LINK":                              "syscall",
	"syscall.KEY_CREATE_SUB_KEY":                           "syscall",
	"syscall.KEY_ENUMERATE_SUB_KEYS":                       "syscall",
	"syscall.KEY_EXECUTE":                                  "syscall",
	"syscall.KEY_NOTIFY":                                   "syscall",
	"syscall.KEY_QUERY_VALUE":                              "syscall",
	"syscall.KEY_READ":                                     "syscall",
	"syscall.KEY_SET_VALUE":                                "syscall",
	"syscall.KEY_WOW64_32KEY":                              "syscall",
	"syscall.KEY_WOW64_64KEY":                              "syscall",
	"syscall.KEY_WRITE":                                    "syscall",
	"syscall.Kevent":                                       "syscall",
	"syscall.Kevent_t":                                     "syscall",
	"syscall.Kill":                                         "syscall",
	"syscall.Klogctl":                                      "syscall",
	"syscall.Kqueue":                                       "syscall",
	"syscall.LANG_ENGLISH":                                 "syscall",
	"syscall.LAYERED_PROTOCOL":                             "syscall",
	"syscall.LCNT_OVERLOAD_FLUSH":                          "syscall",
	"syscall.LINUX_REBOOT_CMD_CAD_OFF":                     "syscall",
	"syscall.LINUX_REBOOT_CMD_CAD_ON":                      "syscall",
	"syscall.LINUX_REBOOT_CMD_HALT":                        "syscall",
	"syscall.LINUX_REBOOT_CMD_KEXEC":                       "syscall",
	"syscall.LINUX_REBOOT_CMD_POWER_OFF":                   "syscall",
	"syscall.LINUX_REBOOT_CMD_RESTART":                     "syscall",
	"syscall.LINUX_REBOOT_CMD_RESTART2":                    "syscall",
	"syscall.LINUX_REBOOT_CMD_SW_SUSPEND":                  "syscall",
	"syscall.LINUX_REBOOT_MAGIC1":                          "syscall",
	"syscall.LINUX_REBOOT_MAGIC2":                          "syscall",
	"syscall.LOCK_EX":                                      "syscall",
	"syscall.LOCK_NB":                                      "syscall",
	"syscall.LOCK_SH":                                      "syscall",
	"syscall.LOCK_UN":                                      "syscall",
	"syscall.LazyDLL":                                      "syscall",
	"syscall.LazyProc":                                     "syscall",
	"syscall.Lchown":                                       "syscall",
	"syscall.Linger":                                       "syscall",
	"syscall.Link":                                         "syscall",
	"syscall.Listen":                                       "syscall",
	"syscall.Listxattr":                                    "syscall",
	"syscall.LoadCancelIoEx":                               "syscall",
	"syscall.LoadConnectEx":                                "syscall",
	"syscall.LoadDLL":                                      "syscall",
	"syscall.LoadGetAddrInfo":                              "syscall",
	"syscall.LoadLibrary":                                  "syscall",
	"syscall.LoadSetFileCompletionNotificationModes":       "syscall",
	"syscall.LocalFree":                                    "syscall",
	"syscall.Log2phys_t":                                   "syscall",
	"syscall.LookupAccountName":                            "syscall",
	"syscall.LookupAccountSid":                             "syscall",
	"syscall.LookupSID":                                    "syscall",
	"syscall.LsfJump":                                      "syscall",
	"syscall.LsfSocket":                                    "syscall",
	"syscall.LsfStmt":                                      "syscall",
	"syscall.Lstat":                                        "syscall",
	"syscall.MADV_AUTOSYNC":                                "syscall",
	"syscall.MADV_CAN_REUSE":                               "syscall",
	"syscall.MADV_CORE":                                    "syscall",
	"syscall.MADV_DOFORK":                                  "syscall",
	"syscall.MADV_DONTFORK":                                "syscall",
	"syscall.MADV_DONTNEED":                                "syscall",
	"syscall.MADV_FREE":                                    "syscall",
	"syscall.MADV_FREE_REUSABLE":                           "syscall",
	"syscall.MADV_FREE_REUSE":                              "syscall",
	"syscall.MADV_HUGEPAGE":                                "syscall",
	"syscall.MADV_HWPOISON":                                "syscall",
	"syscall.MADV_MERGEABLE":                               "syscall",
	"syscall.MADV_NOCORE":                                  "syscall",
	"syscall.MADV_NOHUGEPAGE":                              "syscall",
	"syscall.MADV_NORMAL":                                  "syscall",
	"syscall.MADV_NOSYNC":                                  "syscall",
	"syscall.MADV_PROTECT":                                 "syscall",
	"syscall.MADV_RANDOM":                                  "syscall",
	"syscall.MADV_REMOVE":                                  "syscall",
	"syscall.MADV_SEQUENTIAL":                              "syscall",
	"syscall.MADV_UNMERGEABLE":                             "syscall",
	"syscall.MADV_WILLNEED":                                "syscall",
	"syscall.MADV_ZERO_WIRED_PAGES":                        "syscall",
	"syscall.MAP_32BIT":                                    "syscall",
	"syscall.MAP_ANON":                                     "syscall",
	"syscall.MAP_ANONYMOUS":                                "syscall",
	"syscall.MAP_COPY":                                     "syscall",
	"syscall.MAP_DENYWRITE":                                "syscall",
	"syscall.MAP_EXECUTABLE":                               "syscall",
	"syscall.MAP_FILE":                                     "syscall",
	"syscall.MAP_FIXED":                                    "syscall",
	"syscall.MAP_GROWSDOWN":                                "syscall",
	"syscall.MAP_HASSEMAPHORE":                             "syscall",
	"syscall.MAP_HUGETLB":                                  "syscall",
	"syscall.MAP_JIT":                                      "syscall",
	"syscall.MAP_LOCKED":                                   "syscall",
	"syscall.MAP_NOCACHE":                                  "syscall",
	"syscall.MAP_NOCORE":                                   "syscall",
	"syscall.MAP_NOEXTEND":                                 "syscall",
	"syscall.MAP_NONBLOCK":                                 "syscall",
	"syscall.MAP_NORESERVE":                                "syscall",
	"syscall.MAP_NOSYNC":                                   "syscall",
	"syscall.MAP_POPULATE":                                 "syscall",
	"syscall.MAP_PREFAULT_READ":                            "syscall",
	"syscall.MAP_PRIVATE":                                  "syscall",
	"syscall.MAP_RENAME":                                   "syscall",
	"syscall.MAP_RESERVED0080":                             "syscall",
	"syscall.MAP_RESERVED0100":                             "syscall",
	"syscall.MAP_SHARED":                                   "syscall",
	"syscall.MAP_STACK":                                    "syscall",
	"syscall.MAP_TYPE":                                     "syscall",
	"syscall.MAXLEN_IFDESCR":                               "syscall",
	"syscall.MAXLEN_PHYSADDR":                              "syscall",
	"syscall.MAX_ADAPTER_ADDRESS_LENGTH":                   "syscall",
	"syscall.MAX_ADAPTER_DESCRIPTION_LENGTH":               "syscall",
	"syscall.MAX_ADAPTER_NAME_LENGTH":                      "syscall",
	"syscall.MAX_COMPUTERNAME_LENGTH":                      "syscall",
	"syscall.MAX_INTERFACE_NAME_LEN":                       "syscall",
	"syscall.MAX_LONG_PATH":                                "syscall",
	"syscall.MAX_PATH":                                     "syscall",
	"syscall.MAX_PROTOCOL_CHAIN":                           "syscall",
	"syscall.MCL_CURRENT":                                  "syscall",
	"syscall.MCL_FUTURE":                                   "syscall",
	"syscall.MNT_DETACH":                                   "syscall",
	"syscall.MNT_EXPIRE":                                   "syscall",
	"syscall.MNT_FORCE":                                    "syscall",
	"syscall.MSG_BCAST":                                    "syscall",
	"syscall.MSG_CMSG_CLOEXEC":                             "syscall",
	"syscall.MSG_COMPAT":                                   "syscall",
	"syscall.MSG_CONFIRM":                                  "syscall",
	"syscall.MSG_CONTROLMBUF":                              "syscall",
	"syscall.MSG_CTRUNC":                                   "syscall",
	"syscall.MSG_DONTROUTE":                                "syscall",
	"syscall.MSG_DONTWAIT":                                 "syscall",
	"syscall.MSG_EOF":                                      "syscall",
	"syscall.MSG_EOR":                                      "syscall",
	"syscall.MSG_ERRQUEUE":                                 "syscall",
	"syscall.MSG_FASTOPEN":                                 "syscall",
	"syscall.MSG_FIN":                                      "syscall",
	"syscall.MSG_FLUSH":                                    "syscall",
	"syscall.MSG_HAVEMORE":                                 "syscall",
	"syscall.MSG_HOLD":                                     "syscall",
	"syscall.MSG_IOVUSRSPACE":                              "syscall",
	"syscall.MSG_LENUSRSPACE":                              "syscall",
	"syscall.MSG_MCAST":                                    "syscall",
	"syscall.MSG_MORE":                                     "syscall",
	"syscall.MSG_NAMEMBUF":                                 "syscall",
	"syscall.MSG_NBIO":                                     "syscall",
	"syscall.MSG_NEEDSA":                                   "syscall",
	"syscall.MSG_NOSIGNAL":                                 "syscall",
	"syscall.MSG_NOTIFICATION":                             "syscall",
	"syscall.MSG_OOB":                                      "syscall",
	"syscall.MSG_PEEK":                                     "syscall",
	"syscall.MSG_PROXY":                                    "syscall",
	"syscall.MSG_RCVMORE":                                  "syscall",
	"syscall.MSG_RST":                                      "syscall",
	"syscall.MSG_SEND":                                     "syscall",
	"syscall.MSG_SYN":                                      "syscall",
	"syscall.MSG_TRUNC":                                    "syscall",
	"syscall.MSG_TRYHARD":                                  "syscall",
	"syscall.MSG_USERFLAGS":                                "syscall",
	"syscall.MSG_WAITALL":                                  "syscall",
	"syscall.MSG_WAITFORONE":                               "syscall",
	"syscall.MSG_WAITSTREAM":                               "syscall",
	"syscall.MS_ACTIVE":                                    "syscall",
	"syscall.MS_ASYNC":                                     "syscall",
	"syscall.MS_BIND":                                      "syscall",
	"syscall.MS_DEACTIVATE":                                "syscall",
	"syscall.MS_DIRSYNC":                                   "syscall",
	"syscall.MS_INVALIDATE":                                "syscall",
	"syscall.MS_I_VERSION":                                 "syscall",
	"syscall.MS_KERNMOUNT":                                 "syscall",
	"syscall.MS_KILLPAGES":                                 "syscall",
	"syscall.MS_MANDLOCK":                                  "syscall",
	"syscall.MS_MGC_MSK":                                   "syscall",
	"syscall.MS_MGC_VAL":                                   "syscall",
	"syscall.MS_MOVE":                                      "syscall",
	"syscall.MS_NOATIME":                                   "syscall",
	"syscall.MS_NODEV":                                     "syscall",
	"syscall.MS_NODIRATIME":                                "syscall",
	"syscall.MS_NOEXEC":                                    "syscall",
	"syscall.MS_NOSUID":                                    "syscall",
	"syscall.MS_NOUSER":                                    "syscall",
	"syscall.MS_POSIXACL":                                  "syscall",
	"syscall.MS_PRIVATE":                                   "syscall",
	"syscall.MS_RDONLY":                                    "syscall",
	"syscall.MS_REC":                                       "syscall",
	"syscall.MS_RELATIME":                                  "syscall",
	"syscall.MS_REMOUNT":                                   "syscall",
	"syscall.MS_RMT_MASK":                                  "syscall",
	"syscall.MS_SHARED":                                    "syscall",
	"syscall.MS_SILENT":                                    "syscall",
	"syscall.MS_SLAVE":                                     "syscall",
	"syscall.MS_STRICTATIME":                               "syscall",
	"syscall.MS_SYNC":                                      "syscall",
	"syscall.MS_SYNCHRONOUS":                               "syscall",
	"syscall.MS_UNBINDABLE":                                "syscall",
	"syscall.Madvise":                                      "syscall",
	"syscall.MapViewOfFile":                                "syscall",
	"syscall.MaxTokenInfoClass":                            "syscall",
	"syscall.Mclpool":                                      "syscall",
	"syscall.MibIfRow":                                     "syscall",
	"syscall.Mkdir":                                        "syscall",
	"syscall.Mkdirat":                                      "syscall",
	"syscall.Mkfifo":                                       "syscall",
	"syscall.Mknod":                                        "syscall",
	"syscall.Mknodat":                                      "syscall",
	"syscall.Mlock":                                        "syscall",
	"syscall.Mlockall":                                     "syscall",
	"syscall.Mmap":                                         "syscall",
	"syscall.Mount":                                        "syscall",
	"syscall.MoveFile":                                     "syscall",
	"syscall.Mprotect":                                     "syscall",
	"syscall.Msghdr":                                       "syscall",
	"syscall.Munlock":                                      "syscall",
	"syscall.Munlockall":                                   "syscall",
	"syscall.Munmap":                                       "syscall",
	"syscall.MustLoadDLL":                                  "syscall",
	"syscall.NAME_MAX":                                     "syscall",
	"syscall.NETLINK_ADD_MEMBERSHIP":                       "syscall",
	"syscall.NETLINK_AUDIT":                                "syscall",
	"syscall.NETLINK_BROADCAST_ERROR":                      "syscall",
	"syscall.NETLINK_CONNECTOR":                            "syscall",
	"syscall.NETLINK_DNRTMSG":                              "syscall",
	"syscall.NETLINK_DROP_MEMBERSHIP":                      "syscall",
	"syscall.NETLINK_ECRYPTFS":                             "syscall",
	"syscall.NETLINK_FIB_LOOKUP":                           "syscall",
	"syscall.NETLINK_FIREWALL":                             "syscall",
	"syscall.NETLINK_GENERIC":                              "syscall",
	"syscall.NETLINK_INET_DIAG":                            "syscall",
	"syscall.NETLINK_IP6_FW":                               "syscall",
	"syscall.NETLINK_ISCSI":                                "syscall",
	"syscall.NETLINK_KOBJECT_UEVENT":                       "syscall",
	"syscall.NETLINK_NETFILTER":                            "syscall",
	"syscall.NETLINK_NFLOG":                                "syscall",
	"syscall.NETLINK_NO_ENOBUFS":                           "syscall",
	"syscall.NETLINK_PKTINFO":                              "syscall",
	"syscall.NETLINK_RDMA":                                 "syscall",
	"syscall.NETLINK_ROUTE":                                "syscall",
	"syscall.NETLINK_SCSITRANSPORT":                        "syscall",
	"syscall.NETLINK_SELINUX":                              "syscall",
	"syscall.NETLINK_UNUSED":                               "syscall",
	"syscall.NETLINK_USERSOCK":                             "syscall",
	"syscall.NETLINK_XFRM":                                 "syscall",
	"syscall.NET_RT_DUMP":                                  "syscall",
	"syscall.NET_RT_DUMP2":                                 "syscall",
	"syscall.NET_RT_FLAGS":                                 "syscall",
	"syscall.NET_RT_IFLIST":                                "syscall",
	"syscall.NET_RT_IFLIST2":                               "syscall",
	"syscall.NET_RT_IFLISTL":                               "syscall",
	"syscall.NET_RT_IFMALIST":                              "syscall",
	"syscall.NET_RT_MAXID":                                 "syscall",
	"syscall.NET_RT_OIFLIST":                               "syscall",
	"syscall.NET_RT_OOIFLIST":                              "syscall",
	"syscall.NET_RT_STAT":                                  "syscall",
	"syscall.NET_RT_STATS":                                 "syscall",
	"syscall.NET_RT_TABLE":                                 "syscall",
	"syscall.NET_RT_TRASH":                                 "syscall",
	"syscall.NLA_ALIGNTO":                                  "syscall",
	"syscall.NLA_F_NESTED":                                 "syscall",
	"syscall.NLA_F_NET_BYTEORDER":                          "syscall",
	"syscall.NLA_HDRLEN":                                   "syscall",
	"syscall.NLMSG_ALIGNTO":                                "syscall",
	"syscall.NLMSG_DONE":                                   "syscall",
	"syscall.NLMSG_ERROR":                                  "syscall",
	"syscall.NLMSG_HDRLEN":                                 "syscall",
	"syscall.NLMSG_MIN_TYPE":                               "syscall",
	"syscall.NLMSG_NOOP":                                   "syscall",
	"syscall.NLMSG_OVERRUN":                                "syscall",
	"syscall.NLM_F_ACK":                                    "syscall",
	"syscall.NLM_F_APPEND":                                 "syscall",
	"syscall.NLM_F_ATOMIC":                                 "syscall",
	"syscall.NLM_F_CREATE":                                 "syscall",
	"syscall.NLM_F_DUMP":                                   "syscall",
	"syscall.NLM_F_ECHO":                                   "syscall",
	"syscall.NLM_F_EXCL":                                   "syscall",
	"syscall.NLM_F_MATCH":                                  "syscall",
	"syscall.NLM_F_MULTI":                                  "syscall",
	"syscall.NLM_F_REPLACE":                                "syscall",
	"syscall.NLM_F_REQUEST":                                "syscall",
	"syscall.NLM_F_ROOT":                                   "syscall",
	"syscall.NOFLSH":                                       "syscall",
	"syscall.NOTE_ABSOLUTE":                                "syscall",
	"syscall.NOTE_ATTRIB":                                  "syscall",
	"syscall.NOTE_CHILD":                                   "syscall",
	"syscall.NOTE_DELETE":                                  "syscall",
	"syscall.NOTE_EOF":                                     "syscall",
	"syscall.NOTE_EXEC":                                    "syscall",
	"syscall.NOTE_EXIT":                                    "syscall",
	"syscall.NOTE_EXITSTATUS":                              "syscall",
	"syscall.NOTE_EXTEND":                                  "syscall",
	"syscall.NOTE_FFAND":                                   "syscall",
	"syscall.NOTE_FFCOPY":                                  "syscall",
	"syscall.NOTE_FFCTRLMASK":                              "syscall",
	"syscall.NOTE_FFLAGSMASK":                              "syscall",
	"syscall.NOTE_FFNOP":                                   "syscall",
	"syscall.NOTE_FFOR":                                    "syscall",
	"syscall.NOTE_FORK":                                    "syscall",
	"syscall.NOTE_LINK":                                    "syscall",
	"syscall.NOTE_LOWAT":                                   "syscall",
	"syscall.NOTE_NONE":                                    "syscall",
	"syscall.NOTE_NSECONDS":                                "syscall",
	"syscall.NOTE_PCTRLMASK":                               "syscall",
	"syscall.NOTE_PDATAMASK":                               "syscall",
	"syscall.NOTE_REAP":                                    "syscall",
	"syscall.NOTE_RENAME":                                  "syscall",
	"syscall.NOTE_RESOURCEEND":                             "syscall",
	"syscall.NOTE_REVOKE":                                  "syscall",
	"syscall.NOTE_SECONDS":                                 "syscall",
	"syscall.NOTE_SIGNAL":                                  "syscall",
	"syscall.NOTE_TRACK":                                   "syscall",
	"syscall.NOTE_TRACKERR":                                "syscall",
	"syscall.NOTE_TRIGGER":                                 "syscall",
	"syscall.NOTE_TRUNCATE":                                "syscall",
	"syscall.NOTE_USECONDS":                                "syscall",
	"syscall.NOTE_VM_ERROR":                                "syscall",
	"syscall.NOTE_VM_PRESSURE":                             "syscall",
	"syscall.NOTE_VM_PRESSURE_SUDDEN_TERMINATE":            "syscall",
	"syscall.NOTE_VM_PRESSURE_TERMINATE":                   "syscall",
	"syscall.NOTE_WRITE":                                   "syscall",
	"syscall.NameCanonical":                                "syscall",
	"syscall.NameCanonicalEx":                              "syscall",
	"syscall.NameDisplay":                                  "syscall",
	"syscall.NameDnsDomain":                                "syscall",
	"syscall.NameFullyQualifiedDN":                         "syscall",
	"syscall.NameSamCompatible":                            "syscall",
	"syscall.NameServicePrincipal":                         "syscall",
	"syscall.NameUniqueId":                                 "syscall",
	"syscall.NameUnknown":                                  "syscall",
	"syscall.NameUserPrincipal":                            "syscall",
	"syscall.Nanosleep":                                    "syscall",
	"syscall.NetApiBufferFree":                             "syscall",
	"syscall.NetGetJoinInformation":                        "syscall",
	"syscall.NetSetupDomainName":                           "syscall",
	"syscall.NetSetupUnjoined":                             "syscall",
	"syscall.NetSetupUnknownStatus":                        "syscall",
	"syscall.NetSetupWorkgroupName":                        "syscall",
	"syscall.NetUserGetInfo":                               "syscall",
	"syscall.NetlinkMessage":                               "syscall",
	"syscall.NetlinkRIB":                                   "syscall",
	"syscall.NetlinkRouteAttr":                             "syscall",
	"syscall.NetlinkRouteRequest":                          "syscall",
	"syscall.NewCallback":                                  "syscall",
	"syscall.NewLazyDLL":                                   "syscall",
	"syscall.NlAttr":                                       "syscall",
	"syscall.NlMsgerr":                                     "syscall",
	"syscall.NlMsghdr":                                     "syscall",
	"syscall.NsecToFiletime":                               "syscall",
	"syscall.NsecToTimespec":                               "syscall",
	"syscall.NsecToTimeval":                                "syscall",
	"syscall.Ntohs":                                        "syscall",
	"syscall.OCRNL":                                        "syscall",
	"syscall.OFDEL":                                        "syscall",
	"syscall.OFILL":                                        "syscall",
	"syscall.OFIOGETBMAP":                                  "syscall",
	"syscall.OID_PKIX_KP_SERVER_AUTH":                      "syscall",
	"syscall.OID_SERVER_GATED_CRYPTO":                      "syscall",
	"syscall.OID_SGC_NETSCAPE":                             "syscall",
	"syscall.OLCUC":                                        "syscall",
	"syscall.ONLCR":                                        "syscall",
	"syscall.ONLRET":                                       "syscall",
	"syscall.ONOCR":                                        "syscall",
	"syscall.ONOEOT":                                       "syscall",
	"syscall.OPEN_ALWAYS":                                  "syscall",
	"syscall.OPEN_EXISTING":                                "syscall",
	"syscall.OPOST":                                        "syscall",
	"syscall.O_ACCMODE":                                    "syscall",
	"syscall.O_ALERT":                                      "syscall",
	"syscall.O_ALT_IO":                                     "syscall",
	"syscall.O_APPEND":                                     "syscall",
	"syscall.O_ASYNC":                                      "syscall",
	"syscall.O_CLOEXEC":                                    "syscall",
	"syscall.O_CREAT":                                      "syscall",
	"syscall.O_DIRECT":                                     "syscall",
	"syscall.O_DIRECTORY":                                  "syscall",
	"syscall.O_DSYNC":                                      "syscall",
	"syscall.O_EVTONLY":                                    "syscall",
	"syscall.O_EXCL":                                       "syscall",
	"syscall.O_EXEC":                                       "syscall",
	"syscall.O_EXLOCK":                                     "syscall",
	"syscall.O_FSYNC":                                      "syscall",
	"syscall.O_LARGEFILE":                                  "syscall",
	"syscall.O_NDELAY":                                     "syscall",
	"syscall.O_NOATIME":                                    "syscall",
	"syscall.O_NOCTTY":                                     "syscall",
	"syscall.O_NOFOLLOW":                                   "syscall",
	"syscall.O_NONBLOCK":                                   "syscall",
	"syscall.O_NOSIGPIPE":                                  "syscall",
	"syscall.O_POPUP":                                      "syscall",
	"syscall.O_RDONLY":                                     "syscall",
	"syscall.O_RDWR":                                       "syscall",
	"syscall.O_RSYNC":                                      "syscall",
	"syscall.O_SHLOCK":                                     "syscall",
	"syscall.O_SYMLINK":                                    "syscall",
	"syscall.O_SYNC":                                       "syscall",
	"syscall.O_TRUNC":                                      "syscall",
	"syscall.O_TTY_INIT":                                   "syscall",
	"syscall.O_WRONLY":                                     "syscall",
	"syscall.Open":                                         "syscall",
	"syscall.OpenCurrentProcessToken":                      "syscall",
	"syscall.OpenProcess":                                  "syscall",
	"syscall.OpenProcessToken":                             "syscall",
	"syscall.Openat":                                       "syscall",
	"syscall.Overlapped":                                   "syscall",
	"syscall.PACKET_ADD_MEMBERSHIP":                        "syscall",
	"syscall.PACKET_BROADCAST":                             "syscall",
	"syscall.PACKET_DROP_MEMBERSHIP":                       "syscall",
	"syscall.PACKET_FASTROUTE":                             "syscall",
	"syscall.PACKET_HOST":                                  "syscall",
	"syscall.PACKET_LOOPBACK":                              "syscall",
	"syscall.PACKET_MR_ALLMULTI":                           "syscall",
	"syscall.PACKET_MR_MULTICAST":                          "syscall",
	"syscall.PACKET_MR_PROMISC":                            "syscall",
	"syscall.PACKET_MULTICAST":                             "syscall",
	"syscall.PACKET_OTHERHOST":                             "syscall",
	"syscall.PACKET_OUTGOING":                              "syscall",
	"syscall.PACKET_RECV_OUTPUT":                           "syscall",
	"syscall.PACKET_RX_RING":                               "syscall",
	"syscall.PACKET_STATISTICS":                            "syscall",
	"syscall.PAGE_EXECUTE_READ":                            "syscall",
	"syscall.PAGE_EXECUTE_READWRITE":                       "syscall",
	"syscall.PAGE_EXECUTE_WRITECOPY":                       "syscall",
	"syscall.PAGE_READONLY":                                "syscall",
	"syscall.PAGE_READWRITE":                               "syscall",
	"syscall.PAGE_WRITECOPY":                               "syscall",
	"syscall.PARENB":                                       "syscall",
	"syscall.PARMRK":                                       "syscall",
	"syscall.PARODD":                                       "syscall",
	"syscall.PENDIN":                                       "syscall",
	"syscall.PFL_HIDDEN":                                   "syscall",
	"syscall.PFL_MATCHES_PROTOCOL_ZERO":                    "syscall",
	"syscall.PFL_MULTIPLE_PROTO_ENTRIES":                   "syscall",
	"syscall.PFL_NETWORKDIRECT_PROVIDER":                   "syscall",
	"syscall.PFL_RECOMMENDED_PROTO_ENTRY":                  "syscall",
	"syscall.PF_FLUSH":                                     "syscall",
	"syscall.PKCS_7_ASN_ENCODING":                          "syscall",
	"syscall.PMC5_PIPELINE_FLUSH":                          "syscall",
	"syscall.PRIO_PGRP":                                    "syscall",
	"syscall.PRIO_PROCESS":                                 "syscall",
	"syscall.PRIO_USER":                                    "syscall",
	"syscall.PRI_IOFLUSH":                                  "syscall",
	"syscall.PROCESS_QUERY_INFORMATION":                    "syscall",
	"syscall.PROCESS_TERMINATE":                            "syscall",
	"syscall.PROT_EXEC":                                    "syscall",
	"syscall.PROT_GROWSDOWN":                               "syscall",
	"syscall.PROT_GROWSUP":                                 "syscall",
	"syscall.PROT_NONE":                                    "syscall",
	"syscall.PROT_READ":                                    "syscall",
	"syscall.PROT_WRITE":                                   "syscall",
	"syscall.PROV_DH_SCHANNEL":                             "syscall",
	"syscall.PROV_DSS":                                     "syscall",
	"syscall.PROV_DSS_DH":                                  "syscall",
	"syscall.PROV_EC_ECDSA_FULL":                           "syscall",
	"syscall.PROV_EC_ECDSA_SIG":                            "syscall",
	"syscall.PROV_EC_ECNRA_FULL":                           "syscall",
	"syscall.PROV_EC_ECNRA_SIG":                            "syscall",
	"syscall.PROV_FORTEZZA":                                "syscall",
	"syscall.PROV_INTEL_SEC":                               "syscall",
	"syscall.PROV_MS_EXCHANGE":                             "syscall",
	"syscall.PROV_REPLACE_OWF":                             "syscall",
	"syscall.PROV_RNG":                                     "syscall",
	"syscall.PROV_RSA_AES":                                 "syscall",
	"syscall.PROV_RSA_FULL":                                "syscall",
	"syscall.PROV_RSA_SCHANNEL":                            "syscall",
	"syscall.PROV_RSA_SIG":                                 "syscall",
	"syscall.PROV_SPYRUS_LYNKS":                            "syscall",
	"syscall.PROV_SSL":                                     "syscall",
	"syscall.PR_CAPBSET_DROP":                              "syscall",
	"syscall.PR_CAPBSET_READ":                              "syscall",
	"syscall.PR_CLEAR_SECCOMP_FILTER":                      "syscall",
	"syscall.PR_ENDIAN_BIG":                                "syscall",
	"syscall.PR_ENDIAN_LITTLE":                             "syscall",
	"syscall.PR_ENDIAN_PPC_LITTLE":                         "syscall",
	"syscall.PR_FPEMU_NOPRINT":                             "syscall",
	"syscall.PR_FPEMU_SIGFPE":                              "syscall",
	"syscall.PR_FP_EXC_ASYNC":                              "syscall",
	"syscall.PR_FP_EXC_DISABLED":                           "syscall",
	"syscall.PR_FP_EXC_DIV":                                "syscall",
	"syscall.PR_FP_EXC_INV":                                "syscall",
	"syscall.PR_FP_EXC_NONRECOV":                           "syscall",
	"syscall.PR_FP_EXC_OVF":                                "syscall",
	"syscall.PR_FP_EXC_PRECISE":                            "syscall",
	"syscall.PR_FP_EXC_RES":                                "syscall",
	"syscall.PR_FP_EXC_SW_ENABLE":                          "syscall",
	"syscall.PR_FP_EXC_UND":                                "syscall",
	"syscall.PR_GET_DUMPABLE":                              "syscall",
	"syscall.PR_GET_ENDIAN":                                "syscall",
	"syscall.PR_GET_FPEMU":                                 "syscall",
	"syscall.PR_GET_FPEXC":                                 "syscall",
	"syscall.PR_GET_KEEPCAPS":                              "syscall",
	"syscall.PR_GET_NAME":                                  "syscall",
	"syscall.PR_GET_PDEATHSIG":                             "syscall",
	"syscall.PR_GET_SECCOMP":                               "syscall",
	"syscall.PR_GET_SECCOMP_FILTER":                        "syscall",
	"syscall.PR_GET_SECUREBITS":                            "syscall",
	"syscall.PR_GET_TIMERSLACK":                            "syscall",
	"syscall.PR_GET_TIMING":                                "syscall",
	"syscall.PR_GET_TSC":                                   "syscall",
	"syscall.PR_GET_UNALIGN":                               "syscall",
	"syscall.PR_MCE_KILL":                                  "syscall",
	"syscall.PR_MCE_KILL_CLEAR":                            "syscall",
	"syscall.PR_MCE_KILL_DEFAULT":                          "syscall",
	"syscall.PR_MCE_KILL_EARLY":                            "syscall",
	"syscall.PR_MCE_KILL_GET":                              "syscall",
	"syscall.PR_MCE_KILL_LATE":                             "syscall",
	"syscall.PR_MCE_KILL_SET":                              "syscall",
	"syscall.PR_SECCOMP_FILTER_EVENT":                      "syscall",
	"syscall.PR_SECCOMP_FILTER_SYSCALL":                    "syscall",
	"syscall.PR_SET_DUMPABLE":                              "syscall",
	"syscall.PR_SET_ENDIAN":                                "syscall",
	"syscall.PR_SET_FPEMU":                                 "syscall",
	"syscall.PR_SET_FPEXC":                                 "syscall",
	"syscall.PR_SET_KEEPCAPS":                              "syscall",
	"syscall.PR_SET_NAME":                                  "syscall",
	"syscall.PR_SET_PDEATHSIG":                             "syscall",
	"syscall.PR_SET_PTRACER":                               "syscall",
	"syscall.PR_SET_SECCOMP":                               "syscall",
	"syscall.PR_SET_SECCOMP_FILTER":                        "syscall",
	"syscall.PR_SET_SECUREBITS":                            "syscall",
	"syscall.PR_SET_TIMERSLACK":                            "syscall",
	"syscall.PR_SET_TIMING":                                "syscall",
	"syscall.PR_SET_TSC":                                   "syscall",
	"syscall.PR_SET_UNALIGN":                               "syscall",
	"syscall.PR_TASK_PERF_EVENTS_DISABLE":                  "syscall",
	"syscall.PR_TASK_PERF_EVENTS_ENABLE":                   "syscall",
	"syscall.PR_TIMING_STATISTICAL":                        "syscall",
	"syscall.PR_TIMING_TIMESTAMP":                          "syscall",
	"syscall.PR_TSC_ENABLE":                                "syscall",
	"syscall.PR_TSC_SIGSEGV":                               "syscall",
	"syscall.PR_UNALIGN_NOPRINT":                           "syscall",
	"syscall.PR_UNALIGN_SIGBUS":                            "syscall",
	"syscall.PTRACE_ARCH_PRCTL":                            "syscall",
	"syscall.PTRACE_ATTACH":                                "syscall",
	"syscall.PTRACE_CONT":                                  "syscall",
	"syscall.PTRACE_DETACH":                                "syscall",
	"syscall.PTRACE_EVENT_CLONE":                           "syscall",
	"syscall.PTRACE_EVENT_EXEC":                            "syscall",
	"syscall.PTRACE_EVENT_EXIT":                            "syscall",
	"syscall.PTRACE_EVENT_FORK":                            "syscall",
	"syscall.PTRACE_EVENT_VFORK":                           "syscall",
	"syscall.PTRACE_EVENT_VFORK_DONE":                      "syscall",
	"syscall.PTRACE_GETCRUNCHREGS":                         "syscall",
	"syscall.PTRACE_GETEVENTMSG":                           "syscall",
	"syscall.PTRACE_GETFPREGS":                             "syscall",
	"syscall.PTRACE_GETFPXREGS":                            "syscall",
	"syscall.PTRACE_GETHBPREGS":                            "syscall",
	"syscall.PTRACE_GETREGS":                               "syscall",
	"syscall.PTRACE_GETREGSET":                             "syscall",
	"syscall.PTRACE_GETSIGINFO":                            "syscall",
	"syscall.PTRACE_GETVFPREGS":                            "syscall",
	"syscall.PTRACE_GETWMMXREGS":                           "syscall",
	"syscall.PTRACE_GET_THREAD_AREA":                       "syscall",
	"syscall.PTRACE_KILL":                                  "syscall",
	"syscall.PTRACE_OLDSETOPTIONS":                         "syscall",
	"syscall.PTRACE_O_MASK":                                "syscall",
	"syscall.PTRACE_O_TRACECLONE":                          "syscall",
	"syscall.PTRACE_O_TRACEEXEC":                           "syscall",
	"syscall.PTRACE_O_TRACEEXIT":                           "syscall",
	"syscall.PTRACE_O_TRACEFORK":                           "syscall",
	"syscall.PTRACE_O_TRACESYSGOOD":                        "syscall",
	"syscall.PTRACE_O_TRACEVFORK":                          "syscall",
	"syscall.PTRACE_O_TRACEVFORKDONE":                      "syscall",
	"syscall.PTRACE_PEEKDATA":                              "syscall",
	"syscall.PTRACE_PEEKTEXT":                              "syscall",
	"syscall.PTRACE_PEEKUSR":                               "syscall",
	"syscall.PTRACE_POKEDATA":                              "syscall",
	"syscall.PTRACE_POKETEXT":                              "syscall",
	"syscall.PTRACE_POKEUSR":                               "syscall",
	"syscall.PTRACE_SETCRUNCHREGS":                         "syscall",
	"syscall.PTRACE_SETFPREGS":                             "syscall",
	"syscall.PTRACE_SETFPXREGS":                            "syscall",
	"syscall.PTRACE_SETHBPREGS":                            "syscall",
	"syscall.PTRACE_SETOPTIONS":                            "syscall",
	"syscall.PTRACE_SETREGS":                               "syscall",
	"syscall.PTRACE_SETREGSET":                             "syscall",
	"syscall.PTRACE_SETSIGINFO":                            "syscall",
	"syscall.PTRACE_SETVFPREGS":                            "syscall",
	"syscall.PTRACE_SETWMMXREGS":                           "syscall",
	"syscall.PTRACE_SET_SYSCALL":                           "syscall",
	"syscall.PTRACE_SET_THREAD_AREA":                       "syscall",
	"syscall.PTRACE_SINGLEBLOCK":                           "syscall",
	"syscall.PTRACE_SINGLESTEP":                            "syscall",
	"syscall.PTRACE_SYSCALL":                               "syscall",
	"syscall.PTRACE_SYSEMU":                                "syscall",
	"syscall.PTRACE_SYSEMU_SINGLESTEP":                     "syscall",
	"syscall.PTRACE_TRACEME":                               "syscall",
	"syscall.PT_ATTACH":                                    "syscall",
	"syscall.PT_ATTACHEXC":                                 "syscall",
	"syscall.PT_CONTINUE":                                  "syscall",
	"syscall.PT_DATA_ADDR":                                 "syscall",
	"syscall.PT_DENY_ATTACH":                               "syscall",
	"syscall.PT_DETACH":                                    "syscall",
	"syscall.PT_FIRSTMACH":                                 "syscall",
	"syscall.PT_FORCEQUOTA":                                "syscall",
	"syscall.PT_KILL":                                      "syscall",
	"syscall.PT_MASK":                                      "syscall",
	"syscall.PT_READ_D":                                    "syscall",
	"syscall.PT_READ_I":                                    "syscall",
	"syscall.PT_READ_U":                                    "syscall",
	"syscall.PT_SIGEXC":                                    "syscall",
	"syscall.PT_STEP":                                      "syscall",
	"syscall.PT_TEXT_ADDR":                                 "syscall",
	"syscall.PT_TEXT_END_ADDR":                             "syscall",
	"syscall.PT_THUPDATE":                                  "syscall",
	"syscall.PT_TRACE_ME":                                  "syscall",
	"syscall.PT_WRITE_D":                                   "syscall",
	"syscall.PT_WRITE_I":                                   "syscall",
	"syscall.PT_WRITE_U":                                   "syscall",
	"syscall.ParseDirent":                                  "syscall",
	"syscall.ParseNetlinkMessage":                          "syscall",
	"syscall.ParseNetlinkRouteAttr":                        "syscall",
	"syscall.ParseRoutingMessage":                          "syscall",
	"syscall.ParseRoutingSockaddr":                         "syscall",
	"syscall.ParseSocketControlMessage":                    "syscall",
	"syscall.ParseUnixCredentials":                         "syscall",
	"syscall.ParseUnixRights":                              "syscall",
	"syscall.PathMax":                                      "syscall",
	"syscall.Pathconf":                                     "syscall",
	"syscall.Pause":                                        "syscall",
	"syscall.Pipe":                                         "syscall",
	"syscall.Pipe2":                                        "syscall",
	"syscall.PivotRoot":                                    "syscall",
	"syscall.PostQueuedCompletionStatus":                   "syscall",
	"syscall.Pread":                                        "syscall",
	"syscall.Proc":                                         "syscall",
	"syscall.ProcAttr":                                     "syscall",
	"syscall.ProcessInformation":                           "syscall",
	"syscall.Protoent":                                     "syscall",
	"syscall.PtraceAttach":                                 "syscall",
	"syscall.PtraceCont":                                   "syscall",
	"syscall.PtraceDetach":                                 "syscall",
	"syscall.PtraceGetEventMsg":                            "syscall",
	"syscall.PtraceGetRegs":                                "syscall",
	"syscall.PtracePeekData":                               "syscall",
	"syscall.PtracePeekText":                               "syscall",
	"syscall.PtracePokeData":                               "syscall",
	"syscall.PtracePokeText":                               "syscall",
	"syscall.PtraceRegs":                                   "syscall",
	"syscall.PtraceSetOptions":                             "syscall",
	"syscall.PtraceSetRegs":                                "syscall",
	"syscall.PtraceSingleStep":                             "syscall",
	"syscall.PtraceSyscall":                                "syscall",
	"syscall.Pwrite":                                       "syscall",
	"syscall.REG_BINARY":                                   "syscall",
	"syscall.REG_DWORD":                                    "syscall",
	"syscall.REG_DWORD_BIG_ENDIAN":                         "syscall",
	"syscall.REG_DWORD_LITTLE_ENDIAN":                      "syscall",
	"syscall.REG_EXPAND_SZ":                                "syscall",
	"syscall.REG_FULL_RESOURCE_DESCRIPTOR":                 "syscall",
	"syscall.REG_LINK":                                     "syscall",
	"syscall.REG_MULTI_SZ":                                 "syscall",
	"syscall.REG_NONE":                                     "syscall",
	"syscall.REG_QWORD":                                    "syscall",
	"syscall.REG_QWORD_LITTLE_ENDIAN":                      "syscall",
	"syscall.REG_RESOURCE_LIST":                            "syscall",
	"syscall.REG_RESOURCE_REQUIREMENTS_LIST":               "syscall",
	"syscall.REG_SZ":                                       "syscall",
	"syscall.RLIMIT_AS":                                    "syscall",
	"syscall.RLIMIT_CORE":                                  "syscall",
	"syscall.RLIMIT_CPU":                                   "syscall",
	"syscall.RLIMIT_DATA":                                  "syscall",
	"syscall.RLIMIT_FSIZE":                                 "syscall",
	"syscall.RLIMIT_NOFILE":                                "syscall",
	"syscall.RLIMIT_STACK":                                 "syscall",
	"syscall.RLIM_INFINITY":                                "syscall",
	"syscall.RTAX_ADVMSS":                                  "syscall",
	"syscall.RTAX_AUTHOR":                                  "syscall",
	"syscall.RTAX_BRD":                                     "syscall",
	"syscall.RTAX_CWND":                                    "syscall",
	"syscall.RTAX_DST":                                     "syscall",
	"syscall.RTAX_FEATURES":                                "syscall",
	"syscall.RTAX_FEATURE_ALLFRAG":                         "syscall",
	"syscall.RTAX_FEATURE_ECN":                             "syscall",
	"syscall.RTAX_FEATURE_SACK":                            "syscall",
	"syscall.RTAX_FEATURE_TIMESTAMP":                       "syscall",
	"syscall.RTAX_GATEWAY":                                 "syscall",
	"syscall.RTAX_GENMASK":                                 "syscall",
	"syscall.RTAX_HOPLIMIT":                                "syscall",
	"syscall.RTAX_IFA":                                     "syscall",
	"syscall.RTAX_IFP":                                     "syscall",
	"syscall.RTAX_INITCWND":                                "syscall",
	"syscall.RTAX_INITRWND":                                "syscall",
	"syscall.RTAX_LABEL":                                   "syscall",
	"syscall.RTAX_LOCK":                                    "syscall",
	"syscall.RTAX_MAX":                                     "syscall",
	"syscall.RTAX_MTU":                                     "syscall",
	"syscall.RTAX_NETMASK":                                 "syscall",
	"syscall.RTAX_REORDERING":                              "syscall",
	"syscall.RTAX_RTO_MIN":                                 "syscall",
	"syscall.RTAX_RTT":                                     "syscall",
	"syscall.RTAX_RTTVAR":                                  "syscall",
	"syscall.RTAX_SRC":                                     "syscall",
	"syscall.RTAX_SRCMASK":                                 "syscall",
	"syscall.RTAX_SSTHRESH":                                "syscall",
	"syscall.RTAX_TAG":                                     "syscall",
	"syscall.RTAX_UNSPEC":                                  "syscall",
	"syscall.RTAX_WINDOW":                                  "syscall",
	"syscall.RTA_ALIGNTO":                                  "syscall",
	"syscall.RTA_AUTHOR":                                   "syscall",
	"syscall.RTA_BRD":                                      "syscall",
	"syscall.RTA_CACHEINFO":                                "syscall",
	"syscall.RTA_DST":                                      "syscall",
	"syscall.RTA_FLOW":                                     "syscall",
	"syscall.RTA_GATEWAY":                                  "syscall",
	"syscall.RTA_GENMASK":                                  "syscall",
	"syscall.RTA_IFA":                                      "syscall",
	"syscall.RTA_IFP":                                      "syscall",
	"syscall.RTA_IIF":                                      "syscall",
	"syscall.RTA_LABEL":                                    "syscall",
	"syscall.RTA_MAX":                                      "syscall",
	"syscall.RTA_METRICS":                                  "syscall",
	"syscall.RTA_MULTIPATH":                                "syscall",
	"syscall.RTA_NETMASK":                                  "syscall",
	"syscall.RTA_OIF":                                      "syscall",
	"syscall.RTA_PREFSRC":                                  "syscall",
	"syscall.RTA_PRIORITY":                                 "syscall",
	"syscall.RTA_SRC":                                      "syscall",
	"syscall.RTA_SRCMASK":                                  "syscall",
	"syscall.RTA_TABLE":                                    "syscall",
	"syscall.RTA_TAG":                                      "syscall",
	"syscall.RTA_UNSPEC":                                   "syscall",
	"syscall.RTCF_DIRECTSRC":                               "syscall",
	"syscall.RTCF_DOREDIRECT":                              "syscall",
	"syscall.RTCF_LOG":                                     "syscall",
	"syscall.RTCF_MASQ":                                    "syscall",
	"syscall.RTCF_NAT":                                     "syscall",
	"syscall.RTCF_VALVE":                                   "syscall",
	"syscall.RTF_ADDRCLASSMASK":                            "syscall",
	"syscall.RTF_ADDRCONF":                                 "syscall",
	"syscall.RTF_ALLONLINK":                                "syscall",
	"syscall.RTF_ANNOUNCE":                                 "syscall",
	"syscall.RTF_BLACKHOLE":                                "syscall",
	"syscall.RTF_BROADCAST":                                "syscall",
	"syscall.RTF_CACHE":                                    "syscall",
	"syscall.RTF_CLONED":                                   "syscall",
	"syscall.RTF_CLONING":                                  "syscall",
	"syscall.RTF_CONDEMNED":                                "syscall",
	"syscall.RTF_DEFAULT":                                  "syscall",
	"syscall.RTF_DELCLONE":                                 "syscall",
	"syscall.RTF_DONE":                                     "syscall",
	"syscall.RTF_DYNAMIC":                                  "syscall",
	"syscall.RTF_FLOW":                                     "syscall",
	"syscall.RTF_FMASK":                                    "syscall",
	"syscall.RTF_GATEWAY":                                  "syscall",
	"syscall.RTF_HOST":                                     "syscall",
	"syscall.RTF_IFREF":                                    "syscall",
	"syscall.RTF_IFSCOPE":                                  "syscall",
	"syscall.RTF_INTERFACE":                                "syscall",
	"syscall.RTF_IRTT":                                     "syscall",
	"syscall.RTF_LINKRT":                                   "syscall",
	"syscall.RTF_LLDATA":                                   "syscall",
	"syscall.RTF_LLINFO":                                   "syscall",
	"syscall.RTF_LOCAL":                                    "syscall",
	"syscall.RTF_MASK":                                     "syscall",
	"syscall.RTF_MODIFIED":                                 "syscall",
	"syscall.RTF_MPATH":                                    "syscall",
	"syscall.RTF_MPLS":                                     "syscall",
	"syscall.RTF_MSS":                                      "syscall",
	"syscall.RTF_MTU":                                      "syscall",
	"syscall.RTF_MULTICAST":                                "syscall",
	"syscall.RTF_NAT":                                      "syscall",
	"syscall.RTF_NOFORWARD":                                "syscall",
	"syscall.RTF_NONEXTHOP":                                "syscall",
	"syscall.RTF_NOPMTUDISC":                               "syscall",
	"syscall.RTF_PERMANENT_ARP":                            "syscall",
	"syscall.RTF_PINNED":                                   "syscall",
	"syscall.RTF_POLICY":                                   "syscall",
	"syscall.RTF_PRCLONING":                                "syscall",
	"syscall.RTF_PROTO1":                                   "syscall",
	"syscall.RTF_PROTO2":                                   "syscall",
	"syscall.RTF_PROTO3":                                   "syscall",
	"syscall.RTF_REINSTATE":                                "syscall",
	"syscall.RTF_REJECT":                                   "syscall",
	"syscall.RTF_RNH_LOCKED":                               "syscall",
	"syscall.RTF_SOURCE":                                   "syscall",
	"syscall.RTF_SRC":                                      "syscall",
	"syscall.RTF_STATIC":                                   "syscall",
	"syscall.RTF_STICKY":                                   "syscall",
	"syscall.RTF_THROW":                                    "syscall",
	"syscall.RTF_TUNNEL":                                   "syscall",
	"syscall.RTF_UP":                                       "syscall",
	"syscall.RTF_USETRAILERS":                              "syscall",
	"syscall.RTF_WASCLONED":                                "syscall",
	"syscall.RTF_WINDOW":                                   "syscall",
	"syscall.RTF_XRESOLVE":                                 "syscall",
	"syscall.RTM_ADD":                                      "syscall",
	"syscall.RTM_BASE":                                     "syscall",
	"syscall.RTM_CHANGE":                                   "syscall",
	"syscall.RTM_CHGADDR":                                  "syscall",
	"syscall.RTM_DELACTION":                                "syscall",
	"syscall.RTM_DELADDR":                                  "syscall",
	"syscall.RTM_DELADDRLABEL":                             "syscall",
	"syscall.RTM_DELETE":                                   "syscall",
	"syscall.RTM_DELLINK":                                  "syscall",
	"syscall.RTM_DELMADDR":                                 "syscall",
	"syscall.RTM_DELNEIGH":                                 "syscall",
	"syscall.RTM_DELQDISC":                                 "syscall",
	"syscall.RTM_DELROUTE":                                 "syscall",
	"syscall.RTM_DELRULE":                                  "syscall",
	"syscall.RTM_DELTCLASS":                                "syscall",
	"syscall.RTM_DELTFILTER":                               "syscall",
	"syscall.RTM_DESYNC":                                   "syscall",
	"syscall.RTM_F_CLONED":                                 "syscall",
	"syscall.RTM_F_EQUALIZE":                               "syscall",
	"syscall.RTM_F_NOTIFY":                                 "syscall",
	"syscall.RTM_F_PREFIX":                                 "syscall",
	"syscall.RTM_GET":                                      "syscall",
	"syscall.RTM_GET2":                                     "syscall",
	"syscall.RTM_GETACTION":                                "syscall",
	"syscall.RTM_GETADDR":                                  "syscall",
	"syscall.RTM_GETADDRLABEL":                             "syscall",
	"syscall.RTM_GETANYCAST":                               "syscall",
	"syscall.RTM_GETDCB":                                   "syscall",
	"syscall.RTM_GETLINK":                                  "syscall",
	"syscall.RTM_GETMULTICAST":                             "syscall",
	"syscall.RTM_GETNEIGH":                                 "syscall",
	"syscall.RTM_GETNEIGHTBL":                              "syscall",
	"syscall.RTM_GETQDISC":                                 "syscall",
	"syscall.RTM_GETROUTE":                                 "syscall",
	"syscall.RTM_GETRULE":                                  "syscall",
	"syscall.RTM_GETTCLASS":                                "syscall",
	"syscall.RTM_GETTFILTER":                               "syscall",
	"syscall.RTM_IEEE80211":                                "syscall",
	"syscall.RTM_IFANNOUNCE":                               "syscall",
	"syscall.RTM_IFINFO":                                   "syscall",
	"syscall.RTM_IFINFO2":                                  "syscall",
	"syscall.RTM_LLINFO_UPD":                               "syscall",
	"syscall.RTM_LOCK":                                     "syscall",
	"syscall.RTM_LOSING":                                   "syscall",
	"syscall.RTM_MAX":                                      "syscall",
	"syscall.RTM_MAXSIZE":                                  "syscall",
	"syscall.RTM_MISS":                                     "syscall",
	"syscall.RTM_NEWACTION":                                "syscall",
	"syscall.RTM_NEWADDR":                                  "syscall",
	"syscall.RTM_NEWADDRLABEL":                             "syscall",
	"syscall.RTM_NEWLINK":                                  "syscall",
	"syscall.RTM_NEWMADDR":                                 "syscall",
	"syscall.RTM_NEWMADDR2":                                "syscall",
	"syscall.RTM_NEWNDUSEROPT":                             "syscall",
	"syscall.RTM_NEWNEIGH":                                 "syscall",
	"syscall.RTM_NEWNEIGHTBL":                              "syscall",
	"syscall.RTM_NEWPREFIX":                                "syscall",
	"syscall.RTM_NEWQDISC":                                 "syscall",
	"syscall.RTM_NEWROUTE":                                 "syscall",
	"syscall.RTM_NEWRULE":                                  "syscall",
	"syscall.RTM_NEWTCLASS":                                "syscall",
	"syscall.RTM_NEWTFILTER":                               "syscall",
	"syscall.RTM_NR_FAMILIES":                              "syscall",
	"syscall.RTM_NR_MSGTYPES":                              "syscall",
	"syscall.RTM_OIFINFO":                                  "syscall",
	"syscall.RTM_OLDADD":                                   "syscall",
	"syscall.RTM_OLDDEL":                                   "syscall",
	"syscall.RTM_OOIFINFO":                                 "syscall",
	"syscall.RTM_REDIRECT":                                 "syscall",
	"syscall.RTM_RESOLVE":                                  "syscall",
	"syscall.RTM_RTTUNIT":                                  "syscall",
	"syscall.RTM_SETDCB":                                   "syscall",
	"syscall.RTM_SETGATE":                                  "syscall",
	"syscall.RTM_SETLINK":                                  "syscall",
	"syscall.RTM_SETNEIGHTBL":                              "syscall",
	"syscall.RTM_VERSION":                                  "syscall",
	"syscall.RTNH_ALIGNTO":                                 "syscall",
	"syscall.RTNH_F_DEAD":                                  "syscall",
	"syscall.RTNH_F_ONLINK":                                "syscall",
	"syscall.RTNH_F_PERVASIVE":                             "syscall",
	"syscall.RTNLGRP_IPV4_IFADDR":                          "syscall",
	"syscall.RTNLGRP_IPV4_MROUTE":                          "syscall",
	"syscall.RTNLGRP_IPV4_ROUTE":                           "syscall",
	"syscall.RTNLGRP_IPV4_RULE":                            "syscall",
	"syscall.RTNLGRP_IPV6_IFADDR":                          "syscall",
	"syscall.RTNLGRP_IPV6_IFINFO":                          "syscall",
	"syscall.RTNLGRP_IPV6_MROUTE":                          "syscall",
	"syscall.RTNLGRP_IPV6_PREFIX":                          "syscall",
	"syscall.RTNLGRP_IPV6_ROUTE":                           "syscall",
	"syscall.RTNLGRP_IPV6_RULE":                            "syscall",
	"syscall.RTNLGRP_LINK":                                 "syscall",
	"syscall.RTNLGRP_ND_USEROPT":                           "syscall",
	"syscall.RTNLGRP_NEIGH":                                "syscall",
	"syscall.RTNLGRP_NONE":                                 "syscall",
	"syscall.RTNLGRP_NOTIFY":                               "syscall",
	"syscall.RTNLGRP_TC":                                   "syscall",
	"syscall.RTN_ANYCAST":                                  "syscall",
	"syscall.RTN_BLACKHOLE":                                "syscall",
	"syscall.RTN_BROADCAST":                                "syscall",
	"syscall.RTN_LOCAL":                                    "syscall",
	"syscall.RTN_MAX":                                      "syscall",
	"syscall.RTN_MULTICAST":                                "syscall",
	"syscall.RTN_NAT":                                      "syscall",
	"syscall.RTN_PROHIBIT":                                 "syscall",
	"syscall.RTN_THROW":                                    "syscall",
	"syscall.RTN_UNICAST":                                  "syscall",
	"syscall.RTN_UNREACHABLE":                              "syscall",
	"syscall.RTN_UNSPEC":                                   "syscall",
	"syscall.RTN_XRESOLVE":                                 "syscall",
	"syscall.RTPROT_BIRD":                                  "syscall",
	"syscall.RTPROT_BOOT":                                  "syscall",
	"syscall.RTPROT_DHCP":                                  "syscall",
	"syscall.RTPROT_DNROUTED":                              "syscall",
	"syscall.RTPROT_GATED":                                 "syscall",
	"syscall.RTPROT_KERNEL":                                "syscall",
	"syscall.RTPROT_MRT":                                   "syscall",
	"syscall.RTPROT_NTK":                                   "syscall",
	"syscall.RTPROT_RA":                                    "syscall",
	"syscall.RTPROT_REDIRECT":                              "syscall",
	"syscall.RTPROT_STATIC":                                "syscall",
	"syscall.RTPROT_UNSPEC":                                "syscall",
	"syscall.RTPROT_XORP":                                  "syscall",
	"syscall.RTPROT_ZEBRA":                                 "syscall",
	"syscall.RTV_EXPIRE":                                   "syscall",
	"syscall.RTV_HOPCOUNT":                                 "syscall",
	"syscall.RTV_MTU":                                      "syscall",
	"syscall.RTV_RPIPE":                                    "syscall",
	"syscall.RTV_RTT":                                      "syscall",
	"syscall.RTV_RTTVAR":                                   "syscall",
	"syscall.RTV_SPIPE":                                    "syscall",
	"syscall.RTV_SSTHRESH":                                 "syscall",
	"syscall.RTV_WEIGHT":                                   "syscall",
	"syscall.RT_CACHING_CONTEXT":                           "syscall",
	"syscall.RT_CLASS_DEFAULT":                             "syscall",
	"syscall.RT_CLASS_LOCAL":                               "syscall",
	"syscall.RT_CLASS_MAIN":                                "syscall",
	"syscall.RT_CLASS_MAX":                                 "syscall",
	"syscall.RT_CLASS_UNSPEC":                              "syscall",
	"syscall.RT_DEFAULT_FIB":                               "syscall",
	"syscall.RT_NORTREF":                                   "syscall",
	"syscall.RT_SCOPE_HOST":                                "syscall",
	"syscall.RT_SCOPE_LINK":                                "syscall",
	"syscall.RT_SCOPE_NOWHERE":                             "syscall",
	"syscall.RT_SCOPE_SITE":                                "syscall",
	"syscall.RT_SCOPE_UNIVERSE":                            "syscall",
	"syscall.RT_TABLEID_MAX":                               "syscall",
	"syscall.RT_TABLE_COMPAT":                              "syscall",
	"syscall.RT_TABLE_DEFAULT":                             "syscall",
	"syscall.RT_TABLE_LOCAL":                               "syscall",
	"syscall.RT_TABLE_MAIN":                                "syscall",
	"syscall.RT_TABLE_MAX":                                 "syscall",
	"syscall.RT_TABLE_UNSPEC":                              "syscall",
	"syscall.RUSAGE_CHILDREN":                              "syscall",
	"syscall.RUSAGE_SELF":                                  "syscall",
	"syscall.RUSAGE_THREAD":                                "syscall",
	"syscall.Radvisory_t":                                  "syscall",
	"syscall.RawSockaddr":                                  "syscall",
	"syscall.RawSockaddrAny":                               "syscall",
	"syscall.RawSockaddrDatalink":                          "syscall",
	"syscall.RawSockaddrInet4":                             "syscall",
	"syscall.RawSockaddrInet6":                             "syscall",
	"syscall.RawSockaddrLinklayer":                         "syscall",
	"syscall.RawSockaddrNetlink":                           "syscall",
	"syscall.RawSockaddrUnix":                              "syscall",
	"syscall.RawSyscall":                                   "syscall",
	"syscall.RawSyscall6":                                  "syscall",
	"syscall.Read":                                         "syscall",
	"syscall.ReadConsole":                                  "syscall",
	"syscall.ReadDirectoryChanges":                         "syscall",
	"syscall.ReadDirent":                                   "syscall",
	"syscall.ReadFile":                                     "syscall",
	"syscall.Readlink":                                     "syscall",
	"syscall.Reboot":                                       "syscall",
	"syscall.Recvfrom":                                     "syscall",
	"syscall.Recvmsg":                                      "syscall",
	"syscall.RegCloseKey":                                  "syscall",
	"syscall.RegEnumKeyEx":                                 "syscall",
	"syscall.RegOpenKeyEx":                                 "syscall",
	"syscall.RegQueryInfoKey":                              "syscall",
	"syscall.RegQueryValueEx":                              "syscall",
	"syscall.RemoveDirectory":                              "syscall",
	"syscall.Removexattr":                                  "syscall",
	"syscall.Rename":                                       "syscall",
	"syscall.Renameat":                                     "syscall",
	"syscall.Revoke":                                       "syscall",
	"syscall.Rlimit":                                       "syscall",
	"syscall.Rmdir":                                        "syscall",
	"syscall.RouteMessage":                                 "syscall",
	"syscall.RouteRIB":                                     "syscall",
	"syscall.RtAttr":                                       "syscall",
	"syscall.RtGenmsg":                                     "syscall",
	"syscall.RtMetrics":                                    "syscall",
	"syscall.RtMsg":                                        "syscall",
	"syscall.RtMsghdr":                                     "syscall",
	"syscall.RtNexthop":                                    "syscall",
	"syscall.Rusage":                                       "syscall",
	"syscall.SCM_BINTIME":                                  "syscall",
	"syscall.SCM_CREDENTIALS":                              "syscall",
	"syscall.SCM_CREDS":                                    "syscall",
	"syscall.SCM_RIGHTS":                                   "syscall",
	"syscall.SCM_TIMESTAMP":                                "syscall",
	"syscall.SCM_TIMESTAMPING":                             "syscall",
	"syscall.SCM_TIMESTAMPNS":                              "syscall",
	"syscall.SCM_TIMESTAMP_MONOTONIC":                      "syscall",
	"syscall.SHUT_RD":                                      "syscall",
	"syscall.SHUT_RDWR":                                    "syscall",
	"syscall.SHUT_WR":                                      "syscall",
	"syscall.SID":                                          "syscall",
	"syscall.SIDAndAttributes":                             "syscall",
	"syscall.SIGABRT":                                      "syscall",
	"syscall.SIGALRM":                                      "syscall",
	"syscall.SIGBUS":                                       "syscall",
	"syscall.SIGCHLD":                                      "syscall",
	"syscall.SIGCLD":                                       "syscall",
	"syscall.SIGCONT":                                      "syscall",
	"syscall.SIGEMT":                                       "syscall",
	"syscall.SIGFPE":                                       "syscall",
	"syscall.SIGHUP":                                       "syscall",
	"syscall.SIGILL":                                       "syscall",
	"syscall.SIGINFO":                                      "syscall",
	"syscall.SIGINT":                                       "syscall",
	"syscall.SIGIO":                                        "syscall",
	"syscall.SIGIOT":                                       "syscall",
	"syscall.SIGKILL":                                      "syscall",
	"syscall.SIGLIBRT":                                     "syscall",
	"syscall.SIGLWP":                                       "syscall",
	"syscall.SIGPIPE":                                      "syscall",
	"syscall.SIGPOLL":                                      "syscall",
	"syscall.SIGPROF":                                      "syscall",
	"syscall.SIGPWR":                                       "syscall",
	"syscall.SIGQUIT":                                      "syscall",
	"syscall.SIGSEGV":                                      "syscall",
	"syscall.SIGSTKFLT":                                    "syscall",
	"syscall.SIGSTOP":                                      "syscall",
	"syscall.SIGSYS":                                       "syscall",
	"syscall.SIGTERM":                                      "syscall",
	"syscall.SIGTHR":                                       "syscall",
	"syscall.SIGTRAP":                                      "syscall",
	"syscall.SIGTSTP":                                      "syscall",
	"syscall.SIGTTIN":                                      "syscall",
	"syscall.SIGTTOU":                                      "syscall",
	"syscall.SIGUNUSED":                                    "syscall",
	"syscall.SIGURG":                                       "syscall",
	"syscall.SIGUSR1":                                      "syscall",
	"syscall.SIGUSR2":                                      "syscall",
	"syscall.SIGVTALRM":                                    "syscall",
	"syscall.SIGWINCH":                                     "syscall",
	"syscall.SIGXCPU":                                      "syscall",
	"syscall.SIGXFSZ":                                      "syscall",
	"syscall.SIOCADDDLCI":                                  "syscall",
	"syscall.SIOCADDMULTI":                                 "syscall",
	"syscall.SIOCADDRT":                                    "syscall",
	"syscall.SIOCAIFADDR":                                  "syscall",
	"syscall.SIOCAIFGROUP":                                 "syscall",
	"syscall.SIOCALIFADDR":                                 "syscall",
	"syscall.SIOCARPIPLL":                                  "syscall",
	"syscall.SIOCATMARK":                                   "syscall",
	"syscall.SIOCAUTOADDR":                                 "syscall",
	"syscall.SIOCAUTONETMASK":                              "syscall",
	"syscall.SIOCBRDGADD":                                  "syscall",
	"syscall.SIOCBRDGADDS":                                 "syscall",
	"syscall.SIOCBRDGARL":                                  "syscall",
	"syscall.SIOCBRDGDADDR":                                "syscall",
	"syscall.SIOCBRDGDEL":                                  "syscall",
	"syscall.SIOCBRDGDELS":                                 "syscall",
	"syscall.SIOCBRDGFLUSH":                                "syscall",
	"syscall.SIOCBRDGFRL":                                  "syscall",
	"syscall.SIOCBRDGGCACHE":                               "syscall",
	"syscall.SIOCBRDGGFD":                                  "syscall",
	"syscall.SIOCBRDGGHT":                                  "syscall",
	"syscall.SIOCBRDGGIFFLGS":                              "syscall",
	"syscall.SIOCBRDGGMA":                                  "syscall",
	"syscall.SIOCBRDGGPARAM":                               "syscall",
	"syscall.SIOCBRDGGPRI":                                 "syscall",
	"syscall.SIOCBRDGGRL":                                  "syscall",
	"syscall.SIOCBRDGGSIFS":                                "syscall",
	"syscall.SIOCBRDGGTO":                                  "syscall",
	"syscall.SIOCBRDGIFS":                                  "syscall",
	"syscall.SIOCBRDGRTS":                                  "syscall",
	"syscall.SIOCBRDGSADDR":                                "syscall",
	"syscall.SIOCBRDGSCACHE":                               "syscall",
	"syscall.SIOCBRDGSFD":                                  "syscall",
	"syscall.SIOCBRDGSHT":                                  "syscall",
	"syscall.SIOCBRDGSIFCOST":                              "syscall",
	"syscall.SIOCBRDGSIFFLGS":                              "syscall",
	"syscall.SIOCBRDGSIFPRIO":                              "syscall",
	"syscall.SIOCBRDGSMA":                                  "syscall",
	"syscall.SIOCBRDGSPRI":                                 "syscall",
	"syscall.SIOCBRDGSPROTO":                               "syscall",
	"syscall.SIOCBRDGSTO":                                  "syscall",
	"syscall.SIOCBRDGSTXHC":                                "syscall",
	"syscall.SIOCDARP":                                     "syscall",
	"syscall.SIOCDELDLCI":                                  "syscall",
	"syscall.SIOCDELMULTI":                                 "syscall",
	"syscall.SIOCDELRT":                                    "syscall",
	"syscall.SIOCDEVPRIVATE":                               "syscall",
	"syscall.SIOCDIFADDR":                                  "syscall",
	"syscall.SIOCDIFGROUP":                                 "syscall",
	"syscall.SIOCDIFPHYADDR":                               "syscall",
	"syscall.SIOCDLIFADDR":                                 "syscall",
	"syscall.SIOCDRARP":                                    "syscall",
	"syscall.SIOCGARP":                                     "syscall",
	"syscall.SIOCGDRVSPEC":                                 "syscall",
	"syscall.SIOCGETKALIVE":                                "syscall",
	"syscall.SIOCGETLABEL":                                 "syscall",
	"syscall.SIOCGETPFLOW":                                 "syscall",
	"syscall.SIOCGETPFSYNC":                                "syscall",
	"syscall.SIOCGETSGCNT":                                 "syscall",
	"syscall.SIOCGETVIFCNT":                                "syscall",
	"syscall.SIOCGETVLAN":                                  "syscall",
	"syscall.SIOCGHIWAT":                                   "syscall",
	"syscall.SIOCGIFADDR":                                  "syscall",
	"syscall.SIOCGIFADDRPREF":                              "syscall",
	"syscall.SIOCGIFALIAS":                                 "syscall",
	"syscall.SIOCGIFALTMTU":                                "syscall",
	"syscall.SIOCGIFASYNCMAP":                              "syscall",
	"syscall.SIOCGIFBOND":                                  "syscall",
	"syscall.SIOCGIFBR":                                    "syscall",
	"syscall.SIOCGIFBRDADDR":                               "syscall",
	"syscall.SIOCGIFCAP":                                   "syscall",
	"syscall.SIOCGIFCONF":                                  "syscall",
	"syscall.SIOCGIFCOUNT":                                 "syscall",
	"syscall.SIOCGIFDATA":                                  "syscall",
	"syscall.SIOCGIFDESCR":                                 "syscall",
	"syscall.SIOCGIFDEVMTU":                                "syscall",
	"syscall.SIOCGIFDLT":                                   "syscall",
	"syscall.SIOCGIFDSTADDR":                               "syscall",
	"syscall.SIOCGIFENCAP":                                 "syscall",
	"syscall.SIOCGIFFIB":                                   "syscall",
	"syscall.SIOCGIFFLAGS":                                 "syscall",
	"syscall.SIOCGIFGATTR":                                 "syscall",
	"syscall.SIOCGIFGENERIC":                               "syscall",
	"syscall.SIOCGIFGMEMB":                                 "syscall",
	"syscall.SIOCGIFGROUP":                                 "syscall",
	"syscall.SIOCGIFHWADDR":                                "syscall",
	"syscall.SIOCGIFINDEX":                                 "syscall",
	"syscall.SIOCGIFKPI":                                   "syscall",
	"syscall.SIOCGIFMAC":                                   "syscall",
	"syscall.SIOCGIFMAP":                                   "syscall",
	"syscall.SIOCGIFMEDIA":                                 "syscall",
	"syscall.SIOCGIFMEM":                                   "syscall",
	"syscall.SIOCGIFMETRIC":                                "syscall",
	"syscall.SIOCGIFMTU":                                   "syscall",
	"syscall.SIOCGIFNAME":                                  "syscall",
	"syscall.SIOCGIFNETMASK":                               "syscall",
	"syscall.SIOCGIFPDSTADDR":                              "syscall",
	"syscall.SIOCGIFPFLAGS":                                "syscall",
	"syscall.SIOCGIFPHYS":                                  "syscall",
	"syscall.SIOCGIFPRIORITY":                              "syscall",
	"syscall.SIOCGIFPSRCADDR":                              "syscall",
	"syscall.SIOCGIFRDOMAIN":                               "syscall",
	"syscall.SIOCGIFRTLABEL":                               "syscall",
	"syscall.SIOCGIFSLAVE":                                 "syscall",
	"syscall.SIOCGIFSTATUS":                                "syscall",
	"syscall.SIOCGIFTIMESLOT":                              "syscall",
	"syscall.SIOCGIFTXQLEN":                                "syscall",
	"syscall.SIOCGIFVLAN":                                  "syscall",
	"syscall.SIOCGIFWAKEFLAGS":                             "syscall",
	"syscall.SIOCGIFXFLAGS":                                "syscall",
	"syscall.SIOCGLIFADDR":                                 "syscall",
	"syscall.SIOCGLIFPHYADDR":                              "syscall",
	"syscall.SIOCGLIFPHYRTABLE":                            "syscall",
	"syscall.SIOCGLINKSTR":                                 "syscall",
	"syscall.SIOCGLOWAT":                                   "syscall",
	"syscall.SIOCGPGRP":                                    "syscall",
	"syscall.SIOCGPRIVATE_0":                               "syscall",
	"syscall.SIOCGPRIVATE_1":                               "syscall",
	"syscall.SIOCGRARP":                                    "syscall",
	"syscall.SIOCGSTAMP":                                   "syscall",
	"syscall.SIOCGSTAMPNS":                                 "syscall",
	"syscall.SIOCGVH":                                      "syscall",
	"syscall.SIOCIFCREATE":                                 "syscall",
	"syscall.SIOCIFCREATE2":                                "syscall",
	"syscall.SIOCIFDESTROY":                                "syscall",
	"syscall.SIOCIFGCLONERS":                               "syscall",
	"syscall.SIOCINITIFADDR":                               "syscall",
	"syscall.SIOCPROTOPRIVATE":                             "syscall",
	"syscall.SIOCRSLVMULTI":                                "syscall",
	"syscall.SIOCRTMSG":                                    "syscall",
	"syscall.SIOCSARP":                                     "syscall",
	"syscall.SIOCSDRVSPEC":                                 "syscall",
	"syscall.SIOCSETKALIVE":                                "syscall",
	"syscall.SIOCSETLABEL":                                 "syscall",
	"syscall.SIOCSETPFLOW":                                 "syscall",
	"syscall.SIOCSETPFSYNC":                                "syscall",
	"syscall.SIOCSETVLAN":                                  "syscall",
	"syscall.SIOCSHIWAT":                                   "syscall",
	"syscall.SIOCSIFADDR":                                  "syscall",
	"syscall.SIOCSIFADDRPREF":                              "syscall",
	"syscall.SIOCSIFALTMTU":                                "syscall",
	"syscall.SIOCSIFASYNCMAP":                              "syscall",
	"syscall.SIOCSIFBOND":                                  "syscall",
	"syscall.SIOCSIFBR":                                    "syscall",
	"syscall.SIOCSIFBRDADDR":                               "syscall",
	"syscall.SIOCSIFCAP":                                   "syscall",
	"syscall.SIOCSIFDESCR":                                 "syscall",
	"syscall.SIOCSIFDSTADDR":                               "syscall",
	"syscall.SIOCSIFENCAP":                                 "syscall",
	"syscall.SIOCSIFFIB":                                   "syscall",
	"syscall.SIOCSIFFLAGS":                                 "syscall",
	"syscall.SIOCSIFGATTR":                                 "syscall",
	"syscall.SIOCSIFGENERIC":                               "syscall",
	"syscall.SIOCSIFHWADDR":                                "syscall",
	"syscall.SIOCSIFHWBROADCAST":                           "syscall",
	"syscall.SIOCSIFKPI":                                   "syscall",
	"syscall.SIOCSIFLINK":                                  "syscall",
	"syscall.SIOCSIFLLADDR":                                "syscall",
	"syscall.SIOCSIFMAC":                                   "syscall",
	"syscall.SIOCSIFMAP":                                   "syscall",
	"syscall.SIOCSIFMEDIA":                                 "syscall",
	"syscall.SIOCSIFMEM":                                   "syscall",
	"syscall.SIOCSIFMETRIC":                                "syscall",
	"syscall.SIOCSIFMTU":                                   "syscall",
	"syscall.SIOCSIFNAME":                                  "syscall",
	"syscall.SIOCSIFNETMASK":                               "syscall",
	"syscall.SIOCSIFPFLAGS":                                "syscall",
	"syscall.SIOCSIFPHYADDR":                               "syscall",
	"syscall.SIOCSIFPHYS":                                  "syscall",
	"syscall.SIOCSIFPRIORITY":                              "syscall",
	"syscall.SIOCSIFRDOMAIN":                               "syscall",
	"syscall.SIOCSIFRTLABEL":                               "syscall",
	"syscall.SIOCSIFRVNET":                                 "syscall",
	"syscall.SIOCSIFSLAVE":                                 "syscall",
	"syscall.SIOCSIFTIMESLOT":                              "syscall",
	"syscall.SIOCSIFTXQLEN":                                "syscall",
	"syscall.SIOCSIFVLAN":                                  "syscall",
	"syscall.SIOCSIFVNET":                                  "syscall",
	"syscall.SIOCSIFXFLAGS":                                "syscall",
	"syscall.SIOCSLIFPHYADDR":                              "syscall",
	"syscall.SIOCSLIFPHYRTABLE":                            "syscall",
	"syscall.SIOCSLINKSTR":                                 "syscall",
	"syscall.SIOCSLOWAT":                                   "syscall",
	"syscall.SIOCSPGRP":                                    "syscall",
	"syscall.SIOCSRARP":                                    "syscall",
	"syscall.SIOCSVH":                                      "syscall",
	"syscall.SIOCZIFDATA":                                  "syscall",
	"syscall.SIO_GET_EXTENSION_FUNCTION_POINTER":           "syscall",
	"syscall.SIO_GET_INTERFACE_LIST":                       "syscall",
	"syscall.SOCK_CLOEXEC":                                 "syscall",
	"syscall.SOCK_DCCP":                                    "syscall",
	"syscall.SOCK_DGRAM":                                   "syscall",
	"syscall.SOCK_FLAGS_MASK":                              "syscall",
	"syscall.SOCK_MAXADDRLEN":                              "syscall",
	"syscall.SOCK_NONBLOCK":                                "syscall",
	"syscall.SOCK_NOSIGPIPE":                               "syscall",
	"syscall.SOCK_PACKET":                                  "syscall",
	"syscall.SOCK_RAW":                                     "syscall",
	"syscall.SOCK_RDM":                                     "syscall",
	"syscall.SOCK_SEQPACKET":                               "syscall",
	"syscall.SOCK_STREAM":                                  "syscall",
	"syscall.SOL_AAL":                                      "syscall",
	"syscall.SOL_ATM":                                      "syscall",
	"syscall.SOL_DECNET":                                   "syscall",
	"syscall.SOL_ICMPV6":                                   "syscall",
	"syscall.SOL_IP":                                       "syscall",
	"syscall.SOL_IPV6":                                     "syscall",
	"syscall.SOL_IRDA":                                     "syscall",
	"syscall.SOL_PACKET":                                   "syscall",
	"syscall.SOL_RAW":                                      "syscall",
	"syscall.SOL_SOCKET":                                   "syscall",
	"syscall.SOL_TCP":                                      "syscall",
	"syscall.SOL_X25":                                      "syscall",
	"syscall.SOMAXCONN":                                    "syscall",
	"syscall.SO_ACCEPTCONN":                                "syscall",
	"syscall.SO_ACCEPTFILTER":                              "syscall",
	"syscall.SO_ATTACH_FILTER":                             "syscall",
	"syscall.SO_BINDANY":                                   "syscall",
	"syscall.SO_BINDTODEVICE":                              "syscall",
	"syscall.SO_BINTIME":                                   "syscall",
	"syscall.SO_BROADCAST":                                 "syscall",
	"syscall.SO_BSDCOMPAT":                                 "syscall",
	"syscall.SO_DEBUG":                                     "syscall",
	"syscall.SO_DETACH_FILTER":                             "syscall",
	"syscall.SO_DOMAIN":                                    "syscall",
	"syscall.SO_DONTROUTE":                                 "syscall",
	"syscall.SO_DONTTRUNC":                                 "syscall",
	"syscall.SO_ERROR":                                     "syscall",
	"syscall.SO_KEEPALIVE":                                 "syscall",
	"syscall.SO_LABEL":                                     "syscall",
	"syscall.SO_LINGER":                                    "syscall",
	"syscall.SO_LINGER_SEC":                                "syscall",
	"syscall.SO_LISTENINCQLEN":                             "syscall",
	"syscall.SO_LISTENQLEN":                                "syscall",
	"syscall.SO_LISTENQLIMIT":                              "syscall",
	"syscall.SO_MARK":                                      "syscall",
	"syscall.SO_NETPROC":                                   "syscall",
	"syscall.SO_NKE":                                       "syscall",
	"syscall.SO_NOADDRERR":                                 "syscall",
	"syscall.SO_NOHEADER":                                  "syscall",
	"syscall.SO_NOSIGPIPE":                                 "syscall",
	"syscall.SO_NOTIFYCONFLICT":                            "syscall",
	"syscall.SO_NO_CHECK":                                  "syscall",
	"syscall.SO_NO_DDP":                                    "syscall",
	"syscall.SO_NO_OFFLOAD":                                "syscall",
	"syscall.SO_NP_EXTENSIONS":                             "syscall",
	"syscall.SO_NREAD":                                     "syscall",
	"syscall.SO_NWRITE":                                    "syscall",
	"syscall.SO_OOBINLINE":                                 "syscall",
	"syscall.SO_OVERFLOWED":                                "syscall",
	"syscall.SO_PASSCRED":                                  "syscall",
	"syscall.SO_PASSSEC":                                   "syscall",
	"syscall.SO_PEERCRED":                                  "syscall",
	"syscall.SO_PEERLABEL":                                 "syscall",
	"syscall.SO_PEERNAME":                                  "syscall",
	"syscall.SO_PEERSEC":                                   "syscall",
	"syscall.SO_PRIORITY":                                  "syscall",
	"syscall.SO_PROTOCOL":                                  "syscall",
	"syscall.SO_PROTOTYPE":                                 "syscall",
	"syscall.SO_RANDOMPORT":                                "syscall",
	"syscall.SO_RCVBUF":                                    "syscall",
	"syscall.SO_RCVBUFFORCE":                               "syscall",
	"syscall.SO_RCVLOWAT":                                  "syscall",
	"syscall.SO_RCVTIMEO":                                  "syscall",
	"syscall.SO_RESTRICTIONS":                              "syscall",
	"syscall.SO_RESTRICT_DENYIN":                           "syscall",
	"syscall.SO_RESTRICT_DENYOUT":                          "syscall",
	"syscall.SO_RESTRICT_DENYSET":                          "syscall",
	"syscall.SO_REUSEADDR":                                 "syscall",
	"syscall.SO_REUSEPORT":                                 "syscall",
	"syscall.SO_REUSESHAREUID":                             "syscall",
	"syscall.SO_RTABLE":                                    "syscall",
	"syscall.SO_RXQ_OVFL":                                  "syscall",
	"syscall.SO_SECURITY_AUTHENTICATION":                   "syscall",
	"syscall.SO_SECURITY_ENCRYPTION_NETWORK":               "syscall",
	"syscall.SO_SECURITY_ENCRYPTION_TRANSPORT":             "syscall",
	"syscall.SO_SETFIB":                                    "syscall",
	"syscall.SO_SNDBUF":                                    "syscall",
	"syscall.SO_SNDBUFFORCE":                               "syscall",
	"syscall.SO_SNDLOWAT":                                  "syscall",
	"syscall.SO_SNDTIMEO":                                  "syscall",
	"syscall.SO_SPLICE":                                    "syscall",
	"syscall.SO_TIMESTAMP":                                 "syscall",
	"syscall.SO_TIMESTAMPING":                              "syscall",
	"syscall.SO_TIMESTAMPNS":                               "syscall",
	"syscall.SO_TIMESTAMP_MONOTONIC":                       "syscall",
	"syscall.SO_TYPE":                                      "syscall",
	"syscall.SO_UPCALLCLOSEWAIT":                           "syscall",
	"syscall.SO_UPDATE_ACCEPT_CONTEXT":                     "syscall",
	"syscall.SO_UPDATE_CONNECT_CONTEXT":                    "syscall",
	"syscall.SO_USELOOPBACK":                               "syscall",
	"syscall.SO_USER_COOKIE":                               "syscall",
	"syscall.SO_WANTMORE":                                  "syscall",
	"syscall.SO_WANTOOBFLAG":                               "syscall",
	"syscall.SSLExtraCertChainPolicyPara":                  "syscall",
	"syscall.STANDARD_RIGHTS_ALL":                          "syscall",
	"syscall.STANDARD_RIGHTS_EXECUTE":                      "syscall",
	"syscall.STANDARD_RIGHTS_READ":                         "syscall",
	"syscall.STANDARD_RIGHTS_REQUIRED":                     "syscall",
	"syscall.STANDARD_RIGHTS_WRITE":                        "syscall",
	"syscall.STARTF_USESHOWWINDOW":                         "syscall",
	"syscall.STARTF_USESTDHANDLES":                         "syscall",
	"syscall.STD_ERROR_HANDLE":                             "syscall",
	"syscall.STD_INPUT_HANDLE":                             "syscall",
	"syscall.STD_OUTPUT_HANDLE":                            "syscall",
	"syscall.SUBLANG_ENGLISH_US":                           "syscall",
	"syscall.SW_FORCEMINIMIZE":                             "syscall",
	"syscall.SW_HIDE":                                      "syscall",
	"syscall.SW_MAXIMIZE":                                  "syscall",
	"syscall.SW_MINIMIZE":                                  "syscall",
	"syscall.SW_NORMAL":                                    "syscall",
	"syscall.SW_RESTORE":                                   "syscall",
	"syscall.SW_SHOW":                                      "syscall",
	"syscall.SW_SHOWDEFAULT":                               "syscall",
	"syscall.SW_SHOWMAXIMIZED":                             "syscall",
	"syscall.SW_SHOWMINIMIZED":                             "syscall",
	"syscall.SW_SHOWMINNOACTIVE":                           "syscall",
	"syscall.SW_SHOWNA":                                    "syscall",
	"syscall.SW_SHOWNOACTIVATE":                            "syscall",
	"syscall.SW_SHOWNORMAL":                                "syscall",
	"syscall.SYNCHRONIZE":                                  "syscall",
	"syscall.SYSCTL_VERSION":                               "syscall",
	"syscall.SYSCTL_VERS_0":                                "syscall",
	"syscall.SYSCTL_VERS_1":                                "syscall",
	"syscall.SYSCTL_VERS_MASK":                             "syscall",
	"syscall.SYS_ABORT2":                                   "syscall",
	"syscall.SYS_ACCEPT":                                   "syscall",
	"syscall.SYS_ACCEPT4":                                  "syscall",
	"syscall.SYS_ACCEPT_NOCANCEL":                          "syscall",
	"syscall.SYS_ACCESS":                                   "syscall",
	"syscall.SYS_ACCESS_EXTENDED":                          "syscall",
	"syscall.SYS_ACCT":                                     "syscall",
	"syscall.SYS_ADD_KEY":                                  "syscall",
	"syscall.SYS_ADD_PROFIL":                               "syscall",
	"syscall.SYS_ADJFREQ":                                  "syscall",
	"syscall.SYS_ADJTIME":                                  "syscall",
	"syscall.SYS_ADJTIMEX":                                 "syscall",
	"syscall.SYS_AFS_SYSCALL":                              "syscall",
	"syscall.SYS_AIO_CANCEL":                               "syscall",
	"syscall.SYS_AIO_ERROR":                                "syscall",
	"syscall.SYS_AIO_FSYNC":                                "syscall",
	"syscall.SYS_AIO_READ":                                 "syscall",
	"syscall.SYS_AIO_RETURN":                               "syscall",
	"syscall.SYS_AIO_SUSPEND":                              "syscall",
	"syscall.SYS_AIO_SUSPEND_NOCANCEL":                     "syscall",
	"syscall.SYS_AIO_WRITE":                                "syscall",
	"syscall.SYS_ALARM":                                    "syscall",
	"syscall.SYS_ARCH_PRCTL":                               "syscall",
	"syscall.SYS_ARM_FADVISE64_64":                         "syscall",
	"syscall.SYS_ARM_SYNC_FILE_RANGE":                      "syscall",
	"syscall.SYS_ATGETMSG":                                 "syscall",
	"syscall.SYS_ATPGETREQ":                                "syscall",
	"syscall.SYS_ATPGETRSP":                                "syscall",
	"syscall.SYS_ATPSNDREQ":                                "syscall",
	"syscall.SYS_ATPSNDRSP":                                "syscall",
	"syscall.SYS_ATPUTMSG":                                 "syscall",
	"syscall.SYS_ATSOCKET":                                 "syscall",
	"syscall.SYS_AUDIT":                                    "syscall",
	"syscall.SYS_AUDITCTL":                                 "syscall",
	"syscall.SYS_AUDITON":                                  "syscall",
	"syscall.SYS_AUDIT_SESSION_JOIN":                       "syscall",
	"syscall.SYS_AUDIT_SESSION_PORT":                       "syscall",
	"syscall.SYS_AUDIT_SESSION_SELF":                       "syscall",
	"syscall.SYS_BDFLUSH":                                  "syscall",
	"syscall.SYS_BIND":                                     "syscall",
	"syscall.SYS_BREAK":                                    "syscall",
	"syscall.SYS_BRK":                                      "syscall",
	"syscall.SYS_BSDTHREAD_CREATE":                         "syscall",
	"syscall.SYS_BSDTHREAD_REGISTER":                       "syscall",
	"syscall.SYS_BSDTHREAD_TERMINATE":                      "syscall",
	"syscall.SYS_CAPGET":                                   "syscall",
	"syscall.SYS_CAPSET":                                   "syscall",
	"syscall.SYS_CAP_ENTER":                                "syscall",
	"syscall.SYS_CAP_FCNTLS_GET":                           "syscall",
	"syscall.SYS_CAP_FCNTLS_LIMIT":                         "syscall",
	"syscall.SYS_CAP_GETMODE":                              "syscall",
	"syscall.SYS_CAP_GETRIGHTS":                            "syscall",
	"syscall.SYS_CAP_IOCTLS_GET":                           "syscall",
	"syscall.SYS_CAP_IOCTLS_LIMIT":                         "syscall",
	"syscall.SYS_CAP_NEW":                                  "syscall",
	"syscall.SYS_CAP_RIGHTS_GET":                           "syscall",
	"syscall.SYS_CAP_RIGHTS_LIMIT":                         "syscall",
	"syscall.SYS_CHDIR":                                    "syscall",
	"syscall.SYS_CHFLAGS":                                  "syscall",
	"syscall.SYS_CHMOD":                                    "syscall",
	"syscall.SYS_CHMOD_EXTENDED":                           "syscall",
	"syscall.SYS_CHOWN":                                    "syscall",
	"syscall.SYS_CHOWN32":                                  "syscall",
	"syscall.SYS_CHROOT":                                   "syscall",
	"syscall.SYS_CHUD":                                     "syscall",
	"syscall.SYS_CLOCK_ADJTIME":                            "syscall",
	"syscall.SYS_CLOCK_GETCPUCLOCKID2":                     "syscall",
	"syscall.SYS_CLOCK_GETRES":                             "syscall",
	"syscall.SYS_CLOCK_GETTIME":                            "syscall",
	"syscall.SYS_CLOCK_NANOSLEEP":                          "syscall",
	"syscall.SYS_CLOCK_SETTIME":                            "syscall",
	"syscall.SYS_CLONE":                                    "syscall",
	"syscall.SYS_CLOSE":                                    "syscall",
	"syscall.SYS_CLOSEFROM":                                "syscall",
	"syscall.SYS_CLOSE_NOCANCEL":                           "syscall",
	"syscall.SYS_CONNECT":                                  "syscall",
	"syscall.SYS_CONNECT_NOCANCEL":                         "syscall",
	"syscall.SYS_COPYFILE":                                 "syscall",
	"syscall.SYS_CPUSET":                                   "syscall",
	"syscall.SYS_CPUSET_GETAFFINITY":                       "syscall",
	"syscall.SYS_CPUSET_GETID":                             "syscall",
	"syscall.SYS_CPUSET_SETAFFINITY":                       "syscall",
	"syscall.SYS_CPUSET_SETID":                             "syscall",
	"syscall.SYS_CREAT":                                    "syscall",
	"syscall.SYS_CREATE_MODULE":                            "syscall",
	"syscall.SYS_CSOPS":                                    "syscall",
	"syscall.SYS_DELETE":                                   "syscall",
	"syscall.SYS_DELETE_MODULE":                            "syscall",
	"syscall.SYS_DUP":                                      "syscall",
	"syscall.SYS_DUP2":                                     "syscall",
	"syscall.SYS_DUP3":                                     "syscall",
	"syscall.SYS_EACCESS":                                  "syscall",
	"syscall.SYS_EPOLL_CREATE":                             "syscall",
	"syscall.SYS_EPOLL_CREATE1":                            "syscall",
	"syscall.SYS_EPOLL_CTL":                                "syscall",
	"syscall.SYS_EPOLL_CTL_OLD":                            "syscall",
	"syscall.SYS_EPOLL_PWAIT":                              "syscall",
	"syscall.SYS_EPOLL_WAIT":                               "syscall",
	"syscall.SYS_EPOLL_WAIT_OLD":                           "syscall",
	"syscall.SYS_EVENTFD":                                  "syscall",
	"syscall.SYS_EVENTFD2":                                 "syscall",
	"syscall.SYS_EXCHANGEDATA":                             "syscall",
	"syscall.SYS_EXECVE":                                   "syscall",
	"syscall.SYS_EXIT":                                     "syscall",
	"syscall.SYS_EXIT_GROUP":                               "syscall",
	"syscall.SYS_EXTATTRCTL":                               "syscall",
	"syscall.SYS_EXTATTR_DELETE_FD":                        "syscall",
	"syscall.SYS_EXTATTR_DELETE_FILE":                      "syscall",
	"syscall.SYS_EXTATTR_DELETE_LINK":                      "syscall",
	"syscall.SYS_EXTATTR_GET_FD":                           "syscall",
	"syscall.SYS_EXTATTR_GET_FILE":                         "syscall",
	"syscall.SYS_EXTATTR_GET_LINK":                         "syscall",
	"syscall.SYS_EXTATTR_LIST_FD":                          "syscall",
	"syscall.SYS_EXTATTR_LIST_FILE":                        "syscall",
	"syscall.SYS_EXTATTR_LIST_LINK":                        "syscall",
	"syscall.SYS_EXTATTR_SET_FD":                           "syscall",
	"syscall.SYS_EXTATTR_SET_FILE":                         "syscall",
	"syscall.SYS_EXTATTR_SET_LINK":                         "syscall",
	"syscall.SYS_FACCESSAT":                                "syscall",
	"syscall.SYS_FADVISE64":                                "syscall",
	"syscall.SYS_FADVISE64_64":                             "syscall",
	"syscall.SYS_FALLOCATE":                                "syscall",
	"syscall.SYS_FANOTIFY_INIT":                            "syscall",
	"syscall.SYS_FANOTIFY_MARK":                            "syscall",
	"syscall.SYS_FCHDIR":                                   "syscall",
	"syscall.SYS_FCHFLAGS":                                 "syscall",
	"syscall.SYS_FCHMOD":                                   "syscall",
	"syscall.SYS_FCHMODAT":                                 "syscall",
	"syscall.SYS_FCHMOD_EXTENDED":                          "syscall",
	"syscall.SYS_FCHOWN":                                   "syscall",
	"syscall.SYS_FCHOWN32":                                 "syscall",
	"syscall.SYS_FCHOWNAT":                                 "syscall",
	"syscall.SYS_FCHROOT":                                  "syscall",
	"syscall.SYS_FCNTL":                                    "syscall",
	"syscall.SYS_FCNTL64":                                  "syscall",
	"syscall.SYS_FCNTL_NOCANCEL":                           "syscall",
	"syscall.SYS_FDATASYNC":                                "syscall",
	"syscall.SYS_FEXECVE":                                  "syscall",
	"syscall.SYS_FFCLOCK_GETCOUNTER":                       "syscall",
	"syscall.SYS_FFCLOCK_GETESTIMATE":                      "syscall",
	"syscall.SYS_FFCLOCK_SETESTIMATE":                      "syscall",
	"syscall.SYS_FFSCTL":                                   "syscall",
	"syscall.SYS_FGETATTRLIST":                             "syscall",
	"syscall.SYS_FGETXATTR":                                "syscall",
	"syscall.SYS_FHOPEN":                                   "syscall",
	"syscall.SYS_FHSTAT":                                   "syscall",
	"syscall.SYS_FHSTATFS":                                 "syscall",
	"syscall.SYS_FILEPORT_MAKEFD":                          "syscall",
	"syscall.SYS_FILEPORT_MAKEPORT":                        "syscall",
	"syscall.SYS_FKTRACE":                                  "syscall",
	"syscall.SYS_FLISTXATTR":                               "syscall",
	"syscall.SYS_FLOCK":                                    "syscall",
	"syscall.SYS_FORK":                                     "syscall",
	"syscall.SYS_FPATHCONF":                                "syscall",
	"syscall.SYS_FREEBSD6_FTRUNCATE":                       "syscall",
	"syscall.SYS_FREEBSD6_LSEEK":                           "syscall",
	"syscall.SYS_FREEBSD6_MMAP":                            "syscall",
	"syscall.SYS_FREEBSD6_PREAD":                           "syscall",
	"syscall.SYS_FREEBSD6_PWRITE":                          "syscall",
	"syscall.SYS_FREEBSD6_TRUNCATE":                        "syscall",
	"syscall.SYS_FREMOVEXATTR":                             "syscall",
	"syscall.SYS_FSCTL":                                    "syscall",
	"syscall.SYS_FSETATTRLIST":                             "syscall",
	"syscall.SYS_FSETXATTR":                                "syscall",
	"syscall.SYS_FSGETPATH":                                "syscall",
	"syscall.SYS_FSTAT":                                    "syscall",
	"syscall.SYS_FSTAT64":                                  "syscall",
	"syscall.SYS_FSTAT64_EXTENDED":                         "syscall",
	"syscall.SYS_FSTATAT":                                  "syscall",
	"syscall.SYS_FSTATAT64":                                "syscall",
	"syscall.SYS_FSTATFS":                                  "syscall",
	"syscall.SYS_FSTATFS64":                                "syscall",
	"syscall.SYS_FSTATV":                                   "syscall",
	"syscall.SYS_FSTATVFS1":                                "syscall",
	"syscall.SYS_FSTAT_EXTENDED":                           "syscall",
	"syscall.SYS_FSYNC":                                    "syscall",
	"syscall.SYS_FSYNC_NOCANCEL":                           "syscall",
	"syscall.SYS_FSYNC_RANGE":                              "syscall",
	"syscall.SYS_FTIME":                                    "syscall",
	"syscall.SYS_FTRUNCATE":                                "syscall",
	"syscall.SYS_FTRUNCATE64":                              "syscall",
	"syscall.SYS_FUTEX":                                    "syscall",
	"syscall.SYS_FUTIMENS":                                 "syscall",
	"syscall.SYS_FUTIMES":                                  "syscall",
	"syscall.SYS_FUTIMESAT":                                "syscall",
	"syscall.SYS_GETATTRLIST":                              "syscall",
	"syscall.SYS_GETAUDIT":                                 "syscall",
	"syscall.SYS_GETAUDIT_ADDR":                            "syscall",
	"syscall.SYS_GETAUID":                                  "syscall",
	"syscall.SYS_GETCONTEXT":                               "syscall",
	"syscall.SYS_GETCPU":                                   "syscall",
	"syscall.SYS_GETCWD":                                   "syscall",
	"syscall.SYS_GETDENTS":                                 "syscall",
	"syscall.SYS_GETDENTS64":                               "syscall",
	"syscall.SYS_GETDIRENTRIES":                            "syscall",
	"syscall.SYS_GETDIRENTRIES64":                          "syscall",
	"syscall.SYS_GETDIRENTRIESATTR":                        "syscall",
	"syscall.SYS_GETDTABLECOUNT":                           "syscall",
	"syscall.SYS_GETDTABLESIZE":                            "syscall",
	"syscall.SYS_GETEGID":                                  "syscall",
	"syscall.SYS_GETEGID32":                                "syscall",
	"syscall.SYS_GETEUID":                                  "syscall",
	"syscall.SYS_GETEUID32":                                "syscall",
	"syscall.SYS_GETFH":                                    "syscall",
	"syscall.SYS_GETFSSTAT":                                "syscall",
	"syscall.SYS_GETFSSTAT64":                              "syscall",
	"syscall.SYS_GETGID":                                   "syscall",
	"syscall.SYS_GETGID32":                                 "syscall",
	"syscall.SYS_GETGROUPS":                                "syscall",
	"syscall.SYS_GETGROUPS32":                              "syscall",
	"syscall.SYS_GETHOSTUUID":                              "syscall",
	"syscall.SYS_GETITIMER":                                "syscall",
	"syscall.SYS_GETLCID":                                  "syscall",
	"syscall.SYS_GETLOGIN":                                 "syscall",
	"syscall.SYS_GETLOGINCLASS":                            "syscall",
	"syscall.SYS_GETPEERNAME":                              "syscall",
	"syscall.SYS_GETPGID":                                  "syscall",
	"syscall.SYS_GETPGRP":                                  "syscall",
	"syscall.SYS_GETPID":                                   "syscall",
	"syscall.SYS_GETPMSG":                                  "syscall",
	"syscall.SYS_GETPPID":                                  "syscall",
	"syscall.SYS_GETPRIORITY":                              "syscall",
	"syscall.SYS_GETRESGID":                                "syscall",
	"syscall.SYS_GETRESGID32":                              "syscall",
	"syscall.SYS_GETRESUID":                                "syscall",
	"syscall.SYS_GETRESUID32":                              "syscall",
	"syscall.SYS_GETRLIMIT":                                "syscall",
	"syscall.SYS_GETRTABLE":                                "syscall",
	"syscall.SYS_GETRUSAGE":                                "syscall",
	"syscall.SYS_GETSGROUPS":                               "syscall",
	"syscall.SYS_GETSID":                                   "syscall",
	"syscall.SYS_GETSOCKNAME":                              "syscall",
	"syscall.SYS_GETSOCKOPT":                               "syscall",
	"syscall.SYS_GETTHRID":                                 "syscall",
	"syscall.SYS_GETTID":                                   "syscall",
	"syscall.SYS_GETTIMEOFDAY":                             "syscall",
	"syscall.SYS_GETUID":                                   "syscall",
	"syscall.SYS_GETUID32":                                 "syscall",
	"syscall.SYS_GETVFSSTAT":                               "syscall",
	"syscall.SYS_GETWGROUPS":                               "syscall",
	"syscall.SYS_GETXATTR":                                 "syscall",
	"syscall.SYS_GET_KERNEL_SYMS":                          "syscall",
	"syscall.SYS_GET_MEMPOLICY":                            "syscall",
	"syscall.SYS_GET_ROBUST_LIST":                          "syscall",
	"syscall.SYS_GET_THREAD_AREA":                          "syscall",
	"syscall.SYS_GTTY":                                     "syscall",
	"syscall.SYS_IDENTITYSVC":                              "syscall",
	"syscall.SYS_IDLE":                                     "syscall",
	"syscall.SYS_INITGROUPS":                               "syscall",
	"syscall.SYS_INIT_MODULE":                              "syscall",
	"syscall.SYS_INOTIFY_ADD_WATCH":                        "syscall",
	"syscall.SYS_INOTIFY_INIT":                             "syscall",
	"syscall.SYS_INOTIFY_INIT1":                            "syscall",
	"syscall.SYS_INOTIFY_RM_WATCH":                         "syscall",
	"syscall.SYS_IOCTL":                                    "syscall",
	"syscall.SYS_IOPERM":                                   "syscall",
	"syscall.SYS_IOPL":                                     "syscall",
	"syscall.SYS_IOPOLICYSYS":                              "syscall",
	"syscall.SYS_IOPRIO_GET":                               "syscall",
	"syscall.SYS_IOPRIO_SET":                               "syscall",
	"syscall.SYS_IO_CANCEL":                                "syscall",
	"syscall.SYS_IO_DESTROY":                               "syscall",
	"syscall.SYS_IO_GETEVENTS":                             "syscall",
	"syscall.SYS_IO_SETUP":                                 "syscall",
	"syscall.SYS_IO_SUBMIT":                                "syscall",
	"syscall.SYS_IPC":                                      "syscall",
	"syscall.SYS_ISSETUGID":                                "syscall",
	"syscall.SYS_JAIL":                                     "syscall",
	"syscall.SYS_JAIL_ATTACH":                              "syscall",
	"syscall.SYS_JAIL_GET":                                 "syscall",
	"syscall.SYS_JAIL_REMOVE":                              "syscall",
	"syscall.SYS_JAIL_SET":                                 "syscall",
	"syscall.SYS_KDEBUG_TRACE":                             "syscall",
	"syscall.SYS_KENV":                                     "syscall",
	"syscall.SYS_KEVENT":                                   "syscall",
	"syscall.SYS_KEVENT64":                                 "syscall",
	"syscall.SYS_KEXEC_LOAD":                               "syscall",
	"syscall.SYS_KEYCTL":                                   "syscall",
	"syscall.SYS_KILL":                                     "syscall",
	"syscall.SYS_KLDFIND":                                  "syscall",
	"syscall.SYS_KLDFIRSTMOD":                              "syscall",
	"syscall.SYS_KLDLOAD":                                  "syscall",
	"syscall.SYS_KLDNEXT":                                  "syscall",
	"syscall.SYS_KLDSTAT":                                  "syscall",
	"syscall.SYS_KLDSYM":                                   "syscall",
	"syscall.SYS_KLDUNLOAD":                                "syscall",
	"syscall.SYS_KLDUNLOADF":                               "syscall",
	"syscall.SYS_KQUEUE":                                   "syscall",
	"syscall.SYS_KQUEUE1":                                  "syscall",
	"syscall.SYS_KTIMER_CREATE":                            "syscall",
	"syscall.SYS_KTIMER_DELETE":                            "syscall",
	"syscall.SYS_KTIMER_GETOVERRUN":                        "syscall",
	"syscall.SYS_KTIMER_GETTIME":                           "syscall",
	"syscall.SYS_KTIMER_SETTIME":                           "syscall",
	"syscall.SYS_KTRACE":                                   "syscall",
	"syscall.SYS_LCHFLAGS":                                 "syscall",
	"syscall.SYS_LCHMOD":                                   "syscall",
	"syscall.SYS_LCHOWN":                                   "syscall",
	"syscall.SYS_LCHOWN32":                                 "syscall",
	"syscall.SYS_LGETFH":                                   "syscall",
	"syscall.SYS_LGETXATTR":                                "syscall",
	"syscall.SYS_LINK":                                     "syscall",
	"syscall.SYS_LINKAT":                                   "syscall",
	"syscall.SYS_LIO_LISTIO":                               "syscall",
	"syscall.SYS_LISTEN":                                   "syscall",
	"syscall.SYS_LISTXATTR":                                "syscall",
	"syscall.SYS_LLISTXATTR":                               "syscall",
	"syscall.SYS_LOCK":                                     "syscall",
	"syscall.SYS_LOOKUP_DCOOKIE":                           "syscall",
	"syscall.SYS_LPATHCONF":                                "syscall",
	"syscall.SYS_LREMOVEXATTR":                             "syscall",
	"syscall.SYS_LSEEK":                                    "syscall",
	"syscall.SYS_LSETXATTR":                                "syscall",
	"syscall.SYS_LSTAT":                                    "syscall",
	"syscall.SYS_LSTAT64":                                  "syscall",
	"syscall.SYS_LSTAT64_EXTENDED":                         "syscall",
	"syscall.SYS_LSTATV":                                   "syscall",
	"syscall.SYS_LSTAT_EXTENDED":                           "syscall",
	"syscall.SYS_LUTIMES":                                  "syscall",
	"syscall.SYS_MAC_SYSCALL":                              "syscall",
	"syscall.SYS_MADVISE":                                  "syscall",
	"syscall.SYS_MADVISE1":                                 "syscall",
	"syscall.SYS_MAXSYSCALL":                               "syscall",
	"syscall.SYS_MBIND":                                    "syscall",
	"syscall.SYS_MIGRATE_PAGES":                            "syscall",
	"syscall.SYS_MINCORE":                                  "syscall",
	"syscall.SYS_MINHERIT":                                 "syscall",
	"syscall.SYS_MKCOMPLEX":                                "syscall",
	"syscall.SYS_MKDIR":                                    "syscall",
	"syscall.SYS_MKDIRAT":                                  "syscall",
	"syscall.SYS_MKDIR_EXTENDED":                           "syscall",
	"syscall.SYS_MKFIFO":                                   "syscall",
	"syscall.SYS_MKFIFOAT":                                 "syscall",
	"syscall.SYS_MKFIFO_EXTENDED":                          "syscall",
	"syscall.SYS_MKNOD":                                    "syscall",
	"syscall.SYS_MKNODAT":                                  "syscall",
	"syscall.SYS_MLOCK":                                    "syscall",
	"syscall.SYS_MLOCKALL":                                 "syscall",
	"syscall.SYS_MMAP":                                     "syscall",
	"syscall.SYS_MMAP2":                                    "syscall",
	"syscall.SYS_MODCTL":                                   "syscall",
	"syscall.SYS_MODFIND":                                  "syscall",
	"syscall.SYS_MODFNEXT":                                 "syscall",
	"syscall.SYS_MODIFY_LDT":                               "syscall",
	"syscall.SYS_MODNEXT":                                  "syscall",
	"syscall.SYS_MODSTAT":                                  "syscall",
	"syscall.SYS_MODWATCH":                                 "syscall",
	"syscall.SYS_MOUNT":                                    "syscall",
	"syscall.SYS_MOVE_PAGES":                               "syscall",
	"syscall.SYS_MPROTECT":                                 "syscall",
	"syscall.SYS_MPX":                                      "syscall",
	"syscall.SYS_MQUERY":                                   "syscall",
	"syscall.SYS_MQ_GETSETATTR":                            "syscall",
	"syscall.SYS_MQ_NOTIFY":                                "syscall",
	"syscall.SYS_MQ_OPEN":                                  "syscall",
	"syscall.SYS_MQ_TIMEDRECEIVE":                          "syscall",
	"syscall.SYS_MQ_TIMEDSEND":                             "syscall",
	"syscall.SYS_MQ_UNLINK":                                "syscall",
	"syscall.SYS_MREMAP":                                   "syscall",
	"syscall.SYS_MSGCTL":                                   "syscall",
	"syscall.SYS_MSGGET":                                   "syscall",
	"syscall.SYS_MSGRCV":                                   "syscall",
	"syscall.SYS_MSGRCV_NOCANCEL":                          "syscall",
	"syscall.SYS_MSGSND":                                   "syscall",
	"syscall.SYS_MSGSND_NOCANCEL":                          "syscall",
	"syscall.SYS_MSGSYS":                                   "syscall",
	"syscall.SYS_MSYNC":                                    "syscall",
	"syscall.SYS_MSYNC_NOCANCEL":                           "syscall",
	"syscall.SYS_MUNLOCK":                                  "syscall",
	"syscall.SYS_MUNLOCKALL":                               "syscall",
	"syscall.SYS_MUNMAP":                                   "syscall",
	"syscall.SYS_NAME_TO_HANDLE_AT":                        "syscall",
	"syscall.SYS_NANOSLEEP":                                "syscall",
	"syscall.SYS_NEWFSTATAT":                               "syscall",
	"syscall.SYS_NFSCLNT":                                  "syscall",
	"syscall.SYS_NFSSERVCTL":                               "syscall",
	"syscall.SYS_NFSSVC":                                   "syscall",
	"syscall.SYS_NFSTAT":                                   "syscall",
	"syscall.SYS_NICE":                                     "syscall",
	"syscall.SYS_NLSTAT":                                   "syscall",
	"syscall.SYS_NMOUNT":                                   "syscall",
	"syscall.SYS_NSTAT":                                    "syscall",
	"syscall.SYS_NTP_ADJTIME":                              "syscall",
	"syscall.SYS_NTP_GETTIME":                              "syscall",
	"syscall.SYS_OABI_SYSCALL_BASE":                        "syscall",
	"syscall.SYS_OBREAK":                                   "syscall",
	"syscall.SYS_OLDFSTAT":                                 "syscall",
	"syscall.SYS_OLDLSTAT":                                 "syscall",
	"syscall.SYS_OLDOLDUNAME":                              "syscall",
	"syscall.SYS_OLDSTAT":                                  "syscall",
	"syscall.SYS_OLDUNAME":                                 "syscall",
	"syscall.SYS_OPEN":                                     "syscall",
	"syscall.SYS_OPENAT":                                   "syscall",
	"syscall.SYS_OPENBSD_POLL":                             "syscall",
	"syscall.SYS_OPEN_BY_HANDLE_AT":                        "syscall",
	"syscall.SYS_OPEN_EXTENDED":                            "syscall",
	"syscall.SYS_OPEN_NOCANCEL":                            "syscall",
	"syscall.SYS_OVADVISE":                                 "syscall",
	"syscall.SYS_PACCEPT":                                  "syscall",
	"syscall.SYS_PATHCONF":                                 "syscall",
	"syscall.SYS_PAUSE":                                    "syscall",
	"syscall.SYS_PCICONFIG_IOBASE":                         "syscall",
	"syscall.SYS_PCICONFIG_READ":                           "syscall",
	"syscall.SYS_PCICONFIG_WRITE":                          "syscall",
	"syscall.SYS_PDFORK":                                   "syscall",
	"syscall.SYS_PDGETPID":                                 "syscall",
	"syscall.SYS_PDKILL":                                   "syscall",
	"syscall.SYS_PERF_EVENT_OPEN":                          "syscall",
	"syscall.SYS_PERSONALITY":                              "syscall",
	"syscall.SYS_PID_HIBERNATE":                            "syscall",
	"syscall.SYS_PID_RESUME":                               "syscall",
	"syscall.SYS_PID_SHUTDOWN_SOCKETS":                     "syscall",
	"syscall.SYS_PID_SUSPEND":                              "syscall",
	"syscall.SYS_PIPE":                                     "syscall",
	"syscall.SYS_PIPE2":                                    "syscall",
	"syscall.SYS_PIVOT_ROOT":                               "syscall",
	"syscall.SYS_PMC_CONTROL":                              "syscall",
	"syscall.SYS_PMC_GET_INFO":                             "syscall",
	"syscall.SYS_POLL":                                     "syscall",
	"syscall.SYS_POLLTS":                                   "syscall",
	"syscall.SYS_POLL_NOCANCEL":                            "syscall",
	"syscall.SYS_POSIX_FADVISE":                            "syscall",
	"syscall.SYS_POSIX_FALLOCATE":                          "syscall",
	"syscall.SYS_POSIX_OPENPT":                             "syscall",
	"syscall.SYS_POSIX_SPAWN":                              "syscall",
	"syscall.SYS_PPOLL":                                    "syscall",
	"syscall.SYS_PRCTL":                                    "syscall",
	"syscall.SYS_PREAD":                                    "syscall",
	"syscall.SYS_PREAD64":                                  "syscall",
	"syscall.SYS_PREADV":                                   "syscall",
	"syscall.SYS_PREAD_NOCANCEL":                           "syscall",
	"syscall.SYS_PRLIMIT64":                                "syscall",
	"syscall.SYS_PROCESS_POLICY":                           "syscall",
	"syscall.SYS_PROCESS_VM_READV":                         "syscall",
	"syscall.SYS_PROCESS_VM_WRITEV":                        "syscall",
	"syscall.SYS_PROC_INFO":                                "syscall",
	"syscall.SYS_PROF":                                     "syscall",
	"syscall.SYS_PROFIL":                                   "syscall",
	"syscall.SYS_PSELECT":                                  "syscall",
	"syscall.SYS_PSELECT6":                                 "syscall",
	"syscall.SYS_PSET_ASSIGN":                              "syscall",
	"syscall.SYS_PSET_CREATE":                              "syscall",
	"syscall.SYS_PSET_DESTROY":                             "syscall",
	"syscall.SYS_PSYNCH_CVBROAD":                           "syscall",
	"syscall.SYS_PSYNCH_CVCLRPREPOST":                      "syscall",
	"syscall.SYS_PSYNCH_CVSIGNAL":                          "syscall",
	"syscall.SYS_PSYNCH_CVWAIT":                            "syscall",
	"syscall.SYS_PSYNCH_MUTEXDROP":                         "syscall",
	"syscall.SYS_PSYNCH_MUTEXWAIT":                         "syscall",
	"syscall.SYS_PSYNCH_RW_DOWNGRADE":                      "syscall",
	"syscall.SYS_PSYNCH_RW_LONGRDLOCK":                     "syscall",
	"syscall.SYS_PSYNCH_RW_RDLOCK":                         "syscall",
	"syscall.SYS_PSYNCH_RW_UNLOCK":                         "syscall",
	"syscall.SYS_PSYNCH_RW_UNLOCK2":                        "syscall",
	"syscall.SYS_PSYNCH_RW_UPGRADE":                        "syscall",
	"syscall.SYS_PSYNCH_RW_WRLOCK":                         "syscall",
	"syscall.SYS_PSYNCH_RW_YIELDWRLOCK":                    "syscall",
	"syscall.SYS_PTRACE":                                   "syscall",
	"syscall.SYS_PUTPMSG":                                  "syscall",
	"syscall.SYS_PWRITE":                                   "syscall",
	"syscall.SYS_PWRITE64":                                 "syscall",
	"syscall.SYS_PWRITEV":                                  "syscall",
	"syscall.SYS_PWRITE_NOCANCEL":                          "syscall",
	"syscall.SYS_QUERY_MODULE":                             "syscall",
	"syscall.SYS_QUOTACTL":                                 "syscall",
	"syscall.SYS_RASCTL":                                   "syscall",
	"syscall.SYS_RCTL_ADD_RULE":                            "syscall",
	"syscall.SYS_RCTL_GET_LIMITS":                          "syscall",
	"syscall.SYS_RCTL_GET_RACCT":                           "syscall",
	"syscall.SYS_RCTL_GET_RULES":                           "syscall",
	"syscall.SYS_RCTL_REMOVE_RULE":                         "syscall",
	"syscall.SYS_READ":                                     "syscall",
	"syscall.SYS_READAHEAD":                                "syscall",
	"syscall.SYS_READDIR":                                  "syscall",
	"syscall.SYS_READLINK":                                 "syscall",
	"syscall.SYS_READLINKAT":                               "syscall",
	"syscall.SYS_READV":                                    "syscall",
	"syscall.SYS_READV_NOCANCEL":                           "syscall",
	"syscall.SYS_READ_NOCANCEL":                            "syscall",
	"syscall.SYS_REBOOT":                                   "syscall",
	"syscall.SYS_RECV":                                     "syscall",
	"syscall.SYS_RECVFROM":                                 "syscall",
	"syscall.SYS_RECVFROM_NOCANCEL":                        "syscall",
	"syscall.SYS_RECVMMSG":                                 "syscall",
	"syscall.SYS_RECVMSG":                                  "syscall",
	"syscall.SYS_RECVMSG_NOCANCEL":                         "syscall",
	"syscall.SYS_REMAP_FILE_PAGES":                         "syscall",
	"syscall.SYS_REMOVEXATTR":                              "syscall",
	"syscall.SYS_RENAME":                                   "syscall",
	"syscall.SYS_RENAMEAT":                                 "syscall",
	"syscall.SYS_REQUEST_KEY":                              "syscall",
	"syscall.SYS_RESTART_SYSCALL":                          "syscall",
	"syscall.SYS_REVOKE":                                   "syscall",
	"syscall.SYS_RFORK":                                    "syscall",
	"syscall.SYS_RMDIR":                                    "syscall",
	"syscall.SYS_RTPRIO":                                   "syscall",
	"syscall.SYS_RTPRIO_THREAD":                            "syscall",
	"syscall.SYS_RT_SIGACTION":                             "syscall",
	"syscall.SYS_RT_SIGPENDING":                            "syscall",
	"syscall.SYS_RT_SIGPROCMASK":                           "syscall",
	"syscall.SYS_RT_SIGQUEUEINFO":                          "syscall",
	"syscall.SYS_RT_SIGRETURN":                             "syscall",
	"syscall.SYS_RT_SIGSUSPEND":                            "syscall",
	"syscall.SYS_RT_SIGTIMEDWAIT":                          "syscall",
	"syscall.SYS_RT_TGSIGQUEUEINFO":                        "syscall",
	"syscall.SYS_SBRK":                                     "syscall",
	"syscall.SYS_SCHED_GETAFFINITY":                        "syscall",
	"syscall.SYS_SCHED_GETPARAM":                           "syscall",
	"syscall.SYS_SCHED_GETSCHEDULER":                       "syscall",
	"syscall.SYS_SCHED_GET_PRIORITY_MAX":                   "syscall",
	"syscall.SYS_SCHED_GET_PRIORITY_MIN":                   "syscall",
	"syscall.SYS_SCHED_RR_GET_INTERVAL":                    "syscall",
	"syscall.SYS_SCHED_SETAFFINITY":                        "syscall",
	"syscall.SYS_SCHED_SETPARAM":                           "syscall",
	"syscall.SYS_SCHED_SETSCHEDULER":                       "syscall",
	"syscall.SYS_SCHED_YIELD":                              "syscall",
	"syscall.SYS_SCTP_GENERIC_RECVMSG":                     "syscall",
	"syscall.SYS_SCTP_GENERIC_SENDMSG":                     "syscall",
	"syscall.SYS_SCTP_GENERIC_SENDMSG_IOV":                 "syscall",
	"syscall.SYS_SCTP_PEELOFF":                             "syscall",
	"syscall.SYS_SEARCHFS":                                 "syscall",
	"syscall.SYS_SECURITY":                                 "syscall",
	"syscall.SYS_SELECT":                                   "syscall",
	"syscall.SYS_SELECT_NOCANCEL":                          "syscall",
	"syscall.SYS_SEMCONFIG":                                "syscall",
	"syscall.SYS_SEMCTL":                                   "syscall",
	"syscall.SYS_SEMGET":                                   "syscall",
	"syscall.SYS_SEMOP":                                    "syscall",
	"syscall.SYS_SEMSYS":                                   "syscall",
	"syscall.SYS_SEMTIMEDOP":                               "syscall",
	"syscall.SYS_SEM_CLOSE":                                "syscall",
	"syscall.SYS_SEM_DESTROY":                              "syscall",
	"syscall.SYS_SEM_GETVALUE":                             "syscall",
	"syscall.SYS_SEM_INIT":                                 "syscall",
	"syscall.SYS_SEM_OPEN":                                 "syscall",
	"syscall.SYS_SEM_POST":                                 "syscall",
	"syscall.SYS_SEM_TRYWAIT":                              "syscall",
	"syscall.SYS_SEM_UNLINK":                               "syscall",
	"syscall.SYS_SEM_WAIT":                                 "syscall",
	"syscall.SYS_SEM_WAIT_NOCANCEL":                        "syscall",
	"syscall.SYS_SEND":                                     "syscall",
	"syscall.SYS_SENDFILE":                                 "syscall",
	"syscall.SYS_SENDFILE64":                               "syscall",
	"syscall.SYS_SENDMMSG":                                 "syscall",
	"syscall.SYS_SENDMSG":                                  "syscall",
	"syscall.SYS_SENDMSG_NOCANCEL":                         "syscall",
	"syscall.SYS_SENDTO":                                   "syscall",
	"syscall.SYS_SENDTO_NOCANCEL":                          "syscall",
	"syscall.SYS_SETATTRLIST":                              "syscall",
	"syscall.SYS_SETAUDIT":                                 "syscall",
	"syscall.SYS_SETAUDIT_ADDR":                            "syscall",
	"syscall.SYS_SETAUID":                                  "syscall",
	"syscall.SYS_SETCONTEXT":                               "syscall",
	"syscall.SYS_SETDOMAINNAME":                            "syscall",
	"syscall.SYS_SETEGID":                                  "syscall",
	"syscall.SYS_SETEUID":                                  "syscall",
	"syscall.SYS_SETFIB":                                   "syscall",
	"syscall.SYS_SETFSGID":                                 "syscall",
	"syscall.SYS_SETFSGID32":                               "syscall",
	"syscall.SYS_SETFSUID":                                 "syscall",
	"syscall.SYS_SETFSUID32":                               "syscall",
	"syscall.SYS_SETGID":                                   "syscall",
	"syscall.SYS_SETGID32":                                 "syscall",
	"syscall.SYS_SETGROUPS":                                "syscall",
	"syscall.SYS_SETGROUPS32":                              "syscall",
	"syscall.SYS_SETHOSTNAME":                              "syscall",
	"syscall.SYS_SETITIMER":                                "syscall",
	"syscall.SYS_SETLCID":                                  "syscall",
	"syscall.SYS_SETLOGIN":                                 "syscall",
	"syscall.SYS_SETLOGINCLASS":                            "syscall",
	"syscall.SYS_SETNS":                                    "syscall",
	"syscall.SYS_SETPGID":                                  "syscall",
	"syscall.SYS_SETPRIORITY":                              "syscall",
	"syscall.SYS_SETPRIVEXEC":                              "syscall",
	"syscall.SYS_SETREGID":                                 "syscall",
	"syscall.SYS_SETREGID32":                               "syscall",
	"syscall.SYS_SETRESGID":                                "syscall",
	"syscall.SYS_SETRESGID32":                              "syscall",
	"syscall.SYS_SETRESUID":                                "syscall",
	"syscall.SYS_SETRESUID32":                              "syscall",
	"syscall.SYS_SETREUID":                                 "syscall",
	"syscall.SYS_SETREUID32":                               "syscall",
	"syscall.SYS_SETRLIMIT":                                "syscall",
	"syscall.SYS_SETRTABLE":                                "syscall",
	"syscall.SYS_SETSGROUPS":                               "syscall",
	"syscall.SYS_SETSID":                                   "syscall",
	"syscall.SYS_SETSOCKOPT":                               "syscall",
	"syscall.SYS_SETTID":                                   "syscall",
	"syscall.SYS_SETTID_WITH_PID":                          "syscall",
	"syscall.SYS_SETTIMEOFDAY":                             "syscall",
	"syscall.SYS_SETUID":                                   "syscall",
	"syscall.SYS_SETUID32":                                 "syscall",
	"syscall.SYS_SETWGROUPS":                               "syscall",
	"syscall.SYS_SETXATTR":                                 "syscall",
	"syscall.SYS_SET_MEMPOLICY":                            "syscall",
	"syscall.SYS_SET_ROBUST_LIST":                          "syscall",
	"syscall.SYS_SET_THREAD_AREA":                          "syscall",
	"syscall.SYS_SET_TID_ADDRESS":                          "syscall",
	"syscall.SYS_SGETMASK":                                 "syscall",
	"syscall.SYS_SHARED_REGION_CHECK_NP":                   "syscall",
	"syscall.SYS_SHARED_REGION_MAP_AND_SLIDE_NP":           "syscall",
	"syscall.SYS_SHMAT":                                    "syscall",
	"syscall.SYS_SHMCTL":                                   "syscall",
	"syscall.SYS_SHMDT":                                    "syscall",
	"syscall.SYS_SHMGET":                                   "syscall",
	"syscall.SYS_SHMSYS":                                   "syscall",
	"syscall.SYS_SHM_OPEN":                                 "syscall",
	"syscall.SYS_SHM_UNLINK":                               "syscall",
	"syscall.SYS_SHUTDOWN":                                 "syscall",
	"syscall.SYS_SIGACTION":                                "syscall",
	"syscall.SYS_SIGALTSTACK":                              "syscall",
	"syscall.SYS_SIGNAL":                                   "syscall",
	"syscall.SYS_SIGNALFD":                                 "syscall",
	"syscall.SYS_SIGNALFD4":                                "syscall",
	"syscall.SYS_SIGPENDING":                               "syscall",
	"syscall.SYS_SIGPROCMASK":                              "syscall",
	"syscall.SYS_SIGQUEUE":                                 "syscall",
	"syscall.SYS_SIGQUEUEINFO":                             "syscall",
	"syscall.SYS_SIGRETURN":                                "syscall",
	"syscall.SYS_SIGSUSPEND":                               "syscall",
	"syscall.SYS_SIGSUSPEND_NOCANCEL":                      "syscall",
	"syscall.SYS_SIGTIMEDWAIT":                             "syscall",
	"syscall.SYS_SIGWAIT":                                  "syscall",
	"syscall.SYS_SIGWAITINFO":                              "syscall",
	"syscall.SYS_SOCKET":                                   "syscall",
	"syscall.SYS_SOCKETCALL":                               "syscall",
	"syscall.SYS_SOCKETPAIR":                               "syscall",
	"syscall.SYS_SPLICE":                                   "syscall",
	"syscall.SYS_SSETMASK":                                 "syscall",
	"syscall.SYS_SSTK":                                     "syscall",
	"syscall.SYS_STACK_SNAPSHOT":                           "syscall",
	"syscall.SYS_STAT":                                     "syscall",
	"syscall.SYS_STAT64":                                   "syscall",
	"syscall.SYS_STAT64_EXTENDED":                          "syscall",
	"syscall.SYS_STATFS":                                   "syscall",
	"syscall.SYS_STATFS64":                                 "syscall",
	"syscall.SYS_STATV":                                    "syscall",
	"syscall.SYS_STATVFS1":                                 "syscall",
	"syscall.SYS_STAT_EXTENDED":                            "syscall",
	"syscall.SYS_STIME":                                    "syscall",
	"syscall.SYS_STTY":                                     "syscall",
	"syscall.SYS_SWAPCONTEXT":                              "syscall",
	"syscall.SYS_SWAPCTL":                                  "syscall",
	"syscall.SYS_SWAPOFF":                                  "syscall",
	"syscall.SYS_SWAPON":                                   "syscall",
	"syscall.SYS_SYMLINK":                                  "syscall",
	"syscall.SYS_SYMLINKAT":                                "syscall",
	"syscall.SYS_SYNC":                                     "syscall",
	"syscall.SYS_SYNCFS":                                   "syscall",
	"syscall.SYS_SYNC_FILE_RANGE":                          "syscall",
	"syscall.SYS_SYSARCH":                                  "syscall",
	"syscall.SYS_SYSCALL":                                  "syscall",
	"syscall.SYS_SYSCALL_BASE":                             "syscall",
	"syscall.SYS_SYSFS":                                    "syscall",
	"syscall.SYS_SYSINFO":                                  "syscall",
	"syscall.SYS_SYSLOG":                                   "syscall",
	"syscall.SYS_TEE":                                      "syscall",
	"syscall.SYS_TGKILL":                                   "syscall",
	"syscall.SYS_THREAD_SELFID":                            "syscall",
	"syscall.SYS_THR_CREATE":                               "syscall",
	"syscall.SYS_THR_EXIT":                                 "syscall",
	"syscall.SYS_THR_KILL":                                 "syscall",
	"syscall.SYS_THR_KILL2":                                "syscall",
	"syscall.SYS_THR_NEW":                                  "syscall",
	"syscall.SYS_THR_SELF":                                 "syscall",
	"syscall.SYS_THR_SET_NAME":                             "syscall",
	"syscall.SYS_THR_SUSPEND":                              "syscall",
	"syscall.SYS_THR_WAKE":                                 "syscall",
	"syscall.SYS_TIME":                                     "syscall",
	"syscall.SYS_TIMERFD_CREATE":                           "syscall",
	"syscall.SYS_TIMERFD_GETTIME":                          "syscall",
	"syscall.SYS_TIMERFD_SETTIME":                          "syscall",
	"syscall.SYS_TIMER_CREATE":                             "syscall",
	"syscall.SYS_TIMER_DELETE":                             "syscall",
	"syscall.SYS_TIMER_GETOVERRUN":                         "syscall",
	"syscall.SYS_TIMER_GETTIME":                            "syscall",
	"syscall.SYS_TIMER_SETTIME":                            "syscall",
	"syscall.SYS_TIMES":                                    "syscall",
	"syscall.SYS_TKILL":                                    "syscall",
	"syscall.SYS_TRUNCATE":                                 "syscall",
	"syscall.SYS_TRUNCATE64":                               "syscall",
	"syscall.SYS_TUXCALL":                                  "syscall",
	"syscall.SYS_UGETRLIMIT":                               "syscall",
	"syscall.SYS_ULIMIT":                                   "syscall",
	"syscall.SYS_UMASK":                                    "syscall",
	"syscall.SYS_UMASK_EXTENDED":                           "syscall",
	"syscall.SYS_UMOUNT":                                   "syscall",
	"syscall.SYS_UMOUNT2":                                  "syscall",
	"syscall.SYS_UNAME":                                    "syscall",
	"syscall.SYS_UNDELETE":                                 "syscall",
	"syscall.SYS_UNLINK":                                   "syscall",
	"syscall.SYS_UNLINKAT":                                 "syscall",
	"syscall.SYS_UNMOUNT":                                  "syscall",
	"syscall.SYS_UNSHARE":                                  "syscall",
	"syscall.SYS_USELIB":                                   "syscall",
	"syscall.SYS_USTAT":                                    "syscall",
	"syscall.SYS_UTIME":                                    "syscall",
	"syscall.SYS_UTIMENSAT":                                "syscall",
	"syscall.SYS_UTIMES":                                   "syscall",
	"syscall.SYS_UTRACE":                                   "syscall",
	"syscall.SYS_UUIDGEN":                                  "syscall",
	"syscall.SYS_VADVISE":                                  "syscall",
	"syscall.SYS_VFORK":                                    "syscall",
	"syscall.SYS_VHANGUP":                                  "syscall",
	"syscall.SYS_VM86":                                     "syscall",
	"syscall.SYS_VM86OLD":                                  "syscall",
	"syscall.SYS_VMSPLICE":                                 "syscall",
	"syscall.SYS_VM_PRESSURE_MONITOR":                      "syscall",
	"syscall.SYS_VSERVER":                                  "syscall",
	"syscall.SYS_WAIT4":                                    "syscall",
	"syscall.SYS_WAIT4_NOCANCEL":                           "syscall",
	"syscall.SYS_WAIT6":                                    "syscall",
	"syscall.SYS_WAITEVENT":                                "syscall",
	"syscall.SYS_WAITID":                                   "syscall",
	"syscall.SYS_WAITID_NOCANCEL":                          "syscall",
	"syscall.SYS_WAITPID":                                  "syscall",
	"syscall.SYS_WATCHEVENT":                               "syscall",
	"syscall.SYS_WORKQ_KERNRETURN":                         "syscall",
	"syscall.SYS_WORKQ_OPEN":                               "syscall",
	"syscall.SYS_WRITE":                                    "syscall",
	"syscall.SYS_WRITEV":                                   "syscall",
	"syscall.SYS_WRITEV_NOCANCEL":                          "syscall",
	"syscall.SYS_WRITE_NOCANCEL":                           "syscall",
	"syscall.SYS_YIELD":                                    "syscall",
	"syscall.SYS__LLSEEK":                                  "syscall",
	"syscall.SYS__LWP_CONTINUE":                            "syscall",
	"syscall.SYS__LWP_CREATE":                              "syscall",
	"syscall.SYS__LWP_CTL":                                 "syscall",
	"syscall.SYS__LWP_DETACH":                              "syscall",
	"syscall.SYS__LWP_EXIT":                                "syscall",
	"syscall.SYS__LWP_GETNAME":                             "syscall",
	"syscall.SYS__LWP_GETPRIVATE":                          "syscall",
	"syscall.SYS__LWP_KILL":                                "syscall",
	"syscall.SYS__LWP_PARK":                                "syscall",
	"syscall.SYS__LWP_SELF":                                "syscall",
	"syscall.SYS__LWP_SETNAME":                             "syscall",
	"syscall.SYS__LWP_SETPRIVATE":                          "syscall",
	"syscall.SYS__LWP_SUSPEND":                             "syscall",
	"syscall.SYS__LWP_UNPARK":                              "syscall",
	"syscall.SYS__LWP_UNPARK_ALL":                          "syscall",
	"syscall.SYS__LWP_WAIT":                                "syscall",
	"syscall.SYS__LWP_WAKEUP":                              "syscall",
	"syscall.SYS__NEWSELECT":                               "syscall",
	"syscall.SYS__PSET_BIND":                               "syscall",
	"syscall.SYS__SCHED_GETAFFINITY":                       "syscall",
	"syscall.SYS__SCHED_GETPARAM":                          "syscall",
	"syscall.SYS__SCHED_SETAFFINITY":                       "syscall",
	"syscall.SYS__SCHED_SETPARAM":                          "syscall",
	"syscall.SYS__SYSCTL":                                  "syscall",
	"syscall.SYS__UMTX_LOCK":                               "syscall",
	"syscall.SYS__UMTX_OP":                                 "syscall",
	"syscall.SYS__UMTX_UNLOCK":                             "syscall",
	"syscall.SYS___ACL_ACLCHECK_FD":                        "syscall",
	"syscall.SYS___ACL_ACLCHECK_FILE":                      "syscall",
	"syscall.SYS___ACL_ACLCHECK_LINK":                      "syscall",
	"syscall.SYS___ACL_DELETE_FD":                          "syscall",
	"syscall.SYS___ACL_DELETE_FILE":                        "syscall",
	"syscall.SYS___ACL_DELETE_LINK":                        "syscall",
	"syscall.SYS___ACL_GET_FD":                             "syscall",
	"syscall.SYS___ACL_GET_FILE":                           "syscall",
	"syscall.SYS___ACL_GET_LINK":                           "syscall",
	"syscall.SYS___ACL_SET_FD":                             "syscall",
	"syscall.SYS___ACL_SET_FILE":                           "syscall",
	"syscall.SYS___ACL_SET_LINK":                           "syscall",
	"syscall.SYS___CLONE":                                  "syscall",
	"syscall.SYS___DISABLE_THREADSIGNAL":                   "syscall",
	"syscall.SYS___GETCWD":                                 "syscall",
	"syscall.SYS___GETLOGIN":                               "syscall",
	"syscall.SYS___GET_TCB":                                "syscall",
	"syscall.SYS___MAC_EXECVE":                             "syscall",
	"syscall.SYS___MAC_GETFSSTAT":                          "syscall",
	"syscall.SYS___MAC_GET_FD":                             "syscall",
	"syscall.SYS___MAC_GET_FILE":                           "syscall",
	"syscall.SYS___MAC_GET_LCID":                           "syscall",
	"syscall.SYS___MAC_GET_LCTX":                           "syscall",
	"syscall.SYS___MAC_GET_LINK":                           "syscall",
	"syscall.SYS___MAC_GET_MOUNT":                          "syscall",
	"syscall.SYS___MAC_GET_PID":                            "syscall",
	"syscall.SYS___MAC_GET_PROC":                           "syscall",
	"syscall.SYS___MAC_MOUNT":                              "syscall",
	"syscall.SYS___MAC_SET_FD":                             "syscall",
	"syscall.SYS___MAC_SET_FILE":                           "syscall",
	"syscall.SYS___MAC_SET_LCTX":                           "syscall",
	"syscall.SYS___MAC_SET_LINK":                           "syscall",
	"syscall.SYS___MAC_SET_PROC":                           "syscall",
	"syscall.SYS___MAC_SYSCALL":                            "syscall",
	"syscall.SYS___OLD_SEMWAIT_SIGNAL":                     "syscall",
	"syscall.SYS___OLD_SEMWAIT_SIGNAL_NOCANCEL":            "syscall",
	"syscall.SYS___POSIX_CHOWN":                            "syscall",
	"syscall.SYS___POSIX_FCHOWN":                           "syscall",
	"syscall.SYS___POSIX_LCHOWN":                           "syscall",
	"syscall.SYS___POSIX_RENAME":                           "syscall",
	"syscall.SYS___PTHREAD_CANCELED":                       "syscall",
	"syscall.SYS___PTHREAD_CHDIR":                          "syscall",
	"syscall.SYS___PTHREAD_FCHDIR":                         "syscall",
	"syscall.SYS___PTHREAD_KILL":                           "syscall",
	"syscall.SYS___PTHREAD_MARKCANCEL":                     "syscall",
	"syscall.SYS___PTHREAD_SIGMASK":                        "syscall",
	"syscall.SYS___QUOTACTL":                               "syscall",
	"syscall.SYS___SEMCTL":                                 "syscall",
	"syscall.SYS___SEMWAIT_SIGNAL":                         "syscall",
	"syscall.SYS___SEMWAIT_SIGNAL_NOCANCEL":                "syscall",
	"syscall.SYS___SETLOGIN":                               "syscall",
	"syscall.SYS___SETUGID":                                "syscall",
	"syscall.SYS___SET_TCB":                                "syscall",
	"syscall.SYS___SIGACTION_SIGTRAMP":                     "syscall",
	"syscall.SYS___SIGTIMEDWAIT":                           "syscall",
	"syscall.SYS___SIGWAIT":                                "syscall",
	"syscall.SYS___SIGWAIT_NOCANCEL":                       "syscall",
	"syscall.SYS___SYSCTL":                                 "syscall",
	"syscall.SYS___TFORK":                                  "syscall",
	"syscall.SYS___THREXIT":                                "syscall",
	"syscall.SYS___THRSIGDIVERT":                           "syscall",
	"syscall.SYS___THRSLEEP":                               "syscall",
	"syscall.SYS___THRWAKEUP":                              "syscall",
	"syscall.S_ARCH1":                                      "syscall",
	"syscall.S_ARCH2":                                      "syscall",
	"syscall.S_BLKSIZE":                                    "syscall",
	"syscall.S_IEXEC":                                      "syscall",
	"syscall.S_IFBLK":                                      "syscall",
	"syscall.S_IFCHR":                                      "syscall",
	"syscall.S_IFDIR":                                      "syscall",
	"syscall.S_IFIFO":                                      "syscall",
	"syscall.S_IFLNK":                                      "syscall",
	"syscall.S_IFMT":                                       "syscall",
	"syscall.S_IFREG":                                      "syscall",
	"syscall.S_IFSOCK":                                     "syscall",
	"syscall.S_IFWHT":                                      "syscall",
	"syscall.S_IREAD":                                      "syscall",
	"syscall.S_IRGRP":                                      "syscall",
	"syscall.S_IROTH":                                      "syscall",
	"syscall.S_IRUSR":                                      "syscall",
	"syscall.S_IRWXG":                                      "syscall",
	"syscall.S_IRWXO":                                      "syscall",
	"syscall.S_IRWXU":                                      "syscall",
	"syscall.S_ISGID":                                      "syscall",
	"syscall.S_ISTXT":                                      "syscall",
	"syscall.S_ISUID":                                      "syscall",
	"syscall.S_ISVTX":                                      "syscall",
	"syscall.S_IWGRP":                                      "syscall",
	"syscall.S_IWOTH":                                      "syscall",
	"syscall.S_IWRITE":                                     "syscall",
	"syscall.S_IWUSR":                                      "syscall",
	"syscall.S_IXGRP":                                      "syscall",
	"syscall.S_IXOTH":                                      "syscall",
	"syscall.S_IXUSR":                                      "syscall",
	"syscall.S_LOGIN_SET":                                  "syscall",
	"syscall.SecurityAttributes":                           "syscall",
	"syscall.Seek":                                         "syscall",
	"syscall.Select":                                       "syscall",
	"syscall.Sendfile":                                     "syscall",
	"syscall.Sendmsg":                                      "syscall",
	"syscall.Sendto":                                       "syscall",
	"syscall.Servent":                                      "syscall",
	"syscall.SetBpf":                                       "syscall",
	"syscall.SetBpfBuflen":                                 "syscall",
	"syscall.SetBpfDatalink":                               "syscall",
	"syscall.SetBpfHeadercmpl":                             "syscall",
	"syscall.SetBpfImmediate":                              "syscall",
	"syscall.SetBpfInterface":                              "syscall",
	"syscall.SetBpfPromisc":                                "syscall",
	"syscall.SetBpfTimeout":                                "syscall",
	"syscall.SetCurrentDirectory":                          "syscall",
	"syscall.SetEndOfFile":                                 "syscall",
	"syscall.SetEnvironmentVariable":                       "syscall",
	"syscall.SetFileAttributes":                            "syscall",
	"syscall.SetFileCompletionNotificationModes":           "syscall",
	"syscall.SetFilePointer":                               "syscall",
	"syscall.SetFileTime":                                  "syscall",
	"syscall.SetHandleInformation":                         "syscall",
	"syscall.SetKevent":                                    "syscall",
	"syscall.SetLsfPromisc":                                "syscall",
	"syscall.SetNonblock":                                  "syscall",
	"syscall.Setdomainname":                                "syscall",
	"syscall.Setegid":                                      "syscall",
	"syscall.Setenv":                                       "syscall",
	"syscall.Seteuid":                                      "syscall",
	"syscall.Setfsgid":                                     "syscall",
	"syscall.Setfsuid":                                     "syscall",
	"syscall.Setgid":                                       "syscall",
	"syscall.Setgroups":                                    "syscall",
	"syscall.Sethostname":                                  "syscall",
	"syscall.Setlogin":                                     "syscall",
	"syscall.Setpgid":                                      "syscall",
	"syscall.Setpriority":                                  "syscall",
	"syscall.Setprivexec":                                  "syscall",
	"syscall.Setregid":                                     "syscall",
	"syscall.Setresgid":                                    "syscall",
	"syscall.Setresuid":                                    "syscall",
	"syscall.Setreuid":                                     "syscall",
	"syscall.Setrlimit":                                    "syscall",
	"syscall.Setsid":                                       "syscall",
	"syscall.Setsockopt":                                   "syscall",
	"syscall.SetsockoptByte":                               "syscall",
	"syscall.SetsockoptICMPv6Filter":                       "syscall",
	"syscall.SetsockoptIPMreq":                             "syscall",
	"syscall.SetsockoptIPMreqn":                            "syscall",
	"syscall.SetsockoptIPv6Mreq":                           "syscall",
	"syscall.SetsockoptInet4Addr":                          "syscall",
	"syscall.SetsockoptInt":                                "syscall",
	"syscall.SetsockoptLinger":                             "syscall",
	"syscall.SetsockoptString":                             "syscall",
	"syscall.SetsockoptTimeval":                            "syscall",
	"syscall.Settimeofday":                                 "syscall",
	"syscall.Setuid":                                       "syscall",
	"syscall.Setxattr":                                     "syscall",
	"syscall.Shutdown":                                     "syscall",
	"syscall.SidTypeAlias":                                 "syscall",
	"syscall.SidTypeComputer":                              "syscall",
	"syscall.SidTypeDeletedAccount":                        "syscall",
	"syscall.SidTypeDomain":                                "syscall",
	"syscall.SidTypeGroup":                                 "syscall",
	"syscall.SidTypeInvalid":                               "syscall",
	"syscall.SidTypeLabel":                                 "syscall",
	"syscall.SidTypeUnknown":                               "syscall",
	"syscall.SidTypeUser":                                  "syscall",
	"syscall.SidTypeWellKnownGroup":                        "syscall",
	"syscall.Signal":                                       "syscall",
	"syscall.SizeofBpfHdr":                                 "syscall",
	"syscall.SizeofBpfInsn":                                "syscall",
	"syscall.SizeofBpfProgram":                             "syscall",
	"syscall.SizeofBpfStat":                                "syscall",
	"syscall.SizeofBpfVersion":                             "syscall",
	"syscall.SizeofBpfZbuf":                                "syscall",
	"syscall.SizeofBpfZbufHeader":                          "syscall",
	"syscall.SizeofCmsghdr":                                "syscall",
	"syscall.SizeofICMPv6Filter":                           "syscall",
	"syscall.SizeofIPMreq":                                 "syscall",
	"syscall.SizeofIPMreqn":                                "syscall",
	"syscall.SizeofIPv6MTUInfo":                            "syscall",
	"syscall.SizeofIPv6Mreq":                               "syscall",
	"syscall.SizeofIfAddrmsg":                              "syscall",
	"syscall.SizeofIfAnnounceMsghdr":                       "syscall",
	"syscall.SizeofIfData":                                 "syscall",
	"syscall.SizeofIfInfomsg":                              "syscall",
	"syscall.SizeofIfMsghdr":                               "syscall",
	"syscall.SizeofIfaMsghdr":                              "syscall",
	"syscall.SizeofIfmaMsghdr":                             "syscall",
	"syscall.SizeofIfmaMsghdr2":                            "syscall",
	"syscall.SizeofInet4Pktinfo":                           "syscall",
	"syscall.SizeofInet6Pktinfo":                           "syscall",
	"syscall.SizeofInotifyEvent":                           "syscall",
	"syscall.SizeofLinger":                                 "syscall",
	"syscall.SizeofMsghdr":                                 "syscall",
	"syscall.SizeofNlAttr":                                 "syscall",
	"syscall.SizeofNlMsgerr":                               "syscall",
	"syscall.SizeofNlMsghdr":                               "syscall",
	"syscall.SizeofRtAttr":                                 "syscall",
	"syscall.SizeofRtGenmsg":                               "syscall",
	"syscall.SizeofRtMetrics":                              "syscall",
	"syscall.SizeofRtMsg":                                  "syscall",
	"syscall.SizeofRtMsghdr":                               "syscall",
	"syscall.SizeofRtNexthop":                              "syscall",
	"syscall.SizeofSockFilter":                             "syscall",
	"syscall.SizeofSockFprog":                              "syscall",
	"syscall.SizeofSockaddrAny":                            "syscall",
	"syscall.SizeofSockaddrDatalink":                       "syscall",
	"syscall.SizeofSockaddrInet4":                          "syscall",
	"syscall.SizeofSockaddrInet6":                          "syscall",
	"syscall.SizeofSockaddrLinklayer":                      "syscall",
	"syscall.SizeofSockaddrNetlink":                        "syscall",
	"syscall.SizeofSockaddrUnix":                           "syscall",
	"syscall.SizeofTCPInfo":                                "syscall",
	"syscall.SizeofUcred":                                  "syscall",
	"syscall.SlicePtrFromStrings":                          "syscall",
	"syscall.SockFilter":                                   "syscall",
	"syscall.SockFprog":                                    "syscall",
	"syscall.SockaddrDatalink":                             "syscall",
	"syscall.SockaddrGen":                                  "syscall",
	"syscall.SockaddrInet4":                                "syscall",
	"syscall.SockaddrInet6":                                "syscall",
	"syscall.SockaddrLinklayer":                            "syscall",
	"syscall.SockaddrNetlink":                              "syscall",
	"syscall.SockaddrUnix":                                 "syscall",
	"syscall.Socket":                                       "syscall",
	"syscall.SocketControlMessage":                         "syscall",
	"syscall.SocketDisableIPv6":                            "syscall",
	"syscall.Socketpair":                                   "syscall",
	"syscall.Splice":                                       "syscall",
	"syscall.StartProcess":                                 "syscall",
	"syscall.StartupInfo":                                  "syscall",
	"syscall.Stat":                                         "syscall",
	"syscall.Stat_t":                                       "syscall",
	"syscall.Statfs":                                       "syscall",
	"syscall.Statfs_t":                                     "syscall",
	"syscall.Stderr":                                       "syscall",
	"syscall.Stdin":                                        "syscall",
	"syscall.Stdout":                                       "syscall",
	"syscall.StringBytePtr":                                "syscall",
	"syscall.StringByteSlice":                              "syscall",
	"syscall.StringSlicePtr":                               "syscall",
	"syscall.StringToSid":                                  "syscall",
	"syscall.StringToUTF16":                                "syscall",
	"syscall.StringToUTF16Ptr":                             "syscall",
	"syscall.Symlink":                                      "syscall",
	"syscall.Sync":                                         "syscall",
	"syscall.SyncFileRange":                                "syscall",
	"syscall.SysProcAttr":                                  "syscall",
	"syscall.Syscall":                                      "syscall",
	"syscall.Syscall12":                                    "syscall",
	"syscall.Syscall15":                                    "syscall",
	"syscall.Syscall6":                                     "syscall",
	"syscall.Syscall9":                                     "syscall",
	"syscall.Sysctl":                                       "syscall",
	"syscall.SysctlUint32":                                 "syscall",
	"syscall.Sysctlnode":                                   "syscall",
	"syscall.Sysinfo":                                      "syscall",
	"syscall.Sysinfo_t":                                    "syscall",
	"syscall.Systemtime":                                   "syscall",
	"syscall.TCGETS":                                       "syscall",
	"syscall.TCIFLUSH":                                     "syscall",
	"syscall.TCIOFLUSH":                                    "syscall",
	"syscall.TCOFLUSH":                                     "syscall",
	"syscall.TCPInfo":                                      "syscall",
	"syscall.TCP_CA_NAME_MAX":                              "syscall",
	"syscall.TCP_CONGCTL":                                  "syscall",
	"syscall.TCP_CONGESTION":                               "syscall",
	"syscall.TCP_CONNECTIONTIMEOUT":                        "syscall",
	"syscall.TCP_CORK":                                     "syscall",
	"syscall.TCP_DEFER_ACCEPT":                             "syscall",
	"syscall.TCP_INFO":                                     "syscall",
	"syscall.TCP_KEEPALIVE":                                "syscall",
	"syscall.TCP_KEEPCNT":                                  "syscall",
	"syscall.TCP_KEEPIDLE":                                 "syscall",
	"syscall.TCP_KEEPINIT":                                 "syscall",
	"syscall.TCP_KEEPINTVL":                                "syscall",
	"syscall.TCP_LINGER2":                                  "syscall",
	"syscall.TCP_MAXBURST":                                 "syscall",
	"syscall.TCP_MAXHLEN":                                  "syscall",
	"syscall.TCP_MAXOLEN":                                  "syscall",
	"syscall.TCP_MAXSEG":                                   "syscall",
	"syscall.TCP_MAXWIN":                                   "syscall",
	"syscall.TCP_MAX_SACK":                                 "syscall",
	"syscall.TCP_MAX_WINSHIFT":                             "syscall",
	"syscall.TCP_MD5SIG":                                   "syscall",
	"syscall.TCP_MD5SIG_MAXKEYLEN":                         "syscall",
	"syscall.TCP_MINMSS":                                   "syscall",
	"syscall.TCP_MINMSSOVERLOAD":                           "syscall",
	"syscall.TCP_MSS":                                      "syscall",
	"syscall.TCP_NODELAY":                                  "syscall",
	"syscall.TCP_NOOPT":                                    "syscall",
	"syscall.TCP_NOPUSH":                                   "syscall",
	"syscall.TCP_NSTATES":                                  "syscall",
	"syscall.TCP_QUICKACK":                                 "syscall",
	"syscall.TCP_RXT_CONNDROPTIME":                         "syscall",
	"syscall.TCP_RXT_FINDROP":                              "syscall",
	"syscall.TCP_SACK_ENABLE":                              "syscall",
	"syscall.TCP_SYNCNT":                                   "syscall",
	"syscall.TCP_WINDOW_CLAMP":                             "syscall",
	"syscall.TCSAFLUSH":                                    "syscall",
	"syscall.TCSETS":                                       "syscall",
	"syscall.TF_DISCONNECT":                                "syscall",
	"syscall.TF_REUSE_SOCKET":                              "syscall",
	"syscall.TF_USE_DEFAULT_WORKER":                        "syscall",
	"syscall.TF_USE_KERNEL_APC":                            "syscall",
	"syscall.TF_USE_SYSTEM_THREAD":                         "syscall",
	"syscall.TF_WRITE_BEHIND":                              "syscall",
	"syscall.TIME_ZONE_ID_DAYLIGHT":                        "syscall",
	"syscall.TIME_ZONE_ID_STANDARD":                        "syscall",
	"syscall.TIME_ZONE_ID_UNKNOWN":                         "syscall",
	"syscall.TIOCCBRK":                                     "syscall",
	"syscall.TIOCCDTR":                                     "syscall",
	"syscall.TIOCCONS":                                     "syscall",
	"syscall.TIOCDCDTIMESTAMP":                             "syscall",
	"syscall.TIOCDRAIN":                                    "syscall",
	"syscall.TIOCDSIMICROCODE":                             "syscall",
	"syscall.TIOCEXCL":                                     "syscall",
	"syscall.TIOCEXT":                                      "syscall",
	"syscall.TIOCFLAG_CDTRCTS":                             "syscall",
	"syscall.TIOCFLAG_CLOCAL":                              "syscall",
	"syscall.TIOCFLAG_CRTSCTS":                             "syscall",
	"syscall.TIOCFLAG_MDMBUF":                              "syscall",
	"syscall.TIOCFLAG_PPS":                                 "syscall",
	"syscall.TIOCFLAG_SOFTCAR":                             "syscall",
	"syscall.TIOCFLUSH":                                    "syscall",
	"syscall.TIOCGDEV":                                     "syscall",
	"syscall.TIOCGDRAINWAIT":                               "syscall",
	"syscall.TIOCGETA":                                     "syscall",
	"syscall.TIOCGETD":                                     "syscall",
	"syscall.TIOCGFLAGS":                                   "syscall",
	"syscall.TIOCGICOUNT":                                  "syscall",
	"syscall.TIOCGLCKTRMIOS":                               "syscall",
	"syscall.TIOCGLINED":                                   "syscall",
	"syscall.TIOCGPGRP":                                    "syscall",
	"syscall.TIOCGPTN":                                     "syscall",
	"syscall.TIOCGQSIZE":                                   "syscall",
	"syscall.TIOCGRANTPT":                                  "syscall",
	"syscall.TIOCGRS485":                                   "syscall",
	"syscall.TIOCGSERIAL":                                  "syscall",
	"syscall.TIOCGSID":                                     "syscall",
	"syscall.TIOCGSIZE":                                    "syscall",
	"syscall.TIOCGSOFTCAR":                                 "syscall",
	"syscall.TIOCGTSTAMP":                                  "syscall",
	"syscall.TIOCGWINSZ":                                   "syscall",
	"syscall.TIOCINQ":                                      "syscall",
	"syscall.TIOCIXOFF":                                    "syscall",
	"syscall.TIOCIXON":                                     "syscall",
	"syscall.TIOCLINUX":                                    "syscall",
	"syscall.TIOCMBIC":                                     "syscall",
	"syscall.TIOCMBIS":                                     "syscall",
	"syscall.TIOCMGDTRWAIT":                                "syscall",
	"syscall.TIOCMGET":                                     "syscall",
	"syscall.TIOCMIWAIT":                                   "syscall",
	"syscall.TIOCMODG":                                     "syscall",
	"syscall.TIOCMODS":                                     "syscall",
	"syscall.TIOCMSDTRWAIT":                                "syscall",
	"syscall.TIOCMSET":                                     "syscall",
	"syscall.TIOCM_CAR":                                    "syscall",
	"syscall.TIOCM_CD":                                     "syscall",
	"syscall.TIOCM_CTS":                                    "syscall",
	"syscall.TIOCM_DCD":                                    "syscall",
	"syscall.TIOCM_DSR":                                    "syscall",
	"syscall.TIOCM_DTR":                                    "syscall",
	"syscall.TIOCM_LE":                                     "syscall",
	"syscall.TIOCM_RI":                                     "syscall",
	"syscall.TIOCM_RNG":                                    "syscall",
	"syscall.TIOCM_RTS":                                    "syscall",
	"syscall.TIOCM_SR":                                     "syscall",
	"syscall.TIOCM_ST":                                     "syscall",
	"syscall.TIOCNOTTY":                                    "syscall",
	"syscall.TIOCNXCL":                                     "syscall",
	"syscall.TIOCOUTQ":                                     "syscall",
	"syscall.TIOCPKT":                                      "syscall",
	"syscall.TIOCPKT_DATA":                                 "syscall",
	"syscall.TIOCPKT_DOSTOP":                               "syscall",
	"syscall.TIOCPKT_FLUSHREAD":                            "syscall",
	"syscall.TIOCPKT_FLUSHWRITE":                           "syscall",
	"syscall.TIOCPKT_IOCTL":                                "syscall",
	"syscall.TIOCPKT_NOSTOP":                               "syscall",
	"syscall.TIOCPKT_START":                                "syscall",
	"syscall.TIOCPKT_STOP":                                 "syscall",
	"syscall.TIOCPTMASTER":                                 "syscall",
	"syscall.TIOCPTMGET":                                   "syscall",
	"syscall.TIOCPTSNAME":                                  "syscall",
	"syscall.TIOCPTYGNAME":                                 "syscall",
	"syscall.TIOCPTYGRANT":                                 "syscall",
	"syscall.TIOCPTYUNLK":                                  "syscall",
	"syscall.TIOCRCVFRAME":                                 "syscall",
	"syscall.TIOCREMOTE":                                   "syscall",
	"syscall.TIOCSBRK":                                     "syscall",
	"syscall.TIOCSCONS":                                    "syscall",
	"syscall.TIOCSCTTY":                                    "syscall",
	"syscall.TIOCSDRAINWAIT":                               "syscall",
	"syscall.TIOCSDTR":                                     "syscall",
	"syscall.TIOCSERCONFIG":                                "syscall",
	"syscall.TIOCSERGETLSR":                                "syscall",
	"syscall.TIOCSERGETMULTI":                              "syscall",
	"syscall.TIOCSERGSTRUCT":                               "syscall",
	"syscall.TIOCSERGWILD":                                 "syscall",
	"syscall.TIOCSERSETMULTI":                              "syscall",
	"syscall.TIOCSERSWILD":                                 "syscall",
	"syscall.TIOCSER_TEMT":                                 "syscall",
	"syscall.TIOCSETA":                                     "syscall",
	"syscall.TIOCSETAF":                                    "syscall",
	"syscall.TIOCSETAW":                                    "syscall",
	"syscall.TIOCSETD":                                     "syscall",
	"syscall.TIOCSFLAGS":                                   "syscall",
	"syscall.TIOCSIG":                                      "syscall",
	"syscall.TIOCSLCKTRMIOS":                               "syscall",
	"syscall.TIOCSLINED":                                   "syscall",
	"syscall.TIOCSPGRP":                                    "syscall",
	"syscall.TIOCSPTLCK":                                   "syscall",
	"syscall.TIOCSQSIZE":                                   "syscall",
	"syscall.TIOCSRS485":                                   "syscall",
	"syscall.TIOCSSERIAL":                                  "syscall",
	"syscall.TIOCSSIZE":                                    "syscall",
	"syscall.TIOCSSOFTCAR":                                 "syscall",
	"syscall.TIOCSTART":                                    "syscall",
	"syscall.TIOCSTAT":                                     "syscall",
	"syscall.TIOCSTI":                                      "syscall",
	"syscall.TIOCSTOP":                                     "syscall",
	"syscall.TIOCSTSTAMP":                                  "syscall",
	"syscall.TIOCSWINSZ":                                   "syscall",
	"syscall.TIOCTIMESTAMP":                                "syscall",
	"syscall.TIOCUCNTL":                                    "syscall",
	"syscall.TIOCVHANGUP":                                  "syscall",
	"syscall.TIOCXMTFRAME":                                 "syscall",
	"syscall.TOKEN_ADJUST_DEFAULT":                         "syscall",
	"syscall.TOKEN_ADJUST_GROUPS":                          "syscall",
	"syscall.TOKEN_ADJUST_PRIVILEGES":                      "syscall",
	"syscall.TOKEN_ALL_ACCESS":                             "syscall",
	"syscall.TOKEN_ASSIGN_PRIMARY":                         "syscall",
	"syscall.TOKEN_DUPLICATE":                              "syscall",
	"syscall.TOKEN_EXECUTE":                                "syscall",
	"syscall.TOKEN_IMPERSONATE":                            "syscall",
	"syscall.TOKEN_QUERY":                                  "syscall",
	"syscall.TOKEN_QUERY_SOURCE":                           "syscall",
	"syscall.TOKEN_READ":                                   "syscall",
	"syscall.TOKEN_WRITE":                                  "syscall",
	"syscall.TOSTOP":                                       "syscall",
	"syscall.TRUNCATE_EXISTING":                            "syscall",
	"syscall.TUNATTACHFILTER":                              "syscall",
	"syscall.TUNDETACHFILTER":                              "syscall",
	"syscall.TUNGETFEATURES":                               "syscall",
	"syscall.TUNGETIFF":                                    "syscall",
	"syscall.TUNGETSNDBUF":                                 "syscall",
	"syscall.TUNGETVNETHDRSZ":                              "syscall",
	"syscall.TUNSETDEBUG":                                  "syscall",
	"syscall.TUNSETGROUP":                                  "syscall",
	"syscall.TUNSETIFF":                                    "syscall",
	"syscall.TUNSETLINK":                                   "syscall",
	"syscall.TUNSETNOCSUM":                                 "syscall",
	"syscall.TUNSETOFFLOAD":                                "syscall",
	"syscall.TUNSETOWNER":                                  "syscall",
	"syscall.TUNSETPERSIST":                                "syscall",
	"syscall.TUNSETSNDBUF":                                 "syscall",
	"syscall.TUNSETTXFILTER":                               "syscall",
	"syscall.TUNSETVNETHDRSZ":                              "syscall",
	"syscall.Tee":                                          "syscall",
	"syscall.TerminateProcess":                             "syscall",
	"syscall.Termios":                                      "syscall",
	"syscall.Tgkill":                                       "syscall",
	"syscall.Time":                                         "syscall",
	"syscall.Time_t":                                       "syscall",
	"syscall.Times":                                        "syscall",
	"syscall.Timespec":                                     "syscall",
	"syscall.TimespecToNsec":                               "syscall",
	"syscall.Timeval":                                      "syscall",
	"syscall.Timeval32":                                    "syscall",
	"syscall.TimevalToNsec":                                "syscall",
	"syscall.Timex":                                        "syscall",
	"syscall.Timezoneinformation":                          "syscall",
	"syscall.Tms":                                          "syscall",
	"syscall.Token":                                        "syscall",
	"syscall.TokenAccessInformation":                       "syscall",
	"syscall.TokenAuditPolicy":                             "syscall",
	"syscall.TokenDefaultDacl":                             "syscall",
	"syscall.TokenElevation":                               "syscall",
	"syscall.TokenElevationType":                           "syscall",
	"syscall.TokenGroups":                                  "syscall",
	"syscall.TokenGroupsAndPrivileges":                     "syscall",
	"syscall.TokenHasRestrictions":                         "syscall",
	"syscall.TokenImpersonationLevel":                      "syscall",
	"syscall.TokenIntegrityLevel":                          "syscall",
	"syscall.TokenLinkedToken":                             "syscall",
	"syscall.TokenLogonSid":                                "syscall",
	"syscall.TokenMandatoryPolicy":                         "syscall",
	"syscall.TokenOrigin":                                  "syscall",
	"syscall.TokenOwner":                                   "syscall",
	"syscall.TokenPrimaryGroup":                            "syscall",
	"syscall.TokenPrivileges":                              "syscall",
	"syscall.TokenRestrictedSids":                          "syscall",
	"syscall.TokenSandBoxInert":                            "syscall",
	"syscall.TokenSessionId":                               "syscall",
	"syscall.TokenSessionReference":                        "syscall",
	"syscall.TokenSource":                                  "syscall",
	"syscall.TokenStatistics":                              "syscall",
	"syscall.TokenType":                                    "syscall",
	"syscall.TokenUIAccess":                                "syscall",
	"syscall.TokenUser":                                    "syscall",
	"syscall.TokenVirtualizationAllowed":                   "syscall",
	"syscall.TokenVirtualizationEnabled":                   "syscall",
	"syscall.Tokenprimarygroup":                            "syscall",
	"syscall.Tokenuser":                                    "syscall",
	"syscall.TranslateAccountName":                         "syscall",
	"syscall.TranslateName":                                "syscall",
	"syscall.TransmitFile":                                 "syscall",
	"syscall.TransmitFileBuffers":                          "syscall",
	"syscall.Truncate":                                     "syscall",
	"syscall.USAGE_MATCH_TYPE_AND":                         "syscall",
	"syscall.USAGE_MATCH_TYPE_OR":                          "syscall",
	"syscall.UTF16FromString":                              "syscall",
	"syscall.UTF16PtrFromString":                           "syscall",
	"syscall.UTF16ToString":                                "syscall",
	"syscall.Ucred":                                        "syscall",
	"syscall.Umask":                                        "syscall",
	"syscall.Uname":                                        "syscall",
	"syscall.Undelete":                                     "syscall",
	"syscall.UnixCredentials":                              "syscall",
	"syscall.UnixRights":                                   "syscall",
	"syscall.Unlink":                                       "syscall",
	"syscall.Unlinkat":                                     "syscall",
	"syscall.UnmapViewOfFile":                              "syscall",
	"syscall.Unmount":                                      "syscall",
	"syscall.Unshare":                                      "syscall",
	"syscall.UserInfo10":                                   "syscall",
	"syscall.Ustat":                                        "syscall",
	"syscall.Ustat_t":                                      "syscall",
	"syscall.Utimbuf":                                      "syscall",
	"syscall.Utime":                                        "syscall",
	"syscall.Utimes":                                       "syscall",
	"syscall.UtimesNano":                                   "syscall",
	"syscall.Utsname":                                      "syscall",
	"syscall.VDISCARD":                                     "syscall",
	"syscall.VDSUSP":                                       "syscall",
	"syscall.VEOF":                                         "syscall",
	"syscall.VEOL":                                         "syscall",
	"syscall.VEOL2":                                        "syscall",
	"syscall.VERASE":                                       "syscall",
	"syscall.VERASE2":                                      "syscall",
	"syscall.VINTR":                                        "syscall",
	"syscall.VKILL":                                        "syscall",
	"syscall.VLNEXT":                                       "syscall",
	"syscall.VMIN":                                         "syscall",
	"syscall.VQUIT":                                        "syscall",
	"syscall.VREPRINT":                                     "syscall",
	"syscall.VSTART":                                       "syscall",
	"syscall.VSTATUS":                                      "syscall",
	"syscall.VSTOP":                                        "syscall",
	"syscall.VSUSP":                                        "syscall",
	"syscall.VSWTC":                                        "syscall",
	"syscall.VT0":                                          "syscall",
	"syscall.VT1":                                          "syscall",
	"syscall.VTDLY":                                        "syscall",
	"syscall.VTIME":                                        "syscall",
	"syscall.VWERASE":                                      "syscall",
	"syscall.VirtualLock":                                  "syscall",
	"syscall.VirtualUnlock":                                "syscall",
	"syscall.WAIT_ABANDONED":                               "syscall",
	"syscall.WAIT_FAILED":                                  "syscall",
	"syscall.WAIT_OBJECT_0":                                "syscall",
	"syscall.WAIT_TIMEOUT":                                 "syscall",
	"syscall.WALL":                                         "syscall",
	"syscall.WALLSIG":                                      "syscall",
	"syscall.WALTSIG":                                      "syscall",
	"syscall.WCLONE":                                       "syscall",
	"syscall.WCONTINUED":                                   "syscall",
	"syscall.WCOREFLAG":                                    "syscall",
	"syscall.WEXITED":                                      "syscall",
	"syscall.WLINUXCLONE":                                  "syscall",
	"syscall.WNOHANG":                                      "syscall",
	"syscall.WNOTHREAD":                                    "syscall",
	"syscall.WNOWAIT":                                      "syscall",
	"syscall.WNOZOMBIE":                                    "syscall",
	"syscall.WOPTSCHECKED":                                 "syscall",
	"syscall.WORDSIZE":                                     "syscall",
	"syscall.WSABuf":                                       "syscall",
	"syscall.WSACleanup":                                   "syscall",
	"syscall.WSADESCRIPTION_LEN":                           "syscall",
	"syscall.WSAData":                                      "syscall",
	"syscall.WSAEACCES":                                    "syscall",
	"syscall.WSAEnumProtocols":                             "syscall",
	"syscall.WSAID_CONNECTEX":                              "syscall",
	"syscall.WSAIoctl":                                     "syscall",
	"syscall.WSAPROTOCOL_LEN":                              "syscall",
	"syscall.WSAProtocolChain":                             "syscall",
	"syscall.WSAProtocolInfo":                              "syscall",
	"syscall.WSARecv":                                      "syscall",
	"syscall.WSARecvFrom":                                  "syscall",
	"syscall.WSASYS_STATUS_LEN":                            "syscall",
	"syscall.WSASend":                                      "syscall",
	"syscall.WSASendTo":                                    "syscall",
	"syscall.WSASendto":                                    "syscall",
	"syscall.WSAStartup":                                   "syscall",
	"syscall.WSTOPPED":                                     "syscall",
	"syscall.WTRAPPED":                                     "syscall",
	"syscall.WUNTRACED":                                    "syscall",
	"syscall.Wait4":                                        "syscall",
	"syscall.WaitForSingleObject":                          "syscall",
	"syscall.WaitStatus":                                   "syscall",
	"syscall.Win32FileAttributeData":                       "syscall",
	"syscall.Win32finddata":                                "syscall",
	"syscall.Write":                                        "syscall",
	"syscall.WriteConsole":                                 "syscall",
	"syscall.WriteFile":                                    "syscall",
	"syscall.X509_ASN_ENCODING":                            "syscall",
	"syscall.XCASE":                                        "syscall",
	"syscall.XP1_CONNECTIONLESS":                           "syscall",
	"syscall.XP1_CONNECT_DATA":                             "syscall",
	"syscall.XP1_DISCONNECT_DATA":                          "syscall",
	"syscall.XP1_EXPEDITED_DATA":                           "syscall",
	"syscall.XP1_GRACEFUL_CLOSE":                           "syscall",
	"syscall.XP1_GUARANTEED_DELIVERY":                      "syscall",
	"syscall.XP1_GUARANTEED_ORDER":                         "syscall",
	"syscall.XP1_IFS_HANDLES":                              "syscall",
	"syscall.XP1_MESSAGE_ORIENTED":                         "syscall",
	"syscall.XP1_MULTIPOINT_CONTROL_PLANE":                 "syscall",
	"syscall.XP1_MULTIPOINT_DATA_PLANE":                    "syscall",
	"syscall.XP1_PARTIAL_MESSAGE":                          "syscall",
	"syscall.XP1_PSEUDO_STREAM":                            "syscall",
	"syscall.XP1_QOS_SUPPORTED":                            "syscall",
	"syscall.XP1_SAN_SUPPORT_SDP":                          "syscall",
	"syscall.XP1_SUPPORT_BROADCAST":                        "syscall",
	"syscall.XP1_SUPPORT_MULTIPOINT":                       "syscall",
	"syscall.XP1_UNI_RECV":                                 "syscall",
	"syscall.XP1_UNI_SEND":                                 "syscall",
	"syslog.Dial":                                          "log/syslog",
	"syslog.LOG_ALERT":                                     "log/syslog",
	"syslog.LOG_AUTH":                                      "log/syslog",
	"syslog.LOG_AUTHPRIV":                                  "log/syslog",
	"syslog.LOG_CRIT":                                      "log/syslog",
	"syslog.LOG_CRON":                                      "log/syslog",
	"syslog.LOG_DAEMON":                                    "log/syslog",
	"syslog.LOG_DEBUG":                                     "log/syslog",
	"syslog.LOG_EMERG":                                     "log/syslog",
	"syslog.LOG_ERR":                                       "log/syslog",
	"syslog.LOG_FTP":                                       "log/syslog",
	"syslog.LOG_INFO":                                      "log/syslog",
	"syslog.LOG_KERN":                                      "log/syslog",
	"syslog.LOG_LOCAL0":                                    "log/syslog",
	"syslog.LOG_LOCAL1":                                    "log/syslog",
	"syslog.LOG_LOCAL2":                                    "log/syslog",
	"syslog.LOG_LOCAL3":                                    "log/syslog",
	"syslog.LOG_LOCAL4":                                    "log/syslog",
	"syslog.LOG_LOCAL5":                                    "log/syslog",
	"syslog.LOG_LOCAL6":                                    "log/syslog",
	"syslog.LOG_LOCAL7":                                    "log/syslog",
	"syslog.LOG_LPR":                                       "log/syslog",
	"syslog.LOG_MAIL":                                      "log/syslog",
	"syslog.LOG_NEWS":                                      "log/syslog",
	"syslog.LOG_NOTICE":                                    "log/syslog",
	"syslog.LOG_SYSLOG":                                    "log/syslog",
	"syslog.LOG_USER":                                      "log/syslog",
	"syslog.LOG_UUCP":                                      "log/syslog",
	"syslog.LOG_WARNING":                                   "log/syslog",
	"syslog.New":                                           "log/syslog",
	"syslog.NewLogger":                                     "log/syslog",
	"syslog.Priority":                                      "log/syslog",
	"syslog.Writer":                                        "log/syslog",
	"tabwriter.AlignRight":                                 "text/tabwriter",
	"tabwriter.Debug":                                      "text/tabwriter",
	"tabwriter.DiscardEmptyColumns":                        "text/tabwriter",
	"tabwriter.Escape":                                     "text/tabwriter",
	"tabwriter.FilterHTML":                                 "text/tabwriter",
	"tabwriter.NewWriter":                                  "text/tabwriter",
	"tabwriter.StripEscape":                                "text/tabwriter",
	"tabwriter.TabIndent":                                  "text/tabwriter",
	"tabwriter.Writer":                                     "text/tabwriter",
	"tar.ErrFieldTooLong":                                  "archive/tar",
	"tar.ErrHeader":                                        "archive/tar",
	"tar.ErrWriteAfterClose":                               "archive/tar",
	"tar.ErrWriteTooLong":                                  "archive/tar",
	"tar.FileInfoHeader":                                   "archive/tar",
	"tar.Header":                                           "archive/tar",
	"tar.NewReader":                                        "archive/tar",
	"tar.NewWriter":                                        "archive/tar",
	"tar.Reader":                                           "archive/tar",
	"tar.TypeBlock":                                        "archive/tar",
	"tar.TypeChar":                                         "archive/tar",
	"tar.TypeCont":                                         "archive/tar",
	"tar.TypeDir":                                          "archive/tar",
	"tar.TypeFifo":                                         "archive/tar",
	"tar.TypeGNULongLink":                                  "archive/tar",
	"tar.TypeGNULongName":                                  "archive/tar",
	"tar.TypeLink":                                         "archive/tar",
	"tar.TypeReg":                                          "archive/tar",
	"tar.TypeRegA":                                         "archive/tar",
	"tar.TypeSymlink":                                      "archive/tar",
	"tar.TypeXGlobalHeader":                                "archive/tar",
	"tar.TypeXHeader":                                      "archive/tar",
	"tar.Writer":                                           "archive/tar",
	"template.CSS":                                         "html/template",
	"template.ErrAmbigContext":                             "html/template",
	"template.ErrBadHTML":                                  "html/template",
	"template.ErrBranchEnd":                                "html/template",
	"template.ErrEndContext":                               "html/template",
	"template.ErrNoSuchTemplate":                           "html/template",
	"template.ErrOutputContext":                            "html/template",
	"template.ErrPartialCharset":                           "html/template",
	"template.ErrPartialEscape":                            "html/template",
	"template.ErrRangeLoopReentry":                         "html/template",
	"template.ErrSlashAmbig":                               "html/template",
	"template.Error":                                       "html/template",
	"template.ErrorCode":                                   "html/template",
	// "template.FuncMap" is ambiguous
	"template.HTML":     "html/template",
	"template.HTMLAttr": "html/template",
	// "template.HTMLEscape" is ambiguous
	// "template.HTMLEscapeString" is ambiguous
	// "template.HTMLEscaper" is ambiguous
	"template.JS": "html/template",
	// "template.JSEscape" is ambiguous
	// "template.JSEscapeString" is ambiguous
	// "template.JSEscaper" is ambiguous
	"template.JSStr": "html/template",
	// "template.Must" is ambiguous
	// "template.New" is ambiguous
	"template.OK": "html/template",
	// "template.ParseFiles" is ambiguous
	// "template.ParseGlob" is ambiguous
	// "template.Template" is ambiguous
	"template.URL": "html/template",
	// "template.URLQueryEscaper" is ambiguous
	"testing.AllocsPerRun":                        "testing",
	"testing.B":                                   "testing",
	"testing.Benchmark":                           "testing",
	"testing.BenchmarkResult":                     "testing",
	"testing.Cover":                               "testing",
	"testing.CoverBlock":                          "testing",
	"testing.InternalBenchmark":                   "testing",
	"testing.InternalExample":                     "testing",
	"testing.InternalTest":                        "testing",
	"testing.Main":                                "testing",
	"testing.RegisterCover":                       "testing",
	"testing.RunBenchmarks":                       "testing",
	"testing.RunExamples":                         "testing",
	"testing.RunTests":                            "testing",
	"testing.Short":                               "testing",
	"testing.T":                                   "testing",
	"testing.Verbose":                             "testing",
	"textproto.CanonicalMIMEHeaderKey":            "net/textproto",
	"textproto.Conn":                              "net/textproto",
	"textproto.Dial":                              "net/textproto",
	"textproto.Error":                             "net/textproto",
	"textproto.MIMEHeader":                        "net/textproto",
	"textproto.NewConn":                           "net/textproto",
	"textproto.NewReader":                         "net/textproto",
	"textproto.NewWriter":                         "net/textproto",
	"textproto.Pipeline":                          "net/textproto",
	"textproto.ProtocolError":                     "net/textproto",
	"textproto.Reader":                            "net/textproto",
	"textproto.TrimBytes":                         "net/textproto",
	"textproto.TrimString":                        "net/textproto",
	"textproto.Writer":                            "net/textproto",
	"time.ANSIC":                                  "time",
	"time.After":                                  "time",
	"time.AfterFunc":                              "time",
	"time.April":                                  "time",
	"time.August":                                 "time",
	"time.Date":                                   "time",
	"time.December":                               "time",
	"time.Duration":                               "time",
	"time.February":                               "time",
	"time.FixedZone":                              "time",
	"time.Friday":                                 "time",
	"time.Hour":                                   "time",
	"time.January":                                "time",
	"time.July":                                   "time",
	"time.June":                                   "time",
	"time.Kitchen":                                "time",
	"time.LoadLocation":                           "time",
	"time.Local":                                  "time",
	"time.Location":                               "time",
	"time.March":                                  "time",
	"time.May":                                    "time",
	"time.Microsecond":                            "time",
	"time.Millisecond":                            "time",
	"time.Minute":                                 "time",
	"time.Monday":                                 "time",
	"time.Month":                                  "time",
	"time.Nanosecond":                             "time",
	"time.NewTicker":                              "time",
	"time.NewTimer":                               "time",
	"time.November":                               "time",
	"time.Now":                                    "time",
	"time.October":                                "time",
	"time.Parse":                                  "time",
	"time.ParseDuration":                          "time",
	"time.ParseError":                             "time",
	"time.ParseInLocation":                        "time",
	"time.RFC1123":                                "time",
	"time.RFC1123Z":                               "time",
	"time.RFC3339":                                "time",
	"time.RFC3339Nano":                            "time",
	"time.RFC822":                                 "time",
	"time.RFC822Z":                                "time",
	"time.RFC850":                                 "time",
	"time.RubyDate":                               "time",
	"time.Saturday":                               "time",
	"time.Second":                                 "time",
	"time.September":                              "time",
	"time.Since":                                  "time",
	"time.Sleep":                                  "time",
	"time.Stamp":                                  "time",
	"time.StampMicro":                             "time",
	"time.StampMilli":                             "time",
	"time.StampNano":                              "time",
	"time.Sunday":                                 "time",
	"time.Thursday":                               "time",
	"time.Tick":                                   "time",
	"time.Ticker":                                 "time",
	"time.Time":                                   "time",
	"time.Timer":                                  "time",
	"time.Tuesday":                                "time",
	"time.UTC":                                    "time",
	"time.Unix":                                   "time",
	"time.UnixDate":                               "time",
	"time.Wednesday":                              "time",
	"time.Weekday":                                "time",
	"tls.Certificate":                             "crypto/tls",
	"tls.Client":                                  "crypto/tls",
	"tls.ClientAuthType":                          "crypto/tls",
	"tls.Config":                                  "crypto/tls",
	"tls.Conn":                                    "crypto/tls",
	"tls.ConnectionState":                         "crypto/tls",
	"tls.Dial":                                    "crypto/tls",
	"tls.Listen":                                  "crypto/tls",
	"tls.LoadX509KeyPair":                         "crypto/tls",
	"tls.NewListener":                             "crypto/tls",
	"tls.NoClientCert":                            "crypto/tls",
	"tls.RequestClientCert":                       "crypto/tls",
	"tls.RequireAndVerifyClientCert":              "crypto/tls",
	"tls.RequireAnyClientCert":                    "crypto/tls",
	"tls.Server":                                  "crypto/tls",
	"tls.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA":    "crypto/tls",
	"tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256": "crypto/tls",
	"tls.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA":    "crypto/tls",
	"tls.TLS_ECDHE_ECDSA_WITH_RC4_128_SHA":        "crypto/tls",
	"tls.TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA":     "crypto/tls",
	"tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA":      "crypto/tls",
	"tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256":   "crypto/tls",
	"tls.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA":      "crypto/tls",
	"tls.TLS_ECDHE_RSA_WITH_RC4_128_SHA":          "crypto/tls",
	"tls.TLS_RSA_WITH_3DES_EDE_CBC_SHA":           "crypto/tls",
	"tls.TLS_RSA_WITH_AES_128_CBC_SHA":            "crypto/tls",
	"tls.TLS_RSA_WITH_AES_256_CBC_SHA":            "crypto/tls",
	"tls.TLS_RSA_WITH_RC4_128_SHA":                "crypto/tls",
	"tls.VerifyClientCertIfGiven":                 "crypto/tls",
	"tls.VersionSSL30":                            "crypto/tls",
	"tls.VersionTLS10":                            "crypto/tls",
	"tls.VersionTLS11":                            "crypto/tls",
	"tls.VersionTLS12":                            "crypto/tls",
	"tls.X509KeyPair":                             "crypto/tls",
	"token.ADD":                                   "go/token",
	"token.ADD_ASSIGN":                            "go/token",
	"token.AND":                                   "go/token",
	"token.AND_ASSIGN":                            "go/token",
	"token.AND_NOT":                               "go/token",
	"token.AND_NOT_ASSIGN":                        "go/token",
	"token.ARROW":                                 "go/token",
	"token.ASSIGN":                                "go/token",
	"token.BREAK":                                 "go/token",
	"token.CASE":                                  "go/token",
	"token.CHAN":                                  "go/token",
	"token.CHAR":                                  "go/token",
	"token.COLON":                                 "go/token",
	"token.COMMA":                                 "go/token",
	"token.COMMENT":                               "go/token",
	"token.CONST":                                 "go/token",
	"token.CONTINUE":                              "go/token",
	"token.DEC":                                   "go/token",
	"token.DEFAULT":                               "go/token",
	"token.DEFER":                                 "go/token",
	"token.DEFINE":                                "go/token",
	"token.ELLIPSIS":                              "go/token",
	"token.ELSE":                                  "go/token",
	"token.EOF":                                   "go/token",
	"token.EQL":                                   "go/token",
	"token.FALLTHROUGH":                           "go/token",
	"token.FLOAT":                                 "go/token",
	"token.FOR":                                   "go/token",
	"token.FUNC":                                  "go/token",
	"token.File":                                  "go/token",
	"token.FileSet":                               "go/token",
	"token.GEQ":                                   "go/token",
	"token.GO":                                    "go/token",
	"token.GOTO":                                  "go/token",
	"token.GTR":                                   "go/token",
	"token.HighestPrec":                           "go/token",
	"token.IDENT":                                 "go/token",
	"token.IF":                                    "go/token",
	"token.ILLEGAL":                               "go/token",
	"token.IMAG":                                  "go/token",
	"token.IMPORT":                                "go/token",
	"token.INC":                                   "go/token",
	"token.INT":                                   "go/token",
	"token.INTERFACE":                             "go/token",
	"token.LAND":                                  "go/token",
	"token.LBRACE":                                "go/token",
	"token.LBRACK":                                "go/token",
	"token.LEQ":                                   "go/token",
	"token.LOR":                                   "go/token",
	"token.LPAREN":                                "go/token",
	"token.LSS":                                   "go/token",
	"token.Lookup":                                "go/token",
	"token.LowestPrec":                            "go/token",
	"token.MAP":                                   "go/token",
	"token.MUL":                                   "go/token",
	"token.MUL_ASSIGN":                            "go/token",
	"token.NEQ":                                   "go/token",
	"token.NOT":                                   "go/token",
	"token.NewFileSet":                            "go/token",
	"token.NoPos":                                 "go/token",
	"token.OR":                                    "go/token",
	"token.OR_ASSIGN":                             "go/token",
	"token.PACKAGE":                               "go/token",
	"token.PERIOD":                                "go/token",
	"token.Pos":                                   "go/token",
	"token.Position":                              "go/token",
	"token.QUO":                                   "go/token",
	"token.QUO_ASSIGN":                            "go/token",
	"token.RANGE":                                 "go/token",
	"token.RBRACE":                                "go/token",
	"token.RBRACK":                                "go/token",
	"token.REM":                                   "go/token",
	"token.REM_ASSIGN":                            "go/token",
	"token.RETURN":                                "go/token",
	"token.RPAREN":                                "go/token",
	"token.SELECT":                                "go/token",
	"token.SEMICOLON":                             "go/token",
	"token.SHL":                                   "go/token",
	"token.SHL_ASSIGN":                            "go/token",
	"token.SHR":                                   "go/token",
	"token.SHR_ASSIGN":                            "go/token",
	"token.STRING":                                "go/token",
	"token.STRUCT":                                "go/token",
	"token.SUB":                                   "go/token",
	"token.SUB_ASSIGN":                            "go/token",
	"token.SWITCH":                                "go/token",
	"token.TYPE":                                  "go/token",
	"token.Token":                                 "go/token",
	"token.UnaryPrec":                             "go/token",
	"token.VAR":                                   "go/token",
	"token.XOR":                                   "go/token",
	"token.XOR_ASSIGN":                            "go/token",
	"unicode.ASCII_Hex_Digit":                     "unicode",
	"unicode.Arabic":                              "unicode",
	"unicode.Armenian":                            "unicode",
	"unicode.Avestan":                             "unicode",
	"unicode.AzeriCase":                           "unicode",
	"unicode.Balinese":                            "unicode",
	"unicode.Bamum":                               "unicode",
	"unicode.Batak":                               "unicode",
	"unicode.Bengali":                             "unicode",
	"unicode.Bidi_Control":                        "unicode",
	"unicode.Bopomofo":                            "unicode",
	"unicode.Brahmi":                              "unicode",
	"unicode.Braille":                             "unicode",
	"unicode.Buginese":                            "unicode",
	"unicode.Buhid":                               "unicode",
	"unicode.C":                                   "unicode",
	"unicode.Canadian_Aboriginal":                 "unicode",
	"unicode.Carian":                              "unicode",
	"unicode.CaseRange":                           "unicode",
	"unicode.CaseRanges":                          "unicode",
	"unicode.Categories":                          "unicode",
	"unicode.Cc":                                  "unicode",
	"unicode.Cf":                                  "unicode",
	"unicode.Chakma":                              "unicode",
	"unicode.Cham":                                "unicode",
	"unicode.Cherokee":                            "unicode",
	"unicode.Co":                                  "unicode",
	"unicode.Common":                              "unicode",
	"unicode.Coptic":                              "unicode",
	"unicode.Cs":                                  "unicode",
	"unicode.Cuneiform":                           "unicode",
	"unicode.Cypriot":                             "unicode",
	"unicode.Cyrillic":                            "unicode",
	"unicode.Dash":                                "unicode",
	"unicode.Deprecated":                          "unicode",
	"unicode.Deseret":                             "unicode",
	"unicode.Devanagari":                          "unicode",
	"unicode.Diacritic":                           "unicode",
	"unicode.Digit":                               "unicode",
	"unicode.Egyptian_Hieroglyphs":                "unicode",
	"unicode.Ethiopic":                            "unicode",
	"unicode.Extender":                            "unicode",
	"unicode.FoldCategory":                        "unicode",
	"unicode.FoldScript":                          "unicode",
	"unicode.Georgian":                            "unicode",
	"unicode.Glagolitic":                          "unicode",
	"unicode.Gothic":                              "unicode",
	"unicode.GraphicRanges":                       "unicode",
	"unicode.Greek":                               "unicode",
	"unicode.Gujarati":                            "unicode",
	"unicode.Gurmukhi":                            "unicode",
	"unicode.Han":                                 "unicode",
	"unicode.Hangul":                              "unicode",
	"unicode.Hanunoo":                             "unicode",
	"unicode.Hebrew":                              "unicode",
	"unicode.Hex_Digit":                           "unicode",
	"unicode.Hiragana":                            "unicode",
	"unicode.Hyphen":                              "unicode",
	"unicode.IDS_Binary_Operator":                 "unicode",
	"unicode.IDS_Trinary_Operator":                "unicode",
	"unicode.Ideographic":                         "unicode",
	"unicode.Imperial_Aramaic":                    "unicode",
	"unicode.In":                                  "unicode",
	"unicode.Inherited":                           "unicode",
	"unicode.Inscriptional_Pahlavi":               "unicode",
	"unicode.Inscriptional_Parthian":              "unicode",
	"unicode.Is":                                  "unicode",
	"unicode.IsControl":                           "unicode",
	"unicode.IsDigit":                             "unicode",
	"unicode.IsGraphic":                           "unicode",
	"unicode.IsLetter":                            "unicode",
	"unicode.IsLower":                             "unicode",
	"unicode.IsMark":                              "unicode",
	"unicode.IsNumber":                            "unicode",
	"unicode.IsOneOf":                             "unicode",
	"unicode.IsPrint":                             "unicode",
	"unicode.IsPunct":                             "unicode",
	"unicode.IsSpace":                             "unicode",
	"unicode.IsSymbol":                            "unicode",
	"unicode.IsTitle":                             "unicode",
	"unicode.IsUpper":                             "unicode",
	"unicode.Javanese":                            "unicode",
	"unicode.Join_Control":                        "unicode",
	"unicode.Kaithi":                              "unicode",
	"unicode.Kannada":                             "unicode",
	"unicode.Katakana":                            "unicode",
	"unicode.Kayah_Li":                            "unicode",
	"unicode.Kharoshthi":                          "unicode",
	"unicode.Khmer":                               "unicode",
	"unicode.L":                                   "unicode",
	"unicode.Lao":                                 "unicode",
	"unicode.Latin":                               "unicode",
	"unicode.Lepcha":                              "unicode",
	"unicode.Letter":                              "unicode",
	"unicode.Limbu":                               "unicode",
	"unicode.Linear_B":                            "unicode",
	"unicode.Lisu":                                "unicode",
	"unicode.Ll":                                  "unicode",
	"unicode.Lm":                                  "unicode",
	"unicode.Lo":                                  "unicode",
	"unicode.Logical_Order_Exception":             "unicode",
	"unicode.Lower":                               "unicode",
	"unicode.LowerCase":                           "unicode",
	"unicode.Lt":                                  "unicode",
	"unicode.Lu":                                  "unicode",
	"unicode.Lycian":                              "unicode",
	"unicode.Lydian":                              "unicode",
	"unicode.M":                                   "unicode",
	"unicode.Malayalam":                           "unicode",
	"unicode.Mandaic":                             "unicode",
	"unicode.Mark":                                "unicode",
	"unicode.MaxASCII":                            "unicode",
	"unicode.MaxCase":                             "unicode",
	"unicode.MaxLatin1":                           "unicode",
	"unicode.MaxRune":                             "unicode",
	"unicode.Mc":                                  "unicode",
	"unicode.Me":                                  "unicode",
	"unicode.Meetei_Mayek":                        "unicode",
	"unicode.Meroitic_Cursive":                    "unicode",
	"unicode.Meroitic_Hieroglyphs":                "unicode",
	"unicode.Miao":                                "unicode",
	"unicode.Mn":                                  "unicode",
	"unicode.Mongolian":                           "unicode",
	"unicode.Myanmar":                             "unicode",
	"unicode.N":                                   "unicode",
	"unicode.Nd":                                  "unicode",
	"unicode.New_Tai_Lue":                         "unicode",
	"unicode.Nko":                                 "unicode",
	"unicode.Nl":                                  "unicode",
	"unicode.No":                                  "unicode",
	"unicode.Noncharacter_Code_Point":             "unicode",
	"unicode.Number":                              "unicode",
	"unicode.Ogham":                               "unicode",
	"unicode.Ol_Chiki":                            "unicode",
	"unicode.Old_Italic":                          "unicode",
	"unicode.Old_Persian":                         "unicode",
	"unicode.Old_South_Arabian":                   "unicode",
	"unicode.Old_Turkic":                          "unicode",
	"unicode.Oriya":                               "unicode",
	"unicode.Osmanya":                             "unicode",
	"unicode.Other":                               "unicode",
	"unicode.Other_Alphabetic":                    "unicode",
	"unicode.Other_Default_Ignorable_Code_Point":  "unicode",
	"unicode.Other_Grapheme_Extend":               "unicode",
	"unicode.Other_ID_Continue":                   "unicode",
	"unicode.Other_ID_Start":                      "unicode",
	"unicode.Other_Lowercase":                     "unicode",
	"unicode.Other_Math":                          "unicode",
	"unicode.Other_Uppercase":                     "unicode",
	"unicode.P":                                   "unicode",
	"unicode.Pattern_Syntax":                      "unicode",
	"unicode.Pattern_White_Space":                 "unicode",
	"unicode.Pc":                                  "unicode",
	"unicode.Pd":                                  "unicode",
	"unicode.Pe":                                  "unicode",
	"unicode.Pf":                                  "unicode",
	"unicode.Phags_Pa":                            "unicode",
	"unicode.Phoenician":                          "unicode",
	"unicode.Pi":                                  "unicode",
	"unicode.Po":                                  "unicode",
	"unicode.PrintRanges":                         "unicode",
	"unicode.Properties":                          "unicode",
	"unicode.Ps":                                  "unicode",
	"unicode.Punct":                               "unicode",
	"unicode.Quotation_Mark":                      "unicode",
	"unicode.Radical":                             "unicode",
	"unicode.Range16":                             "unicode",
	"unicode.Range32":                             "unicode",
	"unicode.RangeTable":                          "unicode",
	"unicode.Rejang":                              "unicode",
	"unicode.ReplacementChar":                     "unicode",
	"unicode.Runic":                               "unicode",
	"unicode.S":                                   "unicode",
	"unicode.STerm":                               "unicode",
	"unicode.Samaritan":                           "unicode",
	"unicode.Saurashtra":                          "unicode",
	"unicode.Sc":                                  "unicode",
	"unicode.Scripts":                             "unicode",
	"unicode.Sharada":                             "unicode",
	"unicode.Shavian":                             "unicode",
	"unicode.SimpleFold":                          "unicode",
	"unicode.Sinhala":                             "unicode",
	"unicode.Sk":                                  "unicode",
	"unicode.Sm":                                  "unicode",
	"unicode.So":                                  "unicode",
	"unicode.Soft_Dotted":                         "unicode",
	"unicode.Sora_Sompeng":                        "unicode",
	"unicode.Space":                               "unicode",
	"unicode.SpecialCase":                         "unicode",
	"unicode.Sundanese":                           "unicode",
	"unicode.Syloti_Nagri":                        "unicode",
	"unicode.Symbol":                              "unicode",
	"unicode.Syriac":                              "unicode",
	"unicode.Tagalog":                             "unicode",
	"unicode.Tagbanwa":                            "unicode",
	"unicode.Tai_Le":                              "unicode",
	"unicode.Tai_Tham":                            "unicode",
	"unicode.Tai_Viet":                            "unicode",
	"unicode.Takri":                               "unicode",
	"unicode.Tamil":                               "unicode",
	"unicode.Telugu":                              "unicode",
	"unicode.Terminal_Punctuation":                "unicode",
	"unicode.Thaana":                              "unicode",
	"unicode.Thai":                                "unicode",
	"unicode.Tibetan":                             "unicode",
	"unicode.Tifinagh":                            "unicode",
	"unicode.Title":                               "unicode",
	"unicode.TitleCase":                           "unicode",
	"unicode.To":                                  "unicode",
	"unicode.ToLower":                             "unicode",
	"unicode.ToTitle":                             "unicode",
	"unicode.ToUpper":                             "unicode",
	"unicode.TurkishCase":                         "unicode",
	"unicode.Ugaritic":                            "unicode",
	"unicode.Unified_Ideograph":                   "unicode",
	"unicode.Upper":                               "unicode",
	"unicode.UpperCase":                           "unicode",
	"unicode.UpperLower":                          "unicode",
	"unicode.Vai":                                 "unicode",
	"unicode.Variation_Selector":                  "unicode",
	"unicode.Version":                             "unicode",
	"unicode.White_Space":                         "unicode",
	"unicode.Yi":                                  "unicode",
	"unicode.Z":                                   "unicode",
	"unicode.Zl":                                  "unicode",
	"unicode.Zp":                                  "unicode",
	"unicode.Zs":                                  "unicode",
	"url.Error":                                   "net/url",
	"url.EscapeError":                             "net/url",
	"url.Parse":                                   "net/url",
	"url.ParseQuery":                              "net/url",
	"url.ParseRequestURI":                         "net/url",
	"url.QueryEscape":                             "net/url",
	"url.QueryUnescape":                           "net/url",
	"url.URL":                                     "net/url",
	"url.User":                                    "net/url",
	"url.UserPassword":                            "net/url",
	"url.Userinfo":                                "net/url",
	"url.Values":                                  "net/url",
	"user.Current":                                "os/user",
	"user.Lookup":                                 "os/user",
	"user.LookupId":                               "os/user",
	"user.UnknownUserError":                       "os/user",
	"user.UnknownUserIdError":                     "os/user",
	"user.User":                                   "os/user",
	"utf16.Decode":                                "unicode/utf16",
	"utf16.DecodeRune":                            "unicode/utf16",
	"utf16.Encode":                                "unicode/utf16",
	"utf16.EncodeRune":                            "unicode/utf16",
	"utf16.IsSurrogate":                           "unicode/utf16",
	"utf8.DecodeLastRune":                         "unicode/utf8",
	"utf8.DecodeLastRuneInString":                 "unicode/utf8",
	"utf8.DecodeRune":                             "unicode/utf8",
	"utf8.DecodeRuneInString":                     "unicode/utf8",
	"utf8.EncodeRune":                             "unicode/utf8",
	"utf8.FullRune":                               "unicode/utf8",
	"utf8.FullRuneInString":                       "unicode/utf8",
	"utf8.MaxRune":                                "unicode/utf8",
	"utf8.RuneCount":                              "unicode/utf8",
	"utf8.RuneCountInString":                      "unicode/utf8",
	"utf8.RuneError":                              "unicode/utf8",
	"utf8.RuneLen":                                "unicode/utf8",
	"utf8.RuneSelf":                               "unicode/utf8",
	"utf8.RuneStart":                              "unicode/utf8",
	"utf8.UTFMax":                                 "unicode/utf8",
	"utf8.Valid":                                  "unicode/utf8",
	"utf8.ValidRune":                              "unicode/utf8",
	"utf8.ValidString":                            "unicode/utf8",
	"x509.CANotAuthorizedForThisName":             "crypto/x509",
	"x509.CertPool":                               "crypto/x509",
	"x509.Certificate":                            "crypto/x509",
	"x509.CertificateInvalidError":                "crypto/x509",
	"x509.ConstraintViolationError":               "crypto/x509",
	"x509.CreateCertificate":                      "crypto/x509",
	"x509.DSA":                                    "crypto/x509",
	"x509.DSAWithSHA1":                            "crypto/x509",
	"x509.DSAWithSHA256":                          "crypto/x509",
	"x509.DecryptPEMBlock":                        "crypto/x509",
	"x509.ECDSA":                                  "crypto/x509",
	"x509.ECDSAWithSHA1":                          "crypto/x509",
	"x509.ECDSAWithSHA256":                        "crypto/x509",
	"x509.ECDSAWithSHA384":                        "crypto/x509",
	"x509.ECDSAWithSHA512":                        "crypto/x509",
	"x509.EncryptPEMBlock":                        "crypto/x509",
	"x509.ErrUnsupportedAlgorithm":                "crypto/x509",
	"x509.Expired":                                "crypto/x509",
	"x509.ExtKeyUsage":                            "crypto/x509",
	"x509.ExtKeyUsageAny":                         "crypto/x509",
	"x509.ExtKeyUsageClientAuth":                  "crypto/x509",
	"x509.ExtKeyUsageCodeSigning":                 "crypto/x509",
	"x509.ExtKeyUsageEmailProtection":             "crypto/x509",
	"x509.ExtKeyUsageIPSECEndSystem":              "crypto/x509",
	"x509.ExtKeyUsageIPSECTunnel":                 "crypto/x509",
	"x509.ExtKeyUsageIPSECUser":                   "crypto/x509",
	"x509.ExtKeyUsageMicrosoftServerGatedCrypto":  "crypto/x509",
	"x509.ExtKeyUsageNetscapeServerGatedCrypto":   "crypto/x509",
	"x509.ExtKeyUsageOCSPSigning":                 "crypto/x509",
	"x509.ExtKeyUsageServerAuth":                  "crypto/x509",
	"x509.ExtKeyUsageTimeStamping":                "crypto/x509",
	"x509.HostnameError":                          "crypto/x509",
	"x509.IncompatibleUsage":                      "crypto/x509",
	"x509.IncorrectPasswordError":                 "crypto/x509",
	"x509.InvalidReason":                          "crypto/x509",
	"x509.IsEncryptedPEMBlock":                    "crypto/x509",
	"x509.KeyUsage":                               "crypto/x509",
	"x509.KeyUsageCRLSign":                        "crypto/x509",
	"x509.KeyUsageCertSign":                       "crypto/x509",
	"x509.KeyUsageContentCommitment":              "crypto/x509",
	"x509.KeyUsageDataEncipherment":               "crypto/x509",
	"x509.KeyUsageDecipherOnly":                   "crypto/x509",
	"x509.KeyUsageDigitalSignature":               "crypto/x509",
	"x509.KeyUsageEncipherOnly":                   "crypto/x509",
	"x509.KeyUsageKeyAgreement":                   "crypto/x509",
	"x509.KeyUsageKeyEncipherment":                "crypto/x509",
	"x509.MD2WithRSA":                             "crypto/x509",
	"x509.MD5WithRSA":                             "crypto/x509",
	"x509.MarshalECPrivateKey":                    "crypto/x509",
	"x509.MarshalPKCS1PrivateKey":                 "crypto/x509",
	"x509.MarshalPKIXPublicKey":                   "crypto/x509",
	"x509.NewCertPool":                            "crypto/x509",
	"x509.NotAuthorizedToSign":                    "crypto/x509",
	"x509.PEMCipher":                              "crypto/x509",
	"x509.PEMCipher3DES":                          "crypto/x509",
	"x509.PEMCipherAES128":                        "crypto/x509",
	"x509.PEMCipherAES192":                        "crypto/x509",
	"x509.PEMCipherAES256":                        "crypto/x509",
	"x509.PEMCipherDES":                           "crypto/x509",
	"x509.ParseCRL":                               "crypto/x509",
	"x509.ParseCertificate":                       "crypto/x509",
	"x509.ParseCertificates":                      "crypto/x509",
	"x509.ParseDERCRL":                            "crypto/x509",
	"x509.ParseECPrivateKey":                      "crypto/x509",
	"x509.ParsePKCS1PrivateKey":                   "crypto/x509",
	"x509.ParsePKCS8PrivateKey":                   "crypto/x509",
	"x509.ParsePKIXPublicKey":                     "crypto/x509",
	"x509.PublicKeyAlgorithm":                     "crypto/x509",
	"x509.RSA":                                    "crypto/x509",
	"x509.SHA1WithRSA":                            "crypto/x509",
	"x509.SHA256WithRSA":                          "crypto/x509",
	"x509.SHA384WithRSA":                          "crypto/x509",
	"x509.SHA512WithRSA":                          "crypto/x509",
	"x509.SignatureAlgorithm":                     "crypto/x509",
	"x509.SystemRootsError":                       "crypto/x509",
	"x509.TooManyIntermediates":                   "crypto/x509",
	"x509.UnhandledCriticalExtension":             "crypto/x509",
	"x509.UnknownAuthorityError":                  "crypto/x509",
	"x509.UnknownPublicKeyAlgorithm":              "crypto/x509",
	"x509.UnknownSignatureAlgorithm":              "crypto/x509",
	"x509.VerifyOptions":                          "crypto/x509",
	"xml.Attr":                                    "encoding/xml",
	"xml.CharData":                                "encoding/xml",
	"xml.Comment":                                 "encoding/xml",
	"xml.CopyToken":                               "encoding/xml",
	"xml.Decoder":                                 "encoding/xml",
	"xml.Directive":                               "encoding/xml",
	"xml.Encoder":                                 "encoding/xml",
	"xml.EndElement":                              "encoding/xml",
	"xml.Escape":                                  "encoding/xml",
	"xml.EscapeText":                              "encoding/xml",
	"xml.HTMLAutoClose":                           "encoding/xml",
	"xml.HTMLEntity":                              "encoding/xml",
	"xml.Header":                                  "encoding/xml",
	"xml.Marshal":                                 "encoding/xml",
	"xml.MarshalIndent":                           "encoding/xml",
	"xml.Marshaler":                               "encoding/xml",
	"xml.MarshalerAttr":                           "encoding/xml",
	"xml.Name":                                    "encoding/xml",
	"xml.NewDecoder":                              "encoding/xml",
	"xml.NewEncoder":                              "encoding/xml",
	"xml.ProcInst":                                "encoding/xml",
	"xml.StartElement":                            "encoding/xml",
	"xml.SyntaxError":                             "encoding/xml",
	"xml.TagPathError":                            "encoding/xml",
	"xml.Token":                                   "encoding/xml",
	"xml.Unmarshal":                               "encoding/xml",
	"xml.UnmarshalError":                          "encoding/xml",
	"xml.Unmarshaler":                             "encoding/xml",
	"xml.UnmarshalerAttr":                         "encoding/xml",
	"xml.UnsupportedTypeError":                    "encoding/xml",
	"zip.Compressor":                              "archive/zip",
	"zip.Decompressor":                            "archive/zip",
	"zip.Deflate":                                 "archive/zip",
	"zip.ErrAlgorithm":                            "archive/zip",
	"zip.ErrChecksum":                             "archive/zip",
	"zip.ErrFormat":                               "archive/zip",
	"zip.File":                                    "archive/zip",
	"zip.FileHeader":                              "archive/zip",
	"zip.FileInfoHeader":                          "archive/zip",
	"zip.NewReader":                               "archive/zip",
	"zip.NewWriter":                               "archive/zip",
	"zip.OpenReader":                              "archive/zip",
	"zip.ReadCloser":                              "archive/zip",
	"zip.Reader":                                  "archive/zip",
	"zip.RegisterCompressor":                      "archive/zip",
	"zip.RegisterDecompressor":                    "archive/zip",
	"zip.Store":                                   "archive/zip",
	"zip.Writer":                                  "archive/zip",
	"zlib.BestCompression":                        "compress/zlib",
	"zlib.BestSpeed":                              "compress/zlib",
	"zlib.DefaultCompression":                     "compress/zlib",
	"zlib.ErrChecksum":                            "compress/zlib",
	"zlib.ErrDictionary":                          "compress/zlib",
	"zlib.ErrHeader":                              "compress/zlib",
	"zlib.NewReader":                              "compress/zlib",
	"zlib.NewReaderDict":                          "compress/zlib",
	"zlib.NewWriter":                              "compress/zlib",
	"zlib.NewWriterLevel":                         "compress/zlib",
	"zlib.NewWriterLevelDict":                     "compress/zlib",
	"zlib.NoCompression":                          "compress/zlib",
	"zlib.Writer":                                 "compress/zlib",
}
